/**********************************************************************
 *
 *        unity.h
 *
 *        Prototypes for the Unity accessor functions found in unity.c.
 *        Register and bit definitions for the Unity ASIC.
 * 
 **********************************************************************/

#ifndef _UNITY_H
#define _UNITY_H

#define Bit(n) (1<<(n))

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

/* Use these macro's to read/write PCI memory space.  */
#define READL( ADDR        )  __builtin_ldwio( (ADDR))
#define READW( ADDR        )  __builtin_ldhuio((ADDR))
#define READB( ADDR        )  __builtin_ldbuio((ADDR))

#define WRITEL( DATA, ADDR  )  __builtin_stwio((ADDR),(DATA))
#define WRITEW( DATA, ADDR  )  __builtin_sthio((ADDR),(DATA))
#define WRITEB( DATA, ADDR  )  __builtin_stbio((ADDR),(DATA))

//=====================================

#define UnityClock 44236800L
#define UnityPrescale 0
#define UnityBaudDivisor(baud) (((UnityClock/(16*(UnityPrescale+1)))/baud)-1)

/*  Port Register Offsets */
#define RCV_DATA_BLOCK             0x0000  // + (UNITY_BASE_ADDR) + (ch * 0x40)
#define XMIT_DATA_BLOCK            0x0000
#define RCV_DATA_WORD              0x0004
#define XMIT_DATA_WORD             0x0004
#define RCV_DATA_BYTE_STATUS       0x0008
#define XMIT_DATA_BYTE             0x0008
#define RCV_FIFO_COUNT             0x000C
#define XMIT_FIFO_COUNT            0x000E
#define INT_ID_CHAN_STATUS         0x0010
#define CHANSTATUS		   0x0010
#define INT_IDENT		   0x0012
#define XMIT_CTRL_RCV_CTRL         0x0014
#define RX_CONTROL		   0x0014
#define TX_CONTROL 		   0x0016
#define UART_CTRL                  0x0018
#define BAUD_RATE_GENERATOR        0x001C
#define INTERCHAR_RELOAD_VAL       0x0020
#define PRIORITY_XMIT_WORD         0x0024
#define PRIORITY_XMIT_BYTE         0x0028

/*  Bit defines for RCV_DATA_BYTE_STATUS (offset 0x08) */
#define RCV_BYTE_PARITY_ERROR      Bit(8)
#define RCV_BYTE_RCV_OVERRUN       Bit(9)
#define RCV_BYTE_FRAMING_ERROR     Bit(10)
#define RCV_BYTE_BREAK_DETECTED    Bit(11)
#define RCV_BYTE_ADDRESS_BYTE      Bit(12)
#define RCV_BYTE_TIMEOUT_DETECTED  Bit(13)

#define Rcv_Byte_Errors    ( RCV_BYTE_FRAMING_ERROR | \
                             RCV_BYTE_RCV_OVERRUN   | \
			     RCV_BYTE_PARITY_ERROR )

/*  Bit defines for INT_ID_CHAN_STATUS (offset 0x10 bits 0-15) */
#define CHAN_STATUS_RCV_DATA_AVAIL           Bit(0)
#define CHAN_STATUS_XMIT_SHIFT_REG_EMPTY     Bit(1)
#define CHAN_STATUS_XMIT_FIFO_EMPTY          Bit(2)
#define CHAN_STATUS_CD_ACTIVE                Bit(3)
#define CHAN_STATUS_DSR_ACTIVE               Bit(4)
#define CHAN_STATUS_CTS_ACTIVE               Bit(5)
#define CHAN_STATUS_RI_ACTIVE                Bit(6)
#define CHAN_STATUS_RTS_IN_STATUS            Bit(7)
#define CHAN_STATUS_PARITY_ERROR             Bit(8)
#define CHAN_STATUS_FRAMING_ERROR            Bit(9)
#define CHAN_STATUS_BREAK_DETECTED           Bit(10)
#define CHAN_STATUS_COMPARE1                 Bit(11)
#define CHAN_STATUS_COMPARE2                 Bit(12)
#define CHAN_STATUS_RCV_OVERFLOW             Bit(13)
#define CHAN_STATUS_PRIORITY_BUFFER_FULL     Bit(14)

#define ChanStatus_RxErr  ( CHAN_STATUS_PARITY_ERROR   | \
                            CHAN_STATUS_FRAMING_ERROR  | \
			    CHAN_STATUS_BREAK_DETECTED | \
			    CHAN_STATUS_RCV_OVERFLOW )


/* Bit defines for INT ID (offset 0x10  bits 16-31) */
#define INT_DSR_CHANGE             Bit(16)
#define INT_CTS_CHANGE             Bit(17)
#define INT_CD_CHANGE              Bit(18)
#define INT_SPECIAL_RECEIVE        Bit(19)
#define INT_XMIT_FIFO_EMPTY        Bit(20)
#define INT_RCV_FIFO_TRIGGER       Bit(21)
#define INT_RI_CHANGE              Bit(22)
#define INT_RCV_DATA_RDY           Bit(23)
#define INT_ADDR_RCVD              Bit(24)
#define INT_XMIT_SHIFTREG_EMPTY    Bit(25)
#define INT_XMIT_FIFO_TRIGGER      Bit(26)
#define INT_INTERCHAR_TIMEOUT      Bit(27)

#define INTID_MODEM   ( INT_DSR_CHANGE | \
			INT_CTS_CHANGE | \
			INT_CD_CHANGE )			


/*  Bit defines for XMIT_CTRL_RCV_CTRL (offset 0x14) */
#define MDM_CHANGE_INT_EN          Bit(0)
#define SPECIAL_RCV_INT_EN         Bit(2)
#define RCV_FIFO_TRIGGER0          Bit(3)
#define RCV_FIFO_TRIGGER1          Bit(4)
#define RCV_PROC_EN                Bit(5)
#define RTS_FLOW_EN                Bit(6)
#define DTR_FLOW_EN                Bit(7)
#define ONE_STOP_BIT_EN            Bit(8)
#define RCV_DATA_RDY_INT_EN        Bit(9)
#define ADDRESS_INT_EN             Bit(10)
#define INTERCHAR_TIMEOUT_INT_EN   Bit(11)
#define INTERCHAR_STATUS_EN        Bit(12)

#define XMIT_FIFO_TRIGGER1         Bit(16)
#define XMIT_FIFO_TRIGGER2         Bit(17)
#define DSR_FLOW_EN                Bit(18)
#define XMIT_SHIFTREG_INT_EN       Bit(19)
#define XMIT_FIFO_INT_EN           Bit(20)
#define RTS_TOGGLE_POLARITY        Bit(21)
#define RTS_TOGGLE                 Bit(22)
#define CTS_FLOW_EN                Bit(23)
#define XMIT_EN                    Bit(24)
#define SET_RTS_ACTIVE             Bit(25)
#define SET_DTR_ACTIVE             Bit(26)
#define LOCAL_LOOPBACK             Bit(27)
#define BREAK_ON                   Bit(28)
#define FORCE_PARITY               Bit(29)
#define EVEN_PARITY                Bit(30)
#define PARITY_ON                  Bit(31)


/*  Bit defines for UART_CTRL (offset 0x18) */
#define INTERFACE_SELECT0          Bit(0)
#define INTERFACE_SELECT1          Bit(1)
#define INTERFACE_SELECT2          Bit(2)
#define FLUSH_RCV_FIFO             Bit(3)
#define FLUSH_XMIT_FIFO            Bit(4)
#define RESET_CHANNEL              Bit(5)
#define RCV_PROC_XMIT_EN           Bit(6)
#define FLUSH_PRIORITY_FIFO        Bit(7)
#define DATA_BITS0                 Bit(8)
#define DATA_BITS1                 Bit(9)
#define STOP_BITS                  Bit(10)


/*  Bit defines for the XMIT FIFO Trigger level of the XMIT_CTRL_RCV_CTRL register */
/*  Translations bits 16 and 17  offset 0x14 */
#define UXMIT_FIFO_TRIG_MASK       (Bit(17) | Bit(16))
#define UXMIT_FIFO_TRIG_DISABLED   (0x00)
#define UXMIT_FIFO_TRIG_1_8        (Bit(16))
#define UXMIT_FIFO_TRIG_1_4        (Bit(17))
#define UXMIT_FIFO_TRIG_1_2        (Bit(17) | Bit(16))

/*  Bit defines for the RCV FIFO Trigger level of the XMIT_CTRL_RCV_CTRL register */
/*  Translations bits 3 and 4  offset 0x14 */
#define URCV_FIFO_TRIG_MASK       (Bit(4) | Bit(3))
#define URCV_FIFO_TRIG_DISABLED   (0x00)
#define URCV_FIFO_TRIG_1_CHAR     (Bit(3))
#define URCV_FIFO_TRIG_1_2        (Bit(4))
#define URCV_FIFO_TRIG_7_8        (Bit(4) | Bit(3))

/*  Data bit defines, unique to UNITY.  */
/*  Translations bits 8 and 9  offset 0x18 */
#define DATABITS_MASK (Bit(9) | Bit(8))	
#define DATABITS_5    (0x00)
#define DATABITS_6    (Bit(8))
#define DATABITS_7    (Bit(9))
#define DATABITS_8    (Bit(9) | Bit(8))

/*  RS mode bit defines, unique to UNITY.  */
/*  Translations bits 0, 1, and 2  offset 0x18 */
#define URS_MASK             (Bit(2) | Bit(1) | Bit(0))	
#define URS_DISABLED         (0x00)
#define URS_RS232            (Bit(0))
#define URS_RS422            (Bit(1))
#define URS_RS485_FD_MASTER  (Bit(1))
#define URS_RS485_FD_SLAVE   (Bit(1) | Bit(0))
#define URS_RS485_HD         (Bit(2))
#define URS_RS485_TEST       (Bit(2) | Bit(0))
#define URS_RESERVED         (Bit(2) | Bit(1))
#define URS_DEBUG            (Bit(2) | Bit(1) | Bit(0))

/*  Unity Receive Processor Microcode Function Addresses and Values  */
#define URP_RCV_FIFO                    0x37    /*  Receive FIFO enable/disable  */
#define URP_RCV_FIFO_ENABLE             0x08
#define URP_RCV_FIFO_DISABLE            0x81

#define URP_TX_SW_FLOW_CONTROL          0x02    /*  Transmit software flow control enable/disable */
#define URP_TX_SW_FLOW_CONTROL_ENABLE   0x81 
#define URP_TX_SW_FLOW_CONTROL_DISABLE  0x9D

#define URP_RX_SW_FLOW_CONTROL          0x0C    /*  Receive software flow control enable/disable */
#define URP_RX_SW_FLOW_CONTROL_ENABLE   0x81 
#define URP_RX_SW_FLOW_CONTROL_DISABLE  0x8D

#define URP_XANY                        0x07    /*  Software flow control XANY enable/disable */
#define URP_XANY_ENABLE                 0x83
#define URP_XANY_DISABLE                0x98

#define URP_COMPARE1                    0x21   /*  Compare #1 enable/disable */
#define URP_COMPARE1_ENABLE             0x81
#define URP_COMPARE1_DISABLE            0x89

#define URP_COMPARE_IRQ1                0x25   /*  Compare #1 IRQ enable/disable */
#define URP_COMPARE_IRQ1_ENABLE         0x11
#define URP_COMPARE_IRQ1_DISABLE        0x81

#define URP_COMPARE_ACTION1             0x26   /*  Compare #1 action */
#define URP_COMPARE_ACTION1_SAVE        0x91
#define URP_COMPARE_ACTION1_REPLACE     0x81
#define URP_COMPARE_ACTION1_DISCARD     0x0A

#define URP_COMPARE2                    0x2A   /*  Compare #2 enable/disable */
#define URP_COMPARE2_ENABLE             0x81
#define URP_COMPARE2_DISABLE            0x89

#define URP_COMPARE_IRQ2                0x2E   /*  Compare #2 IRQ enable/disable */
#define URP_COMPARE_IRQ2_ENABLE         0x10
#define URP_COMPARE_IRQ2_DISABLE        0x81

#define URP_COMPARE_ACTION2             0x2F   /*  Compare #2 action */
#define URP_COMPARE_ACTION2_SAVE        0x88
#define URP_COMPARE_ACTION2_REPLACE     0x81
#define URP_COMPARE_ACTION2_DISCARD     0x0A

#define URP_STRIP_NULLS                 0x33   /*  Strip Nulls */
#define URP_STRIP_NULLS_ENABLE          0xC2
#define URP_STRIP_NULLS_DISABLE         0x84

/*  Unity Receive Processor Microcode Variable Addresses  */
#define URP_MASK           0x20
#define URP_COMPARE_BYTE1  0x23
#define URP_REPLACE_BYTE1  0x28
#define URP_COMPARE_BYTE2  0x2C
#define URP_REPLACE_BYTE2  0x31
#define URP_XMIT_XOFF_CHAR 0x11
#define URP_RCV_XOFF_CHAR  0x04
#define URP_XMIT_XON_CHAR  0x16
#define URP_RCV_XON_CHAR   0x06

#define FLOW_CONTROL_NONE    0 
#define FLOW_CONTROL_RTSCTS  1
#define FLOW_CONTROL_DTRDSR  2
#define FLOW_CONTROL_XONXOFF 3

#endif //_UNITY_H
