#include <redboot.h>
#include <net/net.h>
#include <fis.h>
#include <cyg/hal/hal_arch.h>
#include <cyg/hal/hal_intr.h>
#include <cyg/hal/hal_cache.h>
#include <cyg/hal/iic.h>
#include "ident.h"
#include "admin.h"
#include "version.h"

//  For information on console_select, disabling autoloading, etc. see notes at bottom

#define NumElements(a)  ((sizeof (a))/(sizeof ((a)[0])))

extern char IdentString[];

/****  GLOBALS  *****/
int disableAutoload;
int disableDriverload;
int terseMode;
tIdent ident;
int ONLY_232;				// Serial Hub 8 or 16

/****  EXTERNS  *****/
extern int console_echo;
extern unsigned volatile SYSCFG;


// stubs for configurations without port diagnostics
#if !defined(CYGPKG_HAL_ARM_DMRTS) && !defined(CYGPKG_COMTROL_DM2_UNITYDIAG)
int cyg_hal_plf_loopback(int port, int mode, int doMC, int doRI, int useAiopic) {return -1;}
int cyg_hal_plf_485_test(int port1, int port2, int use4530) {return -1;}
#endif

#define Bit(n) (1<<(n))

      
#if defined(CYGPKG_HAL_ARM_DMRTS)
  #include <cyg/hal/ks32c5000.h>
  extern volatile unsigned PioData;
  extern volatile unsigned PioMode;
  #define LEDEnable()  PioMode |= Bit(15)
  #define LEDToggle()  PioData ^= Bit(15)
  #define RESTORE_IRQ_MASK        Bit(3)
  int HAS_RESTORE_FUNC;                                 // unit has restore funtion
  extern volatile tInterruptController ks32c5000_int;   // Interrupt controller registers
  extern volatile unsigned long IOPCON;                 // I/O Port Control reg
#elif defined(CYGPKG_COMTROL_DM2_FLASH)
  cyg_uint32 DM2WatchdogRegShadow;
  #define LedBase ((cyg_uint32*)0x20018c0)
  #define LedDirReg  (LedBase+1)
  #define LedDataReg (LedBase+0)
  #define LEDEnable()  __builtin_stwio(LedDirReg, 1)
  #define LEDToggle()  __builtin_stwio(LedDataReg, (__builtin_ldwio(LedDataReg) ^ 1))
          // defines added to manipulate the restore interrup
  #define RD_32_DIRECT( ADDR        )  __builtin_ldwio(   (void*)(ADDR) )
  #define WR_32_DIRECT( ADDR, DATA  )  __builtin_stwio(  (void*)(ADDR), (DATA) )
  #define Restore_base  0x2001840
#else
  #warning "unkown platform"
  #define LEDEnable() /* noop */
  #define LEDToggle() /* noop */
#endif



static int is4530;  // what kind of processor?

extern void do_flash_init(void);

extern cyg_bool redboot_telnet_disable;
extern char *redboot_telnet_password;
extern unsigned redboot_telnet_timeout;

void dm2reset(void)
{
  diag_printf("dm2reset()\n");
  DM_BOARD_RESET();
  while (1)
    ;
}

// underscore so that this one is first in init sequence
static void _identInit(void)
{
  unsigned long ipAddr,netMask,gateway;

#if defined(CYGPKG_HAL_NIOS2)
  CYGACC_CALL_IF_RESET_SET(dm2reset);
#endif
  
  #if defined(CYGPKG_HAL_ARM_DMRTS)
  is4530 = SYSCFG & Bit(27);
  if (0)
    {
      extern volatile unsigned char EpldInterfaceSelect0;
      EpldInterfaceSelect0 = 0;
    }
  #endif

#if defined(CYGPKG_REDBOOT_FLASH)
  do_flash_init();
#endif  
  
  
#if defined(CYGPKG_HAL_ARM_DMRTS)
  iicInit();
#endif
  
  // read id stuff from flash
  getRedbootDelay(&ident.timeoutValue);
  getTelnetTimeout(&ident.telnetTimeout);
#if defined(CYGPKG_REDBOOT_NETWORKING)
  redboot_telnet_timeout = 1000 * ident.telnetTimeout;
#endif  
  getPassword(ident.password);
  getTelnetEnable(&ident.telnetEnable);
  getSecureDataEnable(&ident.secureDataEnable);
  getSecureConfigEnable(&ident.secureConfigEnable);
  getSnmpEnable(&ident.snmpEnable);
  getDhcpEnable(&ident.dhcpEnable);
  getModelNumber(&ident.modelId);
#if defined(CYGPKG_HAL_NIOS2)
  getSerNumDM2(&ident.serNumDM2);
  getSerNumPrefixDM2(&ident.serNumPrefixDM2);
#endif
  getBoardRev(&ident.boardRev);
  getAppEnableKey(&ident.appEnblKey);

#if defined(CYGPKG_HAL_ARM_DMRTS)
  #define ArchId 3
#elif defined(CYGPKG_COMTROL_DM2_FLASH)
  #define ArchId 4
#else
  #error "unkown platform"
#endif  

  ident.archId = ArchId;
  ident.programId = 0;
  
  switch (ident.modelId)
    {
    case 5002110:		//  DM 8 port DB9
    case 5002112:		//  DM 8 port RJ45
    case 5002115:		//  DM SH 8 port DB9
    case 5002330:		//  DM Pro 8 port DB9
      ident.numPorts = 8;
      break;
    case 5002111:		//  DM 4 port DB9
    case 5002113:		//  DM 4 port RJ45
    case 5002116:		//  IAD UP 4 Port
    case 5002117:		//  DM500 4 Port DB9
      ident.numPorts = 4;
      break;
    case 5002120:		//  DM 16 port RJ45
    case 5002240:		//  DM RTS RM 16 port
    case 5002315:		//  DM SH 16 port DB9
    case 5002325:		//  DM Pro 16 port RJ45
      ident.numPorts = 16;
      break;
    case 5002241:		//  DM RTS RM 32
      ident.numPorts = 32;
      break;
    case 5002525:               //  DM RTS 2P 1E (Configuration 1)
    case 5002535:               //  DM RTS 2P 2E (Configuration 2)
    case 5002595:               //  DM RTS 2P 1E (Configuration 1) DB9
    case 5002600:               //  DM RTS 2P 2E (Configuration 2) DB9
     ident.numPorts = 2;
      break;
    default:
      // all other models are one-ports
      ident.numPorts = 1;
    }
              
  if (ident.modelId == 5002115 || ident.modelId == 5002315)
    ONLY_232 = 1;
  else
    ONLY_232 = 0;
    
  // So far only DM RTS 2P models have restore function
  HAS_RESTORE_FUNC = ((ident.modelId == 5002525) ||
                      (ident.modelId == 5002535) ||
		      (ident.modelId == 5002595) ||
		      (ident.modelId == 5002600));

  ident.versionMajor = VERSION_MAJOR;
  ident.versionMinor = VERSION_MINOR;

    // Set defaults as appropriate
#if defined(CYGSEM_REDBOOT_DEFAULT_NO_BOOTP)
    use_bootp = false;
#else
    use_bootp = true;
#endif

#if defined(CYGPKG_REDBOOT_NETWORKING)
  
  unsigned long temp_ipaddr, temp_netmask, temp_gateway;
  getIpInfo(&temp_ipaddr, &temp_netmask, &temp_gateway);
  ipAddr = htonl(temp_ipaddr);
  netMask = htonl(temp_netmask);
  gateway = htonl(temp_gateway);

  gdb_port = 23;
  net_debug = 0;
  have_net = 1;
  
  if ( (ident.dhcpEnable == 0) && (ipAddr == 0x00000000) ) {
  use_bootp = 0;
  } else if ( ipAddr == 0xffffffff ) {
    have_net = 0;
  }

  memcpy(__local_ip_addr,&ipAddr,sizeof __local_ip_addr);
  memcpy(__local_ip_mask,&netMask,sizeof __local_ip_addr);
  memcpy(__local_ip_gate,&gateway,sizeof __local_ip_addr);
  
  redboot_telnet_disable = !ident.telnetEnable || ident.secureConfigEnable;
  redboot_telnet_password = ident.password;
#endif  
  
  LEDEnable();
  loadStatusInit();
}


// function to return feature bitmask for ID response

cyg_uint32 getFeatureBits(void)
{
  cyg_uint32 r = 0;
  if (ident.telnetEnable)       r |= Bit(0);
  if (ident.snmpEnable)         r |= Bit(1);
  if (ident.secureDataEnable)   r |= Bit(2);
  if (ident.secureConfigEnable) r |= Bit(3);
  if (ident.dhcpEnable)			r |= Bit(4);
  return r;
}

#if defined(CYGPKG_HAL_NIOS2)
// print Unix style timestamp as a nice string
void showTimestamp(unsigned long timestamp)
{
  static unsigned   DaysToMonth[] = 
    {
      0,     
      0+31,
      0+31+28,
      0+31+28+31,
      0+31+28+31+30,
      0+31+28+31+30+31,
      0+31+28+31+30+31+30,
      0+31+28+31+30+31+30+31,
      0+31+28+31+30+31+30+31+31,
      0+31+28+31+30+31+30+31+31+30,
      0+31+28+31+30+31+30+31+31+30+31,
      0+31+28+31+30+31+30+31+31+30+31+30,
      0+31+28+31+30+31+30+31+31+30+31+30+31
    };
  static const char *dayName[] = {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
  static const char *monthName[] = {"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
  
  unsigned long hour;
  unsigned long day;
  unsigned long minute;
  unsigned long second;
  unsigned long month;
  unsigned long year;


  unsigned long whole_minutes;
  unsigned long whole_hours;
  unsigned long whole_days;
  unsigned long whole_days_since_1968;
  unsigned long leap_year_periods;
  unsigned long days_since_current_lyear;
  unsigned long whole_years;
  unsigned long days_since_first_of_year;
  unsigned long days_to_month;
  unsigned long day_of_week;

  whole_minutes = timestamp / 60;
  second = timestamp - (60 * whole_minutes);

  whole_hours = whole_minutes / 60;
  minute = whole_minutes - (60 * whole_hours);

  whole_days = whole_hours / 24;
  hour = whole_hours - (24 * whole_days);

  whole_days_since_1968 = whole_days + 365 + 366;
  leap_year_periods = whole_days_since_1968 / ((4 * 365) + 1);

  days_since_current_lyear = whole_days_since_1968 % ((4 * 365) + 1);
  
  // if days are after a current leap year then add a leap year period
  if ((days_since_current_lyear >= (31 + 29)))
    leap_year_periods++;

  whole_years = (whole_days_since_1968 - leap_year_periods)  / 365;
  days_since_first_of_year = whole_days_since_1968
                             - (whole_years * 365) - leap_year_periods;

  if ((days_since_current_lyear <= 365) && (days_since_current_lyear >= 60))
    days_since_first_of_year++;

  year = whole_years + 68;

  // setup for a search for what month it is based on how many
  // days have past within the current year

  month = 13;
  days_to_month = 366;
  while (days_since_first_of_year < days_to_month)
    {
      month--;
      days_to_month = DaysToMonth[month];
      if ((month >= 2) && ((year % 4) == 0))
        days_to_month++;
    }
  day = days_since_first_of_year - days_to_month + 1;

  day_of_week = (whole_days  + 4) % 7;
  year += 1900;

  diag_printf("%s %s %2lu ", dayName[day_of_week], monthName[month], day);
  diag_printf("%02lu:%02lu:%02lu %lu",hour,minute,second,year);
}
#endif


// this needs to run just before net_init, hence the funny name
static void net_inis_identInit(void)  
{
#if defined(CYGPKG_REDBOOT_NETWORKING)
  getMacAddress(__local_enet_addr);
  __local_enet_addr[0] &= 0x7f;
  // pre-load the Ethernet address so Ethernet driver finds it.
# if defined(CYGPKG_HAL_ARM_DMRTS)
# define CAM_BaseAddr 0x7ff9100
  unsigned *p = (unsigned *)CAM_BaseAddr;
  p[0] = (__local_enet_addr[0]<<24) | (__local_enet_addr[1]<<16) | (__local_enet_addr[2]<<8) | __local_enet_addr[3];
  p[1] &= 0x0000ffff;
  p[1] |= (__local_enet_addr[4]<<24) | (__local_enet_addr[5]<<16);
# else
# error "unkown platform -- how do I set up the Ethernet address?"  
# endif
#endif

#if defined(CYGPKG_HAL_ARM_DMRTS)
  if (HAS_RESTORE_FUNC)
    {
      // set up extern interrupt 3 to rising edge
      IOPCON &= ~(0x1f << 15);
      IOPCON |= (0x19 << 15);
      ks32c5000_int.intmsk &= ~RESTORE_IRQ_MASK;
    }
#endif
}

static unsigned long _cksum(unsigned long *buf, int len)
{
  unsigned long cksum = 0;
  // Round 'len' up to multiple of longwords
  len = (len + (sizeof(unsigned long)-1)) / sizeof(unsigned long);
  while (len-- > 0)
    cksum ^= *buf++;
  return cksum;
}

static int load_program(char *name, unsigned long **entryPtr)
{
  #if defined(CYGOPT_REDBOOT_FIS)
  
  struct fis_image_desc *img;
  unsigned long mem_addr;
  unsigned long cksum;
  
  
  img = fis_lookup(name, NULL);
  if (!img) {
      diag_printf("load_program() - lookup failed \n");
      return 1;
  }

  mem_addr = img->mem_base;

  // Load image from FLASH into RAM
  if ((mem_addr < (unsigned long)ram_start) ||
      ((mem_addr+img->size) >= (unsigned long)ram_end)) {
      diag_printf("load_program() - address errors \n");
      return 2;
  }
  
  memcpy((void *)mem_addr, (void *)img->flash_base, img->size);
  if (entryPtr)
    *entryPtr = (unsigned long *)img->entry_point;
  cksum = _cksum((unsigned long *)mem_addr, img->data_length);
  if (cksum != img->file_cksum) {
      diag_printf("load_program() - checksum error - calc: 0x%x  file_cksum: 0x%x \n", cksum, img->file_cksum);
      return 3;
  }
 #endif
  return 0;
}

static void run_program(unsigned long *entryPtr)
{
  typedef void code_fun(void);
  unsigned long oldints;
  code_fun *fun = (code_fun*)entryPtr;
  HAL_DISABLE_INTERRUPTS(oldints);
  HAL_DCACHE_SYNC();
  HAL_ICACHE_DISABLE();
  HAL_DCACHE_DISABLE();
  HAL_DCACHE_SYNC();
  HAL_ICACHE_INVALIDATE_ALL();
  HAL_DCACHE_INVALIDATE_ALL();
  (*fun)();
}

/*
 *  Idle loop.  In eCos 1.3.1 called every 1ms.  
 */

static void idleLoop(bool is_idle)
{
  static unsigned next;
  static int loadAttempted;
  unsigned now = get_ms_ticks();
  unsigned long *entry;
  static int loopCount;
  static int old_console_selected;

  if ((int)(next-now) < 0)
    {
      /*  Toggle LED every 250ms */
      next = now + 250;
      LEDToggle();
      ++loopCount;
    }
  
  if (old_console_selected != console_selected)
    disableAutoload = disableDriverload = old_console_selected = console_selected;
  
  if (!disableAutoload && !loadAttempted && ident.timeoutValue && loopCount > ident.timeoutValue*4)
    {
      char *name = "default";
      if (load_program(name,&entry) == 0)
	run_program(entry);
      loadAttempted = 1;
    }

#if defined(CYGPKG_HAL_ARM_DMRTS)
  if (HAS_RESTORE_FUNC && (ks32c5000_int.intpnd & RESTORE_IRQ_MASK))
    {
      ks32c5000_int.intmsk |= Bit(21);                  // disable all interrupts
      restoreFactoryDefault();
      DM_BOARD_RESET();
    }
#elif defined(CYGPKG_HAL_NIOS2)
  if( (RD_32_DIRECT( Restore_base + 0xC) & 1) )
    {
      WR_32_DIRECT( Restore_base + 8, 0x0);	        //disable pio interrupt 

      putIpInfo(0xC0A8FAFA, 0xFFFF0000, 0xC0A8FA01);    // restore default ip, mask, gateway
      putPassword("");          // clear password
      putAuthMethod(1);         // authentication to none
      putTelnetEnable(1);       // telnet enabled
      putSecureDataEnable(0);   // encryption disabled
      putSecureConfigEnable(0); // encryption disabled
      putRedbootDelay(15);      // start up delay to 15 seconds
      putTelnetTimeout(300);    // telnet time out to 300 seconds

      dm2reset();
    }
#endif
}

RedBoot_init(_identInit,RedBoot_INIT_FIRST);
RedBoot_init(net_inis_identInit,RedBoot_INIT_LAST);
RedBoot_idle(idleLoop,RedBoot_IDLE_LAST);

unsigned fromHexAscii(char *p)
{
  unsigned r = 0;
  
  while (*p)
    {
      r <<= 4;
      if (*p >= '0' && *p <= '9')
        r += *p - '0';
      else if (*p >= 'a' && *p <= 'f')
        r += 10 + *p - 'a';
      else if (*p >= 'A' && *p <= 'F')
        r += 10 + *p - 'A';
      ++p;
    }
  return r;
}


int external_console(void)
  {
    int cur_console;
    cur_console = CYGACC_CALL_IF_SET_CONSOLE_COMM(CYGNUM_CALL_IF_SET_COMM_ID_QUERY_CURRENT);
    if (cur_console < 2)
      return 0;
    printf("- Item is read only.\n");
    return 1;
  }


/*  DeviceMaster short help */
static void do_short_help(int argc, char *argv[])
{
    diag_printf("help     - Complete help \n");
    diag_printf("reset    - Reset device \n");
    diag_printf("ip       - View/Set IP Address (ip <addr> <mask> <gateway>) \n");
    diag_printf("password - View/Set password (just 'password' to clear) \n");
    diag_printf("timeout  - View/Set time (in seconds) until application loads \n");
    diag_printf("history  - CTRL-P previous cmd, CTRL-N next cmd \n");
//    diag_printf("         - load -r -b 0 -h 10.0.0.3 socket.bin \n");
}

RedBoot_cmd("?","Short help","",do_short_help);

static void do_mac_cmd(int argc, char *argv[])
{
  unsigned char mac_addr[6];
  int i;
  
  --argc;
  ++argv;
  
  if (argc == 6)
    {
      if (external_console())
        return;
      for (i=0; i<6; ++i)
        mac_addr[i] = fromHexAscii(argv[i]);
      putMacAddress(mac_addr);
    }
  if (argc == 0 || argc == 6)
    {
      getMacAddress(mac_addr);
      printf(terseMode ? "+ " : "MAC Addr: ");
      for (i=0; i<6; ++i)
        printf("%02x ",mac_addr[i]);
      printf("\n");
    }
  else
    printf("- Error: MAC address format: XX XX XX XX XX XX\n");
}

static char WrongNumParam[] = "- Error: wrong number of parameters\n";
static char BadNumber[] = "- Error: not a valid number\n";

static void badNumberRecieved(int val)
{
    printf("- Error: invalid number entered - %d \n", val);
}

RedBoot_cmd("mac","Show/set Ethernet MAC address","[XX XX XX XX XX XX XX]",do_mac_cmd);
                           
static void do_model_cmd(int argc, char *argv[])
{
  unsigned long val;
  
  --argc;
  ++argv;
  
  if (argc == 1)
    {
      if (external_console())
        return;
      if (parse_num(argv[0],&val,NULL,NULL))
        putModelNumber(val);
      else
        {
          printf(BadNumber);
          return;
        }
    }
  if (argc == 0 || argc == 1)
    {
      getModelNumber(&val);
      printf(terseMode ? "+ %d\n" : "Model %d\n",val);
    }
  else
    printf(WrongNumParam);
}

RedBoot_cmd("model","Show/set Model number","[model-number]",do_model_cmd);

#define EnableSerialNumberCommand 0

#if EnableSerialNumberCommand
#if defined(CYGPKG_HAL_NIOS2)
static void do_sernum_cmd(int argc, char *argv[])
{
  unsigned long val0, val1;
  
  --argc;
  ++argv;
  
  if (argc == 2)
    {
      if (external_console())
        return;
      if (parse_num(argv[0],&val0,NULL,NULL) &&
          parse_num(argv[1],&val1,NULL,NULL))
        {
          ident.serNumPrefixDM2 = (val0 > 9999) ? 9999: val0;
          ident.serNumDM2 = (val1 > 999999) ? 999999: val1;
          putSerNumPrefixDM2(ident.serNumPrefixDM2);
          putSerNumDM2(ident.serNumDM2);
        }
      else
        {
          printf(BadNumber);
          return;
        }
    }
  else if (argc == 1)
    {
      if (external_console())
        return;
      if (parse_num(argv[0],&val0,NULL,NULL))
        {
          ident.serNumDM2 = (val0 > 999999) ? 999999: val0;
          putSerNumDM2(ident.serNumDM2);
        }
      else
        {
          printf(BadNumber);
          return;
        }
    }
  else if (argc == 0)
    {
      getSerNumPrefixDM2(&ident.serNumPrefixDM2);
      getSerNumDM2(&ident.serNumDM2);
      printf("Serial number: %04u-%06lu\n",
             ident.serNumPrefixDM2, ident.serNumDM2);
    }
  else
    printf(WrongNumParam);
}
#else
static void do_sernum_cmd(int argc, char *argv[])
{
  unsigned short serNum, serNumPrefix;
  unsigned long val0, val1;
  
  --argc;
  ++argv;
  
  if (argc == 2)
    {
      if (external_console())
        return;
      if (parse_num(argv[0],&val0,NULL,NULL) &&
          parse_num(argv[1],&val1,NULL,NULL))
        {
          serNumPrefix = (val0 > 9999) ? 9999: val0;
          serNum = (val1 > 999999) ? 999999: val1;
          putSerNumPrefix(serNumPrefix);
          putSerNum(serNum);
        }
      else
        {
          printf(BadNumber);
          return;
        }
    }
  else if (argc == 1)
    {
      if (external_console())
        return;
      if (parse_num(argv[0],&val0,NULL,NULL))
        {
          serNum = (val0 > 999999) ? 999999: val0;
          putSerNum(serNum);
        }
      else
        {
          printf(BadNumber);
          return;
        }
    }
  else if (argc == 0)
    {
      getSerNumPrefix(&serNumPrefix);
      getSerNum(&serNum);
      printf("Serial number: %04u-%06lu\n",serNumPrefix,serNum);
    }
  else
    printf(WrongNumParam);
}
#endif
RedBoot_cmd("sernum", "Show/set device serial number",
            "[prefix] [serial_number]\n   sernum [serial_number]", do_sernum_cmd);
#endif

static void do_timeout_cmd(int argc, char *argv[])
{
  unsigned long val;
  unsigned char c;

  --argc;
  ++argv;

  if (argc == 1)
    {
      if (parse_num(argv[0],&val,NULL,NULL))
        {
          c = (val > 255) ? 255 : val;
          putRedbootDelay(c);
        }
      else
        {
          printf(BadNumber);
          return;
        }
    }
  
  if (argc == 0 || argc == 1)
    {
      getRedbootDelay(&c);
      printf(terseMode ? "+ %d\n" : "     Timeout %d seconds\n", c);
    }
  else
    printf(WrongNumParam);
}

RedBoot_cmd("timeout","Show/set bootloader timeout","[seconds]",do_timeout_cmd);

#if defined(CYGPKG_REDBOOT_NETWORKING)
static void do_telnettimeout_cmd(int argc, char *argv[])
{
  unsigned long val;

  --argc;
  ++argv;

  if (argc == 1)
    {
      if (parse_num(argv[0],&val,NULL,NULL))
        {
          putTelnetTimeout(val);
        }
      else
        {
          printf(BadNumber);
          return;
        }
    }
  
  if (argc == 0 || argc == 1)
    {
      getTelnetTimeout(&val);
      printf(terseMode ? "+ %d\n" : "Timeout %ld seconds\n", val);
    }
  else
    printf(WrongNumParam);
}

RedBoot_cmd("teltimeout","Show/set telnet timeout","[seconds]",do_telnettimeout_cmd);
#endif

static void do_boardrev_cmd(int argc, char *argv[])
{
  unsigned long val;
  
  --argc;
  ++argv;
  
  if (argc == 1)
    {
      if (external_console())
        return;
      if (parse_num(argv[0],&val,NULL,NULL))
        putBoardRev(val);
      else
        {
          printf(BadNumber);
          return;
        }
    }
  if (argc == 0 || argc == 1)
    {
      getBoardRev(&val);
      printf(terseMode ? "+ %d\n" : "BoardRev %d\n",val);
    }
  else
    printf(WrongNumParam);
}

RedBoot_cmd("boardrev","Show/set Board revision","[rev-number]",do_boardrev_cmd);

static void do_password_cmd(int argc, char *argv[])
{
  unsigned char p[16];
  --argc;
  ++argv;
  
  if (argc == 1)
    putPassword(argv[0]);
  else if (argc == 0)
    putPassword("");

  getPassword(p);

  if (strlen(p) == 0)
      printf("Cleared  \n");
  else if (terseMode)
      printf("+ '%s'\n",p);
  else
      printf("Password '%s'\n",p);
}

RedBoot_cmd("password","Set/Delete password","[password]",do_password_cmd);

static void do_auth_cmd(int argc, char *argv[])
{
  unsigned char m;
  static char *methods[] = {"noaccess","none","basic","md5","invalid"};
  --argc;
  ++argv;

  if (argc == 1)
    {
      int i;
      for (i=0; i<NumElements(methods); ++i)
        if (!strcmp(argv[0],methods[i]))
          {
            putAuthMethod(i);
            break;
          }
      if (i >= NumElements(methods))
        {
          printf("- Unrecognized auth method: '%s'\n",argv[0]);
          return;
        }
    }
  getAuthMethod(&m);
  if (argc == 0 || argc == 1)
    printf((terseMode ? "+ %s\n" : "        Auth %s\n"), ((m >= NumElements(methods)) ? "???" : methods[m]));
}

RedBoot_cmd("auth","Set/show web authentication","[noaccess,none,basic,md5,invalid]",do_auth_cmd);


static const char *enableDisable[] = {"disable","enable"};
static void showEnableDisable(char *s, int m)
{
  char *v = "???";
  if (m >= 0 && m <= 1)
    v = (char*)enableDisable[m];
  if (terseMode)
    printf("+ %s\n",v);
  else
    printf("%12s %sd\n",s,v);
}
static int getEnableDisable(char *s)
{
  if (strcmp(s,enableDisable[0]) == 0)
    return 0;
  if (strcmp(s,enableDisable[1]) == 0)
    return 1;
  printf("- Unrecognized response, please use 'enable' or 'disable'\n");
  return -1;
}


static void do_telnetEnable_cmd(int argc, char *argv[])
{
  unsigned char m;
  --argc;
  ++argv;

  if (argc == 1)
    {
      int i = getEnableDisable(argv[0]);
      if (i >= 0)
        putTelnetEnable(i);
    }
  getTelnetEnable(&m);
  if (argc == 0 || argc == 1)
    showEnableDisable("Telnet",m);
}

RedBoot_cmd("telnet","Set/Show telnet server enable","[disable|enable]",do_telnetEnable_cmd);

static void do_snmpEnable_cmd(int argc, char *argv[])
{
  unsigned char m;
  --argc;
  ++argv;

  if (argc == 1)
    {
      int i = getEnableDisable(argv[0]);
      if (i >= 0)
        putSnmpEnable(i);
    }
  getSnmpEnable(&m);
  if (argc == 0 || argc == 1)
    showEnableDisable("SNMP",m);
}

RedBoot_cmd("snmp","Set/Show SNMP enable","[disable|enable]",do_snmpEnable_cmd);


static void do_secureConfigEnable_cmd(int argc, char *argv[])
{
  unsigned char m;
  --argc;
  ++argv;

  if (argc == 1)
    {
      int i = getEnableDisable(argv[0]);
      if (i >= 0)
        putSecureConfigEnable(i);
    }
  getSecureConfigEnable(&m);
  if (argc == 0 || argc == 1)
    showEnableDisable("Secureconf",m);
}

RedBoot_cmd("secureconf","Set/Show secure config enable","[disable|enable]",do_secureConfigEnable_cmd);

static void do_secureDataEnable_cmd(int argc, char *argv[])
{
  unsigned char m;
  --argc;
  ++argv;

  if (argc == 1)
    {
      int i = getEnableDisable(argv[0]);
      if (i >= 0)
        putSecureDataEnable(i);
    }
  getSecureDataEnable(&m);
  if (argc == 0 || argc == 1)
    showEnableDisable("Securedata",m);
}

RedBoot_cmd("securedata","Set/Show secure data enable","[disable|enable]",do_secureDataEnable_cmd);

#if defined(CYGPKG_REDBOOT_NETWORKING)

static void do_ip_cmd(int argc, char *argv[])
{
  in_addr_t ipAddr;
  in_addr_t netMask;
  in_addr_t gateway;
  int err;
  static char errMsg[] = "Error: parameters requred: ip-addr netmask gateway\n";
  
  --argc;
  ++argv;

  unsigned long temp_ipaddr, temp_netmask, temp_gateway;
  getIpInfo(&temp_ipaddr, &temp_netmask, &temp_gateway);
  ipAddr.s_addr = htonl(temp_ipaddr);
  netMask.s_addr = htonl(temp_netmask);
  gateway.s_addr = htonl(temp_gateway);

  err = argc > 3;
  if (argc >= 1)
    err |= !inet_aton(argv[0],&ipAddr);
  if (argc >= 2)
    err |= !inet_aton(argv[1],&netMask);
  if (argc >= 3)
    err |= !inet_aton(argv[2],&gateway);
  
  if (err)
    {
      printf("- %s",errMsg);
      return;
    }
  else
    putIpInfo(ntohl(ipAddr.s_addr),ntohl(netMask.s_addr),ntohl(gateway.s_addr));
  
  diag_printf("      IP: %s\n", inet_ntoa(&ipAddr));
  diag_printf("    Mask: %s\n", inet_ntoa(&netMask));
  diag_printf(" Gateway: %s\n", inet_ntoa(&gateway));
}

RedBoot_cmd("ip","Show/set IP address config","[addr mask gateway]",do_ip_cmd);

#endif

static void do_disableLoad_cmd(int argc, char *argv[])
{
  disableAutoload = 1;
  disableDriverload = 1;
  printf(terseMode ? "+\n" : "Loading disabled\n");
}

RedBoot_cmd("disable","Disable program loading (auto/default and ns-link)","",do_disableLoad_cmd);

static void do_terse_cmd(int argc, char *argv[])
{
  terseMode = 1;
  console_echo = 0;
  printf("+\n");
}

RedBoot_cmd("terse","Terse command response mode","",do_terse_cmd);

#define BREAK           3
#define FRAMING_ERROR   2
#define PARITY_ERROR    1
#define OVERRUN_ERROR   0

static void printLoopbackError(int error)
{
  // It seems this code only works for the 4530 UART on the 1-port...
  if (!is4530)
    return;
  
    switch (error & 0xFFFF0000) {
    case LOOP_ERROR:  
    case LOOP_CHANNEL_STATUS_ERROR:  
	printf("Channel status error - ");
	if (error & BREAK) printf("Break  ");
	if (error & FRAMING_ERROR) printf("Framing Error  \n");
	if (error & PARITY_ERROR) printf("Parity Error \n");
	if (error & OVERRUN_ERROR) printf("Overrun Error \n");
	printf("\n");
	break;
    case LOOP_DATA_VERIFY_ERROR:  
	printf("Data verify error.  Expected:%d  Recieved:%d \n",
	       (error & 0xFF00) >> 8, error & 0x00FF);
	break;
    case LOOP_CTS_NOT_RTS:  
	printf("Flow Control Test Error - CTS not equal to RTS \n");
	break;
    case LOOP_NO_CTS_EVENT:  
	printf("Flow Control Test Error - No CTS event when CTS toggled \n");
	break;
    case LOOP_RI_NOT_RTS:  
	printf("Flow Control Test Error - RI not equal to RTS \n");
	break;
    case LOOP_DSR_NOT_DTR:  
	printf("Flow Control Test Error - DSR not equal to DTR \n");
	break;
    case LOOP_CD_NOT_DTR:  
	printf("Flow Control Test Error - CD not equal to DTR \n");
	break;
    case LOOP_TIMEOUT:  
	printf("Timeout \n"); 
	break;
    case LOOP_MODE_NOT_SUPPORTED:
        printf("RS422 Mode not supported \n");
	break;
    }
}

static void do_loopback_cmd(int argc, char *argv[])
{
  extern int cyg_hal_plf_loopback(int port, int mode, int doMC, int doRI, int useAiopic);
  unsigned long val;
  int i=0;
  static char usage[] = "- Usage: 'loop mode port'  where mode = 232, 422, int (internal loopback)  port = 0-31 \n";

// In $ECOSAPPS/main.c the cyg_user_start() function has a model ID
// test to determine if the model has RI. These lists should match.
  int doRI = ((ident.modelId == 5002280) ||    // DM1
              (ident.modelId == 5002281) ||    // DM1E
              (ident.modelId == 5002111) ||    // DM4 DB9
              (ident.modelId == 5002110) ||    // DM8 DB9
              (ident.modelId == 5002330) ||    // DM8 PRO DB9
              (ident.modelId == 5002283) ||    // DM8 Air 1
              (ident.modelId == 5002115) ||    // DM SH 8
              (ident.modelId == 5002315) ||    // DM SH 16
              (ident.modelId == 5002362) ||    // DM 500 1 Port Embedded
              (ident.modelId == 5002363) ||    // DM 500 1 Port DB9
              (ident.modelId == 5002117) ||    // DM 500 4 Port DB9
              (ident.modelId == 5002535) ||    // DM RTS 2P 2E Configuration 2
              (ident.modelId == 5002595) ||    // DM RTS 2P 1E Configuration 1 DB9
              (ident.modelId == 5002600));     // DM RTS 2P 2E Configuration 2 DB9

  // DM RTS 2P Configuration 1 does not have modem control lines.
  int doMC = !(ident.modelId == 5002525);

  // One port models and DM RTS 2P do not support RS232 RTS inverted mode
  int doRS232Inv = !((ident.numPorts == 1) ||
                     (ident.modelId == 5002525) ||
                     (ident.modelId == 5002535) ||
                     (ident.modelId == 5002595) ||
                     (ident.modelId == 5002600));

  --argc;
  ++argv;
  
  if (argc != 2)
    {
      printf(usage);
      return;
    }
  if (!parse_num(argv[1],&val,NULL,NULL) || 
      val < 0 ||  
      val > ident.numPorts - 1)
    {
      badNumberRecieved(val);
      return;
    }

  if (!strcmp(argv[0],"232"))
    {
      if ((i=cyg_hal_plf_loopback(val, 0, doMC, doRI, is4530)))		//  0=>rs232
        {
          printf("- Loopback (1) failed RS-232: %x\n",i);
	  printLoopbackError(i);
          return;
        }

       if (doRS232Inv)
         {
           if ((i=cyg_hal_plf_loopback(val, 1, doMC, doRI, is4530)))	// 1=>rs232 inv
             {
               printf("- Loopback (2) failed RS-232: %x\n",i);
               printLoopbackError(i);
               return;
             }
         }
    }
  else if (!strcmp(argv[0],"422"))
    {
      int cur = CYGACC_CALL_IF_SET_CONSOLE_COMM(CYGNUM_CALL_IF_SET_COMM_ID_QUERY_CURRENT);
      // console channel 2 is external serial port 0
      if ( cur == 2 && (val == 0))  {
          printf("- Running the 422 loop test from the serial connection \n  will cause unpredictable results.\n");
          return;
        }
      if (ONLY_232)
        {
          printf("- Loopback failed RS-422: %x\n",i);
          printLoopbackError(LOOP_MODE_NOT_SUPPORTED);
          return;
	}
      if ((i=cyg_hal_plf_loopback(val, 2, 0, 0, is4530)))		// 2=>rs422
        {
          printf("- Loopback failed RS-422: %x\n",i);
          printLoopbackError(i);
          return;
        }
    }
  else if (!strcmp(argv[0],"int"))
    {
      if ((i=cyg_hal_plf_loopback(val, 3, 0, 0, is4530)))		// 3=>internal loopback
        {
          printf("- Loopback failed internal: %x\n",i);
          printLoopbackError(i);
          return;
        }
    }
  else
    {
      printf(usage);
      return;
    }
  printf("%sLoopback pass\n",terseMode ? "+ " : "");
  return;
}

static char loop_usage[] = " 232|422|int port-number - 422 NOT Supported on DvcMstr Serial Hub";
RedBoot_cmd("loop","Run loopback test on port", loop_usage, do_loopback_cmd);

static void do_test485_cmd(int argc, char *argv[])
{
    extern int cyg_hal_plf_485_test(int port1, int port2, int use4530);
    unsigned long p1,p2;
    static char usage[] = "- Usage: 't485 port1 port2 (half duplex mode)'\n"
                          "         't485 port1 port1 (485 test mode, 1 & 2 port models)'\n";
    int i;
    int is_1port = 0;

    if( ONLY_232 )  {		// DM RTS SH 8 or 16
        printf("- 485 Mode not supported \n");
	return;
    }

    // So far only DM RTS 2P supports 485 test mode
    int has485TestMode = ((ident.modelId == 5002525) ||
                          (ident.modelId == 5002535) ||
                          (ident.modelId == 5002595) ||
                          (ident.modelId == 5002600));

    if (!is4530 && ident.modelId != 5002405) {			//not a 1 port
	--argc;
	++argv;

	if (argc != 2)    {
	    printf(usage);
	    return;
	}
	
	if (!parse_num(argv[0],&p1,NULL,NULL) || 
	    p1 < 0 ||  
	    p1 > ident.numPorts - 1) {
	    badNumberRecieved(p1);
	    return;
	}
	
	if (!parse_num(argv[1],&p2,NULL,NULL) || 
	    p2 < 0 ||  
	    p2 > ident.numPorts - 1)  {
	    badNumberRecieved(p2);
	    return;
	}

        // if port1 equals port2, then use RS485 test mode (available on DM RTS 2P only)
        if ((p1 == p2) && !has485TestMode) {
            printf("- 485 test mode not supported.\n");
            return;
        }

        int cur = CYGACC_CALL_IF_SET_CONSOLE_COMM(CYGNUM_CALL_IF_SET_COMM_ID_QUERY_CURRENT);
        // console channel 2 is external serial port 0
        if ( cur == 2 && (p1 == 0 || p2 == 0))  {
            printf("- Running the 485 test from the serial connection \n  will cause unpredictable results.\n");
            return;
        }

    }
    else {      // (ident.modelId == 5002405 || is4530 )
        is_1port =1;
        p1 = p2 = 0;
    }

    if ((i=cyg_hal_plf_485_test(p1, p2, is_1port)))   {
	printf("- Failed 485 test: %x\n",i);
	return;
    }
    printf("%s485 pass\n",terseMode ? "+ " : "");
    return;
}

static char t485_usage[] = " port1 port2 - 485 NOT Supported on DeviceMaster Serial Hub";
RedBoot_cmd("t485","Run port-to-port RS-485 test",t485_usage ,do_test485_cmd);


static void do_flash_info(int argc, char *argv[])
{
  extern void flash_hwr_init(void);
  flash_hwr_init();
}
RedBoot_cmd("flash","Show flash info","",do_flash_info);


#if 0
// Some Engineering-only commands to read/write/erase EEPROM
static void do_eeprom_init(int argc, char *argv[])
{
  extern void iicEepromErase(void);
  iicEepromErase();
}
RedBoot_cmd("eeinit","Initialize EEPROM to all 0xff","",do_eeprom_init);

// read eeprom, bypassing cache
static void do_eeprom_read(int argc, char *argv[])
{
  #define EEPROM_SLAVE_ADDR       0x50
  extern void iicRead(int slave_addr, int addr, void *p, int count);
  static unsigned long addr;
  unsigned char buf[8];
  if (argc == 2)
    parse_num(argv[1],&addr,NULL,NULL);
  iicRead(EEPROM_SLAVE_ADDR, addr, buf, sizeof buf);
  diag_dump_buf(buf,sizeof buf);
  addr += sizeof buf;
}
RedBoot_cmd("eerd","Read EEPROM bytes","",do_eeprom_read);

static void do_eeprom_write(int argc, char *argv[])
{
  extern void iicEepromWrite(unsigned addr, void *p, unsigned count);
  unsigned long addr,val;
  unsigned char b;

  if (argc != 3)
    {
      printf("Usage: eerw addr value\n");
      return;
    }
  parse_num(argv[1],&addr,NULL,NULL);
  parse_num(argv[2],&val,NULL,NULL);
  b = val;
  iicEepromWrite(addr,&b,1);
}
RedBoot_cmd("eewr","Write EEPROM byte","",do_eeprom_write);
#endif


static const char prodNameUP[] = "Comtrol DeviceMaster UP ";
static const char prodNameRTS[] = "Comtrol DeviceMaster RTS ";
static const char prodNameDM2[] = "Comtrol DeviceMaster2 ";
static void do_chassis_cmd(int argc, char *argv[])
{
  getAppEnableKey(&ident.appEnblKey);

  if ((ident.appEnblKey == 0) || (ident.appEnblKey == 0xff)) {
      if (ident.modelId == 5002405)
          printf("%s\n",prodNameDM2);
      else
          printf("%s\n",prodNameRTS);
  } else if (ident.appEnblKey == UP_ENABLE_FLAG){
      // one is only valid setting for DeviceMaster UP
      printf("%s\n",prodNameUP);
  } else {
      // one is only possible setting 
      printf("%s Type: %d\n",prodNameRTS, ident.appEnblKey);
  }
}

RedBoot_cmd("chassis","Show chassis type (RTS, DM2 or UP)","",do_chassis_cmd);

static unsigned long loadAddrLow;
static unsigned long loadAddrHigh;

void loadStatusInit(void)
{
  loadAddrHigh = 0;
  loadAddrLow  = 0xffffffff;
}

void loadStatusUpdate(unsigned long addr, unsigned len)
{
  if (addr < loadAddrLow)
    loadAddrLow = addr;
  if (addr+len > loadAddrHigh)
    loadAddrHigh = addr+len;
}

void loadStatusShow(void)
{
  unsigned long i,count,sum;
  unsigned long *p;
  
  count = loadAddrHigh - loadAddrLow;

  printf("\n%ld bytes loaded (0x%08lx-0x%08lx) ",count,loadAddrLow,loadAddrHigh-1);
    
  p = (unsigned long*)loadAddrLow;
  sum = 0;
  i = count/4;
  
  while (i--)
    sum += *p++;
  
  diag_printf("checksum: 0x%08lx\n", sum);
  
  loadStatusInit();
}

static char *emptryString = "";

void do_summary_cmd(int argc, char *argv[])
{
  extern void do_version(int argc, char *argv[]);

  do_flash_info(1,&emptryString);
  printf("\nRedBoot %s\n",IdentString);
  do_model_cmd(1,&emptryString);
  printf("\n");
  do_mac_cmd(1,&emptryString);
  do_ip_cmd(1,&emptryString);
  printf("\n");
  do_auth_cmd(1,&emptryString);
  do_timeout_cmd(1,&emptryString);
  do_telnetEnable_cmd(1,&emptryString);
  do_secureConfigEnable_cmd(1,&emptryString);
  do_secureDataEnable_cmd(1,&emptryString);
}

RedBoot_cmd("summary", "Show configuration summary", "", do_summary_cmd);

/*************************************************************

  Console Select, Disabling Autoloading, Disabling Driver Load   
  ------------------------------------------------------------

Definitions:

console_select - A Redboot global, set when the "console" is selected
by the operator.  For DeviceMaster, this occurs when the debug header is
connected and the ENTER key is pressed.

disableDriverLoad - Variable used to disable loading of the DeviceMaster
by driver.  Actually is does more (see below).

disableAutoLoad - Variable that disables autoloading of default app.

'dis' entered - The 'dis' command can be entered when  telnetting into RedBoot
or when connecting via the the !DM serial port method.

NOTE:  When the debug header is connected and any keystroke other than
ENTER is pressed, all DeviceMaster processing stops (both loops).  In this 
state Redboot code is executing, no DeviceMaster loops run.  This state is not
the console_selected state.  When the ENTER is pressed, Redboot processes the
entered line as a command.  At that point, the console_select state is 
true and the DeviceMaster loops run again.

During this entering keystrokes state, the TCP-IP stack does not work, you 
cannot telnet into the device or ping it.  No MAC mode commands are processed, 
though, and you cannot make TCP connection to 0x11FE.

This state can be identified by the LED not blinking.

Disable Driver Load 
===================
Used to stop the driver from loading the DeviceMaster, used during
debug.  This state is entered when:
-  Console is selected
-  'dis' entered

What is does:
-  Disables response to all MAC mode commands
-  Disables listening on TCP socket 0x11FE
-  If socket is open, closes that socket

During this state, the device can still be pinged and telnetted into.


Disable Auto Load 
===================
Used to stop Redboot from automatically loading the default application
after <timeout> seconds.  This state is entered into when:
-  Console is selected
-  'dis' entered
-  MAC mode binary load has begun.  Note there is no abort or reset from this.  Success is assumed...
-  TCP socket into TCP port 0x11FE established

NOTE:  Code change Jan. 2005 - disableAutoLoad variable cleared when TCP socket is closed.  It was 
observed that if PortVision did a TCP ID Query to Redboot the default app would not load.  This 
fixes that.


*************************************************************/

/*************************************************************

  Ownership
  ---------

Ownership allows only a single host to load firmware into a DM.  The variable OwnershipGranted, 
if non-zero, indicates that the DM is owned.  This variable is set to 60 when ownership
is granted, and decrements every second.  Therefore, ownership expires after 60 seconds (except 
TCP, see below).


MAC MODE
========
In MAC mode, an incoming ADMIN_ID_QUERY command is used to request ownership.  If Byte[12] bit 0 = 0,
the host is requesting ownership.  The firmware checks OwnershipGranted:
-  If zero, ownership is granted.  Ownership Granted is set to 60
-  If non-zero, ownership is not granted.

The response packet indicates to the host whether ownership was granted or not.
-  Byte[12] = 0  Ownership was granted
-  Byte[12] = 2  Ownership was not granted


IP MODE
=======
In IP mode, the host attempts to create a socket connection into the DM on TCP port 0x11FE.  Redboot
is listening on this port, and it allows only one connection at a time.  Therefore, if the attempt
to connect is successful, the host is granted ownership.  

-  When a socket connection into the DM on TCP port 0x11FE is 1st established, OwnershipGranted is set to 60.
-  When the socket is closed, OwnershipGranted is set to 0.
-  Every packet received over this socket causes OwnershipGranted to be reset to 60.


*************************************************************/
