#include <pkgconf/system.h>
#include <pkgconf/devs_eth_arm_dmrts.h>

#if defined(CYGPKG_IO)
#include <pkgconf/io.h>
#include <cyg/io/io.h>
#include <cyg/io/devtab.h>
#else
// need to prvide fake values for errno
#define EIO 1
#define EINVAL 2
#endif

#define cyg_thread_delay_ms(ms) cyg_thread_delay(ms/((CYGNUM_HAL_RTC_NUMERATOR/CYGNUM_HAL_RTC_DENOMINATOR)/1000000))

#include <string.h>
#include <net/etherio.h>

#include <cyg/infra/cyg_type.h>  // Common type definitions (and endian-ness)

#include <cyg/infra/diag.h>
#include <cyg/io/eth/netdev.h>
#include <cyg/io/eth/eth_drv.h>
#include <cyg/io/eth/eth_drv_stats.h>
#include <cyg/hal/hal_intr.h>
#include <cyg/hal/hal_diag.h>
#include <cyg/hal/iic.h>

#if defined(CYGPKG_REDBOOT)
#include <pkgconf/redboot.h>
#include <net/net.h>
#endif

#include "std.h"
#include "ks5000_regs.h"
#include "ks5000_ether.h"

#if !defined(CYGPKG_NET)
#define cyg_drv_interrupt_unmask(v) /* noop */
#define cyg_drv_interrupt_mask(v)   /* noop */
#define cyg_drv_isr_lock()          /* noop */
#define cyg_drv_isr_unlock()        /* noop */
#define cyg_drv_mutex_init(m)       /* noop */
#define cyg_drv_mutex_lock(m)       /* noop */
#define cyg_drv_mutex_unlock(m)     /* noop */
#define cyg_drv_dsr_lock()          /* noop */
#define cyg_drv_dsr_unlock()        /* noop */
#endif

#if !defined(CYGPKG_DEVS_ETH_ARM_DMRTS_PHY)
#error "Invalid configuration no: PHY setting."
#endif

#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_PHY_NONE)
#define HavePHY 0
#else
#define HavePHY 1
#endif

#if HavePHY
#include "phy.h"
#include "ac101l.h"
#include "ksz8893mql.h"
#endif

static U32 crc32( unsigned char *blk_adr, unsigned blk_len);

#if 1
#define debug_printf(args...) diag_printf(args)
#else
#define debug_printf(args...) /* noop */
#endif
#define Bit(n) (1<<(n))

extern void hal_delay_us(int us);

char *macToStr(unsigned char *mac)
{
    static char hex[] = "0123456789ABCDEF";
    static char buf[18];
    unsigned char *p = buf;
    int i;
    for (i=0; i<6; ++i) {
        *p++ = hex[(mac[i] >> 4) & 0xf];
        *p++ = hex[(mac[i]) & 0xf];
        *p++ = ':';
    }
    buf[17] = '\0';
    return buf;
}

// miscellaneous data structures

typedef  BYTE  ETH_ADDR[6] __attribute__((packed));

typedef  struct tagETH_HEADER {
    ETH_ADDR daddr __attribute__((packed));
    ETH_ADDR saddr __attribute__((packed));
    WORD type      __attribute__((packed));
} ETH_HEADER __attribute__((packed));

#define ETH_HEADER_SIZE   14

// Tx/Rx common descriptor structure
typedef  struct tagFRAME_DESCRIPTOR {
    LWORD FrameDataPtr;
    LWORD Reserved;   /*  cf: RX-reserved, TX-Reserved(25bits) + Control bits(7bits) */
    LWORD StatusAndFrameLength;
    struct tagFRAME_DESCRIPTOR   *NextFD;
} FRAME_DESCRIPTOR;

typedef struct {
    U8 DestinationAddr[6];
    U8 SourceAddr[6];
    U8 LengthOrType[2];
    U8 LLCData[1506];
} MAC_FRAME;

#if defined(CYGPKG_NET)
static cyg_drv_mutex_t txMutex;
struct ether_drv_stats ifStats;
#endif

typedef struct {
    LWORD BTxNLErr;
    LWORD BTxNOErr;
    LWORD BTxEmptyErr;
} BDMA_TX_ERR;

typedef struct {
    LWORD BRxNLErr;
    LWORD BRxNOErr;
    LWORD BRxMSOErr;
    LWORD BRxEmptyErr;
    LWORD sBRxSEarly;
    LWORD noBufferAvail;
    LWORD queueOverflow;
    LWORD bad;
} BDMA_RX_ERR;


// interrupt entry counters
U32 ks5000_MAC_Rx_Cnt;
U32 ks5000_MAC_Tx_Cnt;
U32 ks5000_MAC_Phy_Cnt;
U32 ks5000_BDMA_Tx_Isr_Cnt;
U32 ks5000_BDMA_Tx_Dsr_Cnt;
U32 ks5000_BDMA_Rx_Isr_Cnt;
U32 ks5000_BDMA_Rx_Dsr_Cnt;


// event counters
static U32 Tx_Pkts;
static U32 Rx_Pkts;
static U32 Tx_Full;
static U32 Tx_Restart;
static U32 Tx_Par;
static U32 Tx_Paused;
static U32 Tx_Halted;

// application controlled configuration values
static unsigned int specialTagging = 0;

// configuration values
//static volatile U32 MACConfigVar = MACON_FULL_DUP;
static volatile U32 MACConfigVar;
static volatile U32 CAMConfigVar = CAMCON_COMP_EN | CAMCON_BROAD_ACC;
static volatile U32 MACTxConfigVar =
    MACTXCON_EN_UNDER |
#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWTXCRC)
    MACTXCON_NO_CRC |
#endif
    MACTXCON_EN_DEFER |
    MACTXCON_EN_NCARR |
    MACTXCON_EN_EXCOLL |
    MACTXCON_EN_LATE_COLL |
    MACTXCON_ENTX_PAR |
    MACTXCON_EN_COMP;
static volatile U32 MACRxConfigVar =
    MACRXCON_RX_EN |
    MACRXCON_EN_ALIGN |
    MACRXCON_EN_CRC_ERR |
    MACRXCON_EN_OVER |
    MACRXCON_EN_LONG_ERR |
    MACRXCON_EN_RX_PAR;

#if defined(CYGPKG_HAL_ARM_RPSH2)
# define BdmaTxBurstLen 3
#else
# define BdmaTxBurstLen 7
#endif

static volatile U32 BDMATxConfigVar =
    BDMATXCON_MSL111 |
    BDMATXCON_STP_SKP |
    BdmaTxBurstLen;   /* burst size - 1 */

#define EtherFramePadding 2

#if EtherFramePadding == 0
#define BDMARXCON_ALIGN BDMARXCON_WA00
#elif EtherFramePadding == 1
#define BDMARXCON_ALIGN BDMARXCON_WA01
#elif EtherFramePadding == 2
#define BDMARXCON_ALIGN BDMARXCON_WA10
#elif EtherFramePadding == 3
#define BDMARXCON_ALIGN BDMARXCON_WA11
#else
#error "EtherFramePadding must be 0,1,2 or 3"
#endif

static volatile U32 BDMARxConfigVar =
    BDMARXCON_DIE |
    BDMARXCON_EN |
    BDMARXCON_BIG |
    BDMARXCON_MA_INC |
    BDMARXCON_NOIE |
    BDMARXCON_ALIGN |
    BDMARXCON_STP_SKP |
    15;  /* burst size - 1 */


/* Global variables For BDMA Error Report */

static BDMA_TX_ERR BDMATxErrCnt = {0, 0, 0};
static BDMA_RX_ERR BDMARxErrCnt = {0, 0, 0, 0, 0};

static void Init_TxFrameDescriptorArray(void);
static void Init_RxFrameDescriptorArray(void);

// number of ethernet buffers should be enough to keep both rx
// and tx queues full plus some extras for in-process packets

#if defined(CYGPKG_REDBOOT)
#define NUM_ETH_BUFFERS CYGNUM_REDBOOT_ETHER_BUFS
#define MAX_RX_FRAME_DESCRIPTORS  CYGNUM_REDBOOT_ETHER_DESCR     // Max number of Rx Frame Descriptors
#define MAX_TX_FRAME_DESCRIPTORS  CYGNUM_REDBOOT_ETHER_DESCR     // Max number of Tx Frame Descriptors
#else
#define NUM_ETH_BUFFERS CYGNUM_ECOS_ETHER_BUFS
#define MAX_RX_FRAME_DESCRIPTORS  CYGNUM_ECOS_ETHER_DESCR     // Max number of Rx Frame Descriptors
#define MAX_TX_FRAME_DESCRIPTORS  CYGNUM_ECOS_ETHER_DESCR     // Max number of Tx Frame Descriptors
#endif

static FRAME_DESCRIPTOR _rxFrameDescrArray[MAX_RX_FRAME_DESCRIPTORS] __attribute__((aligned(16)));
static FRAME_DESCRIPTOR _txFrameDescrArray[MAX_TX_FRAME_DESCRIPTORS] __attribute__((aligned(16)));

/* define aliases that will set the no-cache bit */
#define rxFrameDescrArray ((FRAME_DESCRIPTOR*)(((unsigned)_rxFrameDescrArray)|0x4000000))
#define txFrameDescrArray ((FRAME_DESCRIPTOR*)(((unsigned)_txFrameDescrArray)|0x4000000))

static FRAME_DESCRIPTOR *rxReadPointer;
static FRAME_DESCRIPTOR *txDonePointer;
static FRAME_DESCRIPTOR *txWritePointer;

static cyg_drv_mutex_t oldRxMutex;
static cyg_drv_cond_t  oldRxCond;

static void dumpDescriptor(FRAME_DESCRIPTOR *p)
{
    char *s1 = "";
    char *s2 = "";
    if ((((unsigned)p)&0x7fffffff) == (unsigned)rxReadPointer)
        s1 = "<-RxRead";
    if ((((unsigned)p)&0x7fffffff) == (unsigned)txDonePointer)
        s1 = "<-TxDone";
    if ((((unsigned)p)&0x7fffffff) == (unsigned)txWritePointer)
        s2 = "<-TxWrite";

    diag_printf("%x: D=%x R=%08x S=%08x N=%x%s%s\n",
                p,
                p->FrameDataPtr,
                p->Reserved,
                p->StatusAndFrameLength,
                p->NextFD,
                s1,s2);
}

void dmrts_ether_dump_frame_descriptors(void)
{
    int i;

    diag_printf("Rx Descriptors:\n");
    for (i=0; i<MAX_RX_FRAME_DESCRIPTORS; ++i)
        dumpDescriptor(rxFrameDescrArray+i);

    diag_printf("Tx Descriptors:\n");
    for (i=0; i<MAX_TX_FRAME_DESCRIPTORS; ++i)
        dumpDescriptor(txFrameDescrArray+i);
}

static bool configDone;

/*----------------------------------------------------------------------
 * Data structures used to manage ethernet buffers
 */

#define EnableQueueCounts 0

typedef struct {
    tEthBuffer *head;
    tEthBuffer *tail;
#if EnableQueueCounts
    int count;
#endif
} tEthBufQueue;

#if EnableQueueCounts
#define EmptyQueue {NULL,NULL,0}
#else
#define EmptyQueue {NULL,NULL}
#endif

static void ethBufQueueClear(tEthBufQueue *q)
{
    q->head = NULL;
    q->tail = NULL;
#if EnableQueueCounts
    q->count = 0;
#endif
}

//  Get the next EthBuf from the linked list q.
static tEthBuffer *ethBufQueueGet(tEthBufQueue *q)
{
    tEthBuffer *r;

    r = q->head;

    if (r) {
        q->head = r->next;
#if EnableQueueCounts
        --q->count;
#endif
    }
    return r;
}

//  Add an EthBuf to the linked list q.
static void ethBufQueuePut(tEthBufQueue *q, tEthBuffer *b)
{
    b->next = NULL;

    if (!q->head) {
        q->head = b;
        q->tail = b;
    } else {
        q->tail->next = b;
        q->tail = b;
    }
#if EnableQueueCounts
    ++q->count;
#endif
}

static unsigned ethBufQueueCount(tEthBufQueue *q)
{
    unsigned r = 0;
    tEthBuffer *b = q->head;

    while (b) {
        ++r;
        b = b->next;
    }

    return r;
}

#if 0
// not used at the moment
static bool ethBufQueueEmpty(tEthBufQueue *q)
{
    return q->head != NULL;
}
#endif

/*----------------------------------------------------------------------
 * Free pool and routines to manipulate it.
 */
static tEthBufQueue freeList;
static int freeCount;

static tEthBuffer __bufferPool[NUM_ETH_BUFFERS] __attribute__((aligned(16)));
#define bufferPool ((tEthBuffer*)((unsigned)__bufferPool|0x4000000))

//  Frees an EthBuf by putting it back on the freeList
//  Do not call from ISR routine.
//  Input:  Ptr to EthBuf to be freed
static void freeBuffer(tEthBuffer *b)
{
    cyg_drv_isr_lock();
    ++freeCount;
    ethBufQueuePut(&freeList, b);
    cyg_drv_isr_unlock();
}

static int allocFail;

void bufferListError(void)
{
    diag_printf("%s:%d Buffer List Error!\n", __FILE__, __LINE__);
}

//  Get an EthBuf, removing it from the freeList
//  Do not call from ISR routine
//  Returns:  Ptr to the EthBuf allocated, NULL if none available
static tEthBuffer *allocBuffer(void)
{
    tEthBuffer *r;

    cyg_drv_isr_lock();
    r = ethBufQueueGet(&freeList);
    if (r)
        --freeCount;
    else {
        ++allocFail;
        if (freeCount)
            bufferListError();
    }
    cyg_drv_isr_unlock();

    return r;
}

static unsigned freeBufferCount(void)
{
    unsigned r;

    cyg_drv_isr_lock();
    r = ethBufQueueCount(&freeList);
    cyg_drv_isr_unlock();

    return r;
}

//  Frees an EthBuf by putting it back on the freeList
//  Call only from ISR routine or init.
//  Input:  Ptr to EthBuf to be freed
static void isrFreeBuffer(tEthBuffer *b)
{
    ++freeCount;
    ethBufQueuePut(&freeList, b);
}

#if 0
// not used at the moment

// call only from ISR routine or init
static tEthBuffer *isrAllocBuffer(void)
{
    tEthBuffer *r;
    r = ethBufQueueGet(&freeList);
    if (r)
        --freeCount;
    else {
        ++allocFail;
        if (freeCount)
            bufferListError();
    }
    return r;
}
#endif

//  Initialize the freeList, add all the EthBufs from the bufferPool to it.
static void initFreeList(void)
{
    int i;

    ethBufQueueClear(&freeList);
    freeCount = 0;
    for (i=0; i<NUM_ETH_BUFFERS; ++i)
        isrFreeBuffer(bufferPool+i);
}

//----------------------------------------------------------------------
// queue a buffer for transmit
//
// returns true if buffer was queued.

static int ks32c5000_eth_buffer_send(tEthBuffer *buf)
{
    static int fullcnt;
#if defined(CYGPKG_NET)
    while (!configDone)
        cyg_thread_delay_ms(10);
#endif

    if (txWritePointer->FrameDataPtr & FRM_OWNERSHIP_BDMA) {
        // queue is full!  make sure transmit is running
        ++Tx_Full;
        // code to reset TX half of MAC -- doesn't work reliably
        // it usually resets the CPU
#if 0
        if (fullcnt++ > 3) {
            // tx half of Ethernet controller is wedged
            static unsigned restarted;
            ++Tx_Restart;
            fullcnt = -1000;
            if (1 && !restarted) {
                restarted = true;
                HAL_DIAG_WRITE_CHAR('R');
                HAL_DIAG_WRITE_CHAR('\n');
                if (0) {
                    //BDMATXCON |= BDMATXCON_RESET;
                    //BDMATXCON &= ~BDMATXCON_RESET;
                    BDMATXCON = BDMATxConfigVar;
                    BDMATXPTR = (U32)txDonePointer;
                }
            }
        }
#endif
        MACTXCON |= MACTXCON_TX_EN;
        BDMATXCON |= BDMATXCON_EN;
        return 0;
    }

    fullcnt = 0;
    ++Tx_Pkts;

#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWTXCRC)
    // add Ethernet CRC
    {
        unsigned long crc;
        unsigned len = buf->length;
        if (len < 64) {
            memset(buf->data+len, 0, 64-len);
            len = 64;
        }
        crc = crc32(buf->data, len);
        buf->data[len++] = crc;
        buf->data[len++] = crc>>8;
        buf->data[len++] = crc>>16;
        buf->data[len++] = crc>>24;
        buf->length = len;
    }
#endif

    cyg_drv_mutex_lock(&txMutex);
    cyg_drv_isr_lock();

    // free old buffer if we need to
    if (txWritePointer->FrameDataPtr) {
        //freeBuffer((tEthBuffer*)txWritePointer->FrameDataPtr);
        // Note: The next two lines do the freeBuffer functionality.
        //       This way we do not unmask the Ethernet interrupt prematurely.
        ++freeCount;
        ethBufQueuePut(&freeList,(tEthBuffer*)txWritePointer->FrameDataPtr);
        txWritePointer->FrameDataPtr = 0;
    }

    cyg_drv_isr_unlock();

    // fill in the packet descriptor
    txWritePointer->Reserved = (  TXFDCON_PADDING_MODE
#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWTXCRC)
                                  | TXFDCON_NO_CRC_MODE
#endif
                                  | TXFDCON_SRC_ADDR_INC
                                  | TXFDCON_BIG_ENDIAN
                                  | TXFDCON_WIDGET_ALIGN00
                                  | TXFDCON_MAC_TX_INT_EN);

    txWritePointer->StatusAndFrameLength = buf->length;
    txWritePointer->FrameDataPtr = ((unsigned)buf | FRM_OWNERSHIP_BDMA);

    /*
     *  Found during eCos 2.0 Redboot testing - this function fails, adding this small delay
     *  fixes it.  Not required under eCos 1.3.1.  Problem observation: first packet sent does not
     *  transmit, next packet sent causes both packets to be transmitted.  This repeats.
     *  Debug code here shows that the Ownership bit is set in the TX frame, the BDMA is enabled,
     *  but the BDMA finds the ownership bit clear, so it disables itself.  Looks like a race
     *  condition.  Kurt R.  9-14-04.
     */
    hal_delay_us(1);

    txWritePointer = txWritePointer->NextFD;

    cyg_drv_mutex_unlock(&txMutex);

    // start transmit
#if defined(CYGPKG_NET)
    ++ifStats.tx_count;
#endif
    BDMATXCON |= BDMATXCON_EN;
    MACTXCON |= MACTXCON_TX_EN;

    //  Debug code, used during eCos 2.0 Redboot debugging...
#if (0)
    if (!(BDMATXCON &  BDMATXCON_EN)) {
        diag_printf("BDMATXCON_EN not set...");

        if (!(txWritePointer->FrameDataPtr & FRM_OWNERSHIP_BDMA))
            diag_printf("Ownership bit not set...");

        if (!(txWritePointer->NextFD))
            diag_printf("NextFD is NULL...");

        diag_printf("re-setting...");
        BDMATXCON |= BDMATXCON_EN;
        diag_printf("done, new value 0x%x \n", BDMATXCON);
    }
#endif

    return 1;
}


//======================================================================
// check to see if there's a frame waiting

static int rx_frame_avail(void)
{
    if (rxReadPointer->FrameDataPtr & FRM_OWNERSHIP_BDMA) {
        // queue is empty -- make sure Rx is running
        if (!(BDMARXCON & BDMARXCON_EN)) {
            ++BDMARxErrCnt.queueOverflow;
            BDMARXCON |= BDMARXCON_EN;
        }
        return 0;
    } else
        return 1;
}

//======================================================================
// Samsung MACs have a couple different failure modes.  Some models
// corrupt packets while DMAing them, and since CRC has already been
// verified, corrupted packets are marked as good.  Other models will
// mark good packets as having CRC errors when they're OK (this only
// seems to happen at cold temps.)
//
//
//
//   If CYGPKG_DEVS_ETH_ARM_DMRTS_SWRXCRC is defined,
//
//      All packets will have their CRCs verified by software.
//      Packets with software validated CRCs will be accepted.
//
// otherwise,
//
//      Packets will be accepted if HW CRC is valid _or_ if software
//      CRC is valid -- SW CRC will only be done on packets marked by
//      HW as bad (for any reason).

//======================================================================
// de-queue a receive buffer
static tEthBuffer *ks32c5000_eth_get_recv_buffer(void)
{
    unsigned RxStatus;
    unsigned RxLength;
    tEthBuffer *RxBufPtr;
    tEthBuffer *emptyBuf;

    while (1) {
        if (rxReadPointer->FrameDataPtr & FRM_OWNERSHIP_BDMA) {
            // queue is empty -- make sure Rx is running
            if (!(BDMARXCON & BDMARXCON_EN)) {
                ++BDMARxErrCnt.queueOverflow;
                BDMARXCON |= BDMARXCON_EN;
            }
            return NULL;
        }

        RxBufPtr = (tEthBuffer*)rxReadPointer->FrameDataPtr;
        RxStatus = rxReadPointer->StatusAndFrameLength >> 16;
        RxLength = rxReadPointer->StatusAndFrameLength & 0xffff;

        // counting on short-circuit && evaluation below to only
        // allocate a fresh buffer if rx packet is good!!

#if defined (CYGPKG_NET)
        ++ifStats.rx_count;
#endif

#  if defined (CYGPKG_DEVS_ETH_ARM_DMRTS_SWRXCRC)
#    define CheckPacket   (1)
#  else
#    define CheckPacket   (!(RxStatus & MACRXSTAT_GOOD))
#  endif

        if (CheckPacket && RxLength>4) {
            unsigned crc, crclen, crcOK;
            unsigned oldRxStatus = RxStatus;

            crclen = RxLength - 4;
            crc = crc32(RxBufPtr->data+2, crclen);
            crcOK = ((U08)(crc>>0) == RxBufPtr->data[2+crclen+0] &&
                     (U08)(crc>>8) == RxBufPtr->data[2+crclen+1] &&
                     (U08)(crc>>16) == RxBufPtr->data[2+crclen+2] &&
                     (U08)(crc>>24) == RxBufPtr->data[2+crclen+3]);

            //diag_printf("SWRXCRC: crcOK=%d\n", crcOK);

            if (crcOK) {
                RxStatus |= RXFDSTAT_GOOD;
                RxStatus &= ~RXFDSTAT_CRC_ERR;
            } else {
                RxStatus &= ~RXFDSTAT_GOOD;
                RxStatus |= RXFDSTAT_CRC_ERR;
            }

            if (0 && (RxStatus != oldRxStatus))
                diag_printf("BDMA rx status was %04x now %04x\n", oldRxStatus, RxStatus);
        }

        if ((RxStatus & RXFDSTAT_GOOD) && (emptyBuf = allocBuffer())) {
            // good packet and we've got a fresh buffer to take
            // it's place in the receive queue
            rxReadPointer->FrameDataPtr = (unsigned)emptyBuf | FRM_OWNERSHIP_BDMA;
            rxReadPointer = rxReadPointer->NextFD;
            RxBufPtr->length = RxLength - 4;
#if defined(CYGPKG_NET)
            ++ifStats.rx_deliver;
#endif
            return RxBufPtr;
        } else {
            // bad packet or out of buffers.  either way we
            // ignore this packet, and reuse the buffer
#if defined(CYGPKG_NET)
            if (RxStatus & RXFDSTAT_GOOD)
                ++ifStats.rx_resource;
            else {
                if (0) {
                    diag_printf("rx status %04x\n", RxStatus);
                    diag_dump_buf(RxBufPtr->data+2, RxLength);
                }
                ++ifStats.rx_crc_errors;
            }
#endif
            rxReadPointer->FrameDataPtr |= FRM_OWNERSHIP_BDMA;
            rxReadPointer = rxReadPointer->NextFD;
        }
    }
}

static void showCam(void) __attribute__((unused));
static void showCam(void)
{
    unsigned int cam_enabled = CAMEN;
    unsigned char mac[ETHER_ADDR_LEN];
    int i = 0;

    while (cam_enabled) {
        if (cam_enabled & 1) {
            memcpy(mac, (unsigned char *)(CAM_BaseAddr + (i * 6)), ETHER_ADDR_LEN);
            diag_printf("CAM %d: %s\n", i, macToStr(mac));
        }
        cam_enabled = cam_enabled >> 1;
        i++;
    }
}

// CAM registers have to be written as words (32-bits), so we
// can't just use memcpy() to write entries into it.

void setCamEntry(int i, unsigned char *mac)
{
    unsigned *p = (unsigned*)((CAM_BaseAddr + (i*6)) & ~3);
    //diag_printf("setCamEntry(%d,%s)\n",i,macToStr(mac));
    if (i&1) {
        p[0] &= 0xffff0000;
        p[0] |= (mac[0]<<8) | mac[1];
        p[1]  = (mac[2]<<24) | (mac[3]<<16) | (mac[4]<<8) | mac[5];
    } else {
        p[0]  = (mac[0]<<24) | (mac[1]<<16) | (mac[2]<<8) | mac[3];
        p[1] &= 0x0000ffff;
        p[1] |= (mac[4]<<24) | (mac[5]<<16);
    }
    CAMEN |= Bit(i);
    //showCam();
}


#if defined(CYGSEM_DEVS_ETH_ARM_DMRTS_KSZ8893MQL_MULTI_MAC)
# define ENABLE_KSZ8893MQL_HACKS (PhyId == PHY_ID_KSZ8893MQL)
# define PNIO_MCADDR_NUM         2
static unsigned char PNIO_MCADDR_LIST[PNIO_MCADDR_NUM][ETHER_ADDR_LEN] = {
    {0x01, 0x0e, 0xcf, 0x00, 0x00, 0x00},   // PN-DCP multicast address
    {0x01, 0x80, 0xc2, 0x00, 0x00, 0x0e}    // LLDP multicast address
};
#else
# define ENABLE_KSZ8893MQL_HACKS 0
#endif


//======================================================================
static int EthInit(U08* mac_address)
{
    // Set the initial condition of the BDMA.
    BDMARXCON = BDMARXCON_RESET;
    BDMATXCON = BDMATXCON_RESET;
    BDMARXLSZ = MAX_ETH_FRAME_SIZE;

    BDMARXPTR = (U32)rxReadPointer;
    BDMATXPTR = (U32)txWritePointer;

    MACCON = MACON_SW_RESET;

    //this is checked and set again in PhyReset based on
    //what the eth chip determines its link mode really is.
    //only the old kending is always half duplex.
    if (PhyId != PHY_ID_KENDIN) {
        MACConfigVar |= MACON_FULL_DUP;
    } else {
        MACConfigVar &= ~MACON_FULL_DUP;
    }

    MACCON = MACConfigVar;
    CAMCON = CAMConfigVar;
    CAMEN = 0;

    // set up our MAC address

    if (mac_address) {
        int index = 0;
        // set up main MAC address
        setCamEntry(index++, mac_address);
        if (ENABLE_KSZ8893MQL_HACKS) {
            unsigned char port_addr[ETHER_ADDR_LEN];
            cyg_uint16 low_addr;

            // use consecutive MAC addresses for port 1 and port 2
            memcpy(port_addr, mac_address, ETHER_ADDR_LEN);
            low_addr = ntohs(*(cyg_uint16 *)&(port_addr[4]));

            *(cyg_uint16 *)&(port_addr[4]) = htons(++low_addr);
            setCamEntry(index++, port_addr);

            *(cyg_uint16 *)&(port_addr[4]) = htons(++low_addr);
            setCamEntry(index++, port_addr);
        }
#if defined(CYGSEM_DEVS_ETH_ARM_DMRTS_KSZ8893MQL_MULTI_MAC)
        {
            // add multicast addresses used by Profinet
            int i;
            for (i = 0; i < PNIO_MCADDR_NUM; i++)
                setCamEntry(index++, PNIO_MCADDR_LIST[i]);
        }
#endif
    }

    // update the Configuration of BDMA and MAC to begin Rx/Tx

    BDMARXCON = BDMARxConfigVar;
    if (ENABLE_KSZ8893MQL_HACKS)
        MACRxConfigVar = 	MACRXCON_RX_EN | MACRXCON_LONG_EN | MACRXCON_EN_ALIGN |
                            MACRXCON_EN_CRC_ERR | MACRXCON_EN_OVER | MACRXCON_EN_LONG_ERR |
                            MACRXCON_EN_RX_PAR;
    MACRXCON = MACRxConfigVar;
    BDMATXCON = BDMATxConfigVar;
    MACTXCON =  MACTxConfigVar;

#if HavePHY
    PhyReset();
#endif

    //  diag_printf("MAC: %s\n", macToStr((unsigned char*)CAM_BaseAddr));

#if (0)
#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWRXCRC)
    diag_printf("Software Rx CRC\n");
#else
    diag_printf("Hardware Rx CRC\n");
#endif

#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWTXCRC)
    diag_printf("Software Tx CRC\n");
#else
    diag_printf("Hardware Tx CRC\n");
#endif
#endif

    return 0;
}//end EthInit

//======================================================================
static void Init_TxFrameDescriptorArray(void)
{
    FRAME_DESCRIPTOR *pFrameDescriptor;
    int i;

    // Each Frame Descriptor's frame data pointer points is NULL
    // if not in use, otherwise points to an ethBuffer
    pFrameDescriptor = txFrameDescrArray;

    for (i=0; i < MAX_TX_FRAME_DESCRIPTORS; i++) {
        pFrameDescriptor->FrameDataPtr = 0;
        pFrameDescriptor->Reserved = 0;
        pFrameDescriptor->StatusAndFrameLength = 0;
        pFrameDescriptor->NextFD = pFrameDescriptor+1;
        pFrameDescriptor++;
    }

    // fix up the last pointer to loop back to the first
    txFrameDescrArray[MAX_TX_FRAME_DESCRIPTORS-1].NextFD = txFrameDescrArray;
    txDonePointer = txWritePointer = txFrameDescrArray;

    return;
}


//======================================================================
static void Init_RxFrameDescriptorArray(void)
{
    FRAME_DESCRIPTOR *pFrameDescriptor;
    int i;

    // Each Frame Descriptor's frame data pointer points to
    // an ethBuffer struct
    pFrameDescriptor = rxFrameDescrArray;

    for (i=0; i < MAX_RX_FRAME_DESCRIPTORS; i++) {
        pFrameDescriptor->FrameDataPtr = ((unsigned)allocBuffer() | FRM_OWNERSHIP_BDMA);
        pFrameDescriptor->Reserved = 0;
        pFrameDescriptor->StatusAndFrameLength = 0;
        pFrameDescriptor->NextFD = pFrameDescriptor+1;
        pFrameDescriptor++;
    }

    // fix up the last pointer to loop back to the first
    rxFrameDescrArray[MAX_RX_FRAME_DESCRIPTORS-1].NextFD = rxFrameDescrArray;
    rxReadPointer = rxFrameDescrArray;

    return;
}

static cyg_ether_info_t config = {{0x00, 0xc0, 0x4e, 0x10, 0xff, 0xff}, 0, 0, \
    CONFIG_MAGIC, 0, sizeof(cyg_ether_info_t), 0
};


static void ks32c5000_handle_tx_complete(void)
{

    // record status and then free any buffers we're done with

    while (txDonePointer->FrameDataPtr && !(txDonePointer->FrameDataPtr & FRM_OWNERSHIP_BDMA)) {
#if defined(CYGPKG_NET)
        U32 txStatus;

        txStatus = txDonePointer->StatusAndFrameLength>>16;

        ++ks5000_MAC_Tx_Cnt;
        ++ifStats.interrupts;

        if (txStatus & MACTXSTAT_COMP)
            ++ifStats.tx_complete;

        if (txStatus & (MACTXSTAT_EX_COLL | MACTXSTAT_DEFFERED |
                        MACTXSTAT_UNDER | MACTXSTAT_DEFER |
                        MACTXSTAT_NCARR | MACTXSTAT_SIG_QUAL |
                        MACTXSTAT_LATE_COLL | MACTXSTAT_PAR |
                        MACTXSTAT_PAUSED | MACTXSTAT_HALTED)) {
            // transmit failed, log errors
            if (txStatus & MACTXSTAT_EX_COLL)
                ++ifStats.tx_max_collisions;
            if (txStatus & MACTXSTAT_DEFFERED)
                ++ifStats.tx_deferred;
            if (txStatus & MACTXSTAT_UNDER)
                ++ifStats.tx_underrun;
            if (txStatus & MACTXSTAT_DEFER)
                ;
            if (txStatus & MACTXSTAT_NCARR)
                ++ifStats.tx_carrier_loss;
            if (txStatus & MACTXSTAT_SIG_QUAL)
                ++ifStats.tx_sqetesterrors;
            if (txStatus & MACTXSTAT_LATE_COLL)
                ++ifStats.tx_late_collisions;
            if (txStatus & MACTXSTAT_PAR)
                ++Tx_Par;
            if (txStatus & MACTXSTAT_PAUSED)
                ++Tx_Paused;
            if (txStatus & MACTXSTAT_HALTED)
                ++Tx_Halted;
        } else {
            // transmit OK
            int collisionCnt = txStatus & 0x0f;
            ++ifStats.tx_good;
            if (collisionCnt) {
                if (collisionCnt == 1)
                    ++ifStats.tx_single_collisions;
                else
                    ++ifStats.tx_mult_collisions;
                ifStats.tx_total_collisions += collisionCnt;
            }
        }
#endif
        isrFreeBuffer((tEthBuffer*)txDonePointer->FrameDataPtr);
        txDonePointer->FrameDataPtr = 0;
        txDonePointer = txDonePointer->NextFD;
    }
}


//======================================================================
static cyg_uint32 MAC_Tx_isr(cyg_vector_t vector, cyg_addrword_t data)
{
    cyg_drv_interrupt_acknowledge(vector);
    return CYG_ISR_HANDLED|CYG_ISR_CALL_DSR;
}

static void MAC_Tx_dsr(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data)
{
    ks32c5000_handle_tx_complete();
}

static unsigned accumulatedMaxRxStatus=0;

//======================================================================
static cyg_uint32 MAC_Rx_isr(cyg_vector_t vector, cyg_addrword_t data)
{
    U32 IntMACRxStatus;

    cyg_drv_interrupt_acknowledge(vector);

    IntMACRxStatus = MACRXSTAT;
    MACRXSTAT = IntMACRxStatus;

    accumulatedMaxRxStatus |= IntMACRxStatus;

    ++ks5000_MAC_Rx_Cnt;

#if defined(CYGPKG_NET)

    ++ifStats.interrupts;

    if (IntMACRxStatus & MACRXSTAT_GOOD) {
        ++ifStats.rx_good;

        if (IntMACRxStatus & MACRXSTAT_CTL_RECD)
            ;  // we don't do anything with control packets

        return CYG_ISR_HANDLED;
    }

    if (IntMACRxStatus & (MACRXSTAT_ALLIGN_ERR | MACRXSTAT_CRC_ERR |
                          MACRXSTAT_OVERFLOW | MACRXSTAT_LONG_ERR |
                          MACRXSTAT_PAR | MACRXSTAT_HALTED) ) {
        if (IntMACRxStatus & MACRXSTAT_ALLIGN_ERR)
            ++ifStats.rx_align_errors;
        if (IntMACRxStatus & MACRXSTAT_CRC_ERR)
            ++ifStats.rx_crc_errors;
        if (IntMACRxStatus & MACRXSTAT_OVERFLOW)
            ++ifStats.rx_overrun_errors;
        if (IntMACRxStatus & MACRXSTAT_LONG_ERR)
            ++ifStats.rx_too_long_frames;
        if (IntMACRxStatus & MACRXSTAT_PAR)
            ++ifStats.rx_symbol_errors;
        if (IntMACRxStatus & MACRXSTAT_HALTED)
            ;
    }
#endif
    return CYG_ISR_HANDLED;
}



//======================================================================
// This interrupt only happens when errors occur
static cyg_uint32 BDMA_Tx_isr(cyg_vector_t vector, cyg_addrword_t data)
{
    U32 IntBDMATxStatus;

    cyg_drv_interrupt_acknowledge(vector);

    IntBDMATxStatus = BDMASTAT;
    BDMASTAT = IntBDMATxStatus;

    ++ks5000_BDMA_Tx_Isr_Cnt;
#if defined(CYGPKG_NET)
    ++ifStats.interrupts;
#endif
    return CYG_ISR_HANDLED|CYG_ISR_CALL_DSR;
}


static void BDMA_Tx_dsr(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data)
{
    U32 IntBDMATxStatus;

    ++ks5000_BDMA_Tx_Dsr_Cnt;


    IntBDMATxStatus = BDMASTAT;
    BDMASTAT = IntBDMATxStatus;

    ++ks5000_BDMA_Tx_Isr_Cnt;
#if defined(CYGPKG_NET)
    ++ifStats.interrupts;
#endif
    //if (IntBDMATxStatus & BDMASTAT_TX_CCP)
    //  {
    //    debug_printf("+-- Control Packet Transfered : %x\r",ERMPZCNT);
    //    debug_printf("    Tx Control Frame Status : %x\r",ETXSTAT);
    //  }

    if (IntBDMATxStatus & (BDMASTAT_TX_NL|BDMASTAT_TX_NO|BDMASTAT_TX_EMPTY) ) {
        if (IntBDMATxStatus & BDMASTAT_TX_NL)
            BDMATxErrCnt.BTxNLErr++;
        if (IntBDMATxStatus & BDMASTAT_TX_NO)
            BDMATxErrCnt.BTxNOErr++;
        if (IntBDMATxStatus & BDMASTAT_TX_EMPTY)
            BDMATxErrCnt.BTxEmptyErr++;
    }

    // free any buffers we're done with

    while (txDonePointer->FrameDataPtr && !(txDonePointer->FrameDataPtr & FRM_OWNERSHIP_BDMA)) {
        freeBuffer((tEthBuffer*)txDonePointer->FrameDataPtr);
        txDonePointer->FrameDataPtr = 0;
        txDonePointer = txDonePointer->NextFD;
    }

}



//======================================================================
static cyg_uint32 BDMA_Rx_isr(cyg_vector_t vector, cyg_addrword_t data)
{
    U32 IntBDMARxStatus;

    cyg_drv_interrupt_acknowledge(vector);

    IntBDMARxStatus = BDMASTAT;
    BDMASTAT = IntBDMARxStatus;

    ++ks5000_BDMA_Rx_Isr_Cnt;
#if defined(CYGPKG_NET)
    ++ifStats.interrupts;
#endif
    if (IntBDMARxStatus & (BDMASTAT_RX_NL  | BDMASTAT_RX_NO |
                           BDMASTAT_RX_MSO | BDMASTAT_RX_EMPTY |
                           BDMASTAT_RX_SEARLY) ) {
        //printf("RxIsr %u\r\n", (unsigned)cyg_current_time());
        if (IntBDMARxStatus & BDMASTAT_RX_NL)
            BDMARxErrCnt.BRxNLErr++;
        if (IntBDMARxStatus & BDMASTAT_RX_NO)
            BDMARxErrCnt.BRxNOErr++;
        if (IntBDMARxStatus & BDMASTAT_RX_MSO)
            BDMARxErrCnt.BRxMSOErr++;
        if (IntBDMARxStatus & BDMASTAT_RX_EMPTY)
            BDMARxErrCnt.BRxEmptyErr++;
        if (IntBDMARxStatus & BDMASTAT_RX_SEARLY)
            BDMARxErrCnt.sBRxSEarly++;
    }

    return CYG_ISR_HANDLED|CYG_ISR_CALL_DSR;
}

// Enable/disable interfaces to driver.  the first is the raw cyg_io_xxx
// device used to do non IP stuff.  The second is the interface to the
// eCos TCP/IP stack.  Should work in any combination.
#if defined(CYGPKG_NET)
#define KS32C5000_ETH_RAW 1
#else
#define KS32C5000_ETH_RAW 0
#endif

#define KS32C5000_ETH_TCPIP 1

#if KS32C5000_ETH_TCPIP
static void eth_handle_recv_buffer(tEthBuffer*);
#endif

#if KS32C5000_ETH_RAW
static void old_eth_handle_recv_buffer(tEthBuffer*);
#else
#define old_eth_handle_recv_buffer(rxBuffer) /* noop */
#endif

static cyg_handle_t  bdmaRxIntrHandle;
static cyg_handle_t  bdmaTxIntrHandle;
static cyg_handle_t  macRxIntrHandle;
static cyg_handle_t  macTxIntrHandle;

static cyg_interrupt  bdmaRxIntrObject;
static cyg_interrupt  bdmaTxIntrObject;
static cyg_interrupt  macRxIntrObject;
static cyg_interrupt  macTxIntrObject;

static int ethernetRunning;

#if HavePHY
#if !defined(CYGPKG_REDBOOT)
#endif // HavePHY
#endif // REDBOOT

static void ks32c5000_eth_deliver(struct eth_drv_sc *sc)
{
    unsigned short p;
    int not_filtered = 1, src_port = 1;
    tEthBuffer *rxBuffer;
    extern void cyg_interrupt_post_dsr(CYG_ADDRWORD intr_obj);
    //extern volatile unsigned PioData;

    //PioData |= Bit(11);

    ++ks5000_BDMA_Rx_Dsr_Cnt;

    while (1) {
        if (!rx_frame_avail()) {
            // No frames available, this is normal.  No incoming data.
            // PioData &= ~Bit(11);
            return;
        }

        if (!(rxBuffer=ks32c5000_eth_get_recv_buffer())) {
            // No buffers available, this is normal.  No incoming data.
            // PioData &= ~Bit(11);
            return;
        }

        ++Rx_Pkts;

        p = *((unsigned short*)(rxBuffer->data+EtherFramePadding+ETH_HEADER_SIZE-2));

        if (specialTagging) {
            // remove 802.1Q tag or the special tag if present
            if ((p & 0xfff0) == 0x8100) {
                int i;
                src_port = (p & 0xf);
                rxBuffer->length -= 4;
                i = EtherFramePadding + ETH_HEADER_SIZE - 2;
                memcpy(&rxBuffer->data[i], &rxBuffer->data[i+4], rxBuffer->length-(ETH_HEADER_SIZE - 2));
                p = *((unsigned short*)(rxBuffer->data+EtherFramePadding+ETH_HEADER_SIZE-2));
            }
        }

        if (KS32C5000_ETH_RAW && config.protocol && ((config.protocol==0xffff) || (config.protocol==p))) {
            old_eth_handle_recv_buffer(rxBuffer);
        } else {
            if (config.magic == CONFIG_MAGIC && config.func_ptr != NULL) {

                //diag_printf("src_port = %d, Got PNIO pkt of type 0x%04x\n", src_port, p);
                not_filtered = config.func_ptr(rxBuffer->data+EtherFramePadding,
                                               rxBuffer->length, src_port);
            }
            //diag_printf("Got Standard packet: ");
            if (0 != not_filtered && KS32C5000_ETH_TCPIP && ethernetRunning) {
                eth_handle_recv_buffer(rxBuffer);
            }
            freeBuffer(rxBuffer);
        }
    }

//  PioData &= ~Bit(11);
}


static void installInterrupts(void)
{

    extern struct eth_drv_sc ks32c5000_sc;
    bool firstTime = true;

    if (!firstTime) return;

    firstTime = false;

    initFreeList();
    Init_RxFrameDescriptorArray();
    Init_TxFrameDescriptorArray();

    BDMARXPTR = (U32)rxReadPointer;
    BDMATXPTR = (U32)txWritePointer;

    cyg_drv_mutex_init(&txMutex);
    cyg_drv_mutex_init(&oldRxMutex);
    cyg_drv_cond_init(&oldRxCond, &oldRxMutex);

    cyg_drv_interrupt_create(CYGNUM_HAL_INTERRUPT_EtherBDMARx, 0, (unsigned)&ks32c5000_sc, BDMA_Rx_isr, eth_drv_dsr, &bdmaRxIntrHandle, &bdmaRxIntrObject);
    cyg_drv_interrupt_create(CYGNUM_HAL_INTERRUPT_EtherBDMATx, 0, 0, BDMA_Tx_isr, BDMA_Tx_dsr, &bdmaTxIntrHandle, &bdmaTxIntrObject);
    cyg_drv_interrupt_create(CYGNUM_HAL_INTERRUPT_EtherMacRx, 0, 0, MAC_Rx_isr, NULL, &macRxIntrHandle, &macRxIntrObject);
    cyg_drv_interrupt_create(CYGNUM_HAL_INTERRUPT_EtherMacTx,0,0,MAC_Tx_isr,MAC_Tx_dsr,&macTxIntrHandle,&macTxIntrObject);

    cyg_drv_interrupt_attach(bdmaRxIntrHandle);
    cyg_drv_interrupt_attach(bdmaTxIntrHandle);
    cyg_drv_interrupt_attach(macRxIntrHandle);
    cyg_drv_interrupt_attach(macTxIntrHandle);

    cyg_drv_interrupt_acknowledge(CYGNUM_HAL_INTERRUPT_Ext0);
    cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_Ext0);
}//end installInterrupts

#if KS32C5000_ETH_RAW
//============================================================
//   eCos ethernet device driver that impliments old etherio.h
//   interface (not the TCP/IP stack interface)

static tEthBufQueue oldRxQueue = EmptyQueue;
static bool portMacStoredInEeprom = false;

// called from DSR to queue up receive packets
static void old_eth_handle_recv_buffer(tEthBuffer *rxBuffer)
{
    ethBufQueuePut(&oldRxQueue, rxBuffer);
    cyg_drv_cond_signal(&oldRxCond);    // wake up any pending reads
}


//----------------------------------------------------------------------
// get a receive packet from the queue where they are put by
// the routine above.
//
static tEthBuffer *old_get_receive_buffer(void)
{
    tEthBuffer *rxBuffer;
#if defined(CYGPKG_NET)
    while (!configDone)
        cyg_thread_delay_ms(10);
    cyg_drv_mutex_lock(&oldRxMutex);
    cyg_drv_dsr_lock();
#endif

    while (1) {
        rxBuffer = ethBufQueueGet(&oldRxQueue);

        if (rxBuffer)
            break;

        // nothing available

        if (config.flags &  CYG_ETHER_FLAGS_RD_NONBLOCK)
            break;

        // block until another frame is received
        cyg_drv_cond_wait(&oldRxCond);
    }

    cyg_drv_dsr_unlock();

    cyg_drv_mutex_unlock(&oldRxMutex);

    return rxBuffer;
}


static bool old_init_handler(struct cyg_devtab_entry *tab)
{
//  diag_printf("ks32c5000 old_init_handler()\n");

#if !KS32C5000_ETH_TCPIP
    installInterrupts();
    EthInit(config.mac_address);
    configDone = 1;
#endif
    return true;
}

static Cyg_ErrNo old_lookup_handler(struct cyg_devtab_entry **tab,
                                    struct cyg_devtab_entry *sub_tab,
                                    const char *name)
{
    return ENOERR;
}

//#define DEBUG_PORT_MACS

#ifdef DEBUG_PORT_MACS
#define DLOG(msg) \
	{ diag_printf("%lld:%s:%d: ", cyg_current_time(), __func__, __LINE__); \
	  diag_printf msg; \
	  diag_printf("\r"); }
#else
#define DLOG(msg)
#endif

static Cyg_ErrNo old_set_config_handler(cyg_io_handle_t handle, cyg_uint32 key, const void *xbuf, cyg_uint32 *len)
{
    //diag_printf("ks32c500 old_set_config_handler()\n");
    switch (key) {
    case CYG_IO_SET_CONFIG_ETHER_INFO:
        if (*len < sizeof (cyg_ether_info_t))
            return -EINVAL;
        if (memcmp(config.mac_address, ((cyg_ether_info_t*)xbuf)->mac_address, 6))
            EthInit(((cyg_ether_info_t*)xbuf)->mac_address);
        config = *((cyg_ether_info_t*)xbuf);
        break;

    case CYG_IO_SET_CONFIG_MAC_FWD:
        if (ENABLE_KSZ8893MQL_HACKS) {
            tConfigMacFwd *fwd_info = (tConfigMacFwd*)xbuf;
            if (*len < sizeof(tConfigMacFwd))
                return -EINVAL;
            WriteStaticMacTable(fwd_info->addr, 0, 0, 0, 1, fwd_info->port_mask, fwd_info->mac);
        }
        break;

    case CYG_IO_SET_CONFIG_SPEC_TAG_MODE: {
        unsigned char enableBit = ((unsigned char*)xbuf)[0];

        if (ENABLE_KSZ8893MQL_HACKS) {
            EnableSpecialTagging(enableBit);
        }
        specialTagging = enableBit;
        break;
    }
    case CYG_IO_SET_CONFIG_PORT_MACS: {
        cyg_uint8 port_addr[ETHER_ADDR_LEN];
        cyg_uint8 numOfPortMacs;
        cyg_uint8 startPortMac[3];
        cyg_uint16 low_addr;

        if (PhyId != PHY_ID_KSZ8893MQL)
            break;

        // Make sure the CAM is configured in compare mode, multicast-group
        // allowed and broadcast allowed
        CAMConfigVar = CAMCON_COMP_EN | CAMCON_GROUP_ACC | CAMCON_BROAD_ACC;
        CAMCON = CAMConfigVar;

        DLOG(("config.mac_address 0x [%02x %02x %02x %02x %02x %02x]\n",
              config.mac_address[0],
              config.mac_address[1],
              config.mac_address[2],
              config.mac_address[3],
              config.mac_address[4],
              config.mac_address[5]));

        // Use consecutive MAC addresses starting from the port
        // MAC address defined in EEPROM.
        memcpy(port_addr, config.mac_address, ETHER_ADDR_LEN);

        getStartingPortMacRt(&startPortMac[0]);
        getNumOfPortMacsRt(&numOfPortMacs);

        DLOG(("Port Mac Info EEPROM Values 0x[%02x %02x %02x %02x]\n",
              numOfPortMacs, startPortMac[0], startPortMac[1], startPortMac[2]));

        // Check legitimacy of EEPROM values
        if ( numOfPortMacs != 0 && numOfPortMacs != 0xff ) {
            int i;
            unsigned char low_addr_octets[2];

            low_addr_octets[0] = startPortMac[1];
            low_addr_octets[1] = startPortMac[2];

            // Assign fourth octet, same for both port MACs.
            port_addr[3] = startPortMac[0];

            // Assign fifth and sixth octets
            low_addr = ntohs(*(cyg_uint16 *)&(low_addr_octets[0]));

            //
            // Re-set port cam entries to the EEPROM defined values.
            for ( i = 1; i <= numOfPortMacs; i++ ) {
                DLOG(("htons(low_addr) %d = 0x[%04x]\n", i, htons(low_addr)));
                *(cyg_uint16 *)&(port_addr[4]) = htons(low_addr++);
                DLOG(("port_addr %d 0x[%02x %02x %02x %02x %02x %02x]\n", i,
                      port_addr[0],
                      port_addr[1],
                      port_addr[2],
                      port_addr[3],
                      port_addr[4],
                      port_addr[5]));
                setCamEntry(i, port_addr);
            }

            // Flag to specifically allow the port mac packet handling code to be run.
            portMacStoredInEeprom = true;
        } else {
            // This indicates that the EEPROM values have not been configured.
            diag_printf("ERROR: EEPROM values have not been configured properly, %d", numOfPortMacs);
            return -EINVAL;
        }

        break;
    }
    default:
        return -ENOSUPP;
    }

    return ENOERR;
}

static Cyg_ErrNo old_get_config_handler(cyg_io_handle_t handle,
                                        cyg_uint32 key,
                                        void *xbuf,
                                        cyg_uint32 *len)
{
    switch (key) {
    case CYG_IO_GET_CONFIG_ETHER_LINK_STATUS: {
#if HavePHY
        if (ENABLE_KSZ8893MQL_HACKS) {
            if (*len < 2) {
                return(-EINVAL);
            } else {
                int i;
                char *buf = xbuf;
                char phyStat[4];
                memset(phyStat, 0, 4);
                //
                // Bits for port 1 are in phyStat[0] and phyStat[1]
                // Bits for port 2 are in phyStat[2] and phyStat[3]
                //
                IicPhyReadRegs(KSZ8893MQL_PORT1_STATUS0_REG, &phyStat[0], 2);
                IicPhyReadRegs(KSZ8893MQL_PORT2_STATUS0_REG, &phyStat[2], 2);

                // Debugging
                //diag_printf("KSZ8893MQL_PORT1_STATUS0_REG === 0x%x:%x\n",phyStat[0],phyStat[1]);
                //diag_printf("KSZ8893MQL_PORT2_STATUS0_REG === 0x%x:%x\n",phyStat[2],phyStat[3]);

                //
                // For each port set the status bits.
                //
                for (i=0; i<4; i+=2) {
                    buf[i/2] = 0;
                    if (phyStat[i] & 1<<5) {
                        buf[i/2] |= PhyStatus_LinkUp;
                    }
                    if (phyStat[i+1] & 1 << 2) {
                        buf[i/2] |= PhyStatus_100Mb;
                    }
                    if (phyStat[i+1] & 1 << 1) {
                        buf[i/2] |= PhyStatus_FullDuplex;
                    }
                }

                return( ENOERR );
            }
        } else {
            if (*len < sizeof (unsigned))
                return -EINVAL;
            *((unsigned*)xbuf) = dmrtsLinkStatus;
            *len = sizeof (unsigned);
            return ENOERR;
        }
#else
        return -EINVAL;
#endif
        break;
    }

    case CYG_IO_GET_CONFIG_ETHER_INFO: {
        if (*len < sizeof (cyg_ether_info_t))
            return -EINVAL;
        *((cyg_ether_info_t*)xbuf) = config;
        *len = sizeof config;
        return ENOERR;
        break;
    }

    //  Get an EthBuf from the global linked list of buffers
    case CYG_IO_GET_CONFIG_ETHER_ALLOC_BUFFER: {
        tEthBuffer *b = allocBuffer();
        if (b) {
            *(tEthBuffer**)xbuf = b;
            return ENOERR;
        } else {
            *(tEthBuffer**)xbuf = NULL;
            return -ENOMEM;
        }
        break;
    }

    //  Free an EthBuf from the global linked list of buffers
    case CYG_IO_GET_CONFIG_ETHER_FREE_BUFFER: {
        if ((tEthBuffer*)xbuf < bufferPool ||
                (tEthBuffer*)xbuf >= (bufferPool + NUM_ETH_BUFFERS))
            return -EINVAL;
        freeBuffer((tEthBuffer*)xbuf);
        return ENOERR;
        break;
    }

    //  Send a tEthBuffer out the Ethernet using DMA
    case CYG_IO_GET_CONFIG_ETHER_SEND_BUFFER: {
        if ((tEthBuffer*)xbuf < bufferPool ||
                (tEthBuffer*)xbuf >= (bufferPool + NUM_ETH_BUFFERS))
            return -EINVAL;
        while (!ks32c5000_eth_buffer_send((tEthBuffer*)xbuf)) {
            if (config.flags &  CYG_ETHER_FLAGS_WR_NONBLOCK)
                return -EAGAIN;
            else
                cyg_thread_delay(1);
        }
        return ENOERR;
        break;
    }

    //  Retrieve received Ethernet data in form of a tEthBuffer struct.
    case CYG_IO_GET_CONFIG_ETHER_RECV_BUFFER: {
        tEthBuffer *rxBuffer;
        rxBuffer = old_get_receive_buffer();
        if (!rxBuffer) {
            *(tEthBuffer**)xbuf = NULL;
            *len = 0;
            return -EAGAIN;
        }

        *(tEthBuffer**)xbuf = rxBuffer;
        return ENOERR;
    }

    break;

    default:
        return -ENOSUPP;
    }
}

//  Writes data to ethernet.  Takes as input char data, memcpy's it to a
//  tEthBuffer struct, then calls the "new" write handler to push it to
//  the ethernet DMA.
static Cyg_ErrNo old_write_handler(cyg_io_handle_t handle, const void *_buf, cyg_uint32 *len)
{
    tEthBuffer *txBuffer;
    const char *txBuf = _buf;
    cyg_uint32 Datlen = *len;
    char srcPort = 0;

    while (!configDone)
        cyg_thread_delay_ms(10);

    if (*len < 14 || *len > MAX_ETH_FRAME_SIZE) {
        *len = 0;
        return -EINVAL;
    }

    if (ENABLE_KSZ8893MQL_HACKS) {
        DLOG(("Got packet with mac [%02X %02X %02X %02X %02X %02X]\n",
              txBuf[6], txBuf[7], txBuf[8], txBuf[9], txBuf[10], txBuf[11]));

        // check to see if port macs are stored in EEPROM
        if (portMacStoredInEeprom) {
            unsigned char	numOfPortMacs;
            unsigned char   startPortMac[3];

            getStartingPortMacRt(&startPortMac[0]);
            getNumOfPortMacsRt(&numOfPortMacs);
            DLOG(("[%02X %02X %02X %02X]\n", numOfPortMacs, startPortMac[0], startPortMac[1], startPortMac[2]));

            // Check to see that the 4th octet of the stored matches the current
            if ( startPortMac[0] == txBuf[9] ) {
                // the packet has a port mac address
                DLOG(("txBuf[9]:%02X == startPortMac[0]:%02X\n", txBuf[9], startPortMac[0]));
                srcPort = txBuf[11] - startPortMac[2];
                srcPort++;
            } else {
                // Port MAC in EEPROM does not match.
                DLOG(("txBuf[9]:%02X != startPortMac[0]:%02X\n", txBuf[9], startPortMac[0]));
                srcPort = txBuf[11] - config.mac_address[5];
            }
        } else {
            DLOG(("!portMacStoredInEeprom\n"));
            srcPort = txBuf[11] - config.mac_address[5];// txBuf[11] -
        }

        if (srcPort > 2) {
            diag_printf("Got bad src port: %d with mac %2X %2X %2X %2X %2X %2X\n", srcPort,
                        txBuf[6], txBuf[7], txBuf[8], txBuf[9], txBuf[10], txBuf[11]);
            srcPort = 0;
        }
    }

    txBuffer = allocBuffer();

    if (!txBuffer) {
        *len = 0;
        return -ENOMEM;
    }


    if (ENABLE_KSZ8893MQL_HACKS && specialTagging) {
        // If no specified srcPort, just copy packet and send...
        if (srcPort == 0)
            memcpy(txBuffer->data, txBuf, Datlen);
        else if (txBuf[12] == 0x81) {
            // If already an 802.1Q packet, just add src port bits...
            memcpy(txBuffer->data, txBuf, Datlen);
            //diag_printf("S%d\n", srcPort);
            txBuffer->data[13] |= srcPort;
        } else {
            // Need to add Special tagging bits to outbound packet and add 4 to length
            memcpy(txBuffer->data, txBuf, 12);
            txBuffer->data[12] = 0x81;
            txBuffer->data[13] = srcPort;
            txBuffer->data[14] = 0;
            txBuffer->data[15] = 0;
            memcpy(&txBuffer->data[16], &txBuf[12], Datlen-12);
            //diag_printf("srcPort = %d, 12 = 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n", srcPort,  (int)txBuffer->data[12],
            //	(int)txBuffer->data[13],(int)txBuffer->data[14],(int)txBuffer->data[15],(int)txBuffer->data[16], (int)txBuffer->data[17]);
            Datlen = *len + 4;
        }
    } else
        memcpy(txBuffer->data, txBuf, Datlen);

    txBuffer->length = Datlen;
    while (!ks32c5000_eth_buffer_send(txBuffer)) {
        if (config.flags &  CYG_ETHER_FLAGS_WR_NONBLOCK) {
            freeBuffer(txBuffer);
            *len = 0;
            return -EAGAIN;
        } else
            cyg_thread_delay(1);
    }

    return ENOERR;
}

static cyg_bool old_select_handler(cyg_io_handle_t handle, cyg_uint32 which, CYG_ADDRWORD info)
{
    // not supported, so just return true
    return true;
}


static Cyg_ErrNo old_read_handler(cyg_io_handle_t handle, void *_buf, cyg_uint32 *len)
{
    int size;
    tEthBuffer *rxBuffer;

    rxBuffer = old_get_receive_buffer();

    if (!rxBuffer) {
        *len = 0;
        return -EAGAIN;
    }

    // copy data from buffer into user space

    size = rxBuffer->length;

    if (size > *len)
        size = *len;

    memcpy(_buf, rxBuffer->data+2, size);

    freeBuffer(rxBuffer);

    *len = size;
    return ENOERR;
}

DEVIO_TABLE(ks32c5000EtherDevio, old_write_handler, old_read_handler, old_select_handler, old_get_config_handler, old_set_config_handler)

#if defined(CYGPKG_IO)
DEVTAB_ENTRY(ks32c5000EtherDevtab, "/dev/reth0", NULL, &ks32c5000EtherDevio, old_init_handler, old_lookup_handler, NULL)
#endif

#endif // KS32C5000_ETH_RAW

#if KS32C5000_ETH_TCPIP
//======================================================================
// Driver code that interfaces to the TCP/IP stack via the common
// Ethernet interface.


// don't have any private data, but if we did, this is where it would go
typedef struct {
    int j;
} ks32c5000_priv_data_t;

ks32c5000_priv_data_t ks32c5000_priv_data;

#define eth_drv_tx_done(sc,key,retval) (sc)->funs->eth_drv->tx_done(sc,key,retval)
#define eth_drv_init(sc,enaddr)  ((sc)->funs->eth_drv->init)(sc, config.mac_address)
#define eth_drv_recv(sc,len)  ((sc)->funs->eth_drv->recv)(sc, len)

static bool ks32c5000_eth_init(struct cyg_netdevtab_entry *tab)
{
    struct eth_drv_sc *sc = (struct eth_drv_sc *)tab->device_instance;
    memcpy(config.mac_address, (unsigned char*)CAM_BaseAddr, 6);

    diag_printf("ks32c5000_eth_init()\n");
    diag_printf("  found MAC address %s\n", macToStr(config.mac_address));

#if defined(CYGPKG_NET)
    ifStats.duplex = 1;      //unknown
    ifStats.operational = 1; //unknown
    ifStats.tx_queue_len = MAX_TX_FRAME_DESCRIPTORS;
#endif
    load_phy_id();
    installInterrupts();
    EthInit(config.mac_address);
#if defined(CYGPKG_NET)
    cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherBDMARx);
    cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherBDMATx);
    cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherMacRx);
    cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherMacTx);
#endif
    configDone = 1;
    ethernetRunning = 1;
    eth_drv_init(sc, config.mac_address);
    return true;
}//end ks32c5000_eth_init

static void ks32c5000_eth_start(struct eth_drv_sc *sc, unsigned char *enaddr, int flags)
{
//diag_printf("dmrts_ether.c: ks32c5000_eth_start begin.\n");
    if (!ethernetRunning) {
        cyg_drv_interrupt_mask(CYGNUM_HAL_INTERRUPT_EtherBDMARx);
        cyg_drv_interrupt_mask(CYGNUM_HAL_INTERRUPT_EtherBDMATx);
        cyg_drv_interrupt_mask(CYGNUM_HAL_INTERRUPT_EtherMacRx);
        cyg_drv_interrupt_mask(CYGNUM_HAL_INTERRUPT_EtherMacTx);
        EthInit(enaddr);
        cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherBDMARx);
        cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherBDMATx);
        cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherMacRx);
        cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherMacTx);
        ethernetRunning = 1;
        diag_printf("ks32c5000_eth_start(): Ethernet Up\n");
    }
//diag_printf("dmrts_ether.c: ks32c5000_eth_start end.\n");
}

static void ks32c5000_eth_stop(struct eth_drv_sc *sc)
{
    diag_printf("ks32c5000_eth_stop()\n");
    ethernetRunning = 0;
}

static int ks32c5000_eth_control(
    struct eth_drv_sc *sc, unsigned long cmd, void *data, int len)
{

    switch (cmd) {
#if defined(CYGPKG_NET)
    case ETH_DRV_GET_IF_STATS_UD:
    case ETH_DRV_GET_IF_STATS: {
        struct ether_drv_stats *p = (struct ether_drv_stats*)data;
        *p = ifStats;
        strncpy(p->description, "DMRTS Ethernet Interface", sizeof p->description);
        strncpy(p->snmp_chipset, "Samsung S3C45xx", sizeof p->snmp_chipset);
        return 0;
    }
#endif
    case ETH_DRV_SET_MAC_ADDRESS: {
        int act;

        if (ETHER_ADDR_LEN != len)
            return -1;

        if (memcmp(data, config.mac_address, ETHER_ADDR_LEN)) {
            diag_printf("eth_control: ETH_DRV_SET_MAC_ADDRESS %s\n",macToStr(data));
            act = ethernetRunning;
            ks32c5000_eth_stop(sc);
            ks32c5000_eth_start(sc, (cyg_uint8*)data, 0);
            ethernetRunning = act;
        }

        return 0;
    }
#ifdef	ETH_DRV_GET_MAC_ADDRESS
    case ETH_DRV_GET_MAC_ADDRESS: {
        if (len < ETHER_ADDR_LEN)
            return -1;
        diag_printf("eth_control: ETH_DRV_GET_MAC_ADDRESS.\n");
        memcpy(data, (void *)CAM_BaseAddr, 6);
        return 0;
    }
#endif
#ifdef ETH_DRV_SET_MC_LIST
    case ETH_DRV_SET_MC_LIST: {
        int i, next_available_entry;
        struct eth_drv_mc_list *mcl = (struct eth_drv_mc_list *)data;
        if (ENABLE_KSZ8893MQL_HACKS) {
            next_available_entry = 3;
            CAMEN = 0x07;
        } else {
            next_available_entry = 1;
            CAMEN = 0x01;
        }
#if defined(CYGSEM_DEVS_ETH_ARM_DMRTS_KSZ8893MQL_MULTI_MAC)
        // do not change the multicast addresses used by Profinet
        CAMEN |= ((1 << PNIO_MCADDR_NUM) - 1) << next_available_entry;
        next_available_entry += PNIO_MCADDR_NUM;
#endif
        //diag_printf("Setting MC List: ");
        if ((next_available_entry + mcl->len) > 21) {
            //diag_printf("Too many MC addresses: setting MC ALL\n");
            CAMCON |= CAMCON_GROUP_ACC;
            return 0;
        }

        for (i = 0; i < mcl->len; i++) {
            //diag_printf("%s, ",macToStr(mcl->addrs[i]));
            setCamEntry(next_available_entry++, mcl->addrs[i]);
        }
        //showCam();
        CAMCON &= ~CAMCON_GROUP_ACC;
        return 0;
    }
#endif
#ifdef ETH_DRV_SET_MC_ALL
    case ETH_DRV_SET_MC_ALL: {
        CAMCON |= CAMCON_GROUP_ACC;
        if (ENABLE_KSZ8893MQL_HACKS)
            CAMEN = 0x07;
        else
            CAMEN = 0x01;
        return 0;
    }
#endif
    default:
        return -1;
    }
}

static int ks32c5000_eth_can_send_count=0;
static int ks32c5000_eth_can_send_count_OK=0;

// In case there are multiple Tx frames waiting, we should
// return how many empty Tx spots we have.  For now we just
// return 0 or 1.
static int ks32c5000_eth_can_send(struct eth_drv_sc *sc)
{
    FRAME_DESCRIPTOR *TxFp, *StartFp;

    // find the next unused spot in the queue

    ++ks32c5000_eth_can_send_count;

    StartFp = TxFp = (FRAME_DESCRIPTOR*)BDMATXPTR;

    while (TxFp->FrameDataPtr & FRM_OWNERSHIP_BDMA) {
        TxFp = TxFp->NextFD;
        if (TxFp == StartFp)
            return 0;
    }
    ++ks32c5000_eth_can_send_count_OK;
    return 1;
}

static int ks5000_eth_send_count=0;

static void ks32c5000_eth_send(struct eth_drv_sc *sc,
                               struct eth_drv_sg *sg_list,
                               int sg_len,
                               int total_len,
                               unsigned long key)
{
    unsigned char *dest;
    unsigned        len;
    tEthBuffer *buf;

//  diag_printf("ks32c5000_eth_send()...");

    if (total_len >= MAX_ETH_FRAME_SIZE) {
        diag_printf("ks32c5000_eth_send() - Invalid length \n");
        eth_drv_tx_done(sc, key, -EINVAL);
        return;
    }

    ++ks5000_eth_send_count;

    // allocate buffer
    buf = allocBuffer();
    if (!buf) {
        // out of buffers
        diag_printf("ks32c5000_eth_send() - Out of buffers \n");
        eth_drv_tx_done(sc, key, -EIO);
        return;
    }

    // copy data from scatter/gather list into BDMA data buffer
    len = 0;
    dest = buf->data;

    while (sg_len) {
        memcpy(dest, (unsigned char*)sg_list->buf, sg_list->len);
        len += sg_list->len;
        dest += sg_list->len;
        ++sg_list;
        --sg_len;
    }

    buf->length = len;

    // tell upper layer that we're done with this sglist
    eth_drv_tx_done(sc, key, 0);

    // queue packet for transmit
    while (!ks32c5000_eth_buffer_send(buf)) {
#if defined(CYGPKG_NET)
        // wait a tick and try again.
        cyg_thread_delay(1);
#else

        // toss it.
        diag_printf("ks32c5000_eth_send() - Tossing buffer \n");
        freeBuffer(buf);
        break;
#endif
    }
}


static int ks5000_eth_rcv_count=0;
static tEthBuffer *tcpIpRxBuffer;

// called from DSR
static void eth_handle_recv_buffer(tEthBuffer* rxBuffer)
{
    extern struct eth_drv_sc ks32c5000_sc;
    tcpIpRxBuffer = rxBuffer;
    eth_drv_recv(&ks32c5000_sc, tcpIpRxBuffer->length);
}

static void ks32c5000_eth_recv(struct eth_drv_sc *sc,
                               struct eth_drv_sg *sg_list,
                               int sg_len)
{
    unsigned char *source;

    ++ks5000_eth_rcv_count;

    if (!tcpIpRxBuffer)
        return;  // no packet waiting, shouldn't be here!

    // copy data from eth buffer into scatter/gather list
    source = tcpIpRxBuffer->data + EtherFramePadding;

    while (sg_len) {
        if (sg_list->buf)
            memcpy((unsigned char*)sg_list->buf, source, sg_list->len);
        source += sg_list->len;
        ++sg_list;
        --sg_len;
    }
    return;
}

// routine called to handle ethernet controller in polled mode
static void ks32c5000_eth_poll(struct eth_drv_sc *sc)
{
    ks32c5000_eth_deliver(sc);  // handle rx frames
    ks32c5000_handle_tx_complete();
}

static int ks32c5000_eth_int_vector(struct eth_drv_sc *sc)
{
    return CYGNUM_HAL_INTERRUPT_EtherBDMARx;
}

ETH_DRV_SC(ks32c5000_sc,
           &ks32c5000_priv_data, // Driver specific data
           "eth0",                // Name for this interface
           ks32c5000_eth_start,
           ks32c5000_eth_stop,
           ks32c5000_eth_control,
           ks32c5000_eth_can_send,
           ks32c5000_eth_send,
           ks32c5000_eth_recv,
           ks32c5000_eth_deliver,
           ks32c5000_eth_poll,
           ks32c5000_eth_int_vector
          );

NETDEVTAB_ENTRY(ks32c5000_netdev,
                "dmrts",
                ks32c5000_eth_init,
                &ks32c5000_sc);
#endif


static U32 crctable[256] = {
    0x00000000L, 0x77073096L, 0xEE0E612CL, 0x990951BAL,
    0x076DC419L, 0x706AF48FL, 0xE963A535L, 0x9E6495A3L,
    0x0EDB8832L, 0x79DCB8A4L, 0xE0D5E91EL, 0x97D2D988L,
    0x09B64C2BL, 0x7EB17CBDL, 0xE7B82D07L, 0x90BF1D91L,
    0x1DB71064L, 0x6AB020F2L, 0xF3B97148L, 0x84BE41DEL,
    0x1ADAD47DL, 0x6DDDE4EBL, 0xF4D4B551L, 0x83D385C7L,
    0x136C9856L, 0x646BA8C0L, 0xFD62F97AL, 0x8A65C9ECL,
    0x14015C4FL, 0x63066CD9L, 0xFA0F3D63L, 0x8D080DF5L,
    0x3B6E20C8L, 0x4C69105EL, 0xD56041E4L, 0xA2677172L,
    0x3C03E4D1L, 0x4B04D447L, 0xD20D85FDL, 0xA50AB56BL,
    0x35B5A8FAL, 0x42B2986CL, 0xDBBBC9D6L, 0xACBCF940L,
    0x32D86CE3L, 0x45DF5C75L, 0xDCD60DCFL, 0xABD13D59L,
    0x26D930ACL, 0x51DE003AL, 0xC8D75180L, 0xBFD06116L,
    0x21B4F4B5L, 0x56B3C423L, 0xCFBA9599L, 0xB8BDA50FL,
    0x2802B89EL, 0x5F058808L, 0xC60CD9B2L, 0xB10BE924L,
    0x2F6F7C87L, 0x58684C11L, 0xC1611DABL, 0xB6662D3DL,
    0x76DC4190L, 0x01DB7106L, 0x98D220BCL, 0xEFD5102AL,
    0x71B18589L, 0x06B6B51FL, 0x9FBFE4A5L, 0xE8B8D433L,
    0x7807C9A2L, 0x0F00F934L, 0x9609A88EL, 0xE10E9818L,
    0x7F6A0DBBL, 0x086D3D2DL, 0x91646C97L, 0xE6635C01L,
    0x6B6B51F4L, 0x1C6C6162L, 0x856530D8L, 0xF262004EL,
    0x6C0695EDL, 0x1B01A57BL, 0x8208F4C1L, 0xF50FC457L,
    0x65B0D9C6L, 0x12B7E950L, 0x8BBEB8EAL, 0xFCB9887CL,
    0x62DD1DDFL, 0x15DA2D49L, 0x8CD37CF3L, 0xFBD44C65L,
    0x4DB26158L, 0x3AB551CEL, 0xA3BC0074L, 0xD4BB30E2L,
    0x4ADFA541L, 0x3DD895D7L, 0xA4D1C46DL, 0xD3D6F4FBL,
    0x4369E96AL, 0x346ED9FCL, 0xAD678846L, 0xDA60B8D0L,
    0x44042D73L, 0x33031DE5L, 0xAA0A4C5FL, 0xDD0D7CC9L,
    0x5005713CL, 0x270241AAL, 0xBE0B1010L, 0xC90C2086L,
    0x5768B525L, 0x206F85B3L, 0xB966D409L, 0xCE61E49FL,
    0x5EDEF90EL, 0x29D9C998L, 0xB0D09822L, 0xC7D7A8B4L,
    0x59B33D17L, 0x2EB40D81L, 0xB7BD5C3BL, 0xC0BA6CADL,
    0xEDB88320L, 0x9ABFB3B6L, 0x03B6E20CL, 0x74B1D29AL,
    0xEAD54739L, 0x9DD277AFL, 0x04DB2615L, 0x73DC1683L,
    0xE3630B12L, 0x94643B84L, 0x0D6D6A3EL, 0x7A6A5AA8L,
    0xE40ECF0BL, 0x9309FF9DL, 0x0A00AE27L, 0x7D079EB1L,
    0xF00F9344L, 0x8708A3D2L, 0x1E01F268L, 0x6906C2FEL,
    0xF762575DL, 0x806567CBL, 0x196C3671L, 0x6E6B06E7L,
    0xFED41B76L, 0x89D32BE0L, 0x10DA7A5AL, 0x67DD4ACCL,
    0xF9B9DF6FL, 0x8EBEEFF9L, 0x17B7BE43L, 0x60B08ED5L,
    0xD6D6A3E8L, 0xA1D1937EL, 0x38D8C2C4L, 0x4FDFF252L,
    0xD1BB67F1L, 0xA6BC5767L, 0x3FB506DDL, 0x48B2364BL,
    0xD80D2BDAL, 0xAF0A1B4CL, 0x36034AF6L, 0x41047A60L,
    0xDF60EFC3L, 0xA867DF55L, 0x316E8EEFL, 0x4669BE79L,
    0xCB61B38CL, 0xBC66831AL, 0x256FD2A0L, 0x5268E236L,
    0xCC0C7795L, 0xBB0B4703L, 0x220216B9L, 0x5505262FL,
    0xC5BA3BBEL, 0xB2BD0B28L, 0x2BB45A92L, 0x5CB36A04L,
    0xC2D7FFA7L, 0xB5D0CF31L, 0x2CD99E8BL, 0x5BDEAE1DL,
    0x9B64C2B0L, 0xEC63F226L, 0x756AA39CL, 0x026D930AL,
    0x9C0906A9L, 0xEB0E363FL, 0x72076785L, 0x05005713L,
    0x95BF4A82L, 0xE2B87A14L, 0x7BB12BAEL, 0x0CB61B38L,
    0x92D28E9BL, 0xE5D5BE0DL, 0x7CDCEFB7L, 0x0BDBDF21L,
    0x86D3D2D4L, 0xF1D4E242L, 0x68DDB3F8L, 0x1FDA836EL,
    0x81BE16CDL, 0xF6B9265BL, 0x6FB077E1L, 0x18B74777L,
    0x88085AE6L, 0xFF0F6A70L, 0x66063BCAL, 0x11010B5CL,
    0x8F659EFFL, 0xF862AE69L, 0x616BFFD3L, 0x166CCF45L,
    0xA00AE278L, 0xD70DD2EEL, 0x4E048354L, 0x3903B3C2L,
    0xA7672661L, 0xD06016F7L, 0x4969474DL, 0x3E6E77DBL,
    0xAED16A4AL, 0xD9D65ADCL, 0x40DF0B66L, 0x37D83BF0L,
    0xA9BCAE53L, 0xDEBB9EC5L, 0x47B2CF7FL, 0x30B5FFE9L,
    0xBDBDF21CL, 0xCABAC28AL, 0x53B39330L, 0x24B4A3A6L,
    0xBAD03605L, 0xCDD70693L, 0x54DE5729L, 0x23D967BFL,
    0xB3667A2EL, 0xC4614AB8L, 0x5D681B02L, 0x2A6F2B94L,
    0xB40BBE37L, 0xC30C8EA1L, 0x5A05DF1BL, 0x2D02EF8DL
};


static U32 crc32( unsigned char *blk_adr, unsigned blk_len)
{
    U32 crc = 0xffffffff;

    while (blk_len--)
        crc = crctable[(crc ^ *blk_adr++) & 0xffL] ^ (crc >> 8);

    return crc ^ 0xffffffff;
}


void dmrts_ether_show_buffers(void)
{
    diag_printf("Free: %d\n", freeCount);
    diag_printf(" Head: %x\n", freeList.head);
    diag_printf(" Tail: %x\n", freeList.tail);
#if EnableQueueCounts
    diag_printf("  Cnt: %d\n",freeList.count);
#endif
    diag_printf(" Alloc fail: %d\n",allocFail);
    diag_printf("oldRxQueue:\n");
#if KS32C5000_ETH_RAW
    diag_printf(" Head: %x\n",oldRxQueue.head);
    diag_printf(" Tail: %x\n", oldRxQueue.tail);
#if EnableQueueCounts
    diag_printf("  Cnt: %d\n",oldRxQueue.count);
#endif
#endif
}

void dmrts_ether_show_ifstats(void)
{
#if !defined(CYGPKG_NET)
    diag_printf("show ifstats not supported\n");
#else
# define showit(a) diag_printf(" %5d %s\n",ifStats.a,#a)
    showit(duplex);
    showit(operational);
    showit(speed);
    showit(interrupts);
    showit(rx_count);
    showit(rx_good);
    showit(rx_deliver);
    showit(rx_resource);
    showit(rx_crc_errors);
    showit(rx_align_errors);
    showit(rx_overrun_errors);
    showit(rx_too_long_frames);
    showit(rx_symbol_errors);
    showit(tx_count);
    showit(tx_complete);
    showit(tx_good);
    showit(tx_max_collisions);
    showit(tx_deferred);
    showit(tx_underrun);
    showit(tx_carrier_loss);
    showit(tx_sqetesterrors);
    showit(tx_late_collisions);
    showit(tx_single_collisions);
    showit(tx_total_collisions);
#endif
}

#define DumpValue(regname)  diag_printf("%30s: %08x\n",#regname,regname)


static void dumpErrorCounts(void)
{
    DumpValue(BDMARxErrCnt.BRxNLErr);
    DumpValue(BDMARxErrCnt.BRxNOErr);
    DumpValue(BDMARxErrCnt.BRxMSOErr);
    DumpValue(BDMARxErrCnt.BRxEmptyErr);
    DumpValue(BDMARxErrCnt.sBRxSEarly);
    DumpValue(BDMARxErrCnt.noBufferAvail);
    DumpValue(BDMARxErrCnt.queueOverflow);
    DumpValue(BDMARxErrCnt.bad);
    diag_printf("\n");
    DumpValue(BDMATxErrCnt.BTxNLErr);
    DumpValue(BDMATxErrCnt.BTxNOErr);
    DumpValue(BDMATxErrCnt.BTxEmptyErr);
    diag_printf("\n");
}


void dmrts_ether_dump_stats(void)
{
    DumpValue(BDMATXCON);
    DumpValue(BDMARXCON);
    DumpValue(BDMATXPTR);
    DumpValue(BDMARXPTR);
    DumpValue(BDMARXLSZ);
    DumpValue(BDMASTAT);
    DumpValue(CAM_BASE);
    DumpValue(BDMATXBUF);
    DumpValue(BDMARXBUF);
    DumpValue(CAM_BaseAddr);
    diag_printf("\n");
    DumpValue(MACCON);
    DumpValue(CAMCON);
    DumpValue(MACTXCON);
    DumpValue(MACTXSTAT);
    DumpValue(MACRXCON);
    DumpValue(MACRXSTAT);
    DumpValue(STADATA);
    DumpValue(STACON);
    DumpValue(CAMEN);
    DumpValue(EMISSCNT);
    DumpValue(EPZCNT);
    DumpValue(ERMPZCNT);
    DumpValue(ETXSTAT);
    DumpValue(MACRXDESTR);
    DumpValue(MACRXSTATEM);
    diag_printf("\n");
    dumpErrorCounts();
    diag_printf("\n");
    DumpValue(Rx_Pkts);
    DumpValue(Tx_Pkts);
    DumpValue(Tx_Full);
    DumpValue(Tx_Restart);
    DumpValue(Tx_Par);
    DumpValue(Tx_Paused);
    DumpValue(Tx_Halted);
    diag_printf("\n");
    DumpValue(freeCount);
    DumpValue(freeBufferCount());
    diag_printf("\n");
    dmrts_ether_show_ifstats();
    diag_printf("\n");
    dmrts_ether_show_buffers();
    diag_printf("\n");
    dmrts_ether_dump_frame_descriptors();
}
