#include <pkgconf/system.h>
#include <pkgconf/devs_eth_arm_dmrts.h>

#if defined(CYGPKG_IO)
#include <pkgconf/io.h>
#include <cyg/io/io.h>
#include <cyg/io/devtab.h>
#else
// need to prvide fake values for errno
#define EIO 1
#define EINVAL 2
#endif

#include <net/etherio.h>

#include <cyg/infra/cyg_type.h>  // Common type definitions and support
// including endian-ness

#include <cyg/infra/diag.h>
#include <cyg/io/eth/netdev.h>
#include <cyg/io/eth/eth_drv.h>
#include <cyg/io/eth/eth_drv_stats.h>
#include <cyg/hal/hal_intr.h>

#if defined(CYGPKG_REDBOOT)
#include <pkgconf/redboot.h>
#endif

#if !defined(CYGPKG_NET)
#define cyg_drv_interrupt_unmask(v) /* noop */
#define cyg_drv_interrupt_mask(v)   /* noop */
#define cyg_drv_isr_lock()          /* noop */
#define cyg_drv_isr_unlock()        /* noop */
#define cyg_drv_mutex_init(m)       /* noop */
#define cyg_drv_mutex_lock(m)       /* noop */
#define cyg_drv_mutex_unlock(m)     /* noop */
#define cyg_drv_dsr_lock()          /* noop */
#define cyg_drv_dsr_unlock()        /* noop */
#endif

#if !defined(CYGPKG_DEVS_ETH_ARM_DMRTS_PHY)
# error "Invalid configuration no: PHY setting."
#endif

#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_PHY_NONE)
#define HavePHY 0
#else
#define HavePHY 1
#endif

#include "std.h"
#include "ks5000_regs.h"
#include "ks5000_ether.h"
#if HavePHY
#include "phy.h"
#endif

#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWRXCRC) || defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWTXCRC)
static U32 crc32( unsigned char *blk_adr, unsigned blk_len);
#endif

#if 1
#define debug_printf(args...) diag_printf(args)
#else
#define debug_printf(args...) /* noop */
#endif
#define Bit(n) (1<<(n))

extern void hal_delay_us(int us);

char *macToStr(unsigned char *mac)
{
    static char hex[] = "0123456789ABCDEF";
    static char buf[18];
    unsigned char *p = buf;
    int i;
    for (i=0; i<6; ++i) {
        *p++ = hex[(mac[i] >> 4) & 0xf];
        *p++ = hex[(mac[i]) & 0xf];
        *p++ = ':';
    }
    buf[17] = '\0';
    return buf;
}

// miscellaneous data structures

typedef  BYTE  ETH_ADDR[6] __attribute__((packed));

typedef  struct tagETH_HEADER {
    ETH_ADDR daddr __attribute__((packed));
    ETH_ADDR saddr __attribute__((packed));
    WORD type      __attribute__((packed));
} ETH_HEADER __attribute__((packed));

#define ETH_HEADER_SIZE   14

// Tx/Rx common descriptor structure
typedef  struct tagFRAME_DESCRIPTOR {
    LWORD FrameDataPtr;
    LWORD Reserved;   /*  cf: RX-reserved, TX-Reserved(25bits) + Control bits(7bits) */
    LWORD StatusAndFrameLength;
    struct tagFRAME_DESCRIPTOR   *NextFD;
} FRAME_DESCRIPTOR;

typedef struct {
    U8 DestinationAddr[6];
    U8 SourceAddr[6];
    U8 LengthOrType[2];
    U8 LLCData[1506];
} MAC_FRAME;

#if defined(CYGPKG_NET)
static cyg_drv_mutex_t txMutex;
struct ether_drv_stats ifStats;
#endif

typedef struct {
    LWORD BTxNLErr;
    LWORD BTxNOErr;
    LWORD BTxEmptyErr;
} BDMA_TX_ERR;

typedef struct {
    LWORD BRxNLErr;
    LWORD BRxNOErr;
    LWORD BRxMSOErr;
    LWORD BRxEmptyErr;
    LWORD sBRxSEarly;
    LWORD noBufferAvail;
    LWORD queueOverflow;
    LWORD bad;
} BDMA_RX_ERR;


// interrupt entry counters
U32 ks5000_MAC_Rx_Cnt;
U32 ks5000_MAC_Tx_Cnt;
U32 ks5000_MAC_Phy_Cnt;
U32 ks5000_BDMA_Tx_Isr_Cnt;
U32 ks5000_BDMA_Tx_Dsr_Cnt;
U32 ks5000_BDMA_Rx_Isr_Cnt;
U32 ks5000_BDMA_Rx_Dsr_Cnt;


// packet and byte counters
static U32 MAC_Tx_Pkts;
static U32 MAC_Tx_Octets;
// static U32 BDMA_Rx_Pkts;
// static U32 BDMA_Rx_Octets;

// configuration values
//static volatile U32 MACConfigVar = MACON_FULL_DUP;
static volatile U32 MACConfigVar;
static volatile U32 CAMConfigVar = CAMCON_COMP_EN | CAMCON_BROAD_ACC;
static volatile U32 MACTxConfigVar =
    MACTXCON_EN_UNDER |
#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWTXCRC)
    MACTXCON_NO_CRC |
#endif
    MACTXCON_EN_DEFER |
    MACTXCON_EN_NCARR |
    MACTXCON_EN_EXCOLL |
    MACTXCON_EN_LATE_COLL |
    MACTXCON_ENTX_PAR |
    MACTXCON_EN_COMP;
static volatile U32 MACRxConfigVar =
    MACRXCON_RX_EN |
    MACRXCON_EN_ALIGN |
    MACRXCON_EN_CRC_ERR |
    MACRXCON_EN_OVER |
    MACRXCON_EN_LONG_ERR |
    MACRXCON_EN_RX_PAR;

#if defined(CYGPKG_HAL_ARM_RPSH2)
# define BdmaTxBurstLen 3
#else
# define BdmaTxBurstLen 7
#endif

static volatile U32 BDMATxConfigVar =
    BDMATXCON_MSL111 |
    BDMATXCON_STP_SKP |
    BdmaTxBurstLen;   /* burst size - 1 */

#define EtherFramePadding 2

#if EtherFramePadding == 0
#define BDMARXCON_ALIGN BDMARXCON_WA00
#elif EtherFramePadding == 1
#define BDMARXCON_ALIGN BDMARXCON_WA01
#elif EtherFramePadding == 2
#define BDMARXCON_ALIGN BDMARXCON_WA10
#elif EtherFramePadding == 3
#define BDMARXCON_ALIGN BDMARXCON_WA11
#else
#error "EtherFramePadding must be 0,1,2 or 3"
#endif

static volatile U32 BDMARxConfigVar =
    BDMARXCON_DIE |
    BDMARXCON_EN |
    BDMARXCON_BIG |
    BDMARXCON_MA_INC |
    BDMARXCON_NOIE |
    BDMARXCON_ALIGN |
    BDMARXCON_STP_SKP |
    15;  /* burst size - 1 */


/* Global variables For BDMA Error Report */

static BDMA_TX_ERR BDMATxErrCnt = {0,0,0};
static BDMA_RX_ERR BDMARxErrCnt = {0,0,0,0,0};

static void Init_TxFrameDescriptorArray(void);
static void Init_RxFrameDescriptorArray(void);

// number of ethernet buffers should be enough to keep both rx
// and tx queues full plus some extras for in-process packets

#if defined(CYGPKG_REDBOOT)
#define NUM_ETH_BUFFERS CYGNUM_REDBOOT_ETHER_BUFS
#define MAX_RX_FRAME_DESCRIPTORS  CYGNUM_REDBOOT_ETHER_DESCR     // Max number of Rx Frame Descriptors
#define MAX_TX_FRAME_DESCRIPTORS  CYGNUM_REDBOOT_ETHER_DESCR     // Max number of Tx Frame Descriptors
#else
#define NUM_ETH_BUFFERS CYGNUM_ECOS_ETHER_BUFS
#define MAX_RX_FRAME_DESCRIPTORS  CYGNUM_ECOS_ETHER_DESCR     // Max number of Rx Frame Descriptors
#define MAX_TX_FRAME_DESCRIPTORS  CYGNUM_ECOS_ETHER_DESCR     // Max number of Tx Frame Descriptors
#endif

static FRAME_DESCRIPTOR _rxFrameDescrArray[MAX_RX_FRAME_DESCRIPTORS] __attribute__((aligned(16)));
static FRAME_DESCRIPTOR _txFrameDescrArray[MAX_TX_FRAME_DESCRIPTORS] __attribute__((aligned(16)));

/* define aliases that will set the no-cache bit */
#define rxFrameDescrArray ((FRAME_DESCRIPTOR*)(((unsigned)_rxFrameDescrArray)|0x4000000))
#define txFrameDescrArray ((FRAME_DESCRIPTOR*)(((unsigned)_txFrameDescrArray)|0x4000000))

static FRAME_DESCRIPTOR *rxReadPointer;
static FRAME_DESCRIPTOR *txDonePointer;
static FRAME_DESCRIPTOR *txWritePointer;

static cyg_drv_mutex_t oldRxMutex;
static cyg_drv_cond_t  oldRxCond;

static void dumpDescriptor(FRAME_DESCRIPTOR *p)
{
    diag_printf("%x: D=%x R=%08x S=%08x N=%x\n",
                p,
                p->FrameDataPtr,
                p->Reserved,
                p->StatusAndFrameLength,
                p->NextFD);
}

void dmrts_ether_dump_frame_descriptors(void)
{
    int i;

    diag_printf("Rx Descriptors:\n");
    for (i=0; i<MAX_RX_FRAME_DESCRIPTORS; ++i)
        dumpDescriptor(rxFrameDescrArray+i);

    diag_printf("Tx Descriptors:\n");
    for (i=0; i<MAX_TX_FRAME_DESCRIPTORS; ++i)
        dumpDescriptor(txFrameDescrArray+i);
}

static bool configDone;

/*----------------------------------------------------------------------
 * Data structures used to manage ethernet buffers
 */

#define EnableQueueCounts 0

typedef struct {
    tEthBuffer *head;
    tEthBuffer *tail;
#if EnableQueueCounts
    int count;
#endif
} tEthBufQueue;

#if EnableQueueCounts
#define EmptyQueue {NULL,NULL,0}
#else
#define EmptyQueue {NULL,NULL}
#endif

static void ethBufQueueClear(tEthBufQueue *q)
{
    q->head = NULL;
    q->tail = NULL;
#if EnableQueueCounts
    q->count = 0;
#endif
}

//  Get the next EthBuf from the linked list q.
static tEthBuffer *ethBufQueueGet(tEthBufQueue *q)
{
    tEthBuffer *r;

    r = q->head;

    if (r) {
        q->head = r->next;
#if EnableQueueCounts
        --q->count;
#endif
    }
    return r;
}

//  Add an EthBuf to the linked list q.
static void ethBufQueuePut(tEthBufQueue *q, tEthBuffer *b)
{
    b->next = NULL;

    if (!q->head) {
        q->head = b;
        q->tail = b;
    } else {
        q->tail->next = b;
        q->tail = b;
    }
#if EnableQueueCounts
    ++q->count;
#endif

}

#if 0
// not used at the moment
static bool ethBufQueueEmpty(tEthBufQueue *q)
{
    return q->head != NULL;
}
#endif

/*----------------------------------------------------------------------
 * Free pool and routines to manipulate it.
 */
static tEthBufQueue freeList;
static int freeCount;

#define EnableBufferTrace 0

#if EnableBufferTrace
#define BufferTraceCount 256 /* must be power of 2 */
static int bufferTraceIndex;
static struct {
    unsigned lineNumber;
    unsigned event;
    unsigned bufPtr;
    unsigned free;
} bufferTrace[BufferTraceCount];
static void addBufferTrace(unsigned l, unsigned e, void *p)
{
    bufferTrace[bufferTraceIndex].lineNumber = l;
    bufferTrace[bufferTraceIndex].event = e;
    bufferTrace[bufferTraceIndex].bufPtr = (unsigned)p;
    bufferTrace[bufferTraceIndex].free = freeCount;
    ++bufferTraceIndex;
    bufferTraceIndex &= (BufferTraceCount-1);
}
#else
#define addBufferTrace(l,e,p) /* noop */
#endif

static tEthBuffer __bufferPool[NUM_ETH_BUFFERS] __attribute__((aligned(16)));
#define bufferPool ((tEthBuffer*)((unsigned)__bufferPool|0x4000000))

//  Frees an EthBuf by putting it back on the freeList
//  Do not call from ISR routine.
//  Input:  Ptr to EthBuf to be freed
static void freeBuffer(tEthBuffer *b)
{
    cyg_drv_isr_lock();
    ++freeCount;
    ethBufQueuePut(&freeList,b);
    cyg_drv_isr_unlock();
    //addBufferTrace(__LINE__,17,b);
}

static int allocFail;

void bufferListError(void)
{
    diag_printf("%s:%d Buffer List Error!\n",__FILE__,__LINE__);
}

//  Get an EthBuf, removing it from the freeList
//  Do not call from ISR routine
//  Returns:  Ptr to the EthBuf allocated, NULL if none available
static tEthBuffer *allocBuffer(void)
{
    tEthBuffer *r;

    cyg_drv_isr_lock();
    r = ethBufQueueGet(&freeList);
    if (r)
        --freeCount;
    else {
        ++allocFail;
        if (freeCount)
            bufferListError();
    }
    cyg_drv_isr_unlock();
    //addBufferTrace(__LINE__,16,r);
    return r;
}

//  Frees an EthBuf by putting it back on the freeList
//  Call only from ISR routine or init.
//  Input:  Ptr to EthBuf to be freed
static void isrFreeBuffer(tEthBuffer *b)
{
    ++freeCount;
    ethBufQueuePut(&freeList,b);
    //addBufferTrace(__LINE__,19,b);
}

#if 0
// not used at the moment

// call only from ISR routine or init
static tEthBuffer *isrAllocBuffer(void)
{
    tEthBuffer *r;
    r = ethBufQueueGet(&freeList);
    if (r)
        --freeCount;
    else {
        ++allocFail;
        if (freeCount)
            bufferListError();
    }
    //addBufferTrace(__LINE__,18,r);
    return r;
}
#endif

//  Initialize the freeList, add all the EthBufs from the bufferPool to it.
static void initFreeList(void)
{
    int i;

    ethBufQueueClear(&freeList);
    freeCount = 0;
    for (i=0; i<NUM_ETH_BUFFERS; ++i) {
        //addBufferTrace(__LINE__,0,bufferPool+i);
        isrFreeBuffer(bufferPool+i);
    }
}

//----------------------------------------------------------------------
// queue a buffer for transmit
//
// returns true if buffer was queued.

static int ks32c5000_eth_buffer_send(tEthBuffer *buf)
{
#if defined(CYGPKG_NET)
    while (!configDone)
        cyg_thread_delay(10);
#endif

    if (txWritePointer->FrameDataPtr & FRM_OWNERSHIP_BDMA) {
        // queue is full!  make sure transmit is running
        BDMATXCON |= BDMATXCON_EN;
        MACTXCON |= MACTXCON_TX_EN;
        return 0;
    }

    MAC_Tx_Pkts += 1;
    MAC_Tx_Octets += buf->length;

#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWTXCRC)
    // add Ethernet CRC
    {
        unsigned long crc;
        unsigned len = buf->length;
        if (len < 64) {
            memset(buf->data+len,0,64-len);
            len = 64;
        }
        crc = crc32(buf->data,len);
        buf->data[len++] = crc;
        buf->data[len++] = crc>>8;
        buf->data[len++] = crc>>16;
        buf->data[len++] = crc>>24;
        buf->length = len;
    }
#endif

    cyg_drv_mutex_lock(&txMutex);
    cyg_drv_isr_lock();

    // free old buffer if we need to
    if (txWritePointer->FrameDataPtr) {
        //addBufferTrace(__LINE__,3,(void*)txWritePointer->FrameDataPtr);
        freeBuffer((tEthBuffer*)txWritePointer->FrameDataPtr);
        txWritePointer->FrameDataPtr = 0;
    }
    cyg_drv_isr_unlock();

    // fill in the packet descriptor

    txWritePointer->Reserved = (  TXFDCON_PADDING_MODE
#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWTXCRC)
                                  | TXFDCON_NO_CRC_MODE
#endif
                                  | TXFDCON_SRC_ADDR_INC
                                  | TXFDCON_BIG_ENDIAN
                                  | TXFDCON_WIDGET_ALIGN00
                                  | TXFDCON_MAC_TX_INT_EN);

    txWritePointer->StatusAndFrameLength = buf->length;
    txWritePointer->FrameDataPtr = ((unsigned)buf | FRM_OWNERSHIP_BDMA);

    /*
     *  Found during eCos 2.0 Redboot testing - this function fails, adding this small delay
     *  fixes it.  Not required under eCos 1.3.1.  Problem observation: first packet sent does not
     *  transmit, next packet sent causes both packets to be transmitted.  This repeats.
     *  Debug code here shows that the Ownership bit is set in the TX frame, the BDMA is enabled,
     *  but the BDMA finds the ownership bit clear, so it disables itself.  Looks like a race
     *  condition.  Kurt R.  9-14-04.
     */
    hal_delay_us(1);

    txWritePointer = txWritePointer->NextFD;

    cyg_drv_mutex_unlock(&txMutex);

    // start transmit

#if defined(CYGPKG_NET)
    ++ifStats.tx_count;
#endif
    BDMATXCON |= BDMATXCON_EN;
    MACTXCON |= MACTXCON_TX_EN;

    //  Debug code, used during eCos 2.0 Redboot debugging...
#if (0)
    if (!(BDMATXCON &  BDMATXCON_EN)) {
        debug_printf("BDMATXCON_EN not set...");

        if (!(txWritePointer->FrameDataPtr & FRM_OWNERSHIP_BDMA))
            debug_printf("Ownership bit not set...");

        if (!(txWritePointer->NextFD))
            debug_printf("NextFD is NULL...");

        debug_printf("re-setting...");
        BDMATXCON |= BDMATXCON_EN;
        MACTXCON |= MACTXCON_TX_EN;
        debug_printf("done, new value 0x%x \n", BDMATXCON);
    }
#endif

    return 1;
}


//======================================================================
// check to see if there's a frame waiting

static int rx_frame_avail(void)
{
    if (rxReadPointer->FrameDataPtr & FRM_OWNERSHIP_BDMA) {
        // queue is empty -- make sure Rx is running
        if (!(BDMARXCON & BDMARXCON_EN)) {
            ++BDMARxErrCnt.queueOverflow;
            BDMARXCON |= BDMARXCON_EN;
        }
        return 0;
    } else
        return 1;
}


//======================================================================
// de-queue a receive buffer
static tEthBuffer *ks32c5000_eth_get_recv_buffer(void)
{
//	static long loop_count;

    unsigned RxStatusAndLength;
    tEthBuffer *RxBufPtr;
    tEthBuffer *emptyBuf;
    /*
    #if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWRXCRC)
      unsigned crc, crclen;
      int crcOK;
    #else
    # define crcOK 1
    #endif
    */

# define crcOK 1

    while (1) {
//    	loop_count++;

        if (rxReadPointer->FrameDataPtr & FRM_OWNERSHIP_BDMA) {
            // queue is empty -- make sure Rx is running
            if (!(BDMARXCON & BDMARXCON_EN)) {
                ++ifStats.rx_resource_errors;

                ++BDMARxErrCnt.queueOverflow;
                BDMARXCON |= BDMARXCON_EN;
            }
            return NULL;
        }

        RxBufPtr = (tEthBuffer*)rxReadPointer->FrameDataPtr;
        RxStatusAndLength = rxReadPointer->StatusAndFrameLength;

        // counting on short-circuit && evaluation below to only
        // allocate a fresh buffer if rx packet is good!!

#if defined (CYGPKG_NET)
        ++ifStats.rx_count;
#endif

#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWRXCRC)
        crclen = (RxStatusAndLength & 0xffff) - 4;
        crc = crc32(RxBufPtr->data+2,crclen);
        crcOK = ((U08)(crc>>0) == RxBufPtr->data[2+crclen+0] &&
                 (U08)(crc>>8) == RxBufPtr->data[2+crclen+1] &&
                 (U08)(crc>>16) == RxBufPtr->data[2+crclen+2] &&
                 (U08)(crc>>24) == RxBufPtr->data[2+crclen+3]);
#endif

        if ((RxStatusAndLength & (RXFDSTAT_GOOD<<16))
                && crcOK
                && (emptyBuf = allocBuffer())) {
//          addBufferTrace(__LINE__,1,emptyBuf);
            // good packet and we've got a fresh buffer to take
            // it's place in the receive queue
            rxReadPointer->FrameDataPtr = (unsigned)emptyBuf | FRM_OWNERSHIP_BDMA;
            rxReadPointer = rxReadPointer->NextFD;
            RxBufPtr->length = (RxStatusAndLength & 0xffff) - 4;
#if defined(CYGPKG_NET)
            ++ifStats.rx_deliver;
#endif
//          addBufferTrace(__LINE__,20,RxBufPtr);
//          if (!(loop_count % 100)) dmrts_ether_show_ifstats();
//          if (!(loop_count % 500)) dmrts_ether_show_buffer_trace();

            return RxBufPtr;
        } else {
            // bad packet or out of buffers.  either way we
            // ignore this packet, and reuse the buffer
#if defined(CYGPKG_NET)
            if (!crcOK) {
                ++ifStats.rx_crc_errors;
//		  	diag_printf("ERROR: CRC bad in ks32c5000_eth_get_recv_buffer()\n");
            } else if ((RxStatusAndLength & (RXFDSTAT_GOOD<<16))) {
                ++ifStats.rx_resource_errors;
                diag_printf("ERROR: allocate buffer failed. (packet good)\n");
            } else {
                diag_printf("ERROR: RxStatusAndLength = 0x%02x in ks32c5000_eth_get_recv_buffer()\n",RxStatusAndLength);
                diag_printf("ERROR: RxStatusAndLength = 0x%02x in ks32c5000_eth_get_recv_buffer()\n",(RXFDSTAT_GOOD<<16));
                diag_printf("ERROR: RxStatusAndLength = 0x%02x in ks32c5000_eth_get_recv_buffer()\n",RxStatusAndLength & (RXFDSTAT_GOOD<<16));
            }
#endif
//          addBufferTrace(__LINE__,2,(void*)rxReadPointer->FrameDataPtr);

            rxReadPointer->FrameDataPtr |= FRM_OWNERSHIP_BDMA;
            rxReadPointer = rxReadPointer->NextFD;

//		  if (!(loop_count % 100)) dmrts_ether_show_ifstats();
//        if (!(loop_count % 500)) dmrts_ether_show_buffer_trace();
        }//end if ((RxStatusAndLength & (RXFDSTAT_GOOD<<16))
    }//end while(1)
}

static void showCam(void) __attribute__((unused));
static void showCam(void)
{
    int i;
    diag_printf("CAM: \n");
    for (i=0; i<32; i+=4)
        diag_printf(" %08x\n", *((unsigned*)(CAM_BaseAddr+i)));
}

// CAM registers have to be written as words (32-bits), so we
// can't just use memcpy() to write entries into it.

void setCamEntry(int i, unsigned char *mac)
{
    unsigned *p = (unsigned*)((CAM_BaseAddr + (i*6)) & ~3);
    //diag_printf("setCamEntry(%d,%s)\n",i,macToStr(mac));
    if (i&1) {
        p[0] &= 0xffff0000;
        p[0] |= (mac[0]<<8) | mac[1];
        p[1]  = (mac[2]<<24) | (mac[3]<<16) | (mac[4]<<8) | mac[5];
    } else {
        p[0]  = (mac[0]<<24) | (mac[1]<<16) | (mac[2]<<8) | mac[3];
        p[1] &= 0x0000ffff;
        p[1] |= (mac[4]<<24) | (mac[5]<<16);
    }
    //showCam();
}

//======================================================================
static int EthInit(U08* mac_address)
{
#if (0)
    if (mac_address)
        debug_printf("EthInit(%s)\n",macToStr(mac_address));
    else
        debug_printf("EthInit(NULL)\n");
#endif

#if HavePHY
    PhyReset();
#endif

    /* Set the initial condition of the BDMA. */
    BDMARXCON = BDMARXCON_RESET;
    BDMATXCON = BDMATXCON_RESET;
    BDMARXLSZ = MAX_ETH_FRAME_SIZE;

    BDMARXPTR = (U32)rxReadPointer;
    BDMATXPTR = (U32)txWritePointer;

    MACCON = MACON_SW_RESET;
    MACCON = MACConfigVar;

    CAMCON = CAMConfigVar;

    // set up our MAC address

    if (mac_address)
        setCamEntry(0,mac_address);

    // CAM Enable
    CAMEN = 0x0001;

    // update the Configuration of BDMA and MAC to begin Rx/Tx

    BDMARXCON = BDMARxConfigVar;
    MACRXCON = MACRxConfigVar;

    BDMATXCON = BDMATxConfigVar;
    MACTXCON =  MACTxConfigVar;

//  diag_printf("MAC: %s\n", macToStr((unsigned char*)CAM_BaseAddr));

#if (0)
#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWRXCRC)
    diag_printf("Software Rx CRC\n");
#else
    diag_printf("Hardware Rx CRC\n");
#endif

#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWTXCRC)
    diag_printf("Software Tx CRC\n");
#else
    diag_printf("Hardware Tx CRC\n");
#endif
#endif

    return 0;
}

//======================================================================
static void Init_TxFrameDescriptorArray(void)
{
    FRAME_DESCRIPTOR *pFrameDescriptor;
    int i;

    // Each Frame Descriptor's frame data pointer points is NULL
    // if not in use, otherwise points to an ethBuffer
    pFrameDescriptor = txFrameDescrArray;

    for(i=0; i < MAX_TX_FRAME_DESCRIPTORS; i++) {
        pFrameDescriptor->FrameDataPtr = 0;
        pFrameDescriptor->Reserved = 0;
        pFrameDescriptor->StatusAndFrameLength = 0;
        pFrameDescriptor->NextFD = pFrameDescriptor+1;
        pFrameDescriptor++;
    }

    // fix up the last pointer to loop back to the first
    txFrameDescrArray[MAX_TX_FRAME_DESCRIPTORS-1].NextFD = txFrameDescrArray;
    txDonePointer = txWritePointer = txFrameDescrArray;

    return;
}


//======================================================================
static void Init_RxFrameDescriptorArray(void)
{
    FRAME_DESCRIPTOR *pFrameDescriptor;
    int i;

    // Each Frame Descriptor's frame data pointer points to
    // an ethBuffer struct
    pFrameDescriptor = rxFrameDescrArray;

    for(i=0; i < MAX_RX_FRAME_DESCRIPTORS; i++) {
        pFrameDescriptor->FrameDataPtr = ((unsigned)allocBuffer() | FRM_OWNERSHIP_BDMA);
        pFrameDescriptor->Reserved = 0;
        pFrameDescriptor->StatusAndFrameLength = 0;
        pFrameDescriptor->NextFD = pFrameDescriptor+1;
        pFrameDescriptor++;
    }

    // fix up the last pointer to loop back to the first
    rxFrameDescrArray[MAX_RX_FRAME_DESCRIPTORS-1].NextFD = rxFrameDescrArray;
    rxReadPointer = rxFrameDescrArray;

    return;
}

static cyg_ether_info_t config = {{0x00,0xc0,0x4e,0x10,0xff,0xff},0,0, \
    CONFIG_MAGIC,0, sizeof(cyg_ether_info_t), 0
};
#if HavePHY
#if !defined(CYGPKG_REDBOOT)

static unsigned linkStatus;

//  ISR that responds to a link change from the PHY chip.  The purpose is to
//  keep the Ethernet controller (in the CPU) and the PHY's duplex settings
//  in sync.
static cyg_uint32 MAC_Phy_isr(cyg_vector_t vector, cyg_addrword_t data)
{
    static unsigned int oldLinkStatus;

    cyg_drv_interrupt_acknowledge(vector);
    PhyInterruptAck();
    linkStatus = PhyStatus();

    //  Check for link change
    if (linkStatus != oldLinkStatus) {

        diag_printf("Ethernet Phy Status Change:");

        //  If duplex changes, must change the Ethernet controller duplex setting
        if (linkStatus & PhyStatus_FullDuplex) {
            MACConfigVar |= (1<<3);
            diag_printf("  Full Duplex  ");
        } else {
            MACConfigVar &= ~(1<<3);
            diag_printf("  Half Duplex  ");
        }

        if (linkStatus & PhyStatus_100Mb)
            diag_printf("  100MB \n");
        else
            diag_printf("  10MB \n");

        oldLinkStatus = linkStatus;
    }

#if defined(CYGPKG_NET)
    if (linkStatus & PhyStatus_FullDuplex)
        ifStats.duplex = 3;
    else
        ifStats.duplex = 2;

    if (linkStatus & PhyStatus_LinkUp)
        ifStats.operational = 3;
    else
        ifStats.operational = 2;

    if (linkStatus & PhyStatus_100Mb)
        ifStats.speed = 100000000;
    else
        ifStats.speed = 10000000;
#endif  //CYGPKG_NET

    MACCON = MACConfigVar;
    return CYG_ISR_HANDLED;
}
#endif // REDBOOT
#endif // HavePHY

static void ks32c5000_handle_tx_complete(void)
{

    // record status and then free any buffers we're done with

    while (txDonePointer->FrameDataPtr && !(txDonePointer->FrameDataPtr & FRM_OWNERSHIP_BDMA)) {
#if defined(CYGPKG_NET)
        U32 txStatus;

        txStatus = txDonePointer->StatusAndFrameLength>>16;

        if (txStatus & MACTXSTAT_COMP)
            ++ifStats.tx_complete;

        if (txStatus & (MACTXSTAT_EX_COLL | MACTXSTAT_DEFFERED |
                        MACTXSTAT_UNDER | MACTXSTAT_DEFER |
                        MACTXSTAT_NCARR | MACTXSTAT_SIG_QUAL |
                        MACTXSTAT_LATE_COLL | MACTXSTAT_PAR |
                        MACTXSTAT_PAUSED | MACTXSTAT_HALTED)) {
            // transmit failed, log errors
            if (txStatus & MACTXSTAT_EX_COLL)
                ++ifStats.tx_max_collisions;
            if (txStatus & MACTXSTAT_DEFFERED)
                ++ifStats.tx_deferred;
            if (txStatus & MACTXSTAT_UNDER)
                ++ifStats.tx_underrun;
            if (txStatus & MACTXSTAT_DEFER)
                ;
            if (txStatus & MACTXSTAT_NCARR)
                ++ifStats.tx_carrier_loss;
            if (txStatus & MACTXSTAT_SIG_QUAL)
                ++ifStats.tx_sqetesterrors;
            if (txStatus & MACTXSTAT_LATE_COLL)
                ++ifStats.tx_late_collisions;
            if (txStatus & MACTXSTAT_PAR)
                ;
            if (txStatus & MACTXSTAT_PAUSED)
                ;
            if (txStatus & MACTXSTAT_HALTED)
                ;
        } else {
            // transmit OK
            int collisionCnt = txStatus & 0x0f;
            ++ifStats.tx_good;
            if (collisionCnt) {
                if (collisionCnt == 1)
                    ++ifStats.tx_single_collisions;
                else
                    ++ifStats.tx_mult_collisions;
                ifStats.tx_total_collisions += collisionCnt;
            }
        }
#endif
        //addBufferTrace(__LINE__,4,(void*)txDonePointer->FrameDataPtr);
        isrFreeBuffer((tEthBuffer*)txDonePointer->FrameDataPtr);
        txDonePointer->FrameDataPtr = 0;
        txDonePointer = txDonePointer->NextFD;
    }
}


//======================================================================
static cyg_uint32 MAC_Tx_isr(cyg_vector_t vector, cyg_addrword_t data)
{
    cyg_drv_interrupt_acknowledge(vector);
    ks32c5000_handle_tx_complete();
    ++ifStats.interrupts;
    ++ks5000_MAC_Tx_Cnt;
    return CYG_ISR_HANDLED;
}

static unsigned accumulatedMaxRxStatus=0;

//======================================================================
static cyg_uint32 MAC_Rx_isr(cyg_vector_t vector, cyg_addrword_t data)
{
    U32 IntMACRxStatus;

    cyg_drv_interrupt_acknowledge(vector);

    IntMACRxStatus = MACRXSTAT;
    MACRXSTAT = IntMACRxStatus;

    accumulatedMaxRxStatus |= IntMACRxStatus;

    ++ks5000_MAC_Rx_Cnt;

#if defined(CYGPKG_NET)

    ++ifStats.interrupts;

    if (IntMACRxStatus & MACRXSTAT_GOOD) {
        ++ifStats.rx_good;

        if (IntMACRxStatus & MACRXSTAT_CTL_RECD)
            ;  // we don't do anything with control packets

        return CYG_ISR_HANDLED;
    }

    if (IntMACRxStatus & (MACRXSTAT_ALLIGN_ERR | MACRXSTAT_CRC_ERR |
                          MACRXSTAT_OVERFLOW | MACRXSTAT_LONG_ERR |
                          MACRXSTAT_PAR | MACRXSTAT_HALTED) ) {
        if (IntMACRxStatus & MACRXSTAT_ALLIGN_ERR)
            ++ifStats.rx_align_errors;
        if (IntMACRxStatus & MACRXSTAT_CRC_ERR)
            ++ifStats.rx_crc_errors;
        if (IntMACRxStatus & MACRXSTAT_OVERFLOW)
            ++ifStats.rx_overrun_errors;
        if (IntMACRxStatus & MACRXSTAT_LONG_ERR)
            ++ifStats.rx_too_long_frames;
        if (IntMACRxStatus & MACRXSTAT_PAR)
            ++ifStats.rx_symbol_errors;
        if (IntMACRxStatus & MACRXSTAT_HALTED)
            ;
    }
#endif
    return CYG_ISR_HANDLED;
}



//======================================================================
// This interrupt only happens when errors occur
static cyg_uint32 BDMA_Tx_isr(cyg_vector_t vector, cyg_addrword_t data)
{
    U32 IntBDMATxStatus;

    cyg_drv_interrupt_acknowledge(vector);

    IntBDMATxStatus = BDMASTAT;
    BDMASTAT = IntBDMATxStatus;

    ++ks5000_BDMA_Tx_Isr_Cnt;
#if defined(CYGPKG_NET)
    ++ifStats.interrupts;
#endif
    if (IntBDMATxStatus & BDMASTAT_TX_CCP) {
        debug_printf("+-- Control Packet Transfered : %x\r",ERMPZCNT);
        debug_printf("    Tx Control Frame Status : %x\r",ETXSTAT);
    }

    if (IntBDMATxStatus & (BDMASTAT_TX_NL|BDMASTAT_TX_NO|BDMASTAT_TX_EMPTY) ) {
        if (IntBDMATxStatus & BDMASTAT_TX_NL)
            BDMATxErrCnt.BTxNLErr++;
        if (IntBDMATxStatus & BDMASTAT_TX_NO)
            BDMATxErrCnt.BTxNOErr++;
        if (IntBDMATxStatus & BDMASTAT_TX_EMPTY)
            BDMATxErrCnt.BTxEmptyErr++;
    }

    // free any buffers we're done with

    while (txDonePointer->FrameDataPtr && !(txDonePointer->FrameDataPtr & FRM_OWNERSHIP_BDMA)) {
        //addBufferTrace(__LINE__,5,(void*)txDonePointer->FrameDataPtr);
        freeBuffer((tEthBuffer*)txDonePointer->FrameDataPtr);
        txDonePointer->FrameDataPtr = 0;
        txDonePointer = txDonePointer->NextFD;
    }

    // don't call tx dsr for now -- it has nothing to do

    return CYG_ISR_HANDLED;
}


static void BDMA_Tx_dsr(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data)
{
    ++ks5000_BDMA_Tx_Dsr_Cnt;
}



//======================================================================
static cyg_uint32 BDMA_Rx_isr(cyg_vector_t vector, cyg_addrword_t data)
{
    U32 IntBDMARxStatus;

    cyg_drv_interrupt_acknowledge(vector);

    IntBDMARxStatus = BDMASTAT;
    BDMASTAT = IntBDMARxStatus;

    ++ks5000_BDMA_Rx_Isr_Cnt;
#if defined(CYGPKG_NET)
    ++ifStats.interrupts;
#endif
    if (IntBDMARxStatus & (BDMASTAT_RX_NL  | BDMASTAT_RX_NO |
                           BDMASTAT_RX_MSO | BDMASTAT_RX_EMPTY |
                           BDMASTAT_RX_SEARLY) ) {
//      printf("RxIsr %u\r\n", (unsigned)cyg_current_time());
        if (IntBDMARxStatus & BDMASTAT_RX_NL)
            BDMARxErrCnt.BRxNLErr++;
        if (IntBDMARxStatus & BDMASTAT_RX_NO)
            BDMARxErrCnt.BRxNOErr++;
        if (IntBDMARxStatus & BDMASTAT_RX_MSO)
            BDMARxErrCnt.BRxMSOErr++;
        if (IntBDMARxStatus & BDMASTAT_RX_EMPTY)
            BDMARxErrCnt.BRxEmptyErr++;
        if (IntBDMARxStatus & BDMASTAT_RX_SEARLY)
            BDMARxErrCnt.sBRxSEarly++;
    }

    return CYG_ISR_HANDLED|CYG_ISR_CALL_DSR;
}

// Enable/disable interfaces to driver.  the first is the raw cyg_io_xxx
// device used to do non IP stuff.  The second is the interface to the
// eCos TCP/IP stack.  Should work in any combination.
#if defined(CYGPKG_NET)
#define KS32C5000_ETH_RAW 1
#else
#define KS32C5000_ETH_RAW 0
#endif

#define KS32C5000_ETH_TCPIP 1

#if KS32C5000_ETH_TCPIP
static void eth_handle_recv_buffer(tEthBuffer*);
#endif

#if KS32C5000_ETH_RAW
static void old_eth_handle_recv_buffer(tEthBuffer*);
#else
#define old_eth_handle_recv_buffer(rxBuffer) /* noop */
#endif

static cyg_handle_t  bdmaRxIntrHandle;
static cyg_handle_t  bdmaTxIntrHandle;
static cyg_handle_t  macRxIntrHandle;
static cyg_handle_t  macTxIntrHandle;

static cyg_interrupt  bdmaRxIntrObject;
static cyg_interrupt  bdmaTxIntrObject;
static cyg_interrupt  macRxIntrObject;
static cyg_interrupt  macTxIntrObject;

static int ethernetRunning;

#if HavePHY
#if !defined(CYGPKG_REDBOOT)
static cyg_handle_t  macPhyIntrHandle;
static cyg_interrupt  macPhyIntrObject;
#endif // HavePHY
#endif // REDBOOT

static void ks32c5000_eth_deliver(struct eth_drv_sc *sc)
{
    static long loop_count;

//	loop_count++;
//	if (!(loop_count % 100)) dmrts_ether_show_ifstats();
//	if (!(loop_count % 500)) dmrts_ether_show_buffer_trace();
//	static long loop_count;
//	++ifStats.rx_resource_errors;

    unsigned short p;
    int not_filtered = 1;
    tEthBuffer *rxBuffer;
    extern void cyg_interrupt_post_dsr(CYG_ADDRWORD intr_obj);
//  extern volatile unsigned PioData;

//  PioData |= Bit(11);

    ++ks5000_BDMA_Rx_Dsr_Cnt;

    while (1) {

        loop_count++;

        if (!rx_frame_avail()) {
            // No frames available, this is normal.  No incoming data.
//          PioData &= ~Bit(11);
            return;
        }

        if (!(rxBuffer=ks32c5000_eth_get_recv_buffer())) {
            // No buffers available, this is normal.  No incoming data.
//          PioData &= ~Bit(11);
            return;
        }

        p = *((unsigned short*)(rxBuffer->data+EtherFramePadding+ETH_HEADER_SIZE-2));

        if (KS32C5000_ETH_RAW && config.protocol &&
                ((config.protocol==0xffff) || (config.protocol==p))) {
            old_eth_handle_recv_buffer(rxBuffer);
        } else {
            if (config.magic == CONFIG_MAGIC && config.func_ptr != NULL) {
                //diag_printf("Got pkt of type 0x%x\n", p);
                not_filtered = config.func_ptr(rxBuffer->data+EtherFramePadding,
                                               rxBuffer->length, 1);
            }
            //debug_printf("Got Standard packet: ");
            if (0 != not_filtered && KS32C5000_ETH_TCPIP && ethernetRunning) {
                //addBufferTrace(__LINE__,21,rxBuffer);
                eth_handle_recv_buffer(rxBuffer);
            } else {
                if (!not_filtered)
                    diag_printf("TCP: ks32c5000_eth_deliver(): Frame filtered\n");
                else if (!KS32C5000_ETH_TCPIP)
                    diag_printf("TCP: ks32c5000_eth_deliver(): TCPIP mode check failed.\n");
                else if (!ethernetRunning)
                    diag_printf("TCP: ks32c5000_eth_deliver(): Ethernet not running.\n");

                addBufferTrace(__LINE__,10,rxBuffer);
            }

            freeBuffer(rxBuffer);
        }

//	  if (!(loop_count % 1000)) {
//	  	dmrts_ether_show_ifstats();
//	  	dmrts_ether_show_buffers();
//	  }
//      if (!(loop_count % 500)) dmrts_ether_show_buffer_trace();

    }//end while
//  PioData &= ~Bit(11);
}


static void installInterrupts(void)
{
    extern struct eth_drv_sc ks32c5000_sc;
    bool firstTime=true;
    int phyType;

//  debug_printf("ks5000_ether: installInterrupts()\n");
    if (!firstTime)
        return;
    firstTime = false;

    initFreeList();
    Init_RxFrameDescriptorArray();
    Init_TxFrameDescriptorArray();

    BDMARXPTR = (U32)rxReadPointer;
    BDMATXPTR = (U32)txWritePointer;

    cyg_drv_mutex_init(&txMutex);
    cyg_drv_mutex_init(&oldRxMutex);
    cyg_drv_cond_init(&oldRxCond,&oldRxMutex);

    cyg_drv_interrupt_create(CYGNUM_HAL_INTERRUPT_EtherBDMARx,0,(unsigned)&ks32c5000_sc,BDMA_Rx_isr,eth_drv_dsr,&bdmaRxIntrHandle,&bdmaRxIntrObject);
    cyg_drv_interrupt_create(CYGNUM_HAL_INTERRUPT_EtherBDMATx,0,0,BDMA_Tx_isr,BDMA_Tx_dsr,&bdmaTxIntrHandle,&bdmaTxIntrObject);
    cyg_drv_interrupt_create(CYGNUM_HAL_INTERRUPT_EtherMacRx,0,0,MAC_Rx_isr,NULL,&macRxIntrHandle,&macRxIntrObject);
    cyg_drv_interrupt_create(CYGNUM_HAL_INTERRUPT_EtherMacTx,0,0,MAC_Tx_isr,NULL,&macTxIntrHandle,&macTxIntrObject);

#if HavePHY
    //  The RTS design has either the Kendin switch/PHY or the Agere AC101L PHY.
    //  The RPSHSI2P uses Level One LXT970.  We need to determine which we are,
    //  since the Agere/Level One  requires an interrupt handler to switch the
    //  Eth Controller based on duplex mode.  We only do this for the
    //  application.  In Redboot, we simply run the ethernet controller in
    //  half duplex mode.
#if defined(CYGPKG_REDBOOT)
    MACCON  &= ~(1<<3);  //  half duplex mode.
#else
    phyType = PhyIsPresent();
    if (phyType) {      //  0=Kendin (no PHY) 1=AC101L 2=LXT970
        if (phyType == 1)
            debug_printf("Broadcom AC101L PHY Detected \n");
        else if (phyType == 2)
            debug_printf("Level One LXT970 PHY Detected \n");

        //  Set interrupt to low level activated.
        IOPCON &=  ~0x1F;  //  Set interrupt 0 to be active low, enabled
        IOPCON |=   0x10;
        cyg_drv_interrupt_create(CYGNUM_HAL_INTERRUPT_Ext0,
                                 0, 0, MAC_Phy_isr,
                                 NULL, &macPhyIntrHandle,
                                 &macPhyIntrObject);
        cyg_drv_interrupt_attach(macPhyIntrHandle);
    } else {
        debug_printf("Kendin PHY Detected \n");
    }
#endif // REDBOOT
#endif //HavePHY

    cyg_drv_interrupt_attach(bdmaRxIntrHandle);
    cyg_drv_interrupt_attach(bdmaTxIntrHandle);
    cyg_drv_interrupt_attach(macRxIntrHandle);
    cyg_drv_interrupt_attach(macTxIntrHandle);

    cyg_drv_interrupt_acknowledge(CYGNUM_HAL_INTERRUPT_Ext0);
    cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_Ext0);
}

#if KS32C5000_ETH_RAW
//============================================================
//   eCos ethernet device driver that impliments old etherio.h
//   interface (not the TCP/IP stack interface)

static tEthBufQueue oldRxQueue = EmptyQueue;

// called from DSR to queue up receive packets
static void old_eth_handle_recv_buffer(tEthBuffer *rxBuffer)
{
    /*
    	static long loop_count;
    	loop_count++;
    	if (!(loop_count % 100))
    		diag_printf(
    			"RAW: old_eth_handle_recv_buffer()->ethBufQueuePut() called at pkt %lu.\n",
    			loop_count
    		);
    */
    ethBufQueuePut(&oldRxQueue,rxBuffer);
    cyg_drv_cond_signal(&oldRxCond);    // wake up any pending reads
}


//----------------------------------------------------------------------
// get a receive packet from the queue where they are put by
// the routine above.
//
static tEthBuffer *old_get_receive_buffer(void)
{
    tEthBuffer *rxBuffer;
#if defined(CYGPKG_NET)
    while (!configDone)
        cyg_thread_delay(10);
    cyg_drv_mutex_lock(&oldRxMutex);
    cyg_drv_dsr_lock();
#endif

    while (1) {
        rxBuffer = ethBufQueueGet(&oldRxQueue);

        if (rxBuffer)
            break;

        // nothing available

        if (config.flags &  CYG_ETHER_FLAGS_RD_NONBLOCK)
            break;

        // block until another frame is received
        cyg_drv_cond_wait(&oldRxCond);
    }

#if defined(CYGPKG_NET)
    cyg_drv_dsr_unlock();
    cyg_drv_mutex_unlock(&oldRxMutex);
#endif

    //addBufferTrace(__LINE__,14,rxBuffer);
    return rxBuffer;
}


static bool old_init_handler(struct cyg_devtab_entry *tab)
{
//  debug_printf("ks32c5000 old_init_handler()\n");
#if !KS32C5000_ETH_TCPIP
    installInterrupts();
    EthInit(config.mac_address);
    configDone = 1;
#endif
    return true;
}

static Cyg_ErrNo old_lookup_handler(struct cyg_devtab_entry **tab,
                                    struct cyg_devtab_entry *sub_tab,
                                    const char *name)
{
    return ENOERR;
}

static Cyg_ErrNo old_set_config_handler(cyg_io_handle_t handle, cyg_uint32 key, const void *xbuf, cyg_uint32 *len)
{
    //debug_printf("ks32c500 old_set_config_handler()\n");
    switch (key) {
    case CYG_IO_SET_CONFIG_ETHER_INFO:
        if (*len < sizeof (cyg_ether_info_t))
            return -EINVAL;
        if (memcmp(config.mac_address, ((cyg_ether_info_t*)xbuf)->mac_address,6))
            EthInit(((cyg_ether_info_t*)xbuf)->mac_address);
        config = *((cyg_ether_info_t*)xbuf);
        return ENOERR;
        break;

    default:
        return -ENOSUPP;
    }
}

static Cyg_ErrNo old_get_config_handler(cyg_io_handle_t handle,
                                        cyg_uint32 key,
                                        void *xbuf,
                                        cyg_uint32 *len)
{
    switch (key) {
    case CYG_IO_GET_CONFIG_ETHER_LINK_STATUS: {
#if HavePHY
        if (*len < sizeof (unsigned))
            return -EINVAL;
        *((unsigned*)xbuf) = linkStatus;
        *len = sizeof (unsigned);
        return ENOERR;
#else
        return -EINVAL;
#endif
        break;
    }

    case CYG_IO_GET_CONFIG_ETHER_INFO: {
        if (*len < sizeof (cyg_ether_info_t))
            return -EINVAL;
        *((cyg_ether_info_t*)xbuf) = config;
        *len = sizeof config;
        return ENOERR;
        break;
    }

    //  Get an EthBuf from the global linked list of buffers
    case CYG_IO_GET_CONFIG_ETHER_ALLOC_BUFFER: {
        tEthBuffer *b = allocBuffer();
        //addBufferTrace(__LINE__,6,b);
        if (b) {
            *(tEthBuffer**)xbuf = b;
            return ENOERR;
        } else {
            *(tEthBuffer**)xbuf = NULL;

            ++ifStats.rx_resource_errors;
//              diag_printf("ERROR: old_get_config_handler(): Unable to allocate buffer.\n");

            return -ENOMEM;
        }
        break;
    }

    //  Free an EthBuf from the global linked list of buffers
    case CYG_IO_GET_CONFIG_ETHER_FREE_BUFFER: {
        if ((tEthBuffer*)xbuf < bufferPool ||
                (tEthBuffer*)xbuf >= (bufferPool + NUM_ETH_BUFFERS))
            return -EINVAL;
        //addBufferTrace(__LINE__,7,xbuf);
        freeBuffer((tEthBuffer*)xbuf);
        return ENOERR;
        break;
    }

    //  Send a tEthBuffer out the Ethernet using DMA
    case CYG_IO_GET_CONFIG_ETHER_SEND_BUFFER: {
        if ((tEthBuffer*)xbuf < bufferPool ||
                (tEthBuffer*)xbuf >= (bufferPool + NUM_ETH_BUFFERS))
            return -EINVAL;
        //addBufferTrace(__LINE__,9,xbuf);
        while(!ks32c5000_eth_buffer_send((tEthBuffer*)xbuf)) {
            if (config.flags &  CYG_ETHER_FLAGS_WR_NONBLOCK)
                return -EAGAIN;
            else
                cyg_thread_delay(1);
        }
        return ENOERR;
        break;
    }

    //  Retrieve received Ethernet data in form of a tEthBuffer struct.
    case CYG_IO_GET_CONFIG_ETHER_RECV_BUFFER: {
        tEthBuffer *rxBuffer;

        rxBuffer = old_get_receive_buffer();
        addBufferTrace(__LINE__,8,rxBuffer);

        if (!rxBuffer) {
            *(tEthBuffer**)xbuf = NULL;
            *len = 0;
            return -EAGAIN;
        }

        *(tEthBuffer**)xbuf = rxBuffer;
        return ENOERR;
    }

    break;

    default:
        return -ENOSUPP;
    }
}

//  Writes data to ethernet.  Takes as input char data, memcpy's it to a
//  tEthBuffer struct, then calls the "new" write handler to push it to
//  the ethernet DMA.
static Cyg_ErrNo old_write_handler(cyg_io_handle_t handle, const void *_buf, cyg_uint32 *len)
{
    tEthBuffer *txBuffer;

    while (!configDone)
        cyg_thread_delay(10);

    if (*len < 14 || *len > MAX_ETH_FRAME_SIZE) {
        *len = 0;
        return -EINVAL;
    }

    txBuffer = allocBuffer();
    //addBufferTrace(__LINE__,12,txBuffer);

    if (!txBuffer) {

        ++ifStats.rx_resource_errors;
//      diag_printf("ERROR: old_write_handler(): Unable to allocate buffer.\n");
        *len = 0;
        return -ENOMEM;
    }
    memcpy(txBuffer->data, _buf, *len);
    txBuffer->length = *len;
    while(!ks32c5000_eth_buffer_send(txBuffer)) {
// Need priority packet check.
// some transmissions (like acks) should not be discarded. Add
// or check for logic that allows for priority packets.
        if (config.flags &  CYG_ETHER_FLAGS_WR_NONBLOCK) {
            //addBufferTrace(__LINE__,13,txBuffer);
            freeBuffer(txBuffer);
            *len = 0;
            return -EAGAIN;
        } else
            cyg_thread_delay(1);
    }

    return ENOERR;
}

static cyg_bool old_select_handler(cyg_io_handle_t handle, cyg_uint32 which, CYG_ADDRWORD info)
{
    // not supported, so just return true
    return true;
}


static Cyg_ErrNo old_read_handler(cyg_io_handle_t handle, void *_buf, cyg_uint32 *len)
{
    int size;
    tEthBuffer *rxBuffer;

    rxBuffer = old_get_receive_buffer();

    if (!rxBuffer) {
        *len = 0;
        return -EAGAIN;
    }

    // copy data from buffer into user space

    size = rxBuffer->length;

    if (size > *len) {
        diag_printf("WARNING: old_read_handler(): pkt truncation: pkt size larger than length request.\n");
        diag_printf("         old_read_handler(): data[0] = %s\n", *(rxBuffer->data));
        diag_printf("         old_read_handler(): data[2] = %s\n", *(rxBuffer->data+2));
        size = *len;
    }

    memcpy(_buf, rxBuffer->data+2, size);

    //addBufferTrace(__LINE__,11,rxBuffer);
    freeBuffer(rxBuffer);

    *len = size;
    return ENOERR;
}

DEVIO_TABLE(ks32c5000EtherDevio, old_write_handler, old_read_handler, old_select_handler, old_get_config_handler, old_set_config_handler)

#if defined(CYGPKG_IO)
DEVTAB_ENTRY(ks32c5000EtherDevtab, "/dev/reth0", NULL, &ks32c5000EtherDevio, old_init_handler, old_lookup_handler, NULL)
#endif

#endif // KS32C5000_ETH_RAW

#if KS32C5000_ETH_TCPIP
//======================================================================
// Driver code that interfaces to the TCP/IP stack via the common
// Ethernet interface.


// don't have any private data, but if we did, this is where it would go
typedef struct {
    int j;
} ks32c5000_priv_data_t;

ks32c5000_priv_data_t ks32c5000_priv_data;

#define eth_drv_tx_done(sc,key,retval) (sc)->funs->eth_drv->tx_done(sc,key,retval)
#define eth_drv_init(sc,enaddr)  ((sc)->funs->eth_drv->init)(sc, config.mac_address)
#define eth_drv_recv(sc,len)  ((sc)->funs->eth_drv->recv)(sc, len)

static bool ks32c5000_eth_init(struct cyg_netdevtab_entry *tab)
{
    struct eth_drv_sc *sc = (struct eth_drv_sc *)tab->device_instance;
    memcpy(config.mac_address,(unsigned char*)CAM_BaseAddr,6);
//  debug_printf("ks32c5000_eth_init()\n");
//  debug_printf("  found MAC address %s\n", macToStr(config.mac_address));
#if defined(CYGPKG_NET)
    ifStats.duplex = 1;      //unknown
    ifStats.operational = 1; //unknown
    ifStats.tx_queue_len = MAX_TX_FRAME_DESCRIPTORS;
#endif
    installInterrupts();
    EthInit(config.mac_address);
#if defined(CYGPKG_NET)
    cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherBDMARx);
    cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherBDMATx);
    cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherMacRx);
    cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherMacTx);
#endif
    configDone = 1;
    ethernetRunning = 1;
    eth_drv_init(sc, config.mac_address);
    return true;
}

static void ks32c5000_eth_start(struct eth_drv_sc *sc, unsigned char *enaddr, int flags)
{
//  debug_printf("ks32c5000_eth_start()\n");
    if (!ethernetRunning) {
        cyg_drv_interrupt_mask(CYGNUM_HAL_INTERRUPT_EtherBDMARx);
        cyg_drv_interrupt_mask(CYGNUM_HAL_INTERRUPT_EtherBDMATx);
        cyg_drv_interrupt_mask(CYGNUM_HAL_INTERRUPT_EtherMacRx);
        cyg_drv_interrupt_mask(CYGNUM_HAL_INTERRUPT_EtherMacTx);
        EthInit(enaddr);
        cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherBDMARx);
        cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherBDMATx);
        cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherMacRx);
        cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_EtherMacTx);
        ethernetRunning = 1;
    }
}

static void ks32c5000_eth_stop(struct eth_drv_sc *sc)
{
    debug_printf("ks32c5000_eth_stop()\n");
    ethernetRunning = 0;
}

static int ks32c5000_eth_control(struct eth_drv_sc *sc,
                                 unsigned long cmd,
                                 void *data,
                                 int len)
{
    switch (cmd) {
#if defined(CYGPKG_NET)
    case ETH_DRV_GET_IF_STATS_UD:
    case ETH_DRV_GET_IF_STATS: {
        struct ether_drv_stats *p = (struct ether_drv_stats*)data;
        *p = ifStats;
        strncpy(p->description,"DMRTS Ethernet Interface",sizeof p->description);
        strncpy(p->snmp_chipset,"Samsung S3C45xx",sizeof p->snmp_chipset);
        return 0;
    }
#endif
    case ETH_DRV_SET_MAC_ADDRESS: {
        int act;

        if (ETHER_ADDR_LEN != len)
            return -1;
        debug_printf("eth_control: ETH_DRV_SET_MAC_ADDRESS.\n");
        act = ethernetRunning;
        ks32c5000_eth_stop(sc);
        ks32c5000_eth_start(sc, data, 0);
        ethernetRunning = act;
        return 0;
    }
#ifdef	ETH_DRV_GET_MAC_ADDRESS
    case ETH_DRV_GET_MAC_ADDRESS: {
        if (len < ETHER_ADDR_LEN)
            return -1;
        debug1_printf("eth_control: ETH_DRV_GET_MAC_ADDRESS.\n");
        memcpy(data, (void *)CAM_BaseAddr, 6);
        return 0;
    }
#endif
#ifdef ETH_DRV_SET_MC_LIST
    case ETH_DRV_SET_MC_LIST: {
        int i;
        unsigned camMask = 1;
        struct eth_drv_mc_list *mcl = (struct eth_drv_mc_list *)data;
        CAMEN = 1;
//          diag_printf("Setting MC List: ");
        if (mcl->len > 20) {
            diag_printf("Too many MC addresses: setting MC ALL\n");
            CAMCON |= CAMCON_GROUP_ACC;
            return 0;
        }
        for (i=0; i < mcl->len; ++i) {
//              diag_printf("%s, ",macToStr(mcl->addrs[i]));
            setCamEntry(i+1, mcl->addrs[i]);
            camMask = (camMask<<1) | 1;
        }
//          diag_printf("CAMEN = %08x\n", camMask);
        CAMEN = camMask;
        CAMCON &= ~CAMCON_GROUP_ACC;
    }
#endif
#ifdef ETH_DRV_SET_MC_ALL
    case ETH_DRV_SET_MC_ALL: {
        CAMCON |= CAMCON_GROUP_ACC;
        CAMEN = 1;
        return 0;
    }
#endif
    default:
        return -1;
    }
}

static int ks32c5000_eth_can_send_count=0;
static int ks32c5000_eth_can_send_count_OK=0;

// In case there are multiple Tx frames waiting, we should
// return how many empty Tx spots we have.  For now we just
// return 0 or 1.
static int ks32c5000_eth_can_send(struct eth_drv_sc *sc)
{
    FRAME_DESCRIPTOR *TxFp, *StartFp;

    // find the next unused spot in the queue

    ++ks32c5000_eth_can_send_count;

    StartFp = TxFp = (FRAME_DESCRIPTOR*)BDMATXPTR;

    while (TxFp->FrameDataPtr & FRM_OWNERSHIP_BDMA) {
        TxFp = TxFp->NextFD;
        if (TxFp == StartFp)
            return 0;
    }
    ++ks32c5000_eth_can_send_count_OK;
    return 1;
}

static int ks5000_eth_send_count=0;

static void ks32c5000_eth_send(struct eth_drv_sc *sc,
                               struct eth_drv_sg *sg_list,
                               int sg_len,
                               int total_len,
                               unsigned long key)
{
    unsigned char *dest;
    unsigned        len;
    tEthBuffer *buf;

//  debug_printf("ks32c5000_eth_send()...");

    if (total_len >= MAX_ETH_FRAME_SIZE) {
        debug_printf("ks32c5000_eth_send() - Invalid length \n");
        eth_drv_tx_done(sc,key,-EINVAL);
        return;
    }

    ++ks5000_eth_send_count;

    // allocate buffer

    buf = allocBuffer();
    //addBufferTrace(__LINE__,15,buf);
    if (!buf) {
        ++ifStats.rx_resource_errors;
        // out of buffers
        debug_printf("ks32c5000_eth_send() - Out of buffers \n");
        eth_drv_tx_done(sc,key,-EIO);
        return;
    }

    // copy data from scatter/gather list into BDMA data buffer

    len = 0;
    dest = buf->data;

    while (sg_len) {
        memcpy(dest,(unsigned char*)sg_list->buf,sg_list->len);
        len += sg_list->len;
        dest += sg_list->len;
        ++sg_list;
        --sg_len;
    }

    buf->length = len;

    // tell upper layer that we're done with this sglist

    eth_drv_tx_done(sc,key,0);

    // queue packet for transmit

    while(!ks32c5000_eth_buffer_send(buf)) {
#if defined(CYGPKG_NET)
        // wait a tick and try again.
        cyg_thread_delay(1);
#else
        // toss it.
        debug_printf("ks32c5000_eth_send() - Tossing buffer \n");
        freeBuffer(buf);
        break;
#endif
    }

}


static int ks5000_eth_rcv_count=0;
static tEthBuffer *tcpIpRxBuffer;

// called from DSR
static void eth_handle_recv_buffer(tEthBuffer* rxBuffer)
{
    extern struct eth_drv_sc ks32c5000_sc;

    /*
      	static long loop_count;
    	loop_count++;
    	if (!(loop_count % 100))
    		diag_printf(
    			"TCP: eth_handle_recv_buffer()->eth_drv_recv() called at pkt %lu.\n",
    			loop_count
    		);
    */
    tcpIpRxBuffer = rxBuffer;
    eth_drv_recv(&ks32c5000_sc,tcpIpRxBuffer->length);
}

static void ks32c5000_eth_recv(struct eth_drv_sc *sc,
                               struct eth_drv_sg *sg_list,
                               int sg_len)
{
    unsigned char *source;

    ++ks5000_eth_rcv_count;

    if (!tcpIpRxBuffer)
        return;  // no packet waiting, shouldn't be here!

    // copy data from eth buffer into scatter/gather list

    source = tcpIpRxBuffer->data + EtherFramePadding;

    while (sg_len) {
        if (sg_list->buf)
            memcpy((unsigned char*)sg_list->buf,source,sg_list->len);
        source += sg_list->len;
        ++sg_list;
        --sg_len;
    }

//see dmrtslnx-78 issue for why this is here and not in mainline driver (yet).
    freeBuffer(tcpIpRxBuffer);
    tcpIpRxBuffer = NULL;

    return;
}

// routine called to handle ethernet controller in polled mode
static void ks32c5000_eth_poll(struct eth_drv_sc *sc)
{
    ks32c5000_eth_deliver(sc);  // handle rx frames
    ks32c5000_handle_tx_complete();
}

static int ks32c5000_eth_int_vector(struct eth_drv_sc *sc)
{
    return CYGNUM_HAL_INTERRUPT_EtherBDMARx;
}

ETH_DRV_SC(ks32c5000_sc,
           &ks32c5000_priv_data, // Driver specific data
           "eth0",                // Name for this interface
           ks32c5000_eth_start,
           ks32c5000_eth_stop,
           ks32c5000_eth_control,
           ks32c5000_eth_can_send,
           ks32c5000_eth_send,
           ks32c5000_eth_recv,
           ks32c5000_eth_deliver,
           ks32c5000_eth_poll,
           ks32c5000_eth_int_vector
          );

NETDEVTAB_ENTRY(ks32c5000_netdev,
                "dmrts",
                ks32c5000_eth_init,
                &ks32c5000_sc);
#endif


#if defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWRXCRC) || defined(CYGPKG_DEVS_ETH_ARM_DMRTS_SWTXCRC)

static U32 crctable[256] = {
    0x00000000L, 0x77073096L, 0xEE0E612CL, 0x990951BAL,
    0x076DC419L, 0x706AF48FL, 0xE963A535L, 0x9E6495A3L,
    0x0EDB8832L, 0x79DCB8A4L, 0xE0D5E91EL, 0x97D2D988L,
    0x09B64C2BL, 0x7EB17CBDL, 0xE7B82D07L, 0x90BF1D91L,
    0x1DB71064L, 0x6AB020F2L, 0xF3B97148L, 0x84BE41DEL,
    0x1ADAD47DL, 0x6DDDE4EBL, 0xF4D4B551L, 0x83D385C7L,
    0x136C9856L, 0x646BA8C0L, 0xFD62F97AL, 0x8A65C9ECL,
    0x14015C4FL, 0x63066CD9L, 0xFA0F3D63L, 0x8D080DF5L,
    0x3B6E20C8L, 0x4C69105EL, 0xD56041E4L, 0xA2677172L,
    0x3C03E4D1L, 0x4B04D447L, 0xD20D85FDL, 0xA50AB56BL,
    0x35B5A8FAL, 0x42B2986CL, 0xDBBBC9D6L, 0xACBCF940L,
    0x32D86CE3L, 0x45DF5C75L, 0xDCD60DCFL, 0xABD13D59L,
    0x26D930ACL, 0x51DE003AL, 0xC8D75180L, 0xBFD06116L,
    0x21B4F4B5L, 0x56B3C423L, 0xCFBA9599L, 0xB8BDA50FL,
    0x2802B89EL, 0x5F058808L, 0xC60CD9B2L, 0xB10BE924L,
    0x2F6F7C87L, 0x58684C11L, 0xC1611DABL, 0xB6662D3DL,
    0x76DC4190L, 0x01DB7106L, 0x98D220BCL, 0xEFD5102AL,
    0x71B18589L, 0x06B6B51FL, 0x9FBFE4A5L, 0xE8B8D433L,
    0x7807C9A2L, 0x0F00F934L, 0x9609A88EL, 0xE10E9818L,
    0x7F6A0DBBL, 0x086D3D2DL, 0x91646C97L, 0xE6635C01L,
    0x6B6B51F4L, 0x1C6C6162L, 0x856530D8L, 0xF262004EL,
    0x6C0695EDL, 0x1B01A57BL, 0x8208F4C1L, 0xF50FC457L,
    0x65B0D9C6L, 0x12B7E950L, 0x8BBEB8EAL, 0xFCB9887CL,
    0x62DD1DDFL, 0x15DA2D49L, 0x8CD37CF3L, 0xFBD44C65L,
    0x4DB26158L, 0x3AB551CEL, 0xA3BC0074L, 0xD4BB30E2L,
    0x4ADFA541L, 0x3DD895D7L, 0xA4D1C46DL, 0xD3D6F4FBL,
    0x4369E96AL, 0x346ED9FCL, 0xAD678846L, 0xDA60B8D0L,
    0x44042D73L, 0x33031DE5L, 0xAA0A4C5FL, 0xDD0D7CC9L,
    0x5005713CL, 0x270241AAL, 0xBE0B1010L, 0xC90C2086L,
    0x5768B525L, 0x206F85B3L, 0xB966D409L, 0xCE61E49FL,
    0x5EDEF90EL, 0x29D9C998L, 0xB0D09822L, 0xC7D7A8B4L,
    0x59B33D17L, 0x2EB40D81L, 0xB7BD5C3BL, 0xC0BA6CADL,
    0xEDB88320L, 0x9ABFB3B6L, 0x03B6E20CL, 0x74B1D29AL,
    0xEAD54739L, 0x9DD277AFL, 0x04DB2615L, 0x73DC1683L,
    0xE3630B12L, 0x94643B84L, 0x0D6D6A3EL, 0x7A6A5AA8L,
    0xE40ECF0BL, 0x9309FF9DL, 0x0A00AE27L, 0x7D079EB1L,
    0xF00F9344L, 0x8708A3D2L, 0x1E01F268L, 0x6906C2FEL,
    0xF762575DL, 0x806567CBL, 0x196C3671L, 0x6E6B06E7L,
    0xFED41B76L, 0x89D32BE0L, 0x10DA7A5AL, 0x67DD4ACCL,
    0xF9B9DF6FL, 0x8EBEEFF9L, 0x17B7BE43L, 0x60B08ED5L,
    0xD6D6A3E8L, 0xA1D1937EL, 0x38D8C2C4L, 0x4FDFF252L,
    0xD1BB67F1L, 0xA6BC5767L, 0x3FB506DDL, 0x48B2364BL,
    0xD80D2BDAL, 0xAF0A1B4CL, 0x36034AF6L, 0x41047A60L,
    0xDF60EFC3L, 0xA867DF55L, 0x316E8EEFL, 0x4669BE79L,
    0xCB61B38CL, 0xBC66831AL, 0x256FD2A0L, 0x5268E236L,
    0xCC0C7795L, 0xBB0B4703L, 0x220216B9L, 0x5505262FL,
    0xC5BA3BBEL, 0xB2BD0B28L, 0x2BB45A92L, 0x5CB36A04L,
    0xC2D7FFA7L, 0xB5D0CF31L, 0x2CD99E8BL, 0x5BDEAE1DL,
    0x9B64C2B0L, 0xEC63F226L, 0x756AA39CL, 0x026D930AL,
    0x9C0906A9L, 0xEB0E363FL, 0x72076785L, 0x05005713L,
    0x95BF4A82L, 0xE2B87A14L, 0x7BB12BAEL, 0x0CB61B38L,
    0x92D28E9BL, 0xE5D5BE0DL, 0x7CDCEFB7L, 0x0BDBDF21L,
    0x86D3D2D4L, 0xF1D4E242L, 0x68DDB3F8L, 0x1FDA836EL,
    0x81BE16CDL, 0xF6B9265BL, 0x6FB077E1L, 0x18B74777L,
    0x88085AE6L, 0xFF0F6A70L, 0x66063BCAL, 0x11010B5CL,
    0x8F659EFFL, 0xF862AE69L, 0x616BFFD3L, 0x166CCF45L,
    0xA00AE278L, 0xD70DD2EEL, 0x4E048354L, 0x3903B3C2L,
    0xA7672661L, 0xD06016F7L, 0x4969474DL, 0x3E6E77DBL,
    0xAED16A4AL, 0xD9D65ADCL, 0x40DF0B66L, 0x37D83BF0L,
    0xA9BCAE53L, 0xDEBB9EC5L, 0x47B2CF7FL, 0x30B5FFE9L,
    0xBDBDF21CL, 0xCABAC28AL, 0x53B39330L, 0x24B4A3A6L,
    0xBAD03605L, 0xCDD70693L, 0x54DE5729L, 0x23D967BFL,
    0xB3667A2EL, 0xC4614AB8L, 0x5D681B02L, 0x2A6F2B94L,
    0xB40BBE37L, 0xC30C8EA1L, 0x5A05DF1BL, 0x2D02EF8DL
};


static U32 crc32( unsigned char *blk_adr, unsigned blk_len)
{
    U32 crc = 0xffffffff;

    while (blk_len--)
        crc = crctable[(crc ^ *blk_adr++) & 0xffL] ^ (crc >> 8);

    return crc ^ 0xffffffff;
}

#endif



void dmrts_ether_show_buffers(void)
{
    diag_printf("Free: %d\n",freeCount);
    diag_printf(" Head: %x\n",freeList.head);
    diag_printf(" Tail: %x\n",freeList.tail);
#if EnableQueueCounts
    diag_printf("  Cnt: %d\n",freeList.count);
#endif
    diag_printf(" Alloc fail: %d\n",allocFail);
    diag_printf("oldRxQueue:\n");
#if KS32C5000_ETH_RAW
    diag_printf(" Head: %x\n",oldRxQueue.head);
    diag_printf(" Tail: %x\n",oldRxQueue.tail);
#if EnableQueueCounts
    diag_printf("  Cnt: %d\n",oldRxQueue.count);
#endif
#endif
}

#if EnableBufferTrace

void dmrts_ether_show_buffer_trace(void)
{
    static char *str[] = {
        "init     ",
        "alloc repl (get_recv_buffer)",
        "reusing rx buffer",
        "free old (buffer_send)",
        "free tx in MAC isr",
        "free tx in BDMA isr",
        "alloc cyg_io get-buffer",
        "free  cyg_io free-buffer",
        "return cyg_io get-rx-frame",
        "queue-tx cyg_io send-buffer",
        "free not-delivered",
        "free cyg_io_read()",
        "alloc cyg_io_write()",
        "free  cyg_io_write() failed",
        "return old_get_receive_buffer",
        "alloc eth_send()",
        "alloc()",
        "free()",
        "isrAlloc()",
        "isrFree()",
        "return eth_get_recv_buffer",
        "passed to eth_handle_recv_buffer",
        "free (consumed) eth_recv",
        "free (discard) eth_handle_recv_buffer",
    };
    int i = bufferTraceIndex;
    do {
        diag_printf("%5d [%d] %5d %x %s\n",
                    bufferTrace[i].lineNumber,
                    bufferTrace[i].free,
                    bufferTrace[i].event,
                    bufferTrace[i].bufPtr,
                    str[bufferTrace[i].event]);
        ++i;
        i &= BufferTraceCount-1;
    } while (i != bufferTraceIndex);

}
#else
void dmrts_ether_show_buffer_trace(void)
{
    diag_printf("buffer trace not built\n");
}
#endif

void dmrts_ether_show_ifstats(void)
{
#if 0
    diag_printf("show ifstats not built\n");
#else
# define showit(a) diag_printf(" %5d %s\n",ifStats.a,#a)
    showit(duplex);
    showit(operational);
    showit(speed);
    showit(interrupts);

    showit(rx_count);
    showit(rx_good);
    showit(rx_deliver);
    showit(rx_resource);
    showit(rx_crc_errors);
    showit(rx_align_errors);
    showit(rx_overrun_errors);
    showit(rx_too_long_frames);
    showit(rx_symbol_errors);
    showit(rx_resource_errors);

    showit(tx_count);
    showit(tx_complete);
    showit(tx_good);
    showit(tx_max_collisions);
    showit(tx_deferred);
    showit(tx_underrun);
    showit(tx_carrier_loss);
    showit(tx_sqetesterrors);
    showit(tx_late_collisions);
    showit(tx_single_collisions);
    showit(tx_total_collisions);
#endif
}
