
/*
 * See latching low/high explaination in ICS1893 spec for the reason
 * two reads are needed.  Also make sure you use decimal notation to
 * access ics1893 registers...not hex like as shown in the ics1890.c
 * source file.
 */
#include <pkgconf/system.h>
#include <pkgconf/devs_eth_arm_dmrts.h>
#include <cyg/infra/diag.h>

#if !defined(CYGPKG_REDBOOT)
#include <cyg/hal/hal_arch.h>
#include <cyg/kernel/kapi.h>
#endif

#include "phy.h"
#include "ics1893.h"

int ics_mii_isolated(void)
{
    cyg_uint16 status;
    status = MiiStationRead(PHY_ADDR, ICS_PHY_CNTL_REG);
    return (status & ICS_R0_PHY_MAC_ISOLATE) != 0;
}

int ics_islinkup(void)
{
    cyg_uint16 status;
    status = MiiStationRead(PHY_ADDR, ICS_PHY_QPDSTATUS_REG);
    status = MiiStationRead(PHY_ADDR, ICS_PHY_QPDSTATUS_REG);
    return (status & ICS_PHY_QPDSTAT_LINK_UP) != 0;
}

int ics_is100MB(void)
{
    cyg_uint16 status;
    status = MiiStationRead(PHY_ADDR, ICS_PHY_QPDSTATUS_REG);
    status = MiiStationRead(PHY_ADDR, ICS_PHY_QPDSTATUS_REG);
    return (status & ICS_PHY_QPDSTAT_100MB) != 0;
}

int ics_isfullduplex(void)
{
    cyg_uint16 status;
    status = MiiStationRead(PHY_ADDR, ICS_PHY_QPDSTATUS_REG);
    status = MiiStationRead(PHY_ADDR, ICS_PHY_QPDSTATUS_REG);
    return (status & ICS_PHY_QPDSTAT_FULL_DUPLEX) != 0;
}

// return set of bits we care about for link-status-change
int ics_linkchangebits(void)
{
    cyg_uint16 status;
    status = MiiStationRead(PHY_ADDR, ICS_PHY_QPDSTATUS_REG);
    status = MiiStationRead(PHY_ADDR, ICS_PHY_QPDSTATUS_REG);
    return status & (ICS_PHY_QPDSTAT_FULL_DUPLEX | ICS_PHY_QPDSTAT_LINK_UP | ICS_PHY_QPDSTAT_100MB);
}

void ics_restart_autonegotiation(void)
{
    MiiStationWrite(PHY_ADDR, ICS_PHY_CNTL_REG, ICS_R0_ENABLE_AUTONEG);
    MiiStationWrite(PHY_ADDR, ICS_PHY_CNTL_REG, ICS_R0_RESTART_AUTONEG);
}

void ics_phy_status(void)
{
    unsigned newLinkStatus = 0;
    if (ics_islinkup()) {
        newLinkStatus |= PhyStatus_LinkUp;
        diag_printf("ICS1893: Link up ");

        if (ics_is100MB()) {
            newLinkStatus |= PhyStatus_100Mb;
            diag_printf("100M ");
        } else {
                diag_printf("10M ");
        }

        if (ics_isfullduplex()) {
            set_maccon_duplex(1);
            newLinkStatus |= PhyStatus_FullDuplex;
            diag_printf("Full\n");
        } else {
            set_maccon_duplex(0);
            diag_printf("Half\n");
        }
    } else {
        diag_printf("ICS1893: Link down.\n");
    }
    dmrtsLinkStatus = newLinkStatus;
}


#define ShowBit(status,mask) \
  diag_printf("ICS1893:   %-30s %s\n", #mask, status&mask ? "TRUE" : "FALSE")

unsigned ics_phy_R0_state(void)
{
    cyg_uint16 status;
    status = MiiStationRead(PHY_ADDR, ICS_PHY_CNTL_REG);
    diag_printf("ICS1893: ICS_PHY_CNTL_REG status = 0x%04x\n", status);
    ShowBit(status,ICS_R0_DATARATE_100MB);
    ShowBit(status,ICS_R0_PHY_FULLDUPLEX);
    ShowBit(status,ICS_R0_ENABLE_LOOPBACK);
    ShowBit(status,ICS_R0_PHY_COL_TEST);
    ShowBit(status,ICS_R0_PHY_MAC_ISOLATE);
    ShowBit(status,ICS_R0_ENABLE_AUTONEG);
    return status;
}

unsigned ics_phy_R1_state(void)
{
    cyg_uint16 status;
    status = MiiStationRead(PHY_ADDR, ICS_PHY_STATUS_REG);
    status = MiiStationRead(PHY_ADDR, ICS_PHY_STATUS_REG);
    diag_printf("ICS1893: ics_phy_R1_state = 0x%04x\n", status);
    ShowBit(status,ICS_PHY_R1_STAT_REMOTEFAULT);
    ShowBit(status,ICS_PHY_R1_STAT_10BTHSUPP);
    ShowBit(status,ICS_PHY_R1_STAT_10BTFSUPP);
    ShowBit(status,ICS_PHY_R1_STAT_100BTXHSUPP);
    ShowBit(status,ICS_PHY_R1_STAT_100BTXFSUPP);
    ShowBit(status,ICS_PHY_R1_STAT_AUTONEGDONE);
    ShowBit(status,ICS_PHY_R1_STAT_LINK_UP);
    return status;
}

unsigned ics_phy_qpd_state(void)
{
    cyg_uint16 status;
    status = MiiStationRead(PHY_ADDR, ICS_PHY_QPDSTATUS_REG);
    status = MiiStationRead(PHY_ADDR, ICS_PHY_QPDSTATUS_REG);
    diag_printf("ICS1893: ics_phy_qpd_state = 0x%04x\n", status);
    ShowBit(status,ICS_PHY_QPDSTAT_LINK_UP);
    ShowBit(status,ICS_PHY_QPDSTAT_FULL_DUPLEX);
    ShowBit(status,ICS_PHY_QPDSTAT_100MB);
    ShowBit(status,ICS_PHY_QPDSTAT_AUTONEG);
    return status;
}

void ics_reset_phy(void)
{
    cyg_uint16 status;
    diag_printf("ICS1893: ics_reset_phy begin.\n");
    MiiStationWrite(PHY_ADDR, ICS_PHY_CNTL_REG, ICS_R0_RESET_PHY);
    do {
        status = MiiStationRead(PHY_ADDR, ICS_PHY_CNTL_REG);
#if 0
        // spews to diag out so don't enable for production builds.
        diag_printf("ICS1893: ICS_R0_RESET_PHY status = %x @ bit(15) = %x\n", status, (status & ICS_R0_RESET_PHY));
#endif
    } while (status & ICS_R0_RESET_PHY);
    set_maccon_duplex(0);   // start out in half-duplex mode
    diag_printf("ICS1893: ics_reset_phy end.\n");
}


#if !defined(CYGPKG_REDBOOT)
#include "std.h"

static void isc_check_4_link_changes(void)
{
    static int oldstate = -1;
    int newstate =ics_linkchangebits();
    if (newstate != oldstate) {
        ics_phy_status();
        oldstate = newstate;
    }
}

static void ics_link_change_poll(cyg_addrword_t data)
{
    diag_printf("ICS1893 Polling Thread Started.\n");
    while (1) {
        isc_check_4_link_changes();
        cyg_thread_delay_ms(1000);
    }
}
#endif

void ics_init(void)
{
#if !defined(CYGPKG_REDBOOT)
    static cyg_handle_t icspollhandle;
    static U32 icspollstack[1024];
    static cyg_thread icspollthread;
    static int icspollrunning = 0;
    if (!icspollrunning) {
        //start polling thread
        //priority 15 appears to run well and not starve other tasks.
        cyg_thread_create(
            15, // thread priority 0-31 inclusive : 0 = highest
            ics_link_change_poll,
            (cyg_addrword_t) 0,
            "ics_link_change_poll",
            (void *)icspollstack,
            sizeof icspollstack,
            &icspollhandle,
            &icspollthread
        );
        cyg_thread_resume(icspollhandle);
        icspollrunning = 1;
    }
#endif
}
