#ifndef _KSZ8893MQL_H
#define _KSZ8893MQL_H

#define Bit(n)                          (1<<(n))

//for polling loop
#define cyg_thread_delay_ms(ms) cyg_thread_delay(ms/((CYGNUM_HAL_RTC_NUMERATOR/CYGNUM_HAL_RTC_DENOMINATOR)/1000000))

// KSZ8893MQL I2C slave address
#define KSZ8893MQL_SLAVE_ADDR           0x5F

#define STATIC_MAC_TABLE_ENTRIES        8
#define VLAN_TABLE_ENTRIES              16
#define MIB_COUNTERS_PER_PORT           32

// Global Registers
#define KSZ8893MQL_CHIP_ID0_REG         0x00
#define KSZ8893MQL_CHIP_ID1_REG         0x01
#define KSZ8893MQL_GLOBAL_CTRL0_REG     0x02
#define KSZ8893MQL_GLOBAL_CTRL1_REG     0x03
#define KSZ8893MQL_GLOBAL_CTRL2_REG     0x04
#define KSZ8893MQL_GLOBAL_CTRL3_REG     0x05
#define KSZ8893MQL_GLOBAL_CTRL4_REG     0x06
#define KSZ8893MQL_GLOBAL_CTRL5_REG     0x07
#define KSZ8893MQL_GLOBAL_CTRL6_REG     0x08
#define KSZ8893MQL_GLOBAL_CTRL7_REG     0x09
#define KSZ8893MQL_GLOBAL_CTRL8_REG     0x0A
#define KSZ8893MQL_GLOBAL_CTRL9_REG     0x0B

#define KSZ8893MQL_CHIP_ID1_START               Bit(0)
#define KSZ8893MQL_GLOBAL_CTRL9_STPID_ENABLE    Bit(0)

//PHY link status
#define KSZ8893MQL_PORT_LINK_STAT0			Bit(5)
#define KSZ8893MQL_PORT_100MB_STAT1			Bit(2)
#define KSZ8893MQL_PORT_DUPLEX_STAT1		Bit(1)


// Port Registers
#define KSZ8893MQL_PORT1_CTRL0_REG      0x10
#define KSZ8893MQL_PORT2_CTRL0_REG      0x20
#define KSZ8893MQL_PORT3_CTRL0_REG      0x30

#define KSZ8893MQL_PORT_CTRL0_TAG_INSERT        0x04
#define KSZ8893MQL_PORT_CTRL0_TAG_REMOVE        0x02

#define KSZ8893MQL_PORT1_STATUS0_REG    0x1E
#define KSZ8893MQL_PORT2_STATUS0_REG    0x2E
#define KSZ8893MQL_PORT1_STATUS1_REG    0x1F
#define KSZ8893MQL_PORT2_STATUS1_REG    0x2F
#define KSZ8893MQL_PORT3_STATUS1_REG    0x3F

#define KSZ8893MQL_PORT3_STATUS1_SPEED          0x04
#define KSZ8893MQL_PORT3_STATUS1_DUPLEX         0x02

// Indirect Access Control
#define KSZ8893MQL_INDIR_CTRL0_REG      0x79
#define KSZ8893MQL_INDIR_CTRL1_REG      0x7A

#define KSZ8893MQL_INDIR_CTRL0_READ             0x10
#define KSZ8893MQL_INDIR_CTRL0_WRITE            0x00
#define KSZ8893MQL_INDIR_CTRL0_STATIC_MAC       0x00
#define KSZ8893MQL_INDIR_CTRL0_VLAN_TAB         0x04
#define KSZ8893MQL_INDIR_CTRL0_DYNAMIC_MAC      0x08
#define KSZ8893MQL_INDIR_CTRL0_MIB_COUNTER      0x0C

// Indirect Data Register
#define KSZ8893MQL_INDIR_DATA8_REG      0x7B
#define KSZ8893MQL_INDIR_DATA7_REG      0x7C
#define KSZ8893MQL_INDIR_DATA6_REG      0x7D
#define KSZ8893MQL_INDIR_DATA5_REG      0x7E
#define KSZ8893MQL_INDIR_DATA4_REG      0x7F
#define KSZ8893MQL_INDIR_DATA3_REG      0x80
#define KSZ8893MQL_INDIR_DATA2_REG      0x81
#define KSZ8893MQL_INDIR_DATA1_REG      0x82
#define KSZ8893MQL_INDIR_DATA0_REG      0x83

// MII Management (MIIM) Registers
#define KSZ8893MQL_MII_CTRL_REG         0x00
#define KSZ8893MQL_MII_STATUSL_REG      0x01
#define KSZ8893MQL_MII_PHYID_HIGH_REG   0x02
#define KSZ8893MQL_MII_PHYID_LOW_REG    0x03

void Ksz8893mqlPhyReset(void);
void ksz_init(void);
void ksz_phy_status(void);
int ksz_link_up(void);
void EnableSpecialTagging(unsigned char enableBit);
void WriteStaticMacTable(int addr, int fid, int use_fid, int override,
                         int valid, int forward_port, unsigned char *mac_addr);
void PrintPhyTable(int type);
cyg_uint8 IicPhyReadRegs(cyg_uint8 addr, cyg_uint8 *buf, int count);

#endif
