// phy functions for Level1 PHY LXT970

#include <cyg/infra/cyg_type.h>
#include <pkgconf/devs_eth_arm_dmrts.h>
#include "phy.h"
#include "lxt970.h"
#include "ks5000_regs.h"

static cyg_uint32 MiiStationRead(cyg_uint32 RegAddr);
static void MiiStationWrite(cyg_uint32 RegAddr, cyg_uint32 PhyWrData);

void PhyReset(void)
{
    // first software reset the LX970
    MiiStationWrite(LX970_CNTL_REG,  LX970_CNTL_RESET);
    MiiStationWrite(LX970_CNTL_REG,  0);

    // set low level drive for MII lines, enable interrupt output
    MiiStationWrite(17,  Bit(3)+Bit(1));

    // default values for 100M encryption are wrong, so fix them
    // and configure LEDC to be activity indicator
    MiiStationWrite(19,  Bit(7));

    // initialize auto-negotiation capabilities
    MiiStationWrite(LX970_ANA_REG,
                    LX970_ANA_PAUSE_ENA+
                    LX970_ANA_100T4+
                    LX970_ANA_100TX_FULL+
                    LX970_ANA_100TX+
                    LX970_ANA_10T_FULL+
                    LX970_ANA_10T+
                    LX970_ANA_SEL_802_3);
#if 1
    // Now start an auto negotiation
    MiiStationWrite(LX970_CNTL_REG,
                    LX970_CNTL_AN+
                    LX970_CNTL_RSTRT_AN);
#else
    // force to 10M full duplex
    MiiStationWrite(LX970_CNTL_REG,
                    LX970_CNTL_FULL_DUP);
#endif
}

unsigned PhyStatus(void)
{
    unsigned lxt970Status = MiiStationRead(LX970_CHIPSTAT_REG);
    unsigned r = 0;

    if (lxt970Status & LX970_CHIPSTAT_LINKUP)
        r |= PhyStatus_LinkUp;
    if (lxt970Status & LX970_CHIPSTAT_FULLDUP)
        r |= PhyStatus_FullDuplex;
    if (lxt970Status & LX970_CHIPSTAT_100M)
        r |=  PhyStatus_100Mb;
    return r;
}

void PhyInterruptAck(void)
{
    MiiStationRead(1);
    MiiStationRead(18);
}

/*----------------------------------
 *  PhyIsPresent
 *  Determines in the PHY is actually present
 *  Returns:  1 if AC101L, 2 if LXT970, else 0
 *---------------------------------- */
cyg_uint32 PhyIsPresent(void)
{
    return 2;
}

//  NOTE:  The AC101L phy writes the PHY ADDR value differently than
//  the LXT970.  Not sure why, but note the required difference.  Kurt R. 11-26-02
#define PHYREGWRITE	0x0400
#define MiiStart        0x0800

/*------------------------------------------------------------------------
 * MiiStationWrite
 * PHY MII Interface Station Management Register Write
 * Input: RegAddr (address of register), PhyWrtData (data to write)
 *------------------------------------------------------------------------*/
static void MiiStationWrite(cyg_uint32 RegAddr, cyg_uint32 PhyWrData)
{
    STADATA = PhyWrData ;
    STACON = RegAddr | CYGPKG_DEVS_ETH_ARM_DMRTS_PHY_ADDR | MiiStart | PHYREGWRITE;
    while (STACON & MiiStart)
        ;
}

/*------------------------------------------------------------------------
 * MiiStationRead
 * PHY MII Interface Station Management Register Read
 * Input:    RegAddr (address of register)
 * Returns:  The data
 *------------------------------------------------------------------------*/
static cyg_uint32 MiiStationRead(cyg_uint32 RegAddr)
{
    cyg_uint32 PhyRdData;

    STACON = RegAddr | CYGPKG_DEVS_ETH_ARM_DMRTS_PHY_ADDR | MiiStart;
    while (STACON & MiiStart)
        ;
    PhyRdData = STADATA;
    return PhyRdData ;
}
