#if !defined(_PHY_H)
#define _PHY_H

#define Bit(n)                  (1<<(n))

#define PHY_ID_AC101L           0x00225521
#define PHY_ID_KSZ8893MQL       0x00221430
#define PHY_ID_ICS1983          0x00150000
#define PHY_ID_ICS1983_MASK     0xffff0000
#define PHY_ID_KENDIN           0xffffffff

#define PHY_ADDR                CYGPKG_DEVS_ETH_ARM_DMRTS_PHY_ADDR

extern unsigned dmrtsLinkStatus;
extern unsigned dmrtsPhyResetCount;

#define PhyStatus_LinkUp      0x0001
#define PhyStatus_FullDuplex  0x0002
#define PhyStatus_100Mb       0x0004

// The following definitions are copyied from include/linux/mii.h
// Not all MII registers are supported by all PHYs

// Generic MII registers
#define MII_BMCR                0x00    // Basic mode control register
#define MII_BMSR                0x01    // Basic mode status register
#define MII_PHYSID1             0x02    // PHYS ID 1
#define MII_PHYSID2             0x03    // PHYS ID 2
#define MII_ADVERTISE           0x04    // Advertisement control reg
#define MII_LPA                 0x05    // Link partner ability reg
#define MII_EXPANSION           0x06    // Expansion register
#define MII_CTRL1000            0x09    // 1000BASE-T control
#define MII_STAT1000            0x0a    // 1000BASE-T status
#define MII_ESTATUS             0x0f    // Extended Status
#define MII_DCOUNTER            0x12    // Disconnect counter
#define MII_FCSCOUNTER          0x13    // False carrier counter
#define MII_NWAYTEST            0x14    // N-way auto-neg test reg
#define MII_RERRCOUNTER         0x15    // Receive error counter
#define MII_SREVISION           0x16    // Silicon revision
#define MII_RESV1               0x17    // Reserved
#define MII_LBRERROR            0x18    // Lpback, rx, bypass error
#define MII_PHYADDR             0x19    // PHY address
#define MII_RESV2               0x1a    // Reserved
#define MII_TPISTATUS           0x1b    // TPI status for 10mbps
#define MII_NCONFIG             0x1c    // Network interface config

// Basic mode control register
#define BMCR_RESV               0x003f  // Unused
#define BMCR_SPEED1000          0x0040  // MSB of Speed (1000)
#define BMCR_CTST               0x0080  // Collision test
#define BMCR_FULLDPLX           0x0100  // Full duplex
#define BMCR_ANRESTART          0x0200  // Auto negotiation restart
#define BMCR_ISOLATE            0x0400  // Disconnect DP83840 from MII
#define BMCR_PDOWN              0x0800  // Powerdown the DP83840
#define BMCR_ANENABLE           0x1000  // Enable auto negotiation
#define BMCR_SPEED100           0x2000  // Select 100Mbps
#define BMCR_LOOPBACK           0x4000  // TXD loopback bits
#define BMCR_RESET              0x8000  // Reset the DP83840

// Basic mode status register
#define BMSR_ERCAP              0x0001  // Ext-reg capability
#define BMSR_JCD                0x0002  // Jabber detected
#define BMSR_LSTATUS            0x0004  // Link status
#define BMSR_ANEGCAPABLE        0x0008  // Able to do auto-negotiation
#define BMSR_RFAULT             0x0010  // Remote fault detected
#define BMSR_ANEGCOMPLETE       0x0020  // Auto-negotiation complete
#define BMSR_RESV               0x00c0  // Unused
#define BMSR_ESTATEN            0x0100  // Extended Status in R15
#define BMSR_100HALF2           0x0200  // Can do 100BASE-T2 HDX
#define BMSR_100FULL2           0x0400  // Can do 100BASE-T2 FDX
#define BMSR_10HALF             0x0800  // Can do 10mbps, half-duplex
#define BMSR_10FULL             0x1000  // Can do 10mbps, full-duplex
#define BMSR_100HALF            0x2000  // Can do 100mbps, half-duplex
#define BMSR_100FULL            0x4000  // Can do 100mbps, full-duplex
#define BMSR_100BASE4           0x8000  // Can do 100mbps, 4k packets

// Advertisement control register
#define ADVERTISE_SLCT          0x001f  // Selector bits
#define ADVERTISE_CSMA          0x0001  // Only selector supported
#define ADVERTISE_10HALF        0x0020  // Try for 10mbps half-duplex
#define ADVERTISE_1000XFULL     0x0020  // Try for 1000BASE-X full-duplex
#define ADVERTISE_10FULL        0x0040  // Try for 10mbps full-duplex
#define ADVERTISE_1000XHALF     0x0040  // Try for 1000BASE-X half-duplex
#define ADVERTISE_100HALF       0x0080  // Try for 100mbps half-duplex
#define ADVERTISE_1000XPAUSE    0x0080  // Try for 1000BASE-X pause
#define ADVERTISE_100FULL       0x0100  // Try for 100mbps full-duplex
#define ADVERTISE_1000XPSE_ASYM 0x0100  // Try for 1000BASE-X asym pause
#define ADVERTISE_100BASE4      0x0200  // Try for 100mbps 4k packets
#define ADVERTISE_PAUSE_CAP     0x0400  // Try for pause
#define ADVERTISE_PAUSE_ASYM    0x0800  // Try for asymetric pause
#define ADVERTISE_RESV          0x1000  // Unused
#define ADVERTISE_RFAULT        0x2000  // Say we can detect faults
#define ADVERTISE_LPACK         0x4000  // Ack link partners response
#define ADVERTISE_NPAGE         0x8000  // Next page bit

#define ADVERTISE_FULL (ADVERTISE_100FULL | ADVERTISE_10FULL | \
                        ADVERTISE_CSMA)
#define ADVERTISE_ALL (ADVERTISE_10HALF | ADVERTISE_10FULL | \
                       ADVERTISE_100HALF | ADVERTISE_100FULL | \
                       ADVERTISE_CSMA | ADVERTISE_PAUSE_CAP)

extern unsigned int PhyId;

cyg_uint32 PhyStatus(void);
void PhyInterruptAck(void);
cyg_uint32 PhyIsPresent(void);


void load_phy_id(void);
void PhyReset(void);
void MiiStationWrite(int phy_addr, int reg_addr, unsigned int data);
unsigned int MiiStationRead(int phy_addr, int reg_addr);
void set_maccon_duplex(int state);
#endif
