//==========================================================================
//
//      bootblock_flash.c
//
//      Flash programming
//
//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// http://www.redhat.com/                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    gthomas
// Contributors: gthomas, jskov
// Date:         2000-07-26
// Purpose:      
// Description:  
//              
//####DESCRIPTIONEND####
//
//==========================================================================

#define DBG_FLASH 
#define printf diag_printf

#include <pkgconf/hal.h>
#include <cyg/hal/hal_arch.h>
#include <cyg/hal/hal_cache.h>
#include <cyg/infra/diag.h>

#define  _FLASH_PRIVATE_
#include <cyg/io/flash.h>

#include "flash.h"

#include <pkgconf/devs_flash_intel_bootblock.h>

#define _si(p) ((p[0]<<8)|p[1])

#define DEBUG 0

int flash_hwr_init(void)
{
  unsigned short data[4];
  typedef int code_fun(unsigned short *);
  code_fun *_flash_query;
  int stat, num_regions = 0, region_size = 0;
#if defined(CYG_HAL_STARTUP_ROM)
  int code_len;
  extern char flash_query, flash_query_end;
  // Copy 'program' code to RAM for execution
  code_len = (unsigned long) &flash_query_end - (unsigned long) &flash_query;
  _flash_query = (code_fun *) flash_info.work_space;
  memcpy(_flash_query, &flash_query, code_len);
  HAL_DCACHE_SYNC();		// Should guarantee this code will run
  HAL_ICACHE_INVALIDATE_ALL();	// is also required to avoid old contents
#else
  extern code_fun flash_query;
  _flash_query = &flash_query;
#endif

  stat = (*_flash_query) (data);

#if DEBUG
  printf("stat = %04x\n", stat);
  dump_buf(data, sizeof(data));
#endif

  if (data[0] == FLASH_Intel_code) {
      if ((data[1] == (unsigned short) 0x8897) || (data[1] == (unsigned short) 0x88C5))  {
	  printf("\nFLASH:   Intel device type 0x%04x (4Mbyte) \n", data[1]);
	  num_regions = 64;
	  region_size = 0x10000;
      }
      else if (data[1] == (unsigned short) 0x88CD) {
	  printf("\nFLASH:   Intel device type 0x%04x (8Mbyte) \n", data[1]);
	  num_regions = 128;
	  region_size = 0x10000;
      }
      else {
	  printf("\nFLASH:   Intel device found - unknown device type: 0x%04x (setting size to 4Mbyte)  \n", data[1]);
	  num_regions = 64;
	  region_size = 0x10000;
      }
  }

  else if (data[0] == (unsigned short) 0x0020) 
    {
      printf("\nFLASH:  Numonyx device type 0x%04x ",data[1]);
      if ((data[1] == (unsigned short)0x88BA) || (data[1] == (unsigned short)0x88BB))
        {
	  printf("(4Mbyte)\n");
	  num_regions = 64;
	  region_size = 0x10000;
        }
      else
        {
	  printf("unknown (defaulting to 4Mbyte)\n", data[1]);
	  num_regions = 64;
	  region_size = 0x10000;
        }
    }

  else if (data[0] == (unsigned short) 0x00B0) {
      if (data[1] == (unsigned short) 0x00B1) {
	  printf("\nFLASH:   Sharp device type 0x%04x  (8Mbyte) \n", data[1]);
	  num_regions = 128;
	  region_size = 0x10000;
      }
      else {
	  printf("\nFLASH:   Sharp device found - unknown device type: 0x%04x  (setting size to 4Mbyte) \n", data[1]);
	  num_regions = 64;
	  region_size = 0x10000;
      }
  }

  else {
      printf("\nFLASH:   Unknown device found - vendor: 0x%04x device: 0x%04x\n", data[0],  data[1]);
  }

  flash_info.block_size = region_size;
  flash_info.blocks = num_regions;
  flash_info.start = (void *) CYGPKG_DEVS_FLASH_INTEL_BOOTBLOCK_ADDRESS;
  flash_info.end = (void *) (CYGPKG_DEVS_FLASH_INTEL_BOOTBLOCK_ADDRESS + (num_regions * region_size));

  return FLASH_ERR_OK;
}

// Map a hardware status to a package error
int flash_hwr_map_error(int err)
{
   if (!(err & 0x007E007E))
     return FLASH_ERR_OK;

   diag_printf("FLASH:  ERROR\nFLASH:  Status Register Returned non-zero value:  0x%04x ", err);
   
   if (err & 0x00020002)
       diag_printf("-  Prog/Erase attempted on locked block ");
   if (err & 0x00040004) 
       diag_printf("-  Program suspend ");
   if (err & 0x00100010)
       diag_printf("-  Error programming a block ");
   if (err & 0x00200020)
       diag_printf("-  Error in block erase ");
   if (err & 0x00400040) 
       diag_printf("-  Erase suspend issued ");

   diag_printf("\n");

   if (err & 0x00400040) 
     return FLASH_ERR_ERASE_SUSPEND;
   else if (err & 0x00040004) 
     return FLASH_ERR_PROGRAM_SUSPEND;
   else if (err & 0x00020002)
     return FLASH_ERR_PROTECT;
   else if (err & 0x00100010)
     return FLASH_ERR_PROGRAM;
   else if (err & 0x00200020)
     return FLASH_ERR_ERASE;
   else
     return FLASH_ERR_HWR; 
}

// See if a range of FLASH addresses overlaps currently running code
bool flash_code_overlaps(void *start, void *end)
{
  extern char _stext[], _etext[];

  return ((((unsigned long) &_stext >= (unsigned long) start) && ((unsigned long) &_stext < (unsigned long) end)) || (((unsigned long) &_etext >= (unsigned long) start) && ((unsigned long) &_etext < (unsigned long) end)));
}
