//==========================================================================
//
//      flash_program_buf.c
//
//      Flash programming
//
//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// http://www.redhat.com/                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------
// 
//                                                                          
//####COPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    gthomas
// Contributors: gthomas
// Date:         2000-07-14
// Purpose:      
// Description:  
//              
//####DESCRIPTIONEND####
//
//==========================================================================

  
#include "flash.h"

#include <pkgconf/hal.h>
#include <cyg/hal/hal_arch.h>
#include <cyg/hal/hal_cache.h>
#include <cyg/hal/hal_diag.h>
#include <cyg/infra/diag.h>

// CAUTION!  This code must be copied to RAM before execution.  Therefore,
// it must not contain any code which might be position dependent!

#if defined(CYGPKG_KERNEL)
#include <cyg/kernel/kapi.h>
#define doDelay()  do {volatile int i; i=100; while (i) {--i;}} while (0)
#else
#define doDelay()  HAL_DELAY_US(10)
#endif

static volatile unsigned short *ROM;
  
void unlock_sector(volatile unsigned short *addr)
{
  unsigned long flashId;
  ROM[0] = FLASH_Read_ID;
  flashId = (ROM[0]<<16 | ROM[1]);
  ROM[0] = FLASH_Config_Setup;
  *addr  = FLASH_Unlock_Block;
}

static void lock_sector(volatile unsigned short *addr)
{
  unsigned long flashId;
  ROM[0] = FLASH_Read_ID;
  flashId = (ROM[0]<<16 | ROM[1]);
  ROM[0] = FLASH_Config_Setup;
  *addr  = FLASH_Lock_Block;
}



int flash_program_buf(volatile unsigned short *addr, unsigned short *data, int len)
{
  extern volatile unsigned char EpldFlashWriteEnable;
  volatile unsigned short *last = 0;
  unsigned short stat = 0;
  int timeout;
  
  GO_SLOW 
    
  ROM = (volatile unsigned short *) ((unsigned long) addr & 0xFF800000);

  diag_printf("FLASH:  Programming... (target: %08x  source: %08x  bytes: %d)\n",addr, data, len * 2);

  // Clear any error conditions
  EpldFlashWriteEnable = 1;
  
  ROM[0] = FLASH_Clear_Status;

  while (len > 0)
    {
      if ((((unsigned long)addr)^((unsigned long)last)) & ~0x1fff)
        {
          // crossing 8K boundary
          if (last)
            lock_sector(last);
          unlock_sector(addr);
          last = addr;
        }
      *addr = FLASH_Program;
      *addr = *data;
      timeout = 100;
      while (((stat = (flush(),ROM[0])) & FLASH_Status_Ready) != FLASH_Status_Ready)
	{
	  doDelay();
	  if (--timeout == 0)
            goto bad;
	}
      if (stat & 0x007E)
        break;
      ROM[0] = FLASH_Reset;
      if (*addr++ != *data++)
	{
	  stat = 0x9910;
	  break;
	}
      len -= 2;
    }

  // Restore ROM to "normal" mode
bad:
  if (last)
    lock_sector(last);
  
  ROM[0] = FLASH_Reset;
  EpldFlashWriteEnable = 0;

  GO_FAST return stat;
}
