typedef struct
{
  volatile unsigned char byte    __attribute__((packed));
  volatile unsigned char _unused __attribute__((packed));
} tHighByte __attribute__((packed));

typedef struct
{
  volatile unsigned char _unused __attribute__((packed));
  volatile unsigned char byte    __attribute__((packed));
} tLowByte __attribute__((packed));

// don't use fields starting with _, since they will generate
// incorrect accesses.

typedef struct
{
  volatile unsigned short data[8]          __attribute__((packed));
  volatile unsigned short rxFifoCnt[8]     __attribute__((packed));
  volatile unsigned short chanStatus[8]    __attribute__((packed));
  volatile unsigned  char _intID[8]        __attribute__((packed));
  
  volatile unsigned  char _command         __attribute__((packed));
  volatile unsigned  char _intChan         __attribute__((packed));
  
  volatile unsigned  char _intMask         __attribute__((packed));
  volatile unsigned  char _reserved        __attribute__((packed));
  
  volatile unsigned short indexAddr         __attribute__((packed));
  volatile unsigned short indexData         __attribute__((packed));
  unsigned char _unused[0x1000-0x40]        __attribute__((packed));
  
  // offset 0x1000 from the start of this struct

  volatile unsigned short __dataHigh[8]     __attribute__((packed));
  volatile unsigned short __txFifoCnt[8]    __attribute__((packed));
  volatile unsigned short chanStatusHigh[8] __attribute__((packed));
  volatile unsigned  char oddIntID[8]       __attribute__((packed));  // odd chan only
  volatile unsigned  char __command         __attribute__((packed));
  volatile unsigned  char intChan           __attribute__((packed));
  volatile unsigned  char __intMask         __attribute__((packed));
  volatile unsigned  char __reserved        __attribute__((packed));
  volatile unsigned short __indexAddr       __attribute__((packed));
  volatile unsigned short indexDataHigh     __attribute__((packed));
  unsigned char __unused[0x1000-0x40]       __attribute__((packed));

  volatile unsigned short dataLow[8]        __attribute__((packed));
  volatile unsigned short txFifoCnt[8]      __attribute__((packed));
  volatile unsigned short chanStatusLow[8]  __attribute__((packed));
  volatile unsigned  char evenIntID[8]      __attribute__((packed));  // even chan only
  volatile unsigned  char command           __attribute__((packed));
  volatile unsigned  char ___intChan        __attribute__((packed));
  volatile unsigned  char intMask           __attribute__((packed));
  volatile unsigned  char ___reserved       __attribute__((packed));
  volatile unsigned short ___indexAddr      __attribute__((packed));
  volatile unsigned short indexDataLow      __attribute__((packed));
}tAiopic;

#define AiopPutIndexedByte(a,i,b) (/*diag_printf("%08x:%04x <- %02x\n",a,i,b),*/ (a)->indexAddr=(i), *((volatile unsigned char*)&((a)->indexDataHigh))=b)
#define AiopPutIndexedShort(a,i,s) (/*diag_printf("%08x:%04x <- %04x\n",a,i,s),*/ (a)->indexAddr=(i), (a)->indexData=s)
#define AiopGetIndexedByte(a,i) ((a)->indexAddr=(i), *((unsigned char*)&((a)->indexDataHigh)))
#define AiopGetIndexedShort(a,i) ((a)->indexAddr=(i), (a)->indexData)

typedef struct
{
  unsigned short offset __attribute__((packed));
  unsigned short value  __attribute__((packed));
}tRegVal __attribute__((packed));

#define GetChanReg(c,r) ((c)->r.value)
#define PutChanReg(c,r,v) (AiopPutIndexedShort((c)->aiop,(c)->r.offset,((c)->r.value = v)))
#define SetBitsChanReg(c,r,v) (AiopPutIndexedShort((c)->aiop,(c)->r.offset,((c)->r.value |= v)))
#define ClrBitsChanReg(c,r,v) (AiopPutIndexedShort((c)->aiop,(c)->r.offset,((c)->r.value &= ~v)))

typedef struct
{
  tAiopic *aiop;
  unsigned chan;
  unsigned short base;
  volatile unsigned short *dataWord;
  volatile unsigned char  *dataByte;
  volatile unsigned short *rxFifoCnt;
  volatile unsigned char  *txFifoCnt;
  volatile unsigned short *status;
  volatile unsigned char  *statusLow;
  volatile unsigned char  *statusHigh;
  volatile unsigned char  *intID;
  tRegVal txControl;
  tRegVal rxControl;
  tRegVal baudDivisor;
  tRegVal txEnables;
  tRegVal txCompare;
  tRegVal txReplace1;
  tRegVal txReplace2;
  tRegVal R[13];  // this really needs to be better named
}tAiopChan;
  

#define Bit(n) (1<<(n))

#define AiopCommand_ResetAll            Bit(7)
#define AiopCommand_TxOverride          Bit(6)
#define AiopCommand_ResetUart           Bit(5)
#define AiopCommand_ResetTxFifo         Bit(4)
#define AiopCommand_ResetRxFifo         Bit(3)

/* 
 * remember: bytes are swapped in word registers!
 * 
 * That means index address and data are all byte-swapped 
 */


/* status bits that are in low byte of rx data/staus byte pair */

#define AoipRxStatus_RxBreak            Bit(3)
#define AiopRxStatus_RxFrameErr         Bit(2)
#define AiopRxStatus_RxOverrunErr       Bit(1)
#define AiopRxStatus_RxParityErr        Bit(0)

/* channel status register */

#define AiopChanStatus_CTS              Bit(13)
#define AiopChanStatus_DSR              Bit(12)
#define AiopChanStatus_CD               Bit(11)
#define AiopChanStatus_TxFifoAvail      Bit(10)
#define AiopChanStatus_TxShiftEmpty     Bit(9)
#define AiopChanStatus_RxDataAvail      Bit(8)
#define AiopChanStatus_StatusModeEnable Bit(7)
#define AiopChanStatus_RxFifoOverflow   Bit(5)
#define AiopChanStatus_RxCompMatch2     Bit(4)
#define AiopChanStatus_RxCompMatch1     Bit(3)
#define AiopChanStatus_RxBreak          Bit(2)
#define AiopChanStatus_RxFrameErr       Bit(1)
#define AiopChanStatus_RxParityErr      Bit(0)

/* interrupt ID */

#define AiopIntId_RxFifo      Bit(5)
#define AiopIntId_TxFifo      Bit(4)
#define AiopIntId_RxSpecial   Bit(3)
#define AiopIntId_CD          Bit(2)
#define AiopIntId_CTS         Bit(1)
#define AiopIntId_DSR         Bit(0)


#define AiopClock 22118400L
#define AiopPrescale 2
#define AiopBaudDivisor(baud) (((AiopClock/(16*(AiopPrescale+1)))/baud)-1)


/************************************************************************
 Tx Control Register Offsets - Indexed - External - Fixed
************************************************************************/
#define _TX_ENBLS  0x8009    /* Tx Processor Enables Register 8 Read / Write */
#define _TXCMP1    0x8809    /* Transmit Compare Value #1     8 Read / Write */
#define _TXCMP2    0x8909    /* Transmit Compare Value #2     8 Read / Write */
#define _TXREP1B1  0x8A09    /* Tx Replace Value #1 - Byte 1  8 Read / Write */
#define _TXREP1B2  0x8B09    /* Tx Replace Value #1 - Byte 2  8 Read / Write */
#define _TXREP2    0x8C09    /* Transmit Replace Value #2     8 Read / Write */

/************************************************************************
Memory Controller Register Offsets - Indexed - External - Fixed
************************************************************************/
#define _RX_FIFO    0x0000    /* Rx FIFO */
#define _TX_FIFO    0x0008    /* Tx FIFO */
#define _RXF_OUTP   0x9009    /* Rx FIFO OUT pointer        16 Read / Write */
#define _RXF_INP    0x9209    /* Rx FIFO IN pointer         16 Read / Write */
#define _TXF_OUTP   0x9409    /* Tx FIFO OUT pointer        8  Read / Write */
#define _TXF_INP    0x9509    /* Tx FIFO IN pointer         8  Read / Write */
#define _TXP_CNT    0x9609    /* Tx Priority Count          8  Read / Write */
#define _TXP_PNTR   0x9709    /* Tx Priority Pointer        8  Read / Write */

#define PRI_PEND    0x80     /* Priority data pending (bit7, Tx pri cnt) */
#define TXFIFO_SIZE 255      /* size of Tx FIFO */
#define RXFIFO_SIZE 1023     /* size of Rx FIFO */

/************************************************************************
Tx Priority Buffer - Indexed - External - Fixed
************************************************************************/
#define _TXP_BUF    0xC009    /* Tx Priority Buffer  32  Bytes   Read / Write */
#define TXP_SIZE    0x20     /* 32 bytes */

/************************************************************************
Channel Register Offsets - Indexed - Internal - Fixed
************************************************************************/

#define _TX_CTRL    0xF00F    /* Transmit Control               16  Write */
#define _RX_CTRL    0xF20F    /* Receive Control                 8  Write */
#define _BAUD       0xF40F    /* Baud Rate                      16  Write */
#define _CLK_PRE    0xF60F    /* Clock Prescaler                 8  Write */

#define AiopRxCntrl_RtsFlowEnable         Bit(14)
#define AiopRxCntrl_ProcEnable            Bit(13)
#define AiopRxCntrl_RxFifoTrigShift       11
#define AiopRxCntrl_RxFifoTrigMask        (3<<11)
#define AiopRxCntrl_SpecIntEnable         Bit(10)
#define AiopRxCntrl_IntEnable             Bit(9)
#define AiopRxCntrl_ModemIntEnable        Bit(8)

#define AiopTxCntrl_BreakOn      Bit(4)
#define AiopTxCntrl_LocalLoop    Bit(3)
#define AiopTxCntrl_DTR          Bit(2)
#define AiopTxCntrl_RTS          Bit(1)
#define AiopTxCntrl_TxEnable     Bit(0)
#define AiopTxCntrl_CtsFlow      Bit(15)
#define AiopTxCntrl_RtsToggle    Bit(14)
#define AiopTxCntrl_TxIntEnable  Bit(12)
#define AiopTxCntrl_2stopBits    Bit(11)
#define AiopTxCntrl_ParityEnable Bit(10)
#define AiopTxCntrl_EvenParity   Bit(9)
#define AiopTxCntrl_8dataBits    Bit(8)


#define sEnRxStatusMode(ch)  *(ch->status) = AiopChanStatus_StatusModeEnable
#define sDisRxStatusMode(ch) *(ch->status) = 0

#define sEnTransmit(ch) SetBitsChanReg(ch,txControl,AiopTxCntrl_TxEnable)
#define sDisTransmit(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_TxEnable)

#define sEnLoopback(ch)  SetBitsChanReg(ch,txControl,AiopTxCntrl_LocalLoop)
#define sDisLoopback(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_LocalLoop)

#define sSetRTS(ch) SetBitsChanReg(ch,txControl,AiopTxCntrl_RTS)
#define sClrRTS(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_RTS)

#define sSetDTR(ch) SetBitsChanReg(ch,txControl,AiopTxCntrl_DTR)
#define sClrDTR(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_DTR)

#define sSetData8(ch) SetBitsChanReg(ch,txControl,AiopTxCntrl_8dataBits)
#define sSetData7(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_8dataBits)

#define sEnRxProcessor(ch)   SetBitsChanReg(ch,rxControl,AiopRxCntrl_ProcEnable)
#define sDisRxProcessor(ch)  ClrBitsChanReg(ch,rxControl,AiopRxCntrl_ProcEnable)

#define sEnRxFifo(ch) PutChanReg(ch,R[12],0x080a)
#define sDisRxFifo(ch) PutChanReg(ch,R[12],0x0a0a)

#define sGetRxCnt(ch) *(ch->rxFifoCnt)
#define sGetTxCnt(ch) *(ch->txFifoCnt)

#define sEnRTSToggle(ch)  SetBitsChanReg(ch,txControl,AiopTxCntrl_RtsToggle)
#define sDisRTSToggle(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_RtsToggle)
