#include <pkgconf/system.h>
#include <pkgconf/io_nserial.h>
#include <pkgconf/io.h>
#include <cyg/io/io.h>
#include <cyg/hal/hal_intr.h>
#include <cyg/io/devtab.h>
#include <cyg/io/serial.h>
#include <cyg/io/serialio.h>  
#include <cyg/infra/diag.h>

#if 1
#define debug_printf(args...) diag_printf(args)
#else
#define debug_printf(args...) /* noop */
#endif

// don't know UART clock until run time, so macros below can't be used
// in static initializers.

// platform specific stuff to figure out baud rate divisor
extern unsigned long CLKCON;
#define ClkDivisor (CLKCON ? CLKCON<<1 : 1)
#define UartClock (7372800L/ClkDivisor)
#define UartDivisor(b) ((unsigned short)(UartClock/(16L*b)))


static unsigned long select_baud[] = 
{
  0,
  50,
  75,
  110,
  134,
  150,
  200,
  300,
  600,
  1200,
  1800,
  2400,
  3600,
  4800,
  7200,
  9600,
  14400,
  19200,
  38400,
  57600,
  115200,
  230400,
  460800
};

static unsigned short xoffChar = 0x13;

#include "16c850_serial.h"

typedef struct xr16850_serial_info 
{
  CYG_ADDRWORD   base;
  CYG_WORD       int_num;
  int            port_num;
  cyg_interrupt  serial_interrupt;
  cyg_handle_t   serial_interrupt_handle;
  cyg_serial_error_count_t error_count;
  bool           receive_stopped;
  unsigned       interCharTimeout;
  unsigned       idleCount;
  bool           timeoutFlag;
  bool           gotData;
  unsigned       rxFifoLast;
} xr16850_serial_info;

#define UseTimer 0

#if UseTimer
static cyg_interrupt  xr16850_timer_interrupt;
static cyg_handle_t   xr16850_timer_interrupt_handle;
static void timerSetup(unsigned period);
#endif
static bool xr16850_serial_init(struct cyg_devtab_entry *tab);
static bool xr16850_serial_putc(serial_channel *chan, unsigned char c);
static Cyg_ErrNo xr16850_serial_lookup(struct cyg_devtab_entry **tab, 
                                   struct cyg_devtab_entry *sub_tab,
                                   const char *name);
static unsigned char xr16850_serial_getc(serial_channel *chan);
static bool xr16850_serial_set_config(serial_channel *chan, cyg_serial_info_t *config);
static void xr16850_serial_start_xmit(serial_channel *chan);
static void xr16850_serial_stop_xmit(serial_channel *chan);
static void xr16850_serial_start_recv(serial_channel *chan);
static void xr16850_serial_flush_input(serial_channel *chan);
static void xr16850_serial_flush_output(serial_channel *chan);
static int xr16850_serial_get_tx_count(serial_channel *chan);
static int xr16850_serial_get_rx_count(serial_channel *chan);
static void xr16850_serial_stop_recv(serial_channel *chan);
static Cyg_ErrNo xr16850_serial_set_config_handler(serial_channel *chan, cyg_uint32 key, const void *xbuf, cyg_uint32 *len);
static Cyg_ErrNo xr16850_serial_get_config_handler(serial_channel *chan, cyg_uint32 key, void *xbuf, cyg_uint32 *len);
static cyg_uint32 xr16850_serial_ISR(cyg_vector_t vector, cyg_addrword_t data);
static void       xr16850_serial_DSR(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data);
#if UseTimer
static cyg_uint32 xr16850_timer_ISR(cyg_vector_t vector, cyg_addrword_t data);
#endif

static SERIAL_FUNS2(xr16850_serial_funs, 
                   xr16850_serial_putc, 
                   xr16850_serial_getc,
                   xr16850_serial_set_config,
                   xr16850_serial_start_xmit,
                   xr16850_serial_stop_xmit,
                   xr16850_serial_get_config_handler,
                   xr16850_serial_set_config_handler,
                   xr16850_serial_start_recv,
                   xr16850_serial_stop_recv,
                   xr16850_serial_flush_input,
                   xr16850_serial_flush_output,
                   xr16850_serial_get_tx_count,
                   xr16850_serial_get_rx_count
    );

static struct
  {
    cyg_mutex_t lock;
    cyg_cond_t  wait;
  }sync;

#define SerialBufsize 2048

static xr16850_serial_info xr16850_serial_info0 = {0x7604000,2,0};
static unsigned char xr16850_serial_out_buf0[SerialBufsize];
static unsigned char xr16850_serial_in_buf0[SerialBufsize];

serial_channel xr16850_serial_channel0 =
{
  &xr16850_serial_funs,
  &cyg_io_serial_callbacks,
  &xr16850_serial_info0,
  CYG_SERIAL_INFO_INIT(9600, 1, 0, 8, 0, 0x11, 0x13, 0x11, 0x13),
  false,
  CBUF_INIT(xr16850_serial_out_buf0, sizeof xr16850_serial_out_buf0),
  CBUF_INIT(xr16850_serial_in_buf0, sizeof xr16850_serial_in_buf0),
  {&sync.lock,&sync.wait}  
};

DEVTAB_ENTRY(xr16850_serial_io0, 
             "/dev/ser0",
             0,                     // Does not depend on a lower level interface
             &cyg_io_serial_devio, 
             xr16850_serial_init, 
             xr16850_serial_lookup,     // Serial driver may need initializing
             &xr16850_serial_channel0
    );

static xr16850_serial_info xr16850_serial_info1 = {0x7608000,3,1};
static unsigned char xr16850_serial_out_buf1[SerialBufsize];
static unsigned char xr16850_serial_in_buf1[SerialBufsize];

serial_channel xr16850_serial_channel1 =
{
  &xr16850_serial_funs, 
  &cyg_io_serial_callbacks,
  &xr16850_serial_info1,
  CYG_SERIAL_INFO_INIT(9600, 1, 0, 8, 0, 0x11, 0x13, 0x11, 0x13),
  false,
  CBUF_INIT(xr16850_serial_out_buf1, sizeof xr16850_serial_out_buf1),
  CBUF_INIT(xr16850_serial_in_buf1, sizeof xr16850_serial_in_buf1),
  {&sync.lock,&sync.wait}  
};


DEVTAB_ENTRY(xr16850_serial_io1, 
             "/dev/ser1",
             0,                     // Does not depend on a lower level interface
             &cyg_io_serial_devio, 
             xr16850_serial_init, 
             xr16850_serial_lookup,     // Serial driver may need initializing
             &xr16850_serial_channel1
    );



#define DEBUG_BUFFER_ENABLED 0

#if DEBUG_BUFFER_ENABLED

#define DebugBufferSize  512
static unsigned debugBuffer[DebugBufferSize] = {0};
static unsigned debugBufferIndex = 0;

void putDebug(void* port, unsigned char v, short p)
{
  if (port == (void*)0x7608000)
    {
      debugBuffer[debugBufferIndex] = ((v | ((unsigned)port&0xff00) )<<16)+p;
      ++debugBufferIndex;
      debugBufferIndex &= (DebugBufferSize-1);
      debugBuffer[debugBufferIndex] = 0;
    }
}
#else
#define putDebug(port,v,p) /* noop */
#endif

#define min(a,b) ((a)<(b)?(a):(b))

static void handleTransmitBytes(serial_channel *chan, xr16850_serial_info *xr16850_chan, volatile struct serial_port *port)
{
  cbuf_t *cbuf = &chan->out_cbuf;
  int txFifoSpace;
  unsigned char scr;
  int avail,count,size;
  unsigned char *src;

  //putDebug((void*)port,0x20,cyg_current_time()&0xffff);
  
  while (cbuf->get != cbuf->put)
    {
      //putDebug((void*)port,0x21,cbuf->put);
      //putDebug((void*)port,0x22,cbuf->get);
      
      if (cbuf->put > cbuf->get)
        avail = cbuf->put - cbuf->get;
      else
        avail = cbuf->len - cbuf->get;
      
      //putDebug((void*)port,0x23,avail);

      // UART bug:
      // scr register doesn't always read the right value, so
      // read it until we get the same value twice in a row.

      do scr = port->scr; while (scr != port->scr);
      
      txFifoSpace = 128 - scr;
      
      //putDebug((void*)port,0x24,txFifoSpace);
      
      
      // UART bug:
      // loosing bytes when FIFO is filled up?
      
      txFifoSpace -= 2;
      
      if (txFifoSpace <= 0)
        break;
      
      size = count = min(txFifoSpace,avail);
      src = cbuf->data+cbuf->get;

      //putDebug((void*)port,0x25,count);
      //putDebug((void*)port,0x26,*src);
      
      while (count--)
        port->thr = *src++;

      cbuf->get += size;
      
      if (cbuf->get == cbuf->len) 
        cbuf->get = 0;
      
      if (size != avail)
        break;
    }

  //putDebug((void*)port,0x27,0);
  
  if (cbuf->get == cbuf->put)
    {
      (chan->funs->stop_xmit)(chan);  // Done with transmit
      //putDebug((void*)port,0x28,0);
    }
  
  if (cbuf->waiting) 
    {
      // See if there is now enough room to restart writer
      int space = (cbuf->len + cbuf->get) - cbuf->put;
      if (space > cbuf->len) space -= cbuf->len;
      if (space >= cbuf->low_water) 
        {
          cbuf->waiting = false;
          cyg_drv_cond_broadcast(&cbuf->wait);
        }
    }
}

static void handleReceiveBytes(serial_channel *chan, xr16850_serial_info *xr16850_chan, volatile struct serial_port *port)
{
  unsigned char rxStatus;
  unsigned char rxByte=0;
  cbuf_t *cbuf = &chan->in_cbuf;
  int wakeup = 0;
  int nextPut;

  if (xr16850_chan->timeoutFlag)
    {
      chan->rxInterCharTimeoutFlag = 1;
      xr16850_chan->timeoutFlag = 0;
      wakeup = 1;
    }
  
  nextPut = cbuf->put+1;
  if (nextPut == cbuf->len)
    nextPut = 0;

  putDebug((void*)port,0x30,cyg_current_time()&0xffff);
  putDebug((void*)port,0x31,cbuf->put);
  putDebug((void*)port,0x32,port->lsr);
  
  while (port->lsr & LSR_RDR)
    {
      if (nextPut == cbuf->get)  //full?
        {
          putDebug((void*)port,0x33,0);
          if (chan->funs->stop_recv)
            {
              cbuf->rxstopped = true;
              chan->funs->stop_recv(chan);
            }
          break;
        }
      
      wakeup = 1;
      
      rxStatus = port->lsr;
      rxByte = port->rhr;
      
      if (rxStatus & (LSR_RXBRK + LSR_FIFO_ERR + LSR_RXOVR_ERR + LSR_RXPAR_ERR + LSR_RXERR))
        {
          if (rxStatus & LSR_RXBRK)
            ++xr16850_chan->error_count.rxBreak;
          if (rxStatus & LSR_FIFO_ERR)
            ++xr16850_chan->error_count.fifo;
          if (rxStatus & LSR_RXOVR_ERR)
            ++xr16850_chan->error_count.rxOverflow;
          if (rxStatus & LSR_RXPAR_ERR)
            ++xr16850_chan->error_count.rxParity;
          if (rxStatus & LSR_RXFRM_ERR)
            ++xr16850_chan->error_count.rxFraming;
        }
//  Commenting out the 'else' means that bytes received with parity/frame errors are still 
//  passed up to the application.  This is the way that the SI works, and the built-in
//  Windows and Linux drivers work, and apparently our CimNet customers need this to.  Kurt R.  10-13-03
//      else
        {
          if (rxByte=='\0' && chan->config.flags & CYG_SERIAL_FLAGS_IGN_NULLS)
            continue;
          
          cbuf->data[cbuf->put] = rxByte;
          cbuf->put = nextPut;
          
          ++nextPut;
          if (nextPut == cbuf->len)
            nextPut = 0;
        }
    }

  xr16850_chan->rxFifoLast = 0;
  
  putDebug((void*)port,0x34,rxByte);
  putDebug((void*)port,0x35,cbuf->put);
    
  if (cbuf->waiting && wakeup)
    {
      cbuf->waiting = false;
      cyg_drv_cond_signal(&cbuf->wait);
      putDebug((void*)port,0x36,0);
    }
}
 
#define TRACE() diag_printf(__FILE__ " line %d\n",__LINE__)

// Internal function to actually configure the hardware to desired baud rate, etc.
static bool xr16850_serial_config_port(serial_channel *chan, cyg_serial_info_t *new_config, bool init)
{
  extern volatile unsigned char EpldInterfaceSelect[];
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;
  unsigned short baud_divisor;
  unsigned char _lcr, _ier, _efr, isr;

  xr16850_chan->interCharTimeout = new_config->inter_char_timeout;
  xr16850_chan->gotData = 0;
  xr16850_chan->timeoutFlag = 0;

  if (new_config->baud >= 50)
    baud_divisor = UartDivisor(new_config->baud);
  else
    baud_divisor = UartDivisor(select_baud[new_config->baud]);
  
  if (baud_divisor == 0) return false;  // Invalid configuration
  
  _ier = port->ier;
  port->ier = 0;  // Disable port interrupts while changing hardware
  _lcr = select_word_length[new_config->word_length - CYGNUM_SERIAL_WORD_LENGTH_5] | 
    select_stop_bits[new_config->stop] |
    select_parity[new_config->parity];
  port->lcr = _lcr;
  port->lcr |= LCR_DL;
  port->mdl = baud_divisor >> 8;
  port->ldl = baud_divisor & 0xFF;
  port->lcr &= ~LCR_DL;
  if (init) 
    {
      volatile unsigned char scratch;
      port->ier = 0;
      port->fcr = FCR_FIFO_ENABLE | FCR_TXFIFO_RESET | FCR_RXFIFO_RESET;
      
      // flush pending rx data

      while (port->lsr & LSR_RDR) 
        scratch = port->rhr;
      
      // clear pending interrupts

      while ((isr=port->isr&0x3f) != 0x01)
        {
          scratch = port->lsr;
          scratch = port->msr;
        }
      
      if (chan->out_cbuf.len != 0) 
          port->ier = IER_RCV;
      
      port->mcr = MCR_INT;  // Master interrupt enable
    } 
  else 
      port->ier = _ier;

  if (new_config->flags & CYG_SERIAL_FLAGS_EN_LOOPBACK)
    port->mcr |= MCR_LOOP;
  else
    port->mcr &= ~MCR_LOOP;
  
  
  _lcr = port->lcr;  // save so we can restore it later
  port->lcr = 0xbf;  // switch to ehanced register set

  if (init)
    {
      // set FIFO trigger levels
      
      port->fctr = FCTR_TRIGTABLE_D;  // user programmable trigger levels
  
      port->fctr |= FCTR_TRIG_TX_SEL;      // Tx
      port->trg = 64;
      
      port->fctr &= ~FCTR_TRIG_TX_SEL;     // Rx
      port->trg = 64;
      
      // set RTS trigger level for RTS flow control

      port->fctr |= 3;
    }
  
  // configure flow control
  
  port->fctr &= ~FCTR_SCRATCH_SWAP;  // shut off scr swap so we can write to xoff2
  
  port->xon1  = new_config->rxflow_xon_char;
  port->xoff1 = new_config->rxflow_xoff_char;
  port->xon2  = new_config->txflow_xon_char;
  port->xoff2 = new_config->txflow_xoff_char;
  xoffChar = new_config->txflow_xoff_char;

  port->fctr |= FCTR_SCRATCH_SWAP | FCTR_TRIG_TX_SEL;   // re-enable scratch swap

  // Put RTS pin in RS485 mode?

  if (new_config->flags & CYG_SERIAL_FLAGS_RTS_TOGGLE)
    port->fctr |= FCTR_RS485;
  else
    port->fctr &= ~FCTR_RS485;

  _efr = EFR_EXTENABLE;

  if (new_config->flags & CYG_SERIAL_FLAGS_XONXOFF_TXFLOW) 
    _efr |= EFR_TXSWFLOW_SET2;
  
  if (new_config->flags &  CYG_SERIAL_FLAGS_XONXOFF_RXFLOW)
    _efr |= EFR_RXSWFLOW_SET1;

  if (new_config->flags & (CYG_SERIAL_FLAGS_RTS_RXFLOW | CYG_SERIAL_FLAGS_RTSCTS))
    _efr |= EFR_AUTORTS;

  if (new_config->flags & (CYG_SERIAL_FLAGS_CTS_TXFLOW | CYG_SERIAL_FLAGS_RTSCTS))
    _efr |= EFR_AUTOCTS;
  
  port->efr = _efr;
      
  port->lcr = _lcr;    // back to normal register set

  port->emsr = 0x31;   // scratch is tx fifo count, large RTS hyst

  // assert RTS if RTS flow control is enabled
  
  if (new_config->flags & (CYG_SERIAL_FLAGS_RTS_RXFLOW | CYG_SERIAL_FLAGS_RTSCTS))
    port->mcr |= MCR_RTS;

  EpldInterfaceSelect[0] = xr16850_chan->port_num | new_config->interface_mode;
  
  // save config

  if (new_config != &chan->config) 
    chan->config = *new_config;

  return true;
}

// Internal function to handle set_config keys not implimented by higher layers
static Cyg_ErrNo
xr16850_serial_set_config_handler(serial_channel *chan, cyg_uint32 key, const void *xbuf, cyg_uint32 *len)
{
  Cyg_ErrNo res = ENOERR;
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;
  
  switch (key)
    {
     case CYG_IO_SET_CONFIG_SERIAL_MODEM:
      if (*len != sizeof (cyg_uint32))
        res = -EINVAL;
      else
        {
          unsigned char _mcr = port->mcr;
          if (*(cyg_uint32*)xbuf & CYG_SERIAL_MODEM_DTR) 
            _mcr |= MCR_DTR;
          else
            _mcr &= ~MCR_DTR;
          // Ignore RTS flag if it's being controlled automatically
          if (!(chan->config.flags & (CYG_SERIAL_FLAGS_RTSCTS |
                                      CYG_SERIAL_FLAGS_RTS_RXFLOW |
                                      CYG_SERIAL_FLAGS_RTS_TOGGLE)))
            {
              if (*(cyg_uint32*)xbuf & CYG_SERIAL_MODEM_RTS) 
                _mcr |= MCR_RTS;
              else
                _mcr &= ~MCR_RTS;
            }
          port->mcr = _mcr;
        }
      break;
      
     default:
      res = -EINVAL;
    }
  
  return res;
}
      
// Internal function to handle get_config keys not implimented by higher layers
static Cyg_ErrNo
xr16850_serial_get_config_handler(serial_channel *chan, cyg_uint32 key, void *xbuf, cyg_uint32 *len)
{
  Cyg_ErrNo res = ENOERR;
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;
  unsigned char _ldl, _mdl, _ier, _lcr, _efr;  //  Backup register values
  unsigned short baud_divisor;

  switch (key)
    {
     case CYG_IO_GET_CONFIG_SERIAL_MODEM:
      if (*len != sizeof (cyg_uint32))
        res = -EINVAL;
      else
        {
          unsigned char _mcr = port->mcr;
          unsigned char _msr = port->msr;

          *(cyg_uint32*)xbuf = 0;
          
          if (_mcr & MCR_DTR)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_DTR;
          if (_mcr & MCR_RTS)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_RTS;
          
          if (_msr & MSR_DSR)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_DSR;
          if (_msr & MSR_CTS)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_CTS;
          if (_msr & MSR_CD)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_CD;
          if (_msr & MSR_RI)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_RI;

          if (_msr & MSR_DSR_DELTA)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_DSR_DELTA;
          if (_msr & MSR_CTS_DELTA)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_CTS_DELTA;
          if (_msr & MSR_CD_DELTA)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_CD_DELTA;
          if (_msr & MSR_RI_DELTA)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_RI_DELTA;
          
          if ((chan->config.flags & (CYG_SERIAL_FLAGS_CTS_TXFLOW | CYG_SERIAL_FLAGS_RTSCTS)) 
              && !(_msr & MSR_CTS))
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_FLOWTXOFF;

//  According the the UART vendor EXAR, there is no way to determine XOFF/XON state. 
//          if ((chan->config.flags & CYG_SERIAL_FLAGS_XONXOFF_TXFLOW) 
//              && (xr16850_chan->xoffActive))
//            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_FLOWTXOFF;
        }
      break;
      
     case CYG_IO_GET_CONFIG_SERIAL_ERRORS:
      if (*len < sizeof xr16850_chan->error_count)
        res = -EINVAL;
      else
        *((cyg_serial_error_count_t*)(xbuf)) = xr16850_chan->error_count;
      break;
      
     case CYG_IO_GET_CONFIG_SERIAL_CLRERRS:
      memset(&xr16850_chan->error_count, 0, sizeof xr16850_chan->error_count);
      break;

     case CYG_IO_GET_CONFIG_SERIAL_BREAKON:
      port->lcr |= LCR_BRK;
      break;
      
     case CYG_IO_GET_CONFIG_SERIAL_BREAKOFF:
      port->lcr &= ~LCR_BRK;
      break;
      
      //  The algorithms used for SET TXOFF and SET TXON state are from EXAR, the UART vendor
     case CYG_IO_SET_CONFIG_TX_OFF:

	 //  Algorithm: Flush TX FIFO, set baud to 230400, switch to loopback mode, 
	 //  send XOFF to ourself, set baud back, switch out of loopback
	 _ier = port->ier;                    //  Disable port interrupts while changing hardware
	 port->ier = 0;                       
//*
 	 port->fcr |= FCR_TXFIFO_RESET;       //  Flush TX FIFO
 	 cyg_thread_delay(1);                 //  Wait 10ms

	 port->mcr |= MCR_LOOP;               //  Enter loopback mode
	 baud_divisor = UartDivisor(230400);  //  Set baud to 230K
	 port->lcr |= LCR_DL;
	 _mdl = port->mdl;
	 _ldl = port->ldl;
	 port->mdl = baud_divisor >> 8;
	 port->ldl = baud_divisor & 0xFF;
	 port->lcr &= ~LCR_DL;
	 xr16850_serial_putc(chan, xoffChar); //  Send XOFF to ourselves
	 cyg_thread_delay(1);                 //  Wait 10ms
	 port->fcr |= FCR_TXFIFO_RESET;       //  Flush TX FIFO again
	 port->lcr |= LCR_DL;
	 port->mdl = _mdl;                    //  Restore baud
	 port->ldl = _ldl;
	 port->lcr &= ~LCR_DL;
	 port->mcr &= ~MCR_LOOP;              //  Exit loopback mode        
	 port->ier = _ier;                    //  Re-enable interrupts
      break;

    case CYG_IO_GET_CONFIG_TX_OVERRIDE:
	//  Algorithm:  Switch out of XON/XOFF mode, then switch back
	 _ier = port->ier;     //  Disable port interrupts while changing hardware
	 port->ier = 0;                       
	 _lcr = port->lcr;     //  Switch to enhanced register set           
	 port->lcr = 0xbf;  
	 _efr = port->efr;
	 port->efr = 0;        //  Disable then re-enable XON/XOFF  
	 port->efr = _efr;
	 port->lcr = _lcr;
	 port->ier = _ier;     //  Re-enable interrupts
	break;

     default:
      res = -EINVAL;
    }
  
  return res;
}

// Function to initialize the device.  Called at bootstrap time.
static bool xr16850_serial_init(struct cyg_devtab_entry *tab)
{
#if UseTimer  
  static int initDone;
#endif  
  serial_channel *chan = (serial_channel *)tab->priv;
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  diag_printf("16850 Serial init - dev: %x.%d\n", xr16850_chan->base, xr16850_chan->int_num);
  debug_printf("  Clock Divisor = %d\n",ClkDivisor);
  debug_printf("     Uart Clock = %d\n",UartClock);
  (chan->callbacks->serial_init)(chan);  // Really only required for interrupt driven devices
  cyg_drv_interrupt_create(xr16850_chan->int_num,
                           99,                     // Priority - what goes here?
                           (cyg_addrword_t)chan,   //  Data item passed to interrupt handler
                           xr16850_serial_ISR,
                           xr16850_serial_DSR,
                           &xr16850_chan->serial_interrupt_handle,
                           &xr16850_chan->serial_interrupt);
  cyg_drv_interrupt_attach(xr16850_chan->serial_interrupt_handle);
  cyg_drv_interrupt_acknowledge(xr16850_chan->int_num);
  cyg_drv_interrupt_unmask(xr16850_chan->int_num);
  xr16850_serial_config_port(chan, &chan->config, true);
#if UseTimer  
  if (!initDone)
    {
      timerSetup(CYGNUM_HAL_CPUCLOCK/1000L);  // 1ms interrupt      
      cyg_drv_interrupt_create(CYGNUM_HAL_INTERRUPT_Timer1,
                               99,
                               0,
                               xr16850_timer_ISR,
                               xr16850_serial_DSR,
                               &xr16850_timer_interrupt_handle,
                               &xr16850_timer_interrupt);
      cyg_drv_interrupt_attach(xr16850_timer_interrupt_handle);
      initDone = 1;
    }
#endif  
  return true;
}

// This routine is called when the device is "looked" up (i.e. attached)
static Cyg_ErrNo xr16850_serial_lookup(struct cyg_devtab_entry **tab, 
                  struct cyg_devtab_entry *sub_tab,
                  const char *name)
{
  serial_channel *chan = (serial_channel *)(*tab)->priv;
#if UseTimer  
  static int initDone;
  if (!initDone)
    {
      cyg_drv_interrupt_acknowledge(CYGNUM_HAL_INTERRUPT_Timer1);
      cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_Timer1);
      initDone = 1;
    }
#endif  
  chan->callbacks->serial_init(chan);
  return ENOERR;
}

// Send a character to the device output buffer.
// Return 'true' if character is sent to device
static bool xr16850_serial_putc(serial_channel *chan, unsigned char c)
{
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;
  unsigned char txFifoCount;

  txFifoCount = port->scr;
  
  //putDebug((void*)port,0x10,txFifoCount);

  if (txFifoCount < 128) // scratch reg actually contains tx fifo count
    {
      // room in tx FIFO
      port->thr = c;
      //putDebug((void*)port,0x11,c);
      return true;
    } 
  else 
    {
      // No space
      //putDebug((void*)port,0x12,c);
      return false;
    }
}


// Fetch a character from the device input buffer, waiting if necessary
static unsigned char xr16850_serial_getc(serial_channel *chan)
{
    unsigned char c;
    xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
    volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;
    while ((port->lsr & LSR_RDR) == 0) ;   // Wait for char
    c = port->rhr;
    return c;
}

// Set up the device characteristics; baud rate, etc.
static bool xr16850_serial_set_config(serial_channel *chan, cyg_serial_info_t *config)
{
    return xr16850_serial_config_port(chan, config, false);
}

// Enable the transfer of bytes to next layer up
static void xr16850_serial_start_recv(serial_channel *chan)
{
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;
  
  xr16850_chan->receive_stopped = false;
  handleReceiveBytes(chan, xr16850_chan, port);
  if (!xr16850_chan->receive_stopped)
    port->ier |= IER_RCV;
}
  

// Disable the transfer of bytes to next layer up -- we need to disable
// the receive interrupt so that we stop reading bytes from the Rx FIFO
// so that the UART will flow controll the other end.
static void xr16850_serial_stop_recv(serial_channel *chan)
{
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;

  xr16850_chan->receive_stopped = true;
  port->ier &= ~IER_RCV;
}


// Enable the transmit interrupt on the device
static void xr16850_serial_start_xmit(serial_channel *chan)
{
    xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
    volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;
    port->ier |= IER_XMT;  // Enable xmit interrupt
}

// Disable the transmit interrupt on the device
static void xr16850_serial_stop_xmit(serial_channel *chan)
{
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;
  port->ier &= ~IER_XMT;  // Disable xmit interrupt
}

static void xr16850_serial_flush_input(serial_channel *chan)
{
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;
  port->fcr |= FCR_RXFIFO_RESET;
}

static void xr16850_serial_flush_output(serial_channel *chan)
{
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;
  port->fcr |= FCR_TXFIFO_RESET;
}

// return tx fifo count + (1 if we're still sending a byte)
static int xr16850_serial_get_tx_count(serial_channel *chan)
{
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;
  unsigned char scr;
  unsigned char txbusy;
  
  do scr = port->scr; while (scr != port->scr);
      
  txbusy = port->lsr & (LSR_THE+LSR_TSRE);
  
  return scr + (txbusy != (LSR_THE+LSR_TSRE));
}

// for now, we'll just return 1 if any data is available
static int xr16850_serial_get_rx_count(serial_channel *chan)
{
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;
  return (port->lsr & LSR_RDR) != 0;
}

#if UseTimer
static void checkTimeout(xr16850_serial_info *chanInfo)
{
  extern void cyg_interrupt_post_dsr(CYG_ADDRWORD intr_obj);
  volatile struct serial_port *port = (volatile struct serial_port*)chanInfo->base;
  unsigned char rxFifoCount;
  
  port->emsr = 0x30;
  do rxFifoCount = port->scr; while (rxFifoCount != port->scr);
  port->emsr = 0x31;
  
  if (rxFifoCount != chanInfo->rxFifoLast)
    {
      chanInfo->gotData = 1;
      chanInfo->rxFifoLast = rxFifoCount;
      chanInfo->idleCount = 0;
      return;
    }
  
  ++chanInfo->idleCount;
  
  if (chanInfo->gotData &&
      chanInfo->interCharTimeout &&
      chanInfo->idleCount >= chanInfo->interCharTimeout)
    {
      chanInfo->timeoutFlag = 1;
      chanInfo->gotData = 0;
      cyg_interrupt_post_dsr(chanInfo->serial_interrupt_handle);
    }
}

// timer ISR routine
static cyg_uint32 xr16850_timer_ISR(cyg_vector_t vector, cyg_addrword_t data)
{
  checkTimeout(&xr16850_serial_info0);
  checkTimeout(&xr16850_serial_info1);
  cyg_drv_interrupt_acknowledge(CYGNUM_HAL_INTERRUPT_Timer1);
  return CYG_ISR_HANDLED;
}
#endif


// Serial I/O - low level interrupt handler (ISR)
static cyg_uint32 xr16850_serial_ISR(cyg_vector_t vector, cyg_addrword_t data)
{
  serial_channel *chan = (serial_channel *)data;
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  putDebug((void*)xr16850_chan->base,0xf0,0);
  cyg_drv_interrupt_mask(xr16850_chan->int_num);
  cyg_drv_interrupt_acknowledge(xr16850_chan->int_num);
  return CYG_ISR_CALL_DSR;  // Cause DSR to be run
}

  
// Serial I/O - high level interrupt handler (DSR)
static void xr16850_serial_DSR(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data)
{
  serial_channel *chan = (serial_channel *)data;
  xr16850_serial_info *xr16850_chan = (xr16850_serial_info *)chan->dev_priv;
  volatile struct serial_port *port = (volatile struct serial_port *)xr16850_chan->base;
  unsigned char isr;
  
  putDebug((void*)port,0xf1,0);
  
  if (xr16850_chan->timeoutFlag)
    handleReceiveBytes(chan,xr16850_chan,port);
    
  while (1)
    {
      isr = port->isr;
      putDebug((void*)port,0xf4,isr);
      isr &= 0x1E;
      if (isr == 0)
        break;
      else if (((isr == ISR_RxTO) || (isr == ISR_Rx)) && (!xr16850_chan->timeoutFlag))
        handleReceiveBytes(chan,xr16850_chan,port);
      else if (isr == ISR_Tx)
        handleTransmitBytes(chan,xr16850_chan,port);
    }

  cyg_drv_interrupt_unmask(xr16850_chan->int_num);
}

#if UseTimer  
  
//===========================================================================
// Want to do periodic polling, for inter-char timeout, so we
// need a configurable timer interrupt

#define Bit(n) (1<<(n))

typedef volatile unsigned int reg;
typedef struct
{
  reg tmod;      // Timer mode
  reg tdata0;    // Timer 0 Data
  reg tdata1;    // Timer 1 Data
  reg tcnt0;     // Timer 0 Count
  reg tcnt1;     // Timer 1 Count
}tTimers;

#define TimerModeEnable0  Bit(0)
#define TimerModeToggle0  Bit(1)
#define TimerModeInitOut0 Bit(2)
#define TimerModeEnable1  Bit(3)
#define TimerModeToggle1  Bit(4)
#define TimerModeInitOut1 Bit(5)

extern volatile tTimers ks32c5000_timer;

static void timerSetup(unsigned period)
{
  ks32c5000_timer.tmod &= ~TimerModeEnable1;                   // disable timer
  ks32c5000_timer.tdata1 = period;                             // set period
  ks32c5000_timer.tmod |= TimerModeEnable1 + TimerModeToggle1; // enable timer
}
  
#endif
