#ifndef CYGONCE_16850_SERIAL_H
#define CYGONCE_16850_SERIAL_H

// ====================================================================
//
//      pid_serial.h
//
//      Device I/O - Description of ARM PID7T serial hardware
//
// ====================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// http://www.redhat.com/                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
// All Rights Reserved.                                                     
// -------------------------------------------                              
//                                                                          
//####COPYRIGHTEND####
// ====================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):           gthomas
// Contributors:        gthomas
// Date:        1999-02-04
// Purpose:     Internal interfaces for serial I/O drivers
// Description:
//
//####DESCRIPTIONEND####
//
// ====================================================================

// Description of 16c850 UART

#define IoPacked

#ifdef IoPacked

struct serial_port {
    unsigned volatile char _byte[8];
};
#define reg(n) _byte[n]

#else 

//// IO is not packed

struct serial_port {
    unsigned char _byte[32];
};

// Little-endian version
#if (CYG_BYTEORDER == CYG_LSBFIRST)
#define reg(n) _byte[n*4]
#else // Big-endian version
#define reg(n) _byte[(n*4)^3]
#endif

#endif

// Receive control registers
#define rhr reg(0)    // Receive holding register
#define isr reg(2)    // Interrupt status register
#define lsr reg(5)    // Line status register
#define msr reg(6)    // Modem status register
#define scr reg(7)    // Scratch register

// Transmit control registers
#define thr reg(0)    // Transmit holding register
#define ier reg(1)    // Interrupt enable register
#define fcr reg(2)    // FIFO control register
#define lcr reg(3)    // Line control register
#define mcr reg(4)    // Modem control register
#define ldl reg(0)    // LSB of baud rate
#define mdl reg(1)    // MSB of baud rate

// Enhanced register set visible when lcr == 0xbf

#define trg reg(0)    // trigger levels
#define fctr reg(1)   // extra stuff
#define efr  reg(2)   // more extra stuff (mostly flow control)
#define xon1 reg(4)
#define xon2 reg(5)
#define xoff1 reg(6)
#define xoff2 reg(7)
#define emsr reg(7)

// FCR Register
#define FCR_FIFO_ENABLE   0x01
#define FCR_RXFIFO_RESET  0x02
#define FCR_TXFIFO_RESET  0x04
#define FCR_FIFO_MODE_1    0x08

#define FCR_TXFIFO_TRIG_MASK 0x30
#define FCR_TXFIFO_TRIG_SEL0  0x00
#define FCR_TXFIFO_TRIG_SEL1  0x10
#define FCR_TXFIFO_TRIG_SEL2  0x20
#define FCR_TXFIFO_TRIG_SEL3  0x30

#define FCR_RXFIFO_TRIG_MASK 0xc0
#define FCR_RXFIFO_TRIG_SEL0  0x00
#define FCR_RXFIFO_TRIG_SEL1  0x40
#define FCR_RXFIFO_TRIG_SEL2  0x80
#define FCR_RXFIFO_TRIG_SEL3  0xc0


// FCTR Register
#define FCTR_RS485 0x08
#define FCTR_TRIGTABLE_MASK 0x30
#define FCTR_TRIGTABLE_A  0x00
#define FCTR_TRIGTABLE_B  0x10
#define FCTR_TRIGTABLE_C  0x20
#define FCTR_TRIGTABLE_D  0x30
#define FCTR_SCRATCH_SWAP 0x40
#define FCTR_TRIG_TX_SEL  0x80

// EMSR Register
#define EMSR_SCRATCH_TXCNT 0x01
#define EMSR_SCRATCH_ALT   0x02

// EFR  Register
#define EFR_TXSWFLOW_MASK 0x03
#define EFR_TXSWFLOW_SET2 0x01
#define EFR_TXSWFLOW_SET1 0x02
#define EFR_RXSWFLOW_MASK 0x0c
#define EFR_RXSWFLOW_SET2 0x04
#define EFR_RXSWFLOW_SET1 0x08
#define EFR_EXTENABLE     0x10
#define EFR_SPECSEL       0x20
#define EFR_AUTORTS       0x40
#define EFR_AUTOCTS       0x80

// Interrupt Enable Register
#define IER_RCV    0x01
#define IER_XMT    0x02
#define IER_LS     0x04
#define IER_MS     0x08
#define IER_SLEEP  0x10
#define IER_RCVOFF 0x20

// Line Control Register
#define LCR_WL5 0x00    // Word length
#define LCR_WL6 0x01
#define LCR_WL7 0x02
#define LCR_WL8 0x03
#define LCR_SB1 0x00    // Number of stop bits
#define LCR_SB1_5 0x04  // 1.5 -> only valid with 5 bit words
#define LCR_SB2 0x04
#define LCR_PN  0x00    // Parity mode - none
#define LCR_PE  0x18    // Parity mode - even
#define LCR_PO  0x08    // Parity mode - odd
#define LCR_PM  0x28    // Forced "mark" parity
#define LCR_PS  0x38    // Forced "space" parity
#define LCR_BRK 0x40    // Break on
#define LCR_DL  0x80    // Enable baud rate latch

// Line Status Register
#define LSR_RDR 0x01
#define LSR_RXOVR_ERR 0x02
#define LSR_RXPAR_ERR 0x04
#define LSR_RXFRM_ERR 0x08
#define LSR_RXBRK 0x10
#define LSR_THE 0x20
#define LSR_TSRE 0x40
#define LSR_FIFO_ERR 0x80

#define LSR_RXERR (LSR_RXOVR_ERR|LSR_RXPAR_ERR|LSR_RXFRM_ERR)

// Modem Control Register
#define MCR_DTR 0x01
#define MCR_RTS 0x02
#define MCR_INT 0x08   // Enable interrupts
#define MCR_LOOP 0x10

// Modem Status Register
#define MSR_CTS_DELTA 0x01
#define MSR_DSR_DELTA 0x02
#define MSR_RI_DELTA  0x04
#define MSR_CD_DELTA  0x08
#define MSR_CTS 0x10
#define MSR_DSR 0x20
#define MSR_RI  0x40
#define MSR_CD  0x80

// Interrupt status register
#define ISR_Tx  0x02
#define ISR_Rx  0x04
#define ISR_RxTO  0x0C
#define ISR_XOFF  0x10

static unsigned char select_word_length[] = {
    LCR_WL5,    // 5 bits / word (char)
    LCR_WL6,
    LCR_WL7,
    LCR_WL8
};

static unsigned char select_stop_bits[] = {
    0,
    LCR_SB1,    // 1 stop bit
    LCR_SB1_5,  // 1.5 stop bit
    LCR_SB2     // 2 stop bits
};

static unsigned char select_parity[] = {
    LCR_PN,     // No parity
    LCR_PE,     // Even parity
    LCR_PO,     // Odd parity
    LCR_PM,     // Mark parity
    LCR_PS,     // Space parity
};

#endif // CYGONCE_16850_SERIAL_H
