/***************************************************************/
/*       aiopic.h                                              */
/*                                                             */
/*       Include file for aiopic.c                             */
/***************************************************************/

#ifndef _AIOPIC_H
#define _AIOPIC_H

// don't use fields starting with _, since they will generate
// incorrect accesses.

struct tAiopic
{
  volatile unsigned short data[8]          __attribute__((packed));
  volatile unsigned short rxFifoCnt[8]     __attribute__((packed));
  volatile unsigned short chanStatus[8]    __attribute__((packed));
  volatile unsigned  char _intID[8]        __attribute__((packed));
  
  volatile unsigned  char _command         __attribute__((packed));
  volatile unsigned  char _intChan         __attribute__((packed));
  
  volatile unsigned  char _intMask         __attribute__((packed));
  volatile unsigned  char _reserved        __attribute__((packed));
  
  volatile unsigned short indexAddr         __attribute__((packed));
  volatile unsigned short indexData         __attribute__((packed));
  unsigned char _unused[0x1000-0x40]        __attribute__((packed));
  
  // offset 0x1000 from the start of this struct

  volatile unsigned short __dataHigh[8]     __attribute__((packed));
  volatile unsigned short __txFifoCnt[8]    __attribute__((packed));
  volatile unsigned short chanStatusHigh[8] __attribute__((packed));
  volatile unsigned  char oddIntID[8]       __attribute__((packed));  // odd chan only
  volatile unsigned  char __command         __attribute__((packed));
  volatile unsigned  char intChan           __attribute__((packed));
  volatile unsigned  char __intMask         __attribute__((packed));
  volatile unsigned  char __reserved        __attribute__((packed));
  volatile unsigned short __indexAddr       __attribute__((packed));
  volatile unsigned short indexDataHigh     __attribute__((packed));
  unsigned char __unused[0x1000-0x40]       __attribute__((packed));

  volatile unsigned short dataLow[8]        __attribute__((packed));
  volatile unsigned short txFifoCnt[8]      __attribute__((packed));
  volatile unsigned short chanStatusLow[8]  __attribute__((packed));
  volatile unsigned  char evenIntID[8]      __attribute__((packed));  // even chan only
  volatile unsigned  char command           __attribute__((packed));
  volatile unsigned  char ___intChan        __attribute__((packed));
  volatile unsigned  char intMask           __attribute__((packed));
  volatile unsigned  char ___reserved       __attribute__((packed));
  volatile unsigned short ___indexAddr      __attribute__((packed));
  volatile unsigned short indexDataLow      __attribute__((packed));
};
typedef struct tAiopic tAiopic;

#define AiopPutIndexedByte(a,i,b) (/*diag_printf("%08x:%04x <- %02x\n",a,i,b),*/ (a)->indexAddr=(i), *((volatile unsigned char*)&((a)->indexDataHigh))=b)
#define AiopPutIndexedShort(a,i,s) (/*diag_printf("%08x:%04x <- %04x\n",a,i,s),*/ (a)->indexAddr=(i), (a)->indexData=s)
#define AiopGetIndexedByte(a,i) ((a)->indexAddr=(i), *((volatile unsigned char*)&((a)->indexDataHigh)))
#define AiopGetIndexedShort(a,i) ((a)->indexAddr=(i), (a)->indexData)

struct tRegVal
{
  unsigned short offset __attribute__((packed));
  unsigned short value  __attribute__((packed));
}__attribute__((packed));
typedef struct tRegVal tRegVal;

#define GetChanReg(c,r) ((c)->r.value)
#define PutChanReg(c,r,v) (AiopPutIndexedShort((c)->aiop,(c)->r.offset,((c)->r.value = v)))
#define SetBitsChanReg(c,r,v) (AiopPutIndexedShort((c)->aiop,(c)->r.offset,((c)->r.value |= v)))
#define ClrBitsChanReg(c,r,v) (AiopPutIndexedShort((c)->aiop,(c)->r.offset,((c)->r.value &= ~v)))

typedef struct
{
  tAiopic *aiop;
  unsigned chan;
  unsigned base;
  unsigned line;
  volatile unsigned short *dataWord;
  volatile unsigned char  *dataByte;
  volatile unsigned short *rxFifoCnt;
  volatile unsigned char  *txFifoCnt;
  volatile unsigned short *status;
  volatile unsigned char  *statusLow;
  volatile unsigned char  *statusHigh;
  volatile unsigned char  *intID;
  tRegVal txControl;
  tRegVal rxControl;
  tRegVal baudDivisor;
  tRegVal txEnables;
  tRegVal txCompare;
  tRegVal txReplace1;
  tRegVal txReplace2;
  unsigned short txPriorityCntIndex;
  unsigned short txPriorityPtrIndex;
  unsigned short txPriorityBufIndex;
  tRegVal R[13];  // this really needs to be better named
}tChan;
  

#define Bit(n) (1<<(n))

#define AiopCommand_ResetAll            Bit(7)
#define AiopCommand_TxOverride          Bit(6)
#define AiopCommand_ResetUart           Bit(5)
#define AiopCommand_ResetTxFifo         Bit(4)
#define AiopCommand_ResetRxFifo         Bit(3)

/* 
 * remember: bytes are swapped in word registers!
 * 
 * That means index address and data are all byte-swapped 
 */


/* status bits that are in low byte of rx data/staus byte pair */

#define AiopRxStatus_RxBreak            Bit(3)
#define AiopRxStatus_RxFrameErr         Bit(2)
#define AiopRxStatus_RxOverrunErr       Bit(1)
#define AiopRxStatus_RxParityErr        Bit(0)
#define AiopRxStatus_Errors (Bit(2)+Bit(1)+Bit(0))

/* channel status register */

#define AiopChanStatus_CTS              Bit(13)
#define AiopChanStatus_DSR              Bit(12)
#define AiopChanStatus_CD               Bit(11)
#define AiopChanStatus_TxFifoEmpty      Bit(10)
#define AiopChanStatus_TxShiftEmpty     Bit(9)
#define AiopChanStatus_RxDataAvail      Bit(8)
#define AiopChanStatus_StatusModeEnable Bit(7)
#define AiopChanStatus_RxFifoOverflow   Bit(5)
#define AiopChanStatus_RxCompMatch2     Bit(4)
#define AiopChanStatus_RxCompMatch1     Bit(3)
#define AiopChanStatus_RxBreak          Bit(2)
#define AiopChanStatus_RxFrameErr       Bit(1)
#define AiopChanStatus_RxParityErr      Bit(0)

#define AiopChanStatus_RxErr (AiopChanStatus_RxParityErr | \
                    AiopChanStatus_RxFrameErr | \
                    AiopChanStatus_RxFifoOverflow | \
                    AiopChanStatus_RxBreak)

/* interrupt ID */

#define AiopIntId_RxFifo      Bit(5)
#define AiopIntId_TxFifo      Bit(4)
#define AiopIntId_RxSpecial   Bit(3)
#define AiopIntId_CD          Bit(2)
#define AiopIntId_CTS         Bit(1)
#define AiopIntId_DSR         Bit(0)
#define AiopIntId_Modem       (AiopIntId_CD|AiopIntId_CTS|AiopIntId_DSR)


#define AiopPrescale 2
// Calculate _twice_ the desired divisor, add 1, then divide by two so
// that we get the closest integer value to the "real" number rather
// than always truncating towards zero.
#define AiopBaudDivisor(baud) ((((2L*AiopClock/(16L*(AiopPrescale+1)))/baud-2L)+1L)/2L)


/************************************************************************
 Tx Control Register Offsets - Indexed - External - Fixed
************************************************************************/
#define _TX_ENBLS  0x8009    /* Tx Processor Enables Register 8 Read / Write */
#define _TXCMP1    0x8809    /* Transmit Compare Value #1     8 Read / Write */
#define _TXCMP2    0x8909    /* Transmit Compare Value #2     8 Read / Write */
#define _TXREP1B1  0x8A09    /* Tx Replace Value #1 - Byte 1  8 Read / Write */
#define _TXREP1B2  0x8B09    /* Tx Replace Value #1 - Byte 2  8 Read / Write */
#define _TXREP2    0x8C09    /* Transmit Replace Value #2     8 Read / Write */

/************************************************************************
Memory Controller Register Offsets - Indexed - External - Fixed
************************************************************************/
#define _RX_FIFO    0x0000    /* Rx FIFO */
#define _TX_FIFO    0x0008    /* Tx FIFO */
#define _RXF_OUTP   0x9009    /* Rx FIFO OUT pointer        16 Read / Write */
#define _RXF_INP    0x9209    /* Rx FIFO IN pointer         16 Read / Write */
#define _TXF_OUTP   0x9409    /* Tx FIFO OUT pointer        8  Read / Write */
#define _TXF_INP    0x9509    /* Tx FIFO IN pointer         8  Read / Write */
#define _TXP_CNT    0x9609    /* Tx Priority Count          8  Read / Write */
#define _TXP_PNTR   0x9709    /* Tx Priority Pointer        8  Read / Write */

#define PRI_PEND    0x80     /* Priority data pending (bit7, Tx pri cnt) */
#define TXFIFO_SIZE 255      /* size of Tx FIFO */
#define RXFIFO_SIZE 1023     /* size of Rx FIFO */

/************************************************************************
Tx Priority Buffer - Indexed - External - Fixed
************************************************************************/
#define _TXP_BUF    0xC009    /* Tx Priority Buffer  32  Bytes   Read / Write */
#define TXP_SIZE    0x20     /* 32 bytes */

/************************************************************************
Channel Register Offsets - Indexed - Internal - Fixed
************************************************************************/

#define _TX_CTRL    0xF00F    /* Transmit Control               16  Write */
#define _RX_CTRL    0xF20F    /* Receive Control                 8  Write */
#define _BAUD       0xF40F    /* Baud Rate                      16  Write */
#define _CLK_PRE    0xF60F    /* Clock Prescaler                 8  Write */

#define AiopRxCntrl_RtsFlow               Bit(14)
#define AiopRxCntrl_ProcEnable            Bit(13)
#define AiopRxCntrl_RxFifoTrigShift       11
#define AiopRxCntrl_RxFifoTrigMask        (3<<11)
#define AiopRxCntrl_SpecIntEnable         Bit(10)
#define AiopRxCntrl_RxIntEnable           Bit(9)
#define AiopRxCntrl_ModemIntEnable        Bit(8)

#define AiopTxCntrl_BreakOn      Bit(4)
#define AiopTxCntrl_LocalLoop    Bit(3)
#define AiopTxCntrl_DTR          Bit(2)
#define AiopTxCntrl_RTS          Bit(1)
#define AiopTxCntrl_TxEnable     Bit(0)
#define AiopTxCntrl_CtsFlow      Bit(15)
#define AiopTxCntrl_RtsToggle    Bit(14)
#define AiopTxCntrl_TxIntEnable  Bit(12)
#define AiopTxCntrl_2stopBits    Bit(11)
#define AiopTxCntrl_ParityEnable Bit(10)
#define AiopTxCntrl_EvenParity   Bit(9)
#define AiopTxCntrl_8dataBits    Bit(8)


#define sEnRxStatusMode(ch)  *(ch->status) = AiopChanStatus_StatusModeEnable
#define sDisRxStatusMode(ch) *(ch->status) = 0

#define sEnTransmit(ch) SetBitsChanReg(ch,txControl,AiopTxCntrl_TxEnable)
#define sDisTransmit(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_TxEnable)

#define sSetRTS(ch) SetBitsChanReg(ch,txControl,AiopTxCntrl_RTS)
#define sClrRTS(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_RTS)

#define sSetDTR(ch) SetBitsChanReg(ch,txControl,AiopTxCntrl_DTR)
#define sClrDTR(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_DTR)

#define sSetData8(ch) SetBitsChanReg(ch,txControl,AiopTxCntrl_8dataBits)
#define sSetData7(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_8dataBits)

#define sEnRxProcessor(ch)   SetBitsChanReg(ch,rxControl,AiopRxCntrl_ProcEnable)
#define sDisRxProcessor(ch)  ClrBitsChanReg(ch,rxControl,AiopRxCntrl_ProcEnable)

#define sEnRxModemInt(ch)   SetBitsChanReg(ch,rxControl,AiopRxCntrl_ModemIntEnable)
#define sDisRxModemInt(ch)  ClrBitsChanReg(ch,rxControl,AiopRxCntrl_ModemIntEnable)

#define sEnRxCharInt(ch)   SetBitsChanReg(ch,rxControl,AiopRxCntrl_SpecIntEnable|AiopRxCntrl_RxIntEnable|(1<<AiopRxCntrl_RxFifoTrigShift))
#define sDisRxCharInt(ch)  ClrBitsChanReg(ch,rxControl,AiopRxCntrl_SpecIntEnable|AiopRxCntrl_RxIntEnable|AiopRxCntrl_RxFifoTrigMask)

#define sEnRxSpecInt(ch)   SetBitsChanReg(ch,rxControl,AiopRxCntrl_SpecIntEnable)
#define sDisRxSpecInt(ch)  ClrBitsChanReg(ch,rxControl,AiopRxCntrl_SpecIntEnable)

#define sEnRxFifo(ch) PutChanReg(ch,R[12],0x080a)
#define sDisRxFifo(ch) PutChanReg(ch,R[12],0x0a0a)

#define sGetRxCnt(ch) swapshort(*(ch->rxFifoCnt))
#define sGetTxCnt(ch) *(ch->txFifoCnt)

#define sEnRTSToggle(ch)  SetBitsChanReg(ch,txControl,AiopTxCntrl_RtsToggle)
#define sDisRTSToggle(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_RtsToggle)

void    sSetBaud(tChan *ch, int baud);

#define sSetBreak(ch) SetBitsChanReg(ch,txControl,AiopTxCntrl_BreakOn)
#define sClrBreak(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_BreakOn)

#define sEnCTSFlowCtl(ch)  SetBitsChanReg(ch,txControl,AiopTxCntrl_CtsFlow)
#define sDisCTSFlowCtl(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_CtsFlow)

#define sEnRTSFlowCtl(ch)  SetBitsChanReg(ch,rxControl,AiopRxCntrl_RtsFlow)
#define sDisRTSFlowCtl(ch) ClrBitsChanReg(ch,rxControl,AiopRxCntrl_RtsFlow)

#define sEnParity(ch)  SetBitsChanReg(ch,txControl,AiopTxCntrl_ParityEnable)
#define sDisParity(ch) ClrBitsChanReg(ch,txControl,AiopTxCntrl_ParityEnable)
#define sSetEvenParity(ch) SetBitsChanReg(ch,txControl,AiopTxCntrl_EvenParity)
#define sSetOddParity(ch)  ClrBitsChanReg(ch,txControl,AiopTxCntrl_EvenParity)

#define sSetStop2(ch)  SetBitsChanReg(ch,txControl,AiopTxCntrl_2stopBits)
#define sSetStop1(ch)  ClrBitsChanReg(ch,txControl,AiopTxCntrl_2stopBits)

#define sGetChanStatus(ch) (*(ch->status))
#define sGetChanIntID(ch) (*(ch->intID))

#define sTxAvail(ch) (sGetTxCnt(ch) < 254)
#define sRxAvail(ch) (*(ch->status) & AiopChanStatus_RxDataAvail)

#define sEnTxSoftFlowCtl(ch) \
do {\
   ch->R[1].value &= 0x00ff;\
   ch->R[1].value |= 0xc500;\
   AiopPutIndexedShort(ch->aiop,ch->R[1].offset,ch->R[1].value);\
   } while (0)
#define sDisTxSoftFlowCtl(ch) \
do {\
   ch->R[1].value &= 0x00ff;\
   ch->R[1].value |= 0x8a00;\
   AiopPutIndexedShort(ch->aiop,ch->R[1].offset,ch->R[1].value);\
   } while (0)

#define sEnIXANY(ch) \
do {\
   ch->R[3].value &= 0x00ff;\
   ch->R[3].value |= 0x2100;\
   AiopPutIndexedShort(ch->aiop,ch->R[3].offset,ch->R[3].value);\
   } while (0)
#define sDisIXANY(ch) \
do {\
   ch->R[3].value &= 0x00ff;\
   ch->R[3].value |= 0x8600;\
   AiopPutIndexedShort(ch->aiop,ch->R[3].offset,ch->R[3].value);\
   } while (0)

#define sSetTxXOFFChar(ch, c) \
do {\
   ch->R[1].value &= 0xff00;\
   ch->R[1].value |= c;\
   AiopPutIndexedShort(ch->aiop,ch->R[1].offset,ch->R[1].value);\
    /* diag_printf("%d:xoff=%02x\n",ch->chan,c); */  \
   } while (0)
#define sSetTxXONChar(ch, c) \
do {\
   ch->R[2].value &= 0xff00;\
   ch->R[2].value |= c;\
   AiopPutIndexedShort(ch->aiop,ch->R[2].offset,ch->R[2].value);\
   /* diag_printf("%d:xon=%02x\n",ch->chan,c); */ \
   } while (0)

#define sClrTxXOFF(ch) \
do { \
   (ch)->aiop->command = (ch)->chan | AiopCommand_TxOverride; \
   (ch)->aiop->command = (ch)->chan; \
   } while (0)

#define sEnRxIntCompare1(ch) \
do { \
   ch->R[6].value &= 0xff00;\
   ch->R[6].value |= (ch->R[2].value & 0x00ff);\
   AiopPutIndexedShort(ch->aiop,ch->R[6].offset,ch->R[6].value);\
   ch->R[6].value &= 0x00ff;\
   ch->R[6].value |= 0xc400;\
   AiopPutIndexedShort(ch->aiop,ch->R[6].offset,ch->R[6].value);\
   ch->R[9].value &= 0x00ff;\
   ch->R[9].value |= 0x8600;\
   AiopPutIndexedShort(ch->aiop,ch->R[9].offset,ch->R[9].value);\
   ch->R[8].value &= 0xff00;\
   ch->R[8].value |= 0x11;\
   AiopPutIndexedShort(ch->aiop,ch->R[8].offset,ch->R[8].value);\
   } while (0)
#define sDisRxCompare1(ch) \
do { \
   ch->R[6].value &= 0x00ff;\
   ch->R[6].value |= 0x8200;\
   AiopPutIndexedShort(ch->aiop,ch->R[6].offset,ch->R[6].value);\
   /* diag_printf("%d:xon=%02x\n",ch->chan,c); */ \
   ch->R[9].value &= 0x00ff;\
   ch->R[9].value |= 0x8600;\
   AiopPutIndexedShort(ch->aiop,ch->R[9].offset,ch->R[9].value);\
   ch->R[8].value &= 0xff00;\
   ch->R[8].value |= 0x81;\
   AiopPutIndexedShort(ch->aiop,ch->R[8].offset,ch->R[8].value);\
   } while (0)

/*  Prototypes */
bool aiopic_serial_putc(serial_channel *chan, unsigned char c);
unsigned char aiopic_serial_getc(serial_channel *chan);
bool aiopic_serial_set_config(serial_channel *chan, cyg_serial_info_t *config);
void aiopic_serial_start_xmit(serial_channel *chan);
void aiopic_serial_stop_xmit(serial_channel *chan);
Cyg_ErrNo aiopic_serial_get_config_handler(serial_channel *chan, cyg_uint32 key, void *xbuf, cyg_uint32 *len);
Cyg_ErrNo aiopic_serial_set_config_handler(serial_channel *chan, cyg_uint32 key, const void *xbuf, cyg_uint32 *len);
void aiopic_serial_start_recv(serial_channel *chan);
void aiopic_serial_stop_recv(serial_channel *chan);
void aiopic_serial_flush_input(serial_channel *chan);
void aiopic_serial_flush_output(serial_channel *chan);
int aiopic_serial_get_tx_count(serial_channel *chan);
int aiopic_serial_get_rx_count(serial_channel *chan);
bool aiopic_serial_init(struct cyg_devtab_entry *tab);
Cyg_ErrNo aiopic_serial_lookup(struct cyg_devtab_entry **tab, struct cyg_devtab_entry *sub_tab, const char *name);

#endif // _AIOPIC_H
