/*****************************************************************/
/*   dmrts.c                                                     */
/*                                                               */
/*   Top level function for Nserial driver for DeviceMaster.     */
/*   The DeviceMaster RTS either uses the AIOPIC for it's serial */
/*   ports or the S3C4530 UART.  This driver detects which at    */
/*   run time and calls the correct handler functions.           */
/*****************************************************************/

#include <pkgconf/system.h>
#include <pkgconf/io_nserial.h>
#include <pkgconf/io.h>
#include <cyg/io/io.h>
#include <cyg/hal/hal_intr.h>
#include <cyg/io/devtab.h>
#include <cyg/io/serial.h>
#include <cyg/io/serialio.h>
#include <cyg/infra/diag.h>
#include "dmrts.h"
#include "aiopic.h"
#include "uart4530.h"

//  Auto detect which type of device we are
extern volatile unsigned int SYSCFG;
static int is4530 = 1;

#if 1
#define debug_printf(args...) diag_printf(args)
#else
#define debug_printf(args...) /* noop */
#endif

//  Define the serial handler functions

serial_channel serialChannel[32];

//  Create the eCos devices /dev/ser0 - /dev/ser31
#define DevTabEntry(n) \
   DEVTAB_ENTRY(dmrts_serial_io##n,\
   "/dev/ser" #n,\
   0,\
   &cyg_io_serial_devio,\
   dmrts_serial_init,\
   dmrts_serial_lookup,\
   &serialChannel[n])

DevTabEntry(0);
DevTabEntry(1);
DevTabEntry(2);
DevTabEntry(3);
DevTabEntry(4);
DevTabEntry(5);
DevTabEntry(6);
DevTabEntry(7);
DevTabEntry(8);
DevTabEntry(9);
DevTabEntry(10);
DevTabEntry(11);
DevTabEntry(12);
DevTabEntry(13);
DevTabEntry(14);
DevTabEntry(15);
DevTabEntry(16);
DevTabEntry(17);
DevTabEntry(18);
DevTabEntry(19);
DevTabEntry(20);
DevTabEntry(21);
DevTabEntry(22);
DevTabEntry(23);
DevTabEntry(24);
DevTabEntry(25);
DevTabEntry(26);
DevTabEntry(27);
DevTabEntry(28);
DevTabEntry(29);
DevTabEntry(30);
DevTabEntry(31);

unsigned long select_baud[] = 
{
  0,
  50,
  75,
  110,
  134,
  150,
  200,
  300,
  600,
  1200,
  1800,
  2400,
  3600,
  4800,
  7200,
  9600,
  14400,
  19200,
  38400,
  57600,
  115200,
  230400,
  460800
};


bool dmrts_serial_init(struct cyg_devtab_entry *tab)
{
    static int init;

    if (!init) {
	is4530 = SYSCFG & Bit(27);
	init = 1;
    }

    if (is4530)
	return uart4530_serial_init(tab);
    else
	return aiopic_serial_init(tab);
}

Cyg_ErrNo dmrts_serial_lookup(struct cyg_devtab_entry **tab, 
			      struct cyg_devtab_entry *sub_tab,
			      const char *name)
{
    if (is4530)
	debug_printf("4530 UART detected.  Dev Name:  %s \n", (*tab)->name);
    else
	debug_printf("AIOPIC detected.  Dev Name:  %s \n", (*tab)->name);
	
    if (is4530)
	return uart4530_serial_lookup(tab, sub_tab, name);
    else
	return aiopic_serial_lookup(tab, sub_tab, name);
}

void timerSetup(unsigned period)
{
    ks32c5000_timer.tmod &= ~TimerModeEnable1;                   // disable timer
    ks32c5000_timer.tdata1 = period;                             // set period
    ks32c5000_timer.tmod |= TimerModeEnable1 + TimerModeToggle1; // enable timer
}
