/***************************************************************/
/*       uart4530.h                                              */
/*                                                             */
/*       Include file for uart4530.c                             */
/***************************************************************/

#ifndef _UART4530_H
#define _UART4530_H

/*  Debugging macros */
#define UART4530_STATUS  /*  General status messages */
//#define UART4530_MC      /*  Modem Control Lines */
//#define UART4530_FC      /*  Flow Control */
//#define UART4530_DRV     /*  Driver Calls */

#ifdef UART4530_STATUS
#  define DS(fmt, args...) diag_printf("DS: " fmt, ## args)
#else
#  define DS(fmt, args...) 
#endif

#ifdef UART4530_MC
#  define DMC(fmt, args...) diag_printf("DMC: " fmt, ## args)
#else
#  define DMC(fmt, args...) 
#endif

#ifdef UART4530_FC
#  define DFC(fmt, args...) diag_printf("DFC: " fmt, ## args)
#else
#  define DFC(fmt, args...) 
#endif

#ifdef UART4530_DRV
#  define DRV(fmt, args...) diag_printf("DRV: " fmt, ## args)
#else
#  define DRV(fmt, args...) 
#endif

#define CLEAR 0
#define SET 1

#define DISABLE 0
#define ENABLE 1

#define FLOW_CONTROL_NONE     0
#define FLOW_CONTROL_HARDWARE 1
#define FLOW_CONTROL_SOFTWARE 2

#define  XON  1
#define  XOFF 2

#define Bit(n) (1<<(n))

/*  Modem control lines that are accessed as IO pins */
#define DCD_BIT Bit(5)
#define CTS_BIT Bit(6)
#define RTS_BIT Bit(7)
#define RI_BIT  Bit(10)
#define DTR_BIT Bit(25)

/*  AIOPIC UART status bits, must convert UART status into these AIOPIC status bits */
#define Aiop_RxBreak            Bit(3)
#define Aiop_RxFrameErr         Bit(2)
#define Aiop_RxOverrunErr       Bit(1)
#define Aiop_RxParityErr        Bit(0)

/*  Struct representing the S3C4530 UART control registers */
typedef struct
{
  reg ucon;    // UART Control
  reg ustat;   // UART Status
  reg inten;   // UART Interupt enable
  reg utxbuf;  // UART Tx Buffer
  reg urxbuf;  // UART Rx Buffer
  reg brdiv;   // UART Baud Rate Divisor
  reg conchar1;// UART Control Chars
  reg conchar2;//UART Control Chars
} tS3C4530Uart;

//  S3C4530 DMA
typedef struct
{
  reg control;    // DMA control
  reg saddr;      // DMA source address
  reg daddr;      // DMA destination address
  reg xfercount;  // DMA transfer count
} tS3C4530Dma;

#define  S3C4530UartStatusRxDataAvail     Bit(0)
#define  S3C4530UartStatusRxBreak         Bit(1)
#define  S3C4530UartStatusRxFrameError    Bit(2)
#define  S3C4530UartStatusRxParityError   Bit(3)
#define  S3C4530UartStatusRxOverrunError  Bit(4)
#define  S3C4530UartStatusCntrlCharDet    Bit(5)
#define  S3C4530UartStatusDCD             Bit(6)
#define  S3C4530UartStatusRxFifoTrig      Bit(7)
#define  S3C4530UartStatusRxFifoEmpty     Bit(8)
#define  S3C4530UartStatusRxFifoFull      Bit(9)
#define  S3C4530UartStatusRxFifoOverrun   Bit(10)
#define  S3C4530UartStatusRxIdle          Bit(11)
#define  S3C4530UartStatusRxTimeout       Bit(12)

#define  S3C4530UartStatusDSR             Bit(14)
#define  S3C4530UartStatusCTS             Bit(15)

#define  S3C4530UartStatusCTSEvent        Bit(16)
#define  S3C4530UartStatusTxComplete      Bit(17)
#define  S3C4530UartStatusTxHoldRegEmpty  Bit(18)

#define  S3C4530UartStatusTxFifoEmpty     Bit(19)
#define  S3C4530UartStatusTxFifoFull      Bit(20)


#define S3C4530UartControlTxModeMask  0x3
#define S3C4530UartControlTxModeDis   0x0
#define S3C4530UartControlTxModeInt   0x1
#define S3C4530UartControlTxModeGDMA0 0x2
#define S3C4530UartControlTxModeGDMA1 0x3

#define S3C4530UartControlRxModeMask  (0x3<<2)
#define S3C4530UartControlRxModeDis   (0x0<<2)
#define S3C4530UartControlRxModeInt   (0x1<<2)
#define S3C4530UartControlRxModeGDMA0 (0x2<<2)
#define S3C4530UartControlRxModeGDMA1 (0x3<<2)

#define S3C4530UartControlSendBreak   Bit(4)
#define S3C4530UartControlClockExt    Bit(5)
#define S3C4530UartControlAutoBaud    Bit(6)
#define S3C4530UartControlLoopback    Bit(7)

#define S3C4530UartControlParityMask  (0x7<<8)
#define S3C4530UartControlParityNone  (0x0<<8)
#define S3C4530UartControlParityOdd   (0x4<<8)
#define S3C4530UartControlParityEven  (0x5<<8)
#define S3C4530UartControlParityMark  (0x6<<8)
#define S3C4530UartControlParitySpace (0x7<<8)

#define S3C4530UartControlStop2       Bit(11)

#define S3C4530UartControlWordLenMask (0x3<<12)
#define S3C4530UartControlWordLen5    (0x0<<12)
#define S3C4530UartControlWordLen6    (0x1<<12)
#define S3C4530UartControlWordLen7    (0x2<<12)
#define S3C4530UartControlWordLen8    (0x3<<12)

#define S3C4530UartControlEnableIR    Bit(14)
#define S3C4530UartControlTxFifoEn    Bit(16)
#define S3C4530UartControlRxFifoEn    Bit(17)
#define S3C4530UartControlTxFifoRst   Bit(18)
#define S3C4530UartControlRxFifoRst   Bit(19)

#define S3C4530UartControlTxFifoTrigMask (0x3<<20)
#define S3C4530UartControlTxFifoTrig1    (0x0<<20)
#define S3C4530UartControlTxFifoTrig8    (0x1<<20)
#define S3C4530UartControlTxFifoTrig18   (0x2<<20)
#define S3C4530UartControlTxFifoTrig28   (0x3<<20)

#define S3C4530UartControlRxFifoTrigMask (0x3<<22)
#define S3C4530UartControlRxFifoTrig1    (0x0<<22)
#define S3C4530UartControlRxFifoTrig8    (0x1<<22)
#define S3C4530UartControlRxFifoTrig18   (0x2<<22)
#define S3C4530UartControlRxFifoTrig28   (0x3<<22)

#define S3C4530UartControlDTR         Bit(24)
#define S3C4530UartControlRTS         Bit(25)

#define S3C4530UartControlHwFlowEn    Bit(28)
#define S3C4530UartControlSwFlowEn    Bit(29)

// Interrupt Enable
#define  S3C4530UartIntEnaRxData     Bit(0)
#define  S3C4530UartIntEnaBreak      Bit(1)
#define  S3C4530UartIntEnaFrameErr   Bit(2)
#define  S3C4530UartIntEnaParityErr  Bit(3)
#define  S3C4530UartIntEnaOverrun    Bit(4)
#define  S3C4530UartIntEnaCtrlChar   Bit(5)
#define  S3C4530UartIntEnaDCD        Bit(6)
#define  S3C4530UartIntEnaRxFifoTrig Bit(7)
#define  S3C4530UartIntEnaRxFifoOver Bit(10)
#define  S3C4530UartIntEnaRxTimeout  Bit(12)
#define  S3C4530UartIntEnaCTS        Bit(16)
#define  S3C4530UartIntEnaTxEmpty    Bit(18)

/*  Prototypes */
bool uart4530_serial_init(struct cyg_devtab_entry *tab);
bool uart4530_putc(serial_channel *priv, unsigned char c);
unsigned char uart4530_getc(serial_channel *priv);
bool uart4530_set_config(serial_channel *priv, cyg_serial_info_t *config);
void uart4530_start_xmit(serial_channel *priv);
void uart4530_stop_xmit(serial_channel *priv);
void uart4530_start_recv(serial_channel *priv);
void uart4530_stop_recv(serial_channel *priv);
Cyg_ErrNo uart4530_get_config_handler(serial_channel *priv, cyg_uint32 key, void *buf, cyg_uint32 *len);
Cyg_ErrNo uart4530_set_config_handler(serial_channel *priv, cyg_uint32 key, const void *buf, cyg_uint32 *len);
void uart4530_flush_input(serial_channel *priv);
void uart4530_flush_output(serial_channel *priv);
int uart4530_get_tx_count(serial_channel *priv);
int uart4530_get_rx_count(serial_channel *priv);
Cyg_ErrNo uart4530_serial_lookup(struct cyg_devtab_entry **tab, 
				 struct cyg_devtab_entry *sub_tab,
				 const char *name);

#endif // _UART4530_H
