/******************************************************************
 *
 *  unity.c
 *
 *       Structs and functions for the serial driver for       
 *       Comtrol DeviceMaster RTS devices that have UNITY asics.  
 ******************************************************************/

#include <pkgconf/system.h>
#include <pkgconf/io_nserial.h>
#include <pkgconf/io.h>
#include <cyg/io/io.h>
#include <cyg/hal/hal_intr.h>
#include <cyg/io/devtab.h>
#include <cyg/io/serial.h>
#include <cyg/io/serialio.h>
#include <cyg/infra/diag.h>
#include <stdlib.h>
#include <string.h>
#include <cyg/kernel/kapi.h>

#include <cyg/hal/hal_tables.h>
//#include <cyg/hal/plf_stub.h>
#include <cyg/hal/iic.h>

#include "unity.h"
#include "dm2.h"
#include "ident.h"
tIdent identity;

//krw: 12/13/06 - for timestamping
cyg_resolution_t mCCR;		//call cyg_clock_get_resolution(cyg_real_time_clock()) to use


//extern volatile unsigned short *RingIndicatorRegister;

#define CYGNUM_HAL_INTERRUPT_Ext5	5	// unity uart interrupt - 5

#define restore_int RESTORE_IRQ			// restore interrupt - 4
#define RESTORE_DATA 0				// defines to access pio config regs
#define RESTORE_DIRECTION 1
#define RESTORE_MASK 2
#define RESTORE_EDGE 3

/******************************************************************************
 *  Nios II specific code  	(this was borrowed from the oc eth driver)
 *  These are slightly modified from altera's io.h
 *  When accessing registers, these must be used to avoid weird caching issues
 ******************************************************************************/
#define RD_32_DIRECT( ADDR        )  __builtin_ldwio(   (void*)(ADDR) )
#define WR_32_DIRECT( ADDR, DATA  )  __builtin_stwio(  (void*)(ADDR), (DATA) )

// this is used to access pio configuration register
static int *restore_regs = (int *) RESTORE_BASE;	// Base I/O address of restore pio - 0x2001840
#define write_restore_reg(reg,data)  WR_32_DIRECT(&restore_regs[reg],data)
#define read_restore_reg(reg)        RD_32_DIRECT(&restore_regs[reg])

#define NumUnity 2
#define NumChan (NumUnity*16)

static int numChanPresent = 0;
static int scanPeriod = 10;

typedef struct
{
  tChan          chan;
  cyg_serial_error_count_t error_count;
  bool           receive_stopped;
  bool           sentXoff;
  unsigned char  *in_buf;
  unsigned char  *out_buf;

  unsigned       interCharTimeout;
  unsigned       idleCount;
  bool           timeoutFlag;
  bool           gotData;
  unsigned       rxFifoLast;
  bool           fakeXoff;			//used with 'set xoff'
} tUnityChanInfo;


static tUnity unity[MAX_NUMBEROF_UNITIES];
static tUnityChanInfo unityChanInfo[MAX_NUMBEROF_PORTS];

/*  Alarm variables */
static cyg_handle_t unityAlarmHandle;
static cyg_alarm unityAlarmObj;
static cyg_alarm_t unity_serial_poll;
static unsigned long alarmIndex = 0;

static cyg_uint32 unity_serial_aiopic_ISR(cyg_vector_t vector, cyg_addrword_t data);
static void       unity_serial_aiopic_DSR(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data);

static cyg_uint32 unity_restore_ISR(cyg_vector_t vector, cyg_addrword_t data);
static void       unity_restore_DSR(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data);

static struct
  {
    cyg_mutex_t lock;
    cyg_cond_t  wait; 
  }sync;


/*  Defines used in the Receive Processor Compare function */
#define COMPARE_SAVE_BYTE    0
#define COMPARE_REPLACE_BYTE 1
#define COMPARE_DISCARD_BYTE 2

//============ 
/*  Use of a global here may be non-recursive */
static unsigned short sval;
static unsigned long lval;

/*  Macros to set/clear a bit in a PCI memory word (short - 16 bits) */
#define SETBITW(addr, bit)  \
do { \
  sval = READW(addr);  \
  WRITEW(sval | bit, addr); \
} while (0)

#define CLEARBITW(addr, bit)  \
do { \
  sval = READW(addr);  \
  WRITEW(sval & ~bit, addr); \
} while (0)

/*  Macros to set/clear a bit in a PCI memory double word (long - 32 bits) */
#define SETBITL(addr, bit)  \
do { \
  lval = READL(addr);  \
  WRITEL(lval | bit, addr); \
} while (0)

#define CLEARBITL(addr, bit)  \
do { \
  lval = READL(addr);  \
  WRITEL(lval & ~bit, addr); \
} while (0)


/****************  Channel microcode ***************8*/
#define UCODESIZE 0x3F

static unsigned char uCode[UCODESIZE] = {
	0xF6, 0x8B, 0x9D, 0xC4,     /*  0x00 */
	0x13, 0xC4, 0x11, 0x98,     /*  0x04 */
	0x20, 0x0A, 0x21, 0x0A,     /*  0x08 */
	0x8D, 0xFA, 0x86, 0x01,     /*  0x0c */
	0x40, 0x11, 0x19, 0x0A,     /*  0x10 */
	0x00, 0x40, 0x13, 0x19,     /*  0x14 */
	0x0A, 0xFA, 0x83, 0x01,     /*  0x18 */
	0x0A, 0x00, 0x0A, 0x41,     /*  0x1C */
	0xFF, 0x89, 0xC2, 0x66,     /*  0x20 */
	0x86, 0x81, 0x91, 0x40,     /*  0x24 */
	0x65, 0x8E, 0x89, 0xC2,     /*  0x28 */
	0x66, 0x86, 0x81, 0x88,     /*  0x2C */
	0x40, 0x65, 0x85, 0x84,     /*  0x30 */
	0x00, 0x82, 0x0A, 0x08,     /*  0x34 */
	0x0A, 0x0A, 0x0A, 0x0A,     /*  0x38 */
	0x0A, 0x08, 0x0A            /*  0x3C */
};


//=========================
void initUnityChan(tUnity *a, int chanNum, tChan *ChP, int lineNum)
{

  memset(ChP, 0, sizeof(tChan));
  if (chanNum < 16) {
    ChP->pBaseRegister = (void *)(UNITY_BASE_ADDR + (chanNum * 0x40));
    ChP->ucodeAddr     = (void *)(UNITY_BASE_ADDR + UCODE_SRAM_CORE0 + (chanNum * 0x80));
  }
  else {
    ChP->pBaseRegister = a->pBasePciMemory + 0x1000 + ((chanNum - 16) * 0x40);
    ChP->ucodeAddr     = a->pBasePciMemory + 0x1000 + UCODE_SRAM_CORE0 + ((chanNum - 16) * 0x80);
  }

  ChP->chan = chanNum;
  ChP->line = lineNum;
  ChP->unity = a;

  /*  Init hardware to good defaults */
  uSetBaudDivisor(ChP, UnityBaudDivisor(9600));

  SetFlowControl(ChP, FLOW_CONTROL_NONE);
  
  // reset Uart
  uChannelReset(ChP);
		  
  uFlushTxFifo(ChP);
  uFlushRxFifo(ChP);
  uFlushPriorityFifo(ChP);
  
  uLoadUcode(ChP);	//load chan ucode
  uStartRxProcessor(ChP);
  uEnTransmit(ChP);
  uEnRxFIFO(ChP);
  uSetData8(ChP);                         // 8N1
  uDisParity(ChP);
  uSetStop1(ChP);
  uClrDtr(ChP);
  uClrRts(ChP);
  uSetTxXONChar(ChP,0x11);
  uSetTxXOFFChar(ChP,0x13);

uSetInterfaceMode(ChP, 0x1);	//rs232

}


//=====================	// modified from aiopic.c
void initUnity(int unityNum)
{
  tUnity *CtlP;

  CtlP = (tUnity *)(&unity[unityNum]);
  memset(CtlP, 0, sizeof(tUnity));
  CtlP->pBasePciMemory = (void *)(UNITY_BASE_ADDR + (0x1000*unityNum));

  if(identity.modelId == 5002405)
    {
    CtlP->numPorts = 1;
    CtlP->has_2nd_asic = 0;
    }

  uGlobalReset(CtlP);
  uSetClockPrescale(CtlP, DEFAULT_PRESCALE);

  // FIXME  ??  this is CORE 0  additional ports ??
  WRITEW( 0x00ff, ((void *)(UNITY_BASE_ADDR + (0x1000 * unityNum) + CHANNEL_INT_MASK)) );
}

//===========================	
static void initUnitiesAndChannels(void)
{
  int i;

  for (i=0; i<numChanPresent%16; ++i)
    initUnity(i);
  
  for (i=0; i<numChanPresent; ++i)
    {
      tChan *ChP = &unityChanInfo[i].chan;
      initUnityChan(&(unity[i/16]),i%16,ChP,i);
    }
}

//============================================================


#define TRACE_BUFFER_ENABLED 0

#if TRACE_BUFFER_ENABLED
#define TraceBufferSize  1024
static unsigned traceBuffer[TraceBufferSize] = {0};
static unsigned traceBufferIndex = 0;

void putTrace(unsigned char port, unsigned char v, unsigned short p)
{
  traceBuffer[traceBufferIndex] = (port<<24) | (v<<16) | p;
  ++traceBufferIndex;
  traceBufferIndex &= (TraceBufferSize-1);
  traceBuffer[traceBufferIndex] = 0;
}
#else	//trace not on
#define putTrace(port,v,p) /* noop */
#endif

#define TRACE_PINS_ENABLED 0

#if TRACE_PINS_ENABLED
extern volatile unsigned PioData;
#define TracePinOn(n)  PioData |= Bit(n)
#define TracePinOff(n)  PioData &= ~Bit(n)
#else
#define TracePinOn(n)  /* noop */
#define TracePinOff(n) /* noop */
#endif

#define min(a,b) ((a)<(b)?(a):(b))

//==============================================================
static void handleTransmitBytes(serial_channel *chan)
{
  tChan *ch = &(((tUnityChanInfo*)(chan->dev_priv))->chan);
  cbuf_t *cbuf = &chan->out_cbuf;
  int txFifoSpace;
  int avail,count,size;
  unsigned char *src;

  if (cbuf->get == cbuf->put)
    {
      if (cbuf->waiting && uGetTxCnt(ch)==0);
        {
          cbuf->waiting = false;
          cyg_drv_cond_broadcast(&cbuf->wait);
        }
      return;
    }
  //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x20,cyg_current_time()&0xffff);
  
  while (cbuf->get != cbuf->put)
    {
      //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x21,cbuf->put);
      //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x22,cbuf->get);
      
      if (cbuf->put > cbuf->get)
        avail = cbuf->put - cbuf->get;
      else
        avail = cbuf->len - cbuf->get;
      //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x23,avail);

      txFifoSpace = 511 - uGetTxCnt(ch);
      //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x24,txFifoSpace);
      
      if (txFifoSpace <= 0)
        break;
      
      size = count = min(txFifoSpace,avail);
      src = cbuf->data+cbuf->get;
      //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x25,count);
      //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x26,*src);

#if 1
      // need hword aligned src pointer
      if (((int)src) & 1)
        {
          uXmitByte(ch, *src);
          src++;
          --count;
        }
      // xfer 16 bits at a time
      while (count > 1)
        {
          uXmitWord(ch, *((unsigned short*)src) );
          src += 2;
          count -= 2;
        }
      // finish up last byte if there is one
      if (count)
        {
          uXmitByte(ch, *src);
          src++;
          --count;
        }
#else
      while (count)
        {
          uXmitByte(ch, *src);
          src++;
          --count;
        }
#endif
      //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x27,src[-1]);

      cbuf->get += size;
      
      if (cbuf->get == cbuf->len) 
        cbuf->get = 0;
      
      if (size != avail)
        break;
    }	//while pointers not equal

  if (cbuf->get == cbuf->put)
    {
      (chan->funs->stop_xmit)(chan);  // Done with transmit
      //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x28,0);
    }
  
  if (cbuf->waiting) 
    {
      // See if there is now enough room to restart writer
      int space = (cbuf->len + cbuf->get) - cbuf->put;
      if (space > cbuf->len) space -= cbuf->len;
      if (space >= cbuf->low_water) 
        {
          cbuf->waiting = false;
          cyg_drv_cond_broadcast(&cbuf->wait);
        }
    }
}

//=======================

#define FlowLowWater  64
#define FlowHighWater 512

//=======================
static void handleReceiveBytes(serial_channel *chan)
{
  tUnityChanInfo *unity_chan = (tUnityChanInfo*)chan->dev_priv;
  tChan *ch = &unity_chan->chan;

  cbuf_t *cbuf = &chan->in_cbuf;
  int nextPut, rxCount;
  unsigned short chanStatus, statusAndRxByte;
  unsigned char *dst, rxByte;
  int avail,count;
  int wakeup = 0;

  // is data being handled by callback instead of "normal" method?
  if (chan->config.rx_callback)
    return;

  rxCount = uGetRxCnt(ch);

  if (unity_chan->timeoutFlag)
    {
      chan->rxInterCharTimeoutFlag = 1;
      unity_chan->timeoutFlag = 0;
      wakeup = 1;
    }
  
  // send Xoff if Rx FIFO is getting full
  if ((chan->config.flags & CYG_SERIAL_FLAGS_XONXOFF_RXFLOW)
       && !unity_chan->sentXoff && (rxCount > FlowHighWater))
    {
//    putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x38,rxCount);
      unity_chan->sentXoff = 1;
      uXmitBytePriority(ch,chan->config.rxflow_xoff_char);
//    putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x3b,chan->config.rxflow_xoff_char);
    }
  
//  putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x30,cyg_current_time()&0xffff);
//  putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x31,cbuf->put);
//  putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x37,cbuf->get);
  
  // find available contiguous space
  
  if (cbuf->get == 0)
    avail = (cbuf->len - cbuf->put) - 1;
  else if (cbuf->get > cbuf->put)
    avail = (cbuf->get - cbuf->put) - 1;
  else
    avail = (cbuf->len - cbuf->put);

  
//  putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x39,avail);
  
  if (avail == 0)  
    {
      //full
//      putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x33,0);

#if 0      
      if (chan->funs->stop_recv)
        {
          cbuf->rxstopped = true;
          chan->funs->stop_recv(chan);
        }
#endif      
    }

  count = min(rxCount,avail);
//  putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x3a,count);
  
  if (count)
    {
      wakeup = 1;
      
      nextPut = cbuf->put + count;
      if (nextPut == cbuf->len)
        nextPut = 0;
  
      dst = cbuf->data+cbuf->put;
  
      chanStatus = uGetChanStatus(ch);
      if (chanStatus & ChanStatus_RxErr)
        {
        while (count)
          {
          --count;
          statusAndRxByte = uRxByteStatus(ch);
          rxByte = statusAndRxByte & 0xFF;
          if (statusAndRxByte & (Rcv_Byte_Errors | RCV_BYTE_BREAK_DETECTED))
                {
		if (statusAndRxByte & RCV_BYTE_BREAK_DETECTED) 
			++unity_chan->error_count.rxBreak;
		if (statusAndRxByte & RCV_BYTE_RCV_OVERRUN) 
			++unity_chan->error_count.rxOverflow;
                if (statusAndRxByte & RCV_BYTE_PARITY_ERROR) 
                    ++unity_chan->error_count.rxParity;
                if (statusAndRxByte & RCV_BYTE_FRAMING_ERROR) 
                    ++unity_chan->error_count.rxFraming;
                }
		//  Removed "else", which causes the byte to be passed to the application
		//  even if there is an error.  This is the way SI, Windows and Linux serial 
		//  drivers work, and we had a customer problem in this area.  Kurt R.  10-8-03
		//              else{*dst++ = rxByte;} 
	      
                *dst++ = rxByte;
/*
 * DO NOT REMOVE				
				if(ident.debugICTimeout){
					ctc = cyg_current_time();
					clkticks2ms(ctc, &mCCR, &ms);
					//if((aiopic_chan->interCharTimeout < (ms - lastStamp)) & (lastStamp != 0))
					//	diag_printf("aiopic-handleRxBytes%u: (A) rxByte @ clock %llu (%llums) (last stamp = %llu) [GAP = %llu]\n", ch->chan, (unsigned long long)ctc, ms, lastStamp, (ms - lastStamp));	
//					else if(((ms - lastStamp) != 0) && ((ms - lastStamp) > aiopic_chan->interCharTimeout))
						diag_printf("aiopic-handleRxBytes%u: (A) rxByte @ clock %llu (%llums) (last stamp = %llu) GAP = %llu\n", ch->chan, (unsigned long long)ctc, ms, lastStamp, (ms - lastStamp));	
					
					lastStamp = ms;	
				}
*/				
	      
            }
        }	//this is (chanstatus &statusmodeenable)
      else
        {
          if (((int)dst) & 1)      // need hword alaigned pointer
            {
              *dst++ = (uRxByteStatus(ch) & 0xFF);
              --count;
/*
 * DO NOT REMOVE    	        
    	        if(ident.debugICTimeout){
					ctc = cyg_current_time();
					clkticks2ms(ctc, &mCCR, &ms);
					//if((aiopic_chan->interCharTimeout < (ms - lastStamp)) & (lastStamp != 0))
					//	diag_printf("aiopic-handleRxBytes%u: (B) dataByte @ clock %llu (%llums) (last stamp = %llu) [GAP = %llu]\n", ch->chan, (unsigned long long)ctc, ms, lastStamp, (ms - lastStamp));	
					//else if(((ms - lastStamp) != 0) && ((ms - lastStamp) > aiopic_chan->interCharTimeout))
						diag_printf("aiopic-handleRxBytes%u: (B) dataByte @ clock %llu (%llums) (last stamp = %llu) GAP = %llu\n", ch->chan, (unsigned long long)ctc, ms, lastStamp, (ms - lastStamp));	
					
					lastStamp = ms;	
				}
*/				
            }
#if 0
//            {
              int odd = count & 1;
              count &= ~1;
              asm(" mov r0,%0"::"r" (dst)          : "r0");
              asm(" mov r1,%0"::"r" (ch->dataWord) : "r1");
              asm(" mov r2,%0"::"r" (count)        : "r2");
              asm(" bl _hword_inblock" : : : "r0","r1","r2","r3","r4");
              dst += count;
              count = odd;
//            }
#else      
          while (count > 1)
            {
              *((unsigned short*)dst) = uRxWord(ch);
              dst += 2;
              count -= 2;

/*
 * DO NOT REMOVE            	
            	if(ident.debugICTimeout){
					ctc = cyg_current_time();
					clkticks2ms(ctc, &mCCR, &ms);
					//if((aiopic_chan->interCharTimeout < (ms - lastStamp)) & (lastStamp != 0))
					//	diag_printf("aiopic-handleRxBytes%u: (C) dataWord @ clock %llu (%llums) (last stamp = %llu) [GAP = %llu]\n", ch->chan, (unsigned long long)ctc, ms, lastStamp, (ms - lastStamp));	
					//else if(((ms - lastStamp) != 0) && ((ms - lastStamp) > aiopic_chan->interCharTimeout))
						diag_printf("aiopic-handleRxBytes%u: (C) dataWord @ clock %llu (%llums) (last stamp = %llu) GAP = %llu\n", ch->chan, (unsigned long long)ctc, ms, lastStamp, (ms - lastStamp));	
					
					lastStamp = ms;	
				}
*/				
            }
#endif      
          if (count)
            {
              *dst++ = (uRxByteStatus(ch) & 0xFF);
              --count;

/*
 * DO NOT REMOVE              	
              	if(ident.debugICTimeout){
					ctc = cyg_current_time();
					clkticks2ms(ctc, &mCCR, &ms);
					//if((aiopic_chan->interCharTimeout < (ms - lastStamp)) & (lastStamp != 0))
					//	diag_printf("aiopic-handleRxBytes%u: (D) dataByte @ clock %llu (%llums) (last stamp = %llu) [GAP = %llu]\n", ch->chan, (unsigned long long)ctc, ms, lastStamp, (ms - lastStamp));	
					//else if(((ms - lastStamp) != 0) && ((ms - lastStamp) > aiopic_chan->interCharTimeout))
						diag_printf("aiopic-handleRxBytes%u: (D) dataByte @ clock %llu (%llums) (last stamp = %llu) GAP = %llu\n", ch->chan, (unsigned long long)ctc, ms, lastStamp, (ms - lastStamp));	
						
					lastStamp = ms;
				}
*/				
            }
        }	//else --- this is not (chanstatus or statusmodeenable)

//      putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x34,dst[-1]);
      cbuf->put = nextPut;
//      putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x35,nextPut);
  
      rxCount = uGetRxCnt(ch);
    }

  unity_chan->rxFifoLast = 0;
  
  // send Xon if there's room in Rx FIFO
  if ((chan->config.flags & CYG_SERIAL_FLAGS_XONXOFF_RXFLOW)
      && unity_chan->sentXoff && (rxCount < FlowLowWater))
    {
      //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x38,rxCount);
      unity_chan->sentXoff = 0;
      uXmitBytePriority(ch, chan->config.rxflow_xon_char);
      //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x3c,chan->config.rxflow_xon_char);
    }
  
  if (cbuf->waiting && wakeup)
    {
      cbuf->waiting = false;
      cyg_drv_cond_signal(&cbuf->wait);
    }
}

//============================================================
// Internal function to actually configure the hardware to desired baud rate, etc.
static bool unity_serial_config_port(serial_channel *chan, 
				      cyg_serial_info_t *new_config, 
				      bool init)
{
  tUnityChanInfo *unityChanInfo = (tUnityChanInfo*)(chan->dev_priv);
  tChan *ch = &unityChanInfo->chan;
  unsigned long baud;
  int mode;

  // interchar timeout
  unityChanInfo->interCharTimeout = new_config->inter_char_timeout;
  unityChanInfo->gotData = 0;
  unityChanInfo->timeoutFlag = 0;

  // ====  baud rate  ====
  if (new_config->baud >= 50)
    baud = new_config->baud;
  else
    baud = select_baud[new_config->baud];
  
  if (baud == 0) 
    return false;  
  
  uSetBaudDivisor(ch, UnityBaudDivisor(baud));

  // ====  word length  ====
  switch(new_config->word_length)
    {
     case CYGNUM_SERIAL_WORD_LENGTH_5:
      uSetData5(ch);
      break;
      
     case CYGNUM_SERIAL_WORD_LENGTH_6:
      uSetData6(ch);
      break;
      
     case CYGNUM_SERIAL_WORD_LENGTH_7:
      uSetData7(ch);
      break;
      
     case CYGNUM_SERIAL_WORD_LENGTH_8:
      uSetData8(ch);
      break;
      
     default:
      return false;
      break;
    }
  
  // ====  parity  ====
  switch (new_config->parity)
    {
     case CYGNUM_SERIAL_PARITY_ODD:
      uSetOddParity(ch);
      uEnParity(ch);
      break;
      
     case CYGNUM_SERIAL_PARITY_EVEN:
      uSetEvenParity(ch);
      uEnParity(ch);
      break;
      
     case  CYGNUM_SERIAL_PARITY_NONE:
      uDisParity(ch);
      break;
      
     case CYGNUM_SERIAL_PARITY_SPACE:
      uSetOddParity(ch);
      uEnForcedParity(ch);
      uEnParity(ch);
      break;
      
     case CYGNUM_SERIAL_PARITY_MARK:
      uSetEvenParity(ch);
      uEnForcedParity(ch);
      uEnParity(ch);
      break;
      
     default:
      return false;
      break;
    }
  
  // ====  stop bits  ====
  switch(new_config->stop)
    {
     case CYGNUM_SERIAL_STOP_1:
      uSetStop1(ch);
      break;

     //?? FIXME - do we need a 5 bit check? - 1 1/2 supported 5 data bit mode only
     case CYGNUM_SERIAL_STOP_1_5:
      uSetStop2(ch);
      break;
      
     case CYGNUM_SERIAL_STOP_2:
      uSetStop2(ch);
      break;
      
     default:
      return false;
      break;
    }

  // ====  loop back  ====

  // ====  flow control  ====
  if (new_config->txflow_xon_char != chan->config.txflow_xon_char)
    uSetTxXONChar(ch, new_config->txflow_xon_char);
  if (new_config->txflow_xoff_char != chan->config.txflow_xoff_char)
    uSetTxXOFFChar(ch,new_config->txflow_xoff_char);
  
  if (new_config->flags & CYG_SERIAL_FLAGS_XONXOFF_TXFLOW)
    uEnSoftFlowCtrl(ch);
  else
    uDisSoftFlowCtrl(ch);
  
  if (new_config->flags & (CYG_SERIAL_FLAGS_RTS_RXFLOW | CYG_SERIAL_FLAGS_RTSCTS))
    uEnRtsFlowCtrl(ch);
  else
    uDisRtsFlowCtrl(ch);

  if (new_config->flags & (CYG_SERIAL_FLAGS_CTS_TXFLOW | CYG_SERIAL_FLAGS_RTSCTS))
    {
      uClrRts(ch);
      uEnCtsFlowCtrl(ch);
    }
  else
    uDisCtsFlowCtrl(ch);

  if (new_config->flags & CYG_SERIAL_FLAGS_RTS_TOGGLE)
    {
      uEnRTSToggle(ch);
      uRtsTogglePolarityHigh(ch);		//need to toggle polarity when rts toggle is set
    }    
  else
    {
      uDisRTSToggle(ch);
      uRtsTogglePolarityLow(ch);
    }    
  
  // ====  interface mode  ====
  mode = new_config->interface_mode;
  uSetInterfaceMode(ch, mode);

  //  Sanity check.  HW flow control can be active only when in RS232 mode
  if (mode != RS232_MODE) {
    uDisRtsFlowCtrl(ch);
    uDisCtsFlowCtrl(ch);
    new_config->flags &= ~CYG_SERIAL_FLAGS_RTS_RXFLOW;
    new_config->flags &= ~CYG_SERIAL_FLAGS_CTS_TXFLOW;
    new_config->flags &= ~CYG_SERIAL_FLAGS_RTSCTS;
  }

  // save config
  if (new_config != &chan->config) 
    chan->config = *new_config;

  // interrupts for callbacks?
  if (chan->config.modem_callback)
    uEnModemChangeInt(ch);
  else
    uDisModemChangeInt(ch);
  
		//#ifdef FIXME	//?? status mode handled differently in unity
				//   as is rx char interrupt
  if (chan->config.rx_callback)
    {
		//      sEnRxStatusMode(ch);
		//      sEnRxCharInt(ch);
      uEnRcvDataReadyTimeoutInt(ch);
    }
  else
    {
		//      sDisRxStatusMode(ch);
		//      sDisRxCharInt(ch);
      uDisRcvDataReadyTimeoutInt(ch);
    }
		//#endif
  
  return true;
}

//===============================================================
// Internal function to handle set_config keys not implimented by higher layers
Cyg_ErrNo unity_serial_set_config_handler(serial_channel *chan, 
					   cyg_uint32 key, 
					   const void *xbuf, 
					   cyg_uint32 *len)
{
  tChan *ch = &(((tUnityChanInfo*)(chan->dev_priv))->chan);
  Cyg_ErrNo res = ENOERR;
  
  switch (key)
    {
     case CYG_IO_SET_CONFIG_SERIAL_MODEM:
      if (*len != sizeof (cyg_uint32))
        res = -EINVAL;
      else
        {
          if (*(cyg_uint32*)xbuf & CYG_SERIAL_MODEM_DTR) 
            uSetDtr(ch);
          else
            uClrDtr(ch);
          // Ignore RTS flag if it's being controlled automatically
          if (!(chan->config.flags & (CYG_SERIAL_FLAGS_RTSCTS |
                                      CYG_SERIAL_FLAGS_RTS_RXFLOW |
                                      CYG_SERIAL_FLAGS_RTS_TOGGLE)))
            {
              if (*(cyg_uint32*)xbuf & CYG_SERIAL_MODEM_RTS) 
                uSetRts(ch);
              else
                uClrRts(ch);
            }
        }
      break;

     case CYG_IO_SET_CONFIG_SERIAL_SCAN_PERIOD:
        {
          unsigned period_ms = *((unsigned*)xbuf);
          if ((*len != sizeof (cyg_uint32)) || (period_ms < 1) || (period_ms > 100))
            res = -EINVAL;
          else
            scanPeriod = period_ms;
          break;
        }
                     
     default:
      res = -EINVAL;
    }
  
  return res;
}



//==============================================================
// Internal function to handle get_config keys not implimented by higher layers
Cyg_ErrNo unity_serial_get_config_handler(serial_channel *chan, 
					   cyg_uint32 key, 
					   void *xbuf, 
					   cyg_uint32 *len)
{
  tUnityChanInfo *serinfo = (tUnityChanInfo *)chan->dev_priv;
  tChan *ch = &(((tUnityChanInfo*)(chan->dev_priv))->chan);
  Cyg_ErrNo res = ENOERR;

  switch (key)
    {
     case CYG_IO_GET_CONFIG_SERIAL_MODEM:
      if (*len != sizeof (cyg_uint32))
        res = -EINVAL;
      else
        {
          unsigned short chanStatus = uGetChanStatus(ch);
          unsigned short txControl  = uGetTxControl(ch);
          unsigned short chanIntID;
          
          // reading the interrupt ID here may clear interrupt
          // and cause edge to be missed by the callback
          // routine, so don't return deltas if callback is
          // attached.
          
          if (chan->config.modem_callback)
            chanIntID = 0;
          else
            chanIntID = uGetChanIntID(ch);
          
          *(cyg_uint32*)xbuf = 0;
          
          if (txControl & SET_DTR_ACTIVE)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_DTR;
          if (txControl & SET_RTS_ACTIVE)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_RTS;
          
          if (chanStatus & CHAN_STATUS_DSR_ACTIVE)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_DSR;
          if (chanStatus & CHAN_STATUS_CTS_ACTIVE)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_CTS;
          if (chanStatus & CHAN_STATUS_CD_ACTIVE)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_CD;
          if (chanStatus & CHAN_STATUS_RI_ACTIVE)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_RI;
          if (chanIntID & INT_DSR_CHANGE)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_DSR_DELTA;
          if (chanIntID & INT_CTS_CHANGE)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_CTS_DELTA;
          if (chanIntID & INT_CD_CHANGE)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_CD_DELTA;
          if (chanIntID & INT_RI_CHANGE)
            *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_RI_DELTA;
          
	  //  Set the flag indicating our transmitter is flowed off.  
	  //  First hardware flow control.  If CTS cleared, must be 
	  //  flowed off.
          if ((chan->config.flags & (CYG_SERIAL_FLAGS_CTS_TXFLOW | CYG_SERIAL_FLAGS_RTSCTS))
              && !(chanStatus & CHAN_STATUS_CTS_ACTIVE)) 
	  {
	      *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_FLOWTXOFF;
	  }

	  //  Next software flow control.  If XON/XOFF is active, check
	  //  if SET_TXOFF is active.  If not, check if automatic SW flow
	  //  control in the AIOPIC has our transmitter flowed off.  There
	  //  is no status flag in the AIOPIC for this, so we test is by 
	  //  checking (twice) that the TX FIFO is not empty but the shift 
	  //  register is empty.  This is not perfect, but is OK, since the 
	  //  Rocketport does it this way...
          else if ((chan->config.flags & CYG_SERIAL_FLAGS_XONXOFF_TXFLOW)) {
	      if (serinfo->fakeXoff) {
		  *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_FLOWTXOFF;
	      }
	      else {
		  if ((chanStatus & (CHAN_STATUS_XMIT_FIFO_EMPTY | CHAN_STATUS_XMIT_SHIFT_REG_EMPTY)) == CHAN_STATUS_XMIT_SHIFT_REG_EMPTY) {
		      chanStatus = uGetChanStatus(ch);
		      if ((chanStatus & (CHAN_STATUS_XMIT_FIFO_EMPTY | CHAN_STATUS_XMIT_SHIFT_REG_EMPTY)) == CHAN_STATUS_XMIT_SHIFT_REG_EMPTY) {
			  *(cyg_uint32*)xbuf |= CYG_SERIAL_MODEM_FLOWTXOFF;
		      }
		  }
	      }
	  }
        }
      break;
      
     case CYG_IO_GET_CONFIG_SERIAL_ERRORS:
      if (*len < sizeof serinfo->error_count)
        res = -EINVAL;
      else
        *((cyg_serial_error_count_t*)(xbuf)) = serinfo->error_count;
      break;
      
     case CYG_IO_GET_CONFIG_SERIAL_CLRERRS:
      memset(&serinfo->error_count, 0, sizeof serinfo->error_count);
      break;

     case CYG_IO_GET_CONFIG_SERIAL_BREAKON:
      uSendBreak(ch);
      break;
      
     case CYG_IO_GET_CONFIG_SERIAL_BREAKOFF:
      uClrBreak(ch);
      break;

     case CYG_IO_SET_CONFIG_TX_OFF:
      if (!(chan->config.flags & CYG_SERIAL_FLAGS_XONXOFF_TXFLOW))
        break;
      uDisSoftFlowCtrl(ch);
      uDisTransmit(ch);
      serinfo->fakeXoff = true;
		//#ifdef FIXME   uEnCompare1 is not completely the same function
		//      sEnRxIntCompare1(ch);	
      uEnCompare1(ch, COMPARE_DISCARD_BYTE, (READB(ch->ucodeAddr + URP_XMIT_XON_CHAR)), 0x0);
		//#endif      
      uEnSpecialReceiveInt(ch);
      break;
      
     case CYG_IO_GET_CONFIG_TX_OVERRIDE:
      if(serinfo->fakeXoff) 
        {
          uEnTransmit(ch);
          uDisSpecialReceiveInt(ch);
          uDisCompare1(ch);
          serinfo->fakeXoff = false;
          uEnSoftFlowCtrl(ch);
        }
      uSetXon(ch);	//??sClrTxXOFF(ch);
      break;
      
     default:
      res = -EINVAL;
    }
  
  return res;
}



//=======================================================
  // Function to initialize the device.  Called at bootstrap time.
SERIAL_FUNS2(unity_serial_functions, 
	     unity_serial_putc, 
	     unity_serial_getc,
	     unity_serial_set_config,
	     unity_serial_start_xmit,
	     unity_serial_stop_xmit,
	     unity_serial_get_config_handler,
	     unity_serial_set_config_handler,
	     unity_serial_start_recv,
	     unity_serial_stop_recv,
	     unity_serial_flush_input,
	     unity_serial_flush_output,
	     unity_serial_get_tx_count,
	     unity_serial_get_rx_count
);

//==========================================================
static cyg_handle_t  unityAiopicInterrupt0Handle;
static cyg_interrupt unityAiopicInterrupt0Object;
	//?? FIXME	 ???? -- need a 2nd interrupt for additional unity's???
	//??static cyg_handle_t  unityAiopicInterrupt1Handle; 
	//??static cyg_interrupt unityAiopicInterrupt1Object;

static cyg_handle_t  unityRestoreInterruptHandle;
static cyg_interrupt unityRestoreInterruptObject;

//==========================================================
bool unity_serial_init(struct cyg_devtab_entry *tab)
{
  serial_channel *chan;
  static int init;
  int i;

  if (init)
    return true;
  else
    init = 1;

  tab->lookup = unity_serial_lookup;
  
  getModelNumber(&identity.modelId);
  
  if(identity.modelId == 5002405)
    {
      numChanPresent = 1;
    }
  else
    {
      numChanPresent = 0;
    }  

  //  If there are no Unities present, this driver fails
  if (numChanPresent == 0)	//this shows up as table look up error 
      return false;
  
  // we're going to initialize all channels at once, because we don't want
  // to enable the interrupt unless all channels are initialized.
  diag_printf("\n\nUNITY serial driver: %d channels\n",numChanPresent);	//?? cr's

  initUnitiesAndChannels();

  for (chan = serialChannel; chan < serialChannel + numChanPresent; ++chan)
    unity_serial_config_port(chan, &chan->config, true);
  
  cyg_mutex_init(&sync.lock);
  cyg_cond_init(&sync.wait,&sync.lock);

  cyg_drv_interrupt_create(CYGNUM_HAL_INTERRUPT_Ext5,
                           99,
                           0,
                           unity_serial_aiopic_ISR,
                           unity_serial_aiopic_DSR,
                           &unityAiopicInterrupt0Handle,
                           &unityAiopicInterrupt0Object);
  cyg_drv_interrupt_attach(unityAiopicInterrupt0Handle);

#ifdef FIXME	// ???? do we need additional interrupts in future, for each unity ??
  if ( (numChanPresent == 16) && (identity.modelId != 5002240) )
    {
      cyg_drv_interrupt_create(CYGNUM_HAL_INTERRUPT_Ext2,
                               99,
                               1,
                               unity_serial_aiopic_ISR,
                               unity_serial_aiopic_DSR,
                               &unityAiopicInterrupt1Handle,
                               &unityAiopicInterrupt1Object);
      cyg_drv_interrupt_attach(unityAiopicInterrupt1Handle);
      cyg_drv_interrupt_acknowledge(CYGNUM_HAL_INTERRUPT_Ext2);
      cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_Ext2);
    }
#endif

  // setup RESTORE ITERRUPT  -  4
  cyg_drv_interrupt_create(restore_int,
                           99,
                           0,
                           unity_restore_ISR,
                           unity_restore_DSR,
                           &unityRestoreInterruptHandle,
                           &unityRestoreInterruptObject);
  cyg_drv_interrupt_attach(unityRestoreInterruptHandle);



  //  Set up the serial channel structures.  Declared in dm2.c, defined here.
  for (i = 0; i < numChanPresent; i++) {
      memset(&serialChannel[i], 0, sizeof(serialChannel[i]));
      serialChannel[i].funs = &unity_serial_functions;
      serialChannel[i].callbacks = &cyg_io_serial_callbacks;
      serialChannel[i].dev_priv = &unityChanInfo[i];
      serialChannel[i].config.baud = 9600;
      serialChannel[i].config.stop = 1;
      serialChannel[i].config.parity = 0;
      serialChannel[i].config.word_length = 8;
      serialChannel[i].config.flags = 0;
      serialChannel[i].config.interface_mode = RS232_MODE;
      serialChannel[i].config.rxflow_xon_char = 0x11;
      serialChannel[i].config.rxflow_xoff_char = 0x13;
      serialChannel[i].config.txflow_xon_char = 0x11;
      serialChannel[i].config.txflow_xoff_char = 0x13;
      serialChannel[i].init = false;
      serialChannel[i].sync.lock = &sync.lock;
      serialChannel[i].sync.wait = &sync.wait;
  }

/*
 * krw 04/07 - DO NOT REMOVE

	ident.debugICTimeout = 0; 	//set this to turn on timestamping of character xmits.
	
	//get the clock resolution for use with clkticks2ms()
	//these values do not change so safe to use member level vars.
	if(ident.debugICTimeout){
		mCCR = cyg_clock_get_resolution(cyg_real_time_clock());
		diag_printf("aiopic_serial_init: *** WARNING *** Debug IC Timeout ENABLED\n");
	}
*/

  return true;
}


//============================================================================
//   This routine is called when the device is "looked" up (i.e. attached)
//   The buffer areas must be malloc's here, since the malloc library is not
//   initialized when the driver init function is called.
Cyg_ErrNo unity_serial_lookup(struct cyg_devtab_entry **tab, 
                  	      struct cyg_devtab_entry *sub_tab,
                              const char *name)
{
  static int dataBuffersAllocated;
  int i;    
  cyg_handle_t systemCounterHandle;

  diag_printf("UNITY detected.  Dev Name:  %s \n", (*tab)->name);
  serial_channel *chan = (serial_channel *)(*tab)->priv;

  if (!dataBuffersAllocated) 
    {
      for (i = 0; i < numChanPresent; i++) 
        {
	  serialChannel[i].out_cbuf.data = (unsigned char*)malloc(BufSize);
	  serialChannel[i].out_cbuf.len = BufSize;
	  serialChannel[i].in_cbuf.data = (unsigned char*)malloc(BufSize);
	  serialChannel[i].in_cbuf.len =BufSize;
        }

      cyg_drv_interrupt_acknowledge(CYGNUM_HAL_INTERRUPT_Ext5);
      cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_Ext5);

#ifdef FIXME // ???? need additional ints for external unity parts ??
      if (numChanPresent > 16)
        {
          cyg_drv_interrupt_acknowledge(CYGNUM_HAL_INTERRUPT_Ext2);
          cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_Ext2);
        }
#endif	

      // enable Restore Interrupt
      cyg_drv_interrupt_acknowledge(restore_int);
      cyg_drv_interrupt_unmask(restore_int);
      
      write_restore_reg(RESTORE_EDGE, 0x0);	//clear interrupt status bit on pio
      write_restore_reg(RESTORE_MASK, 0x01);	//enable interrupt mask on pio

      /*  
       *  When driver is first looked up, register the alarm so the poll
       *  function can get called every ms.
       */
      cyg_clock_to_counter(cyg_real_time_clock(), &systemCounterHandle);
      cyg_alarm_create(systemCounterHandle, unity_serial_poll, (cyg_addrword_t)alarmIndex,
		       &unityAlarmHandle, &unityAlarmObj);
      cyg_alarm_initialize(unityAlarmHandle, 10, 1);

      dataBuffersAllocated = 1;
    }

  if (chan >= serialChannel + numChanPresent)
    return -ENODEV;

  (chan->callbacks->serial_init)(chan);  // Really only required for interrupt driven devices
  return ENOERR;
}

//================================================================================
// Send a character to the device output buffer.
// Return 'true' if character is sent to device
bool unity_serial_putc(serial_channel *chan, unsigned char c)
{
  tChan *ch = &(((tUnityChanInfo*)(chan->dev_priv))->chan);

  //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x10,txFifoCount);
  if (  511 - (uGetTxCnt(ch)) > 0)
    {
      uXmitByte(ch, c);
      //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x11,c);

/*  
 * KRW    
 * I never seen this come through on the debug header so not sure 
 * if it is in play. Will add timestamp to handleReceive/TransmitBytes() 
 * functions instead of put/getc.
		if(ident.debugICTimeout){
			ctc = cyg_current_time();
			clkticks2ms(ctc, &mCCR, &ms);
			diag_printf("aiopicputc%u: [%c] @ clock %llu (%llums)\n", ch->chan, *(ch->dataByte), (unsigned long long)ctc, ms);	
		}	
*/      
      return true;
    } 
  else 
    {
      // No space
      //putTrace((((unsigned)(ch->unity) & 0xff)|ch->chan),0x12,c);
      return false;
    }
}

//============================
// Fetch a character from the device input buffer, waiting if necessary
unsigned char unity_serial_getc(serial_channel *chan)
{
  tChan *ch = &(((tUnityChanInfo*)(chan->dev_priv))->chan);

  while (!uRcvDataAvail(ch))
    ;

/*
 * KRW    
 * I never seen this come through on the debug header so not sure 
 * if it is in play. Will add timestamp to handleReceive/TransmitBytes() 
 * functions instead of put/getc.
    if(ident.debugICTimeout){
		ctc = cyg_current_time();
		clkticks2ms(ctc, &mCCR, &ms);
		diag_printf("aiopicgetc%u: [%c] @ clock %llu (%llums)\n", ch->chan, *(ch->dataByte), (unsigned long long)ctc, ms);	
	}
*/
  return (uRxByteStatus(ch) & 0xFF);
}

//============================
// Set up the device characteristics; baud rate, etc.
bool unity_serial_set_config(serial_channel *chan, cyg_serial_info_t *config)
{
  return unity_serial_config_port(chan, config, false);
}

//============================
// Enable the transfer of bytes to next layer up
void unity_serial_start_recv(serial_channel *chan)
{
  tUnityChanInfo *unity_chan = (tUnityChanInfo *)chan->dev_priv;
  unity_chan->receive_stopped = false;
  handleReceiveBytes(chan);
}

//============================
// Get tx FIFO count
int unity_serial_get_tx_count(serial_channel *chan)
{
  tChan *ch = &(((tUnityChanInfo*)(chan->dev_priv))->chan);
  return uGetTxCnt(ch) + ((uGetChanStatus(ch)&CHAN_STATUS_XMIT_SHIFT_REG_EMPTY) == 0);
}

//============================
// Get rx FIFO count
int unity_serial_get_rx_count(serial_channel *chan)
{
  tChan *ch = &(((tUnityChanInfo*)(chan->dev_priv))->chan);
  return uGetRxCnt(ch);
}

//============================
// Flush input data
void unity_serial_flush_input(serial_channel *chan)
{
  tUnityChanInfo *unity_chan = (tUnityChanInfo *)chan->dev_priv;
  tChan *ch = &(unity_chan->chan);

  uDisRxFIFO(ch);
  diag_printf("");   // need to delay 2us

#ifdef FIXME  // ???? unity doesn't have command to reset fifo, only flush
  // reset rx fifo logic
  ch->aiop->command = AiopCommand_ResetRxFifo | ch->chan;
  ch->aiop->command = ch->chan;
  // set rx fifo pointers to 0
  AiopPutIndexedShort(ch->aiop,ch->base+_RXF_OUTP,0);
  AiopPutIndexedShort(ch->aiop,ch->base+_RXF_INP,0);
#endif	

  uFlushRxFifo(ch);
  uEnRxFIFO(ch);
}
  
//============================  
// Flush output data
void unity_serial_flush_output(serial_channel *chan)
{
  tUnityChanInfo *unity_chan = (tUnityChanInfo *)chan->dev_priv;
  tChan *ch = &unity_chan->chan;
  int txEnabled;
  
  txEnabled = uGetTxControl(ch) & XMIT_EN;
  if (txEnabled)
    uDisTransmit(ch);
  uStopRxProcessor(ch);

#ifdef FIXME  // ???? unity doesn't have command to reset fifo, only flush
  ch->aiop->command = AiopCommand_ResetTxFifo | ch->chan;
  ch->aiop->command = ch->chan;
  // set tx fifo pointers to 0
  AiopPutIndexedShort(ch->aiop,ch->base+_TXF_OUTP,0);
#endif	

  uFlushTxFifo(ch);

  if (txEnabled)
    uEnTransmit(ch);
  uStartRxProcessor(ch);
}

//============================  
// Disable the transfer of bytes to next layer up -- we need to disable
// the receive interrupt so that we stop reading bytes from the Rx FIFO
// so that the UART will flow control the other end.
void unity_serial_stop_recv(serial_channel *chan)
{
  tUnityChanInfo *unity_chan = (tUnityChanInfo *)chan->dev_priv;
  unity_chan->receive_stopped = true;
}

//============================
// Enable the transmit interrupt on the device
void unity_serial_start_xmit(serial_channel *chan)
{
}

//============================
// Disable the transmit interrupt on the device
void unity_serial_stop_xmit(serial_channel *chan)
{
}

//============================
static int unity0DsrCount;
//??static int unity1DsrCount;
static int unityIsrCount;
static cyg_addrword_t isrDataValue;

//============================
static cyg_uint32 unity_serial_aiopic_ISR(cyg_vector_t vector, cyg_addrword_t data)
{
  isrDataValue = data;
  ++unityIsrCount;

  if (data==0)
    cyg_drv_interrupt_mask(CYGNUM_HAL_INTERRUPT_Ext5);
#ifdef FIXME		//multiple interrupts ???
  else
    cyg_drv_interrupt_mask(CYGNUM_HAL_INTERRUPT_Ext2);
#endif	//FIXME 
  return CYG_ISR_HANDLED+CYG_ISR_CALL_DSR;
}

//============================
static void unity_serial_aiopic_DSR(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data)
{
  int i;

  for (i=0; i<numChanPresent; ++i)
    {
      tChan *ch = &unityChanInfo[i].chan;
      unsigned short intID = uGetChanIntID(ch);
      if (intID)
        {
          unsigned short status = uGetChanStatus(ch);
	  cyg_serial_info_t *config = &serialChannel[i].config;

          if ((intID & INTID_MODEM) && config->modem_callback)
            {
              unsigned current = 0;
              unsigned delta = 0;
              unsigned short txControl = uGetTxControl(ch);
              
              if (status & CHAN_STATUS_CTS_ACTIVE )
                current |= CYG_SERIAL_MODEM_CTS;
              if (status & CHAN_STATUS_DSR_ACTIVE )
                current |= CYG_SERIAL_MODEM_DSR;
              if (status & CHAN_STATUS_CD_ACTIVE )
                current |= CYG_SERIAL_MODEM_CD;
              if ( uGetChanRI(ch) )
                current |= CYG_SERIAL_MODEM_RI;
              if (txControl & SET_DTR_ACTIVE)
                current |= CYG_SERIAL_MODEM_DTR;
              if (txControl & SET_RTS_ACTIVE)
                current |= CYG_SERIAL_MODEM_RTS;
              
              if (intID & INT_CTS_CHANGE)
                delta |= CYG_SERIAL_MODEM_CTS;
              if (intID & INT_DSR_CHANGE)
                delta |= CYG_SERIAL_MODEM_DSR;
              if (intID & INT_CD_CHANGE)
                delta |= CYG_SERIAL_MODEM_CD;
              config->modem_callback(config->modem_callback_data,current,delta);
            }
          
          if (config->rx_callback)
            {
              while (uGetChanStatus(ch) & CHAN_STATUS_RCV_DATA_AVAIL)
                {
                  unsigned short rxCharAndStatus = uRxByteStatus(ch);
                  config->rx_callback(config->rx_callback_data,rxCharAndStatus &0xff,rxCharAndStatus>>8);
                }
            }
          
          if ((intID & INT_XMIT_FIFO_EMPTY) && 0)
            {
            }
          if(intID & INT_SPECIAL_RECEIVE)		//if it was 'set txoff'
            {
              if(unityChanInfo[i].fakeXoff) 
                {
                  volatile unsigned char databyte __attribute((unused));
                  databyte = ( uRxByteStatus(ch) & 0xff );	//remove xon char
                  uEnTransmit(ch);
                  uDisSpecialReceiveInt(ch);
                  uDisCompare1(ch);
                  uEnSoftFlowCtrl(ch);
                  unityChanInfo[i].fakeXoff = false;
                }
              uSetXon(ch);		//??sClrTxXOFF(ch);
            }
        }
    }

  if (isrDataValue == 0)
    {
      ++unity0DsrCount;
      cyg_drv_interrupt_acknowledge(CYGNUM_HAL_INTERRUPT_Ext5);
      cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_Ext5);
    }
  else
    {
#ifdef FIXME   // ???? multiple interrupts ???
      ++unity1DsrCount;
      cyg_drv_interrupt_acknowledge(CYGNUM_HAL_INTERRUPT_Ext2);
      cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_Ext2);
#endif      
    }
}

//============================
static int restoreDsrCount;
static int restoreIsrCount;

//============================
static cyg_uint32 unity_restore_ISR(cyg_vector_t vector, cyg_addrword_t data)
{
  isrDataValue = data;
  ++restoreIsrCount;

  cyg_drv_interrupt_mask(restore_int);
  return CYG_ISR_HANDLED+CYG_ISR_CALL_DSR;
}

//============================
static void unity_restore_DSR(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data)
{

   ++restoreDsrCount;
   write_restore_reg(RESTORE_MASK, 0x0);		//disable interrupt

   putIpInfo(0xC0A8FAFA, 0xFFFF0000, 0xC0A8FA01);	// restore default ip, mask, gateway
   putPassword("");					// clear password
   putAuthMethod(0x1);					// authentication to none
   putTelnetEnable(1);					// telnet enabled
   putRedbootDelay( (unsigned long)15 );		// start up delay to 15 seconds
   putSecureDataEnable(0x0);
   putSecureConfigEnable(0x0);
   putTelnetTimeout( (unsigned long)300);		// telnet time out to 300 seconds

   cyg_drv_interrupt_acknowledge(restore_int);		// handle interrupt in ecos
   cyg_drv_interrupt_unmask(restore_int);		// "      "         "  "

   write_restore_reg(RESTORE_EDGE, 0x0);		//clear interrupt on pio part
   write_restore_reg(RESTORE_MASK, 0x01);		//reenable interrupt on pio part

   HAL_PLATFORM_RESET();				// Reset DM2 for changes to take effec
}


//=============
#define EnableProfiling 0

#if EnableProfiling
  // number of profile snapshots must be power of 2
#define NumProfileSnapshots (32*1024)
static unsigned long profileSnapshots[NumProfileSnapshots];
static int profileIndex;
#endif

/*
 *  Serial polling loop, called every 1ms from the alarm.  Handles intercharacter
 *  timeout.  Global variable scanPeriod indicates how often to poll the serial 
 *  ports (some multiple of 1ms).
 */
static void unity_serial_poll(cyg_handle_t alarm_handle, cyg_addrword_t data)
{
    int i, pollSerialPorts = 1;
    tUnityChanInfo *chanInfo;
    static unsigned count;
    serial_channel *chan;

#if EnableProfiling
    profileSnapshots[profileIndex++] = cyg_thread_self();
    profileIndex &= NumProfileSnapshots-1;
#endif

    ++count;

    if (count < scanPeriod)
	pollSerialPorts = 0;
 
    for (i=0, chanInfo=unityChanInfo; i<numChanPresent; ++i,++chanInfo)
    {
	tChan *ch = &chanInfo->chan;
	unsigned rxFifoCount = uGetRxCnt(ch);

	/*  If RX FIFO count has changes, byte(s) have been received */
	if (rxFifoCount != chanInfo->rxFifoLast)
        {
	    chanInfo->gotData = 1;
	    chanInfo->rxFifoLast = rxFifoCount;
	    chanInfo->idleCount = 0;
        }

	/*  No bytes received this poll */
	else
        {
	    /*  
	     *  If intercharacter timeout is enabled, we've got data, and timeout 
	     *  is exceeded, set the timeout flag and poll the serial ports. 
	     */
	    ++chanInfo->idleCount;
	    if (chanInfo->gotData &&
		chanInfo->interCharTimeout &&
		chanInfo->idleCount >= chanInfo->interCharTimeout)
            {
		pollSerialPorts = 1;
		chanInfo->timeoutFlag = 1;
		chanInfo->gotData = 0;

/*
 * KRW 04/07: DO NOT REMOVE				
				if(ident.debugICTimeout)
					diag_printf("\n\naiopic_serial_poll%u: interCharTimeout (%u) triggered (idle = %u) @ clock %llu\n\n", 
								ch->chan, chanInfo->interCharTimeout, chanInfo->idleCount, 
								(unsigned long long)cyg_current_time());
*/								
            }
        }	//else
    }	//for

    if (pollSerialPorts) {
	for (i = 0, chan = serialChannel; i < numChanPresent; ++i, ++chan)
	{
	    handleReceiveBytes(chan);
	    handleTransmitBytes(chan);
	}
	cyg_drv_cond_broadcast(&sync.wait);
	count = 0;
    }
/*
 * KRW	
 * this was to check that alarm was being processed every ms 
    if(ident.debugICTimeout){ //check to see if this is getting called correctly.
    	++ncalls;
    	if(tperiod == 0){ 
    		tperiod = cyg_current_time();
    		lastScanPeriod = scanPeriod;
    		diag_printf("\naiopic_serial_poll call annouced @ %llu\n", tperiod);
    		diag_printf("aiopic_serial_poll scan period = %d\n", lastScanPeriod);
    		diag_printf("aiopic_serial_poll called %llu times (10 second report interval - %llu)\n", ncalls, tperiod);
    	}else if(cyg_current_time() > (tperiod + 1000)){
    		tperiod = cyg_current_time();
    		diag_printf("aiopic_serial_poll called %llu times (10 second report interval - %llu)\n", ncalls, tperiod);
    		if(lastScanPeriod != scanPeriod){
    			lastScanPeriod = scanPeriod;
    			diag_printf("aiopic_serial_poll scan period changed to %d @ %llu\n", lastScanPeriod, tperiod);
    		}
    	}
    }
*/    
}

//======================================================================
/*  
 *  Set flow control (FLOW_CONTROL_xxx) for the port
 *  Inputs:  Channel pointer and flow control setting (FLOW_CONTROL_xxx)
 */
void SetFlowControl(tChan *ChP, int flowControl)
{
	switch (flowControl) {
	case FLOW_CONTROL_NONE:
		uDisRtsFlowCtrl(ChP);
		uDisCtsFlowCtrl(ChP);
		uDisDtrFlowCtrl(ChP);
		uDisDsrFlowCtrl(ChP);
		uDisSoftFlowCtrl(ChP);
		break;
		
	case FLOW_CONTROL_RTSCTS:
		uEnRtsFlowCtrl(ChP);
		uEnCtsFlowCtrl(ChP);
		uDisDtrFlowCtrl(ChP);
		uDisDsrFlowCtrl(ChP);
		uDisSoftFlowCtrl(ChP);
		break;
		
	case FLOW_CONTROL_DTRDSR:
		uDisRtsFlowCtrl(ChP);
		uDisCtsFlowCtrl(ChP);
		uEnDtrFlowCtrl(ChP);
		uEnDsrFlowCtrl(ChP);
		uDisSoftFlowCtrl(ChP);
		break;
		
	case FLOW_CONTROL_XONXOFF:
		uDisRtsFlowCtrl(ChP);
		uDisCtsFlowCtrl(ChP);
		uDisDtrFlowCtrl(ChP);
		uDisDsrFlowCtrl(ChP);
		uEnSoftFlowCtrl(ChP);
		break;
	}
}


/************************************************************************
 *  Sets the clock prescale value.
 *  Inputs:   CtlP pointer to controller struct, prescale value. 
 */
void uSetClockPrescale(tUnity *CtlP, unsigned long prescale)
{
    WRITEL(prescale & 0xFF, CtlP->pBasePciMemory + ASIC_CLOCK_PRESCALER);
    if (CtlP->has_2nd_asic)
	WRITEL(prescale & 0xFF, CtlP->pBasePciMemory + 0x1000 + ASIC_CLOCK_PRESCALER);
}

/*********************************************************************
 *  Resets the channel.  
 *  Inputs:   ChP pointer to channel struct, baud divisor value. 
 */
void uChannelReset(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + UART_CTRL, RESET_CHANNEL);
}


/************************************************************************
 *  Sets the baud divisor on a channel.
 *  Inputs:   ChP pointer to channel struct, baud divisor value. 
 */
void uSetBaudDivisor(tChan *ChP, unsigned long divisor)
{
    WRITEW(divisor, ChP->pBaseRegister + BAUD_RATE_GENERATOR);
}

/*  
 *  Sets the Intercharacter timeout reload value.
 *  Inputs:   ChP pointer to channel struct, timeout value in ms. 
 */
void uSetIntercharValue(tChan *ChP, unsigned long val)
{
    WRITEL(val, ChP->pBaseRegister + INTERCHAR_RELOAD_VAL);
}

/********************************************************************
 *  Flushes the TX/RX FIFOs.
 *  Inputs:   ChP pointer to channel struct. 
 */
void uFlushTxFifo(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + UART_CTRL, FLUSH_XMIT_FIFO);
    CLEARBITL(ChP->pBaseRegister + UART_CTRL, FLUSH_XMIT_FIFO);
}

void uFlushRxFifo(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + UART_CTRL, FLUSH_RCV_FIFO);
    CLEARBITL(ChP->pBaseRegister + UART_CTRL, FLUSH_RCV_FIFO);
}

void uFlushPriorityFifo(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + UART_CTRL, FLUSH_PRIORITY_FIFO);
    CLEARBITL(ChP->pBaseRegister + UART_CTRL, FLUSH_PRIORITY_FIFO);
}


/***************************************************************************
 *  Gets the contents tx control
 *  Inputs:   ChP pointer to channel struct. 
 */
 
unsigned short uGetTxControl(tChan *ChP)
{
    return READW(ChP->pBaseRegister + TX_CONTROL);
}


/***************************************************************
 *  Send/Stop sending a transmit BREAK signal.
 *  Inputs:   ChP pointer to channel struct. 
 */
void uSendBreak(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, BREAK_ON);
}

void uClrBreak(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, BREAK_ON);
}


/******************************************************************
 *  Set/clear the DTR output pin.
 *  Inputs:   ChP pointer to channel struct. 
 */
void uSetDtr(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, SET_DTR_ACTIVE);
}

void uClrDtr(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, SET_DTR_ACTIVE);
}


/******************************************************************
 *  Set/clear RTS output pin.
 *  Inputs:   ChP pointer to channel struct. 
 */
void uSetRts(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, SET_RTS_ACTIVE);
}

void uClrRts(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, SET_RTS_ACTIVE);
}

/**********************************************************************
 *  Enable/disable CTS flow control.
 *   Inputs:   ChP pointer to channel struct. 
 */
void uEnCtsFlowCtrl(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, CTS_FLOW_EN);
}

void uDisCtsFlowCtrl(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, CTS_FLOW_EN);
}


/**********************************************************************
 *  Enable/disable RTS output flow control.
 *   Inputs:   ChP pointer to channel struct. 
 */
void uEnRtsFlowCtrl(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, RTS_FLOW_EN);
}

void uDisRtsFlowCtrl(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, RTS_FLOW_EN);
}


/**********************************************************************
 *  Enable/disable parity.
 *  Inputs:   ChP pointer to channel struct. 
 */
void uEnParity(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, PARITY_ON);
}

void uDisParity(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, PARITY_ON);
}

/*********************************************************************
 *  Enable/disable forced parity.
 *  Inputs:   ChP pointer to channel struct. 
 */
void uEnForcedParity(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, FORCE_PARITY);
}

void uDisForcedParity(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, FORCE_PARITY);
}


/********************************************************************
 *  Enable/disable RTS toggle.
 *  Inputs:   ChP pointer to channel struct. 
 */
void uEnRTSToggle(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, RTS_TOGGLE);
}

void uDisRTSToggle(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, RTS_TOGGLE);
}


/**********************************************************************
 *  Start/stop the Receive Processor.  Procedure is to have the processor stopped, 
 *  load ucode, then start the processor.  To subsequently disable the recieve 
 *  processor, don't use this bit.  Instead use the uDisRxFIFO()/uEnRxFIFO() functions
 *  which uses the microcode flag.
 *
 *  Background:  Clearing this flag causes the received bytes to be handled by
 *  a 16550 UART core rather than the receive processor.  This only will work for
 *  channel 0, and should never be used by the drivers.  It is for ASIC debug only.
 *  Setting this bit causes received serial bytes to be handled by the receive processor.
 *
 *  Inputs:   ChP pointer to channel struct. 
 */
void uStartRxProcessor(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, RCV_PROC_EN);
}

void uStopRxProcessor(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, RCV_PROC_EN);
}

/*************************************************************************
 *  Enable/disable the RX FIFO.
 *  Inputs:   ChP pointer to channel struct. 
 */
void uEnRxFIFO(tChan *ChP)
{                                               
    WRITEB(URP_RCV_FIFO_ENABLE, ChP->ucodeAddr + URP_RCV_FIFO);
}

void uDisRxFIFO(tChan *ChP)
{
    WRITEB(URP_RCV_FIFO_DISABLE, ChP->ucodeAddr + URP_RCV_FIFO);
}

/*********************************************************************
 *  Enable/disable the transmitter.
 *  Inputs:   ChP pointer to channel struct. 
 */
void uEnTransmit(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, XMIT_EN);
}

void uDisTransmit(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, XMIT_EN);
}


/**************************************************************************
 *  Enable/disable the strip nulls function in the receive processor.
 *  Inputs:   ChP pointer to channel struct. 
 */
void uEnStripNulls(tChan *ChP)
{
    WRITEB(URP_STRIP_NULLS_ENABLE, ChP->ucodeAddr + URP_STRIP_NULLS);
}

void uDisStripNulls(tChan *ChP)
{
    WRITEB(URP_STRIP_NULLS_DISABLE, ChP->ucodeAddr + URP_STRIP_NULLS);
}


/***********************************************************************
 *  Enable/disable TX software flow control.
 *  Inputs:   ChP pointer to channel struct. 
 */
void uEnSoftFlowCtrl(tChan *ChP)
{
    WRITEB(URP_TX_SW_FLOW_CONTROL_ENABLE, ChP->ucodeAddr + URP_TX_SW_FLOW_CONTROL);
    WRITEB(URP_RX_SW_FLOW_CONTROL_ENABLE, ChP->ucodeAddr + URP_RX_SW_FLOW_CONTROL);
}

void uDisSoftFlowCtrl(tChan *ChP)
{
    WRITEB(URP_TX_SW_FLOW_CONTROL_DISABLE, ChP->ucodeAddr + URP_TX_SW_FLOW_CONTROL);
    WRITEB(URP_RX_SW_FLOW_CONTROL_DISABLE, ChP->ucodeAddr + URP_RX_SW_FLOW_CONTROL);
}

/**********************************************************************
 *  Sets the mask in the receive processor.
 *  Inputs:   ChP pointer to channel struct, mask. 
 */
void uSetMask(tChan *ChP, unsigned char mask)
{
    WRITEB(mask, ChP->ucodeAddr + URP_MASK);
}

// kdr  Perhaps this should be moved into the application...
/*  Defines used in the Receive Processor Compare function */
//#define COMPARE_SAVE_BYTE    0
//#define COMPARE_REPLACE_BYTE 1
//#define COMPARE_DISCARD_BYTE 2

/***********************************************************************
 *  Enable/disable compare #1 & 2.
 *  Inputs:   ChP pointer to channel struct. 
 */
void uEnCompare1(tChan *ChP, unsigned char compareAction, unsigned char compareChar, unsigned char replaceChar)
{
    switch (compareAction) {
    case COMPARE_DISCARD_BYTE:
	WRITEB(compareChar, ChP->ucodeAddr + URP_COMPARE_BYTE1);
	WRITEB(URP_COMPARE_ACTION1_DISCARD, ChP->ucodeAddr + URP_COMPARE_ACTION1);
	WRITEB(URP_COMPARE1_ENABLE, ChP->ucodeAddr + URP_COMPARE1);
	WRITEB(URP_COMPARE_IRQ1_ENABLE, ChP->ucodeAddr + URP_COMPARE_IRQ1);
	break;
    case COMPARE_REPLACE_BYTE:
	WRITEB(compareChar, ChP->ucodeAddr + URP_COMPARE_BYTE1);
	WRITEB(replaceChar, ChP->ucodeAddr + URP_REPLACE_BYTE1);
	WRITEB(URP_COMPARE_ACTION1_REPLACE, ChP->ucodeAddr + URP_COMPARE_ACTION1);
	WRITEB(URP_COMPARE1_ENABLE, ChP->ucodeAddr + URP_COMPARE1);
	WRITEB(URP_COMPARE_IRQ1_ENABLE, ChP->ucodeAddr + URP_COMPARE_IRQ1);
	break;
    case COMPARE_SAVE_BYTE:
    default: 
	WRITEB(compareChar, ChP->ucodeAddr + URP_COMPARE_BYTE1);
	WRITEB(URP_COMPARE_ACTION1_SAVE, ChP->ucodeAddr + URP_COMPARE_ACTION1);
	WRITEB(URP_COMPARE1_ENABLE, ChP->ucodeAddr + URP_COMPARE1);
	WRITEB(URP_COMPARE_IRQ1_ENABLE, ChP->ucodeAddr + URP_COMPARE_IRQ1);
	break;
    }
}

void uEnCompare2(tChan *ChP, unsigned char compareAction, unsigned char compareChar, unsigned char replaceChar)
{
    switch (compareAction) {
    case COMPARE_DISCARD_BYTE:
	WRITEB(compareChar, ChP->ucodeAddr + URP_COMPARE_BYTE2);
	WRITEB(URP_COMPARE_ACTION2_DISCARD, ChP->ucodeAddr + URP_COMPARE_ACTION2);
	WRITEB(URP_COMPARE2_ENABLE, ChP->ucodeAddr + URP_COMPARE2);
	WRITEB(URP_COMPARE_IRQ2_ENABLE, ChP->ucodeAddr + URP_COMPARE_IRQ2);
	break;
    case COMPARE_REPLACE_BYTE:
	WRITEB(compareChar, ChP->ucodeAddr + URP_COMPARE_BYTE2);
	WRITEB(replaceChar, ChP->ucodeAddr + URP_REPLACE_BYTE2);
	WRITEB(URP_COMPARE_ACTION2_REPLACE, ChP->ucodeAddr + URP_COMPARE_ACTION2);
	WRITEB(URP_COMPARE2_ENABLE, ChP->ucodeAddr + URP_COMPARE2);
	WRITEB(URP_COMPARE_IRQ2_ENABLE, ChP->ucodeAddr + URP_COMPARE_IRQ2);
	break;
    case COMPARE_SAVE_BYTE:
    default: 
	WRITEB(compareChar, ChP->ucodeAddr + URP_COMPARE_BYTE2);
	WRITEB(URP_COMPARE_ACTION2_SAVE, ChP->ucodeAddr + URP_COMPARE_ACTION2);
	WRITEB(URP_COMPARE2_ENABLE, ChP->ucodeAddr + URP_COMPARE2);
	WRITEB(URP_COMPARE_IRQ2_ENABLE, ChP->ucodeAddr + URP_COMPARE_IRQ2);
	break;
    }
}

void uDisCompare1(tChan *ChP)
{
    WRITEB(URP_COMPARE1_DISABLE, ChP->ucodeAddr + URP_COMPARE1);
    WRITEB(URP_COMPARE_IRQ1_DISABLE, ChP->ucodeAddr + URP_COMPARE_IRQ1);
}

void uDisCompare2(tChan *ChP)
{
    WRITEB(URP_COMPARE2_DISABLE, ChP->ucodeAddr + URP_COMPARE2);
    WRITEB(URP_COMPARE_IRQ2_DISABLE, ChP->ucodeAddr + URP_COMPARE_IRQ2);
}

/**************************************************************************
 *   Set the Rx/Tx XOFF/XON flow control characters
 *  Inputs:   ChP pointer to channel struct, the character. 
 */
void uSetTxXOFFChar(tChan *ChP, char ch)
{
    WRITEB(ch, ChP->ucodeAddr + URP_XMIT_XOFF_CHAR);
}

void uSetTxXONChar(tChan *ChP, char ch)
{
    WRITEB(ch, ChP->ucodeAddr + URP_XMIT_XON_CHAR);
}

void uSetRxXOFFChar(tChan *ChP, char ch)
{
    WRITEB(ch, ChP->ucodeAddr + URP_RCV_XOFF_CHAR);
}

void uSetRxXONChar(tChan *ChP, char ch)
{
    WRITEB(ch, ChP->ucodeAddr + URP_RCV_XON_CHAR);
}


/*************************************************************************
 *  Set stop bits to 1 or 2
 *  Inputs:   ChP pointer to channel struct. 
 */
void uSetStop1(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + UART_CTRL, STOP_BITS);
}

void uSetStop2(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + UART_CTRL, STOP_BITS);
}


/*************************************************************************
 *  Set TX FIFO trigger level
 *  Inputs:   ChP pointer to channel struct, level sets the number of characters in 
 *            FIFO at which the  interrupt will be generated.  
 *            -  UXMIT_FIFO_TRIG_DISABLED  disabled the XMIT FIFO Trigger
 *            -  UXMIT_FIFO_TRIG_1_8       1/8 full
 *            -  UXMIT_FIFO_TRIG_1_4       1/4 full
 *            -  UXMIT_FIFO_TRIG_1_2       1/2 full
 */
void uSetTxTrigger(tChan *ChP, unsigned long level)
{
    unsigned long aval;

    aval = READL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL);  
    aval &= ~UXMIT_FIFO_TRIG_MASK;
    WRITEL(aval | level, ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL); 
}

/*************************************************************************
 *  Set RX FIFO trigger level
 *  Inputs:   ChP pointer to channel struct, level sets the number of characters in 
 *            FIFO at which the  interrupt will be generated.  
 *            -  URCV_FIFO_TRIG_DISABLED  disabled the RCV FIFO Trigger
 *            -  URCV_FIFO_TRIG_1_CHAR    1 Character
 *            -  URCV_FIFO_TRIG_1_2       1/2 full
 *            -  URCV_FIFO_TRIG_7_8       7/8 full
 */
void uSetRxTrigger(tChan *ChP, unsigned long level)
{
    unsigned long aval;

    aval = READL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL);  
    aval &= ~URCV_FIFO_TRIG_MASK;
    WRITEL(aval | level, ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL); 
}
   

/****************************************************************
 *  Set even/odd parity
 *  Inputs:   ChP pointer to channel struct. 
 */
void uSetEvenParity(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, EVEN_PARITY);
}

void uSetOddParity(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, EVEN_PARITY);
}


/*******************************************************************
 *  Functions to set number of data bits
 *  Inputs:   ChP pointer to channel struct. 
 */
void uSetDataBits(tChan *ChP, unsigned long level)
{
    unsigned long aval;

    aval = READL(ChP->pBaseRegister + UART_CTRL);  
    aval &= ~DATABITS_MASK;
    WRITEL(aval | level, ChP->pBaseRegister + UART_CTRL); 
}

void uSetData5(tChan *ChP)
{
    uSetDataBits(ChP, DATABITS_5);
}

void uSetData6(tChan *ChP)
{
    uSetDataBits(ChP, DATABITS_6);
}

void uSetData7(tChan *ChP)
{
    uSetDataBits(ChP, DATABITS_7);
}

void uSetData8(tChan *ChP)
{
    uSetDataBits(ChP, DATABITS_8);
}


/**********************************************************************
 *  Get the number of data bytes in the TX/RX FIFOs
 *  Inputs:   ChP pointer to channel struct. 
 *  Returns:  FIFO count
 */
unsigned short uGetTxCnt(tChan *ChP)
{
    return READW(ChP->pBaseRegister + XMIT_FIFO_COUNT);
}

unsigned short uGetRxCnt(tChan *ChP)
{
    return READW(ChP->pBaseRegister + RCV_FIFO_COUNT);
}


/*********************************************************************
 *  Set the RS interface mode.
 *  Inputs:   ChP pointer to channel struct, mode byte indicating the mode.
 *  NOTE:  Different modes than AIOPIC, see URSxxx in unity.h.
 */
void uSetInterfaceMode(tChan *ChP, unsigned long mode)
{
    unsigned long aval;

// FIXME  when selecting all modes, must be fixed
//        for now supporting original modes 
//??    if(mode==3 || mode==4 || mode==5)
//??       mode=4;
    aval = READL(ChP->pBaseRegister + UART_CTRL);  
    aval &= ~URS_MASK;
    WRITEL(aval | mode, ChP->pBaseRegister + UART_CTRL); 
}

/********************************************************************
 *  Enable/disable tranmitter loopback mode.
 *  Inputs:   ChP pointer to channel struct
 */
void uEnLoopbackMode(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, LOCAL_LOOPBACK);
}

void uDisLoopbackMode(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, LOCAL_LOOPBACK);
}


/************************************************************************
 *  Set RTS toggle polarity to active low/high.
 *  Inputs:   ChP pointer to channel struct
 */
void uRtsTogglePolarityHigh(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, RTS_TOGGLE_POLARITY);
}

void uRtsTogglePolarityLow(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, RTS_TOGGLE_POLARITY);
}


/*********************************************************************
 *  Enable/disable tranmitter DSR flow control.
 *  Inputs:   ChP pointer to channel struct
 */
void uEnDsrFlowCtrl(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, DSR_FLOW_EN);
}

void uDisDsrFlowCtrl(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, DSR_FLOW_EN);
}


/**********************************************************************
 *  Enable/disable receiver DTR flow control.
 *  Inputs:   ChP pointer to channel struct
 */
void uEnDtrFlowCtrl(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, DTR_FLOW_EN);
}

void uDisDtrFlowCtrl(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, DTR_FLOW_EN);
}


/********************************************************************
 *  Returns the RI line state.
 *  Returns:  1 if RI is set, else 0.
 */
int uGetChanRI(tChan * ChP)
{
    return (READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & CHAN_STATUS_RI_ACTIVE);
}


/********************************************************************
 *  Transmits a serial byte/word/long.  No check of FIFO is done first, that must be done
 *  in the calling function.
 *  Inputs:   ChP pointer to channel struct, Data to send
 */
void uXmitByte(tChan *ChP, unsigned char pval)
{
    WRITEB(pval, ChP->pBaseRegister + XMIT_DATA_BYTE);
}

void uXmitWord(tChan *ChP, unsigned short pval)
{
    WRITEW(pval, ChP->pBaseRegister + XMIT_DATA_WORD);
}

void uXmitDword(tChan *ChP, unsigned long pval)
{
    WRITEL(pval, ChP->pBaseRegister + XMIT_DATA_BLOCK);
}

/********************************************************************
 *  Transmits a serial byte/word/long using the priority buffer.
 *  Inputs:   ChP pointer to channel struct, Data to send
 */
void uXmitBytePriority(tChan *ChP, unsigned char pval)
{
    WRITEB(pval, ChP->pBaseRegister + PRIORITY_XMIT_BYTE);
}

void uXmitWordPriority(tChan *ChP, unsigned short pval)
{
    WRITEW(pval, ChP->pBaseRegister + PRIORITY_XMIT_WORD);
}

void uXmitDwordPriority(tChan *ChP, unsigned long pval)
{
    WRITEW((unsigned short)(pval & 0xFFFF), ChP->pBaseRegister + PRIORITY_XMIT_WORD);
    WRITEW((unsigned short)((pval & 0xFFFF0000) >> 16), ChP->pBaseRegister + PRIORITY_XMIT_WORD);
}

/***********************************************************************
 *  Reads a serial byte/word/long.  No check of FIFO is done first, that must be done
 *  in the calling function.
 *  Inputs:   ChP pointer to channel struct, Data to send
 */
unsigned short uRxByteStatus(tChan *ChP)
{
    return READW(ChP->pBaseRegister + RCV_DATA_BYTE_STATUS);
}

unsigned short uRxWord(tChan *ChP)
{
    return READW(ChP->pBaseRegister + RCV_DATA_WORD);
}

unsigned long uRxDword(tChan *ChP)
{
    return READL(ChP->pBaseRegister + RCV_DATA_BLOCK);
}

//?????????????????????????????????
#ifdef FIXME	//do we care about this interrupt??? 
/*******************************************************************
 *  Enable/Disable global interrupts.
 *  Inputs:   ChP pointer to channel struct
 */
void uEnableGlobalInterrupts(tUnity *CtlP)
{
    unsigned long intMask = (PCI_INTERRUPT_ENABLE | PCI_WISHBONE_ERROR_ENABLE | PCI_ERROR_ENABLE | PCI_PARITY_ERROR_ENABLE | PCI_SYSTEM_ERROR_ENABLE);

    SETBITL(CtlP->pBasePciRegion0 + PCI_INTERRUPT_CONTROL, intMask);
}

void uDisableGlobalInterrupts(tUnity *CtlP)
{
    WRITEL(0, CtlP->pBasePciRegion0 + PCI_INTERRUPT_CONTROL);
}

int uGlobalInterruptsAreEnabled(tUnity *CtlP)
{
    return READL(CtlP->pBasePciRegion0 + PCI_INTERRUPT_CONTROL) & PCI_INTERRUPT_ENABLE;
}

/******************************************************************
 *  Clears global hardware interrupts.  This is done in Region 0, the Flextronics core. 
 *  Inputs:   CtrP pointer to controller struct
 */
void uClearGlobalInts(tUnity *CtlP)
{
    WRITEL(0, CtlP->pBasePciRegion0 + PCI_INTERRUPT_STATUS);
}


/*****************************************************************
 *  Enable/Disable AIOP interrupts.
 *  Inputs:   ChP pointer to channel struct
 */
void uEnableAiopInterrupts(tUnity *CtlP)
{
    SETBITL(CtlP->pBasePciMemory + ASIC_INT_STATUS, AIOP_INT_EN);
}

void uDisableAiopInterrupts(tUnity *CtlP)
{
    CLEARBITL(CtlP->pBasePciMemory + ASIC_INT_STATUS, AIOP_INT_EN);
}

#endif	//fixme
//???????????????????????????????

/*********************************************************************
 *  Enable/disable the AIOP interrupts
 *  Inputs:   ChP pointer to channel struct
 */
void uEnXmitFifoEmptyInt(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, XMIT_FIFO_INT_EN);
}

void uDisXmitFifoEmptyInt(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, XMIT_FIFO_INT_EN);
}

void uEnXmitShiftRegEmptyInt(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, XMIT_SHIFTREG_INT_EN);
}

void uDisXmitShiftRegEmptyInt(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, XMIT_SHIFTREG_INT_EN);
}

void uEnInterCharTimeoutInt(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, INTERCHAR_TIMEOUT_INT_EN);
}

void uDisInterCharTimeoutInt(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, INTERCHAR_TIMEOUT_INT_EN);
}

void uEnInterCharTimeoutStatus(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, INTERCHAR_STATUS_EN);
}

void uDisInterCharTimeoutStatus(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, INTERCHAR_STATUS_EN);
}

void uEnAddRcvdInt(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, ADDRESS_INT_EN);
}

void uDisAddRcvdInt(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, ADDRESS_INT_EN);
}

void uEnRcvDataReadyTimeoutInt(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, RCV_DATA_RDY_INT_EN);
}

void uDisRcvDataReadyTimeoutInt(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, RCV_DATA_RDY_INT_EN);
}

void uEnSpecialReceiveInt(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, SPECIAL_RCV_INT_EN);
}

void uDisSpecialReceiveInt(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, SPECIAL_RCV_INT_EN);
}

void uEnModemChangeInt(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, MDM_CHANGE_INT_EN);
}

void uDisModemChangeInt(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, MDM_CHANGE_INT_EN);
}



#ifdef FIXME  //???? need to look at each chan on aiop for status
		// because asic_int_status is wrapper, not included in dm2
/*************************************************************************
 *  Reports whether this PCI card was the source of the interrupt.  Reads the Flextronics
 *  PCI Interrupt Status register, which also detects other PCI interrupts.
 *  Returns:  1 if yes, else 0
 */
int uInterruptActive(tUnity *CtlP)
{
    unsigned long intStatus, intMask = (PCI_INTERRUPT_STATUS_BIT | PCI_WISHBONE_ERROR | PCI_ERROR | PCI_PARITY_ERROR | PCI_SYSTEM_ERROR);

    intStatus = READL(CtlP->pBasePciRegion0 + PCI_INTERRUPT_STATUS);
    if (intStatus & intMask)
	return 1;
    else
	return 0;
}

/*************************************************************************
 *  Functions that report whether a particular interrupt has occurred
 *  Returns:  1 if yes, else 0
 */
int uDmaInterruptActive(tUnity *CtlP)
{
    return READL(CtlP->pBasePciMemory + ASIC_INT_STATUS) & DMA_INT_ACTIVE;
}
#endif	//fixme

int uAiop0InterruptActive(tUnity *CtlP)
{
    return READL(CtlP->pBasePciMemory + CHANNEL_INT_STATUS) & 0x00FF;
}

int uAiop1InterruptActive(tUnity *CtlP)
{
    return READL(CtlP->pBasePciMemory + CHANNEL_INT_STATUS) & 0xFF00;
}

#ifdef FIXME	//more than 1 unity
int uAiop2InterruptActive(tUnity *CtlP)
{
    return READL(CtlP->pBasePciMemory + CHANNEL_INT_STATUS) & 0x00FF;
}

int uAiop3InterruptActive(tUnity *CtlP)
{
    return READL(CtlP->pBasePciMemory + CHANNEL_INT_STATUS) & 0xFF00;
}

int uAnyAiopInterrruptActive(tUnity *CtlP)
{
    return READL(CtlP->pBasePciMemory + CHANNEL_INT_STATUS) & (AIOP0_INT_ACTIVE | AIOP1_INT_ACTIVE | AIOP2_INT_ACTIVE | AIOP3_INT_ACTIVE) ;
}
#endif	//fixme

int uAnyAiopInterrruptActive(tUnity *CtlP)
{
    return READL(CtlP->pBasePciMemory + CHANNEL_INT_STATUS) & (0xFFFF);
}

/**************************************************************************
 *  Interrupt identification functions. Bits must be cleared if 
 *  set by writing a '1' to them.  The special receive is not immediately 
 *  cleared here, since it has multiple sources.
 *  Returns:  1 if yes, else 0
 */
int uInterCharTimeoutInt(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & INT_INTERCHAR_TIMEOUT))
	WRITEL(INT_INTERCHAR_TIMEOUT, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uXmitFifoTriggerInt(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & INT_XMIT_FIFO_TRIGGER))
	WRITEL(INT_XMIT_FIFO_TRIGGER, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uXmitShiftRegEmptyInt(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & INT_XMIT_SHIFTREG_EMPTY))
	WRITEL(INT_XMIT_SHIFTREG_EMPTY, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uAddrRcvdInt(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & INT_ADDR_RCVD))
	WRITEL(INT_ADDR_RCVD, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uRcvDataRdyInt(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & INT_RCV_DATA_RDY))
	WRITEL(INT_RCV_DATA_RDY, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uIntRiChange(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & INT_RI_CHANGE))
	WRITEL(INT_RI_CHANGE, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uRcvFifoTriggerInt(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & INT_RCV_FIFO_TRIGGER))
	WRITEL(INT_RCV_FIFO_TRIGGER, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uXmitFifoEmptyInt(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & INT_XMIT_FIFO_EMPTY))
	WRITEL(INT_XMIT_FIFO_EMPTY, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

/***********************************************************************
 *  Retrieve/clear the Special Receive interrupt flag.  Should read it, them 
 *  determine and clear the source flags (parity, framing, etc), then clear this bit.
 */
int uSpecialReceiveInt(tChan *ChP)
{
    return (READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & INT_SPECIAL_RECEIVE);
}

void uClrSpecialReceiveInt(tChan *ChP)
{
    WRITEL(INT_SPECIAL_RECEIVE, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
}

int uIntCdChange(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & INT_CD_CHANGE))
	WRITEL(INT_CD_CHANGE, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uIntCtsChange(tChan *ChP)
{
    int ret = 0;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & INT_CTS_CHANGE))
	WRITEL(INT_CTS_CHANGE, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uIntDsrChange(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & INT_DSR_CHANGE))
	WRITEL(INT_DSR_CHANGE, ChP->pBaseRegister + INT_ID_CHAN_STATUS);

    return ret;
}


/***************************************************************************
 *  Gets the interrupt Id register contents
 *  Inputs:   ChP pointer to channel struct. 
 */
 
unsigned short uGetChanIntID(tChan *ChP)
{
    return READW(ChP->pBaseRegister + INT_IDENT);
}

 
/***************************************************************************
 *  Get/Clr channel status register 
 *  Inputs:   ChP pointer to channel struct. 
 */
unsigned short uGetChanStatus(tChan *ChP)
{
    return READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS);
}

int uClrChanStatus(tChan *ChP)
{
    WRITEL(CHAN_STATUS_RCV_OVERFLOW, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    WRITEL(CHAN_STATUS_BREAK_DETECTED, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    WRITEL(CHAN_STATUS_FRAMING_ERROR, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    WRITEL(CHAN_STATUS_PARITY_ERROR, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return 0;
}


/*******************************************************************
 *  Channel status functions (offset 0x10).  Bits 0-7 are read only.  Bits 8-15
 *  must be cleared if set by writing a '1' to them.  Exception - Priority Transmit
 *  FIFO Full bit is read only.
 *  Returns:  1 if yes, else 0
 */
int uPriorityBufferFull(tChan *ChP)
{
    return READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & CHAN_STATUS_PRIORITY_BUFFER_FULL;
}

int uChanStatusRcvOverflow(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & CHAN_STATUS_RCV_OVERFLOW))
	WRITEL(CHAN_STATUS_RCV_OVERFLOW, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uChanStatusCompare2(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & CHAN_STATUS_COMPARE2)) 
	WRITEL(CHAN_STATUS_COMPARE2, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uChanStatusCompare1(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & CHAN_STATUS_COMPARE1))
	WRITEL(CHAN_STATUS_COMPARE1, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uChanStatusBreakDetected(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & CHAN_STATUS_BREAK_DETECTED))
	WRITEL(CHAN_STATUS_BREAK_DETECTED, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uChanStatusFramingError(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & CHAN_STATUS_FRAMING_ERROR))
	WRITEL(CHAN_STATUS_FRAMING_ERROR, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uChanStatusParityError(tChan *ChP)
{
    int ret;

    if ((ret = READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & CHAN_STATUS_PARITY_ERROR))
	WRITEL(CHAN_STATUS_PARITY_ERROR, ChP->pBaseRegister + INT_ID_CHAN_STATUS);
    return ret;
}

int uRiActive(tChan *ChP)
{
    return READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & CHAN_STATUS_RI_ACTIVE;
}

int uCtsActive(tChan *ChP)
{
    return READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & CHAN_STATUS_CTS_ACTIVE;
}

int uDsrActive(tChan *ChP)
{
    return READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & CHAN_STATUS_DSR_ACTIVE;
}

int uCdActive(tChan *ChP)
{
    return READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & CHAN_STATUS_CD_ACTIVE;
}

int uXmitFifoEmpty(tChan *ChP)
{
    return READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) &  CHAN_STATUS_XMIT_FIFO_EMPTY;
}

int uXmitShiftRegEmpty(tChan *ChP)
{
    return READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) &  CHAN_STATUS_XMIT_SHIFT_REG_EMPTY;
}

int uDtrActive(tChan *ChP)
{
    return READL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL) & SET_DTR_ACTIVE;
}

int uRtsActive(tChan *ChP)
{
    return READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) & CHAN_STATUS_RTS_IN_STATUS;
}

int uRcvDataAvail(tChan *ChP)
{
    return READL(ChP->pBaseRegister + INT_ID_CHAN_STATUS) &  CHAN_STATUS_RCV_DATA_AVAIL;
}

/*****************************************************************************
 *  Gets the bit in the Interrupt Status register corresponding to a channel (serial port).
 *  Inputs:   CtlP pointer to controller struct, port number
 *  Returns:  1 if set, 0 else
 */
int uGetInterruptStatus(tUnity *CtlP, unsigned long port_number)
{
    if (port_number < 16) {
	return READL(CtlP->pBasePciMemory + CHANNEL_INT_STATUS) & Bit(port_number);
    }
    else {
	if (CtlP->has_2nd_asic)
	    return READL(CtlP->pBasePciMemory + 0x1000 + CHANNEL_INT_STATUS) & Bit(port_number - 16);
	else
	    return 0;
    }
}

/*************************************************************************
 *  Enables/disables channel interrupts by clearing/setting the bit in the Interrupt Mask
 *  register corresponding to the channel.  
 *  Inputs:   ChP pointer to the channel struct
 */
void uEnChannelInterrupt(tChan *ChP)
{
    if (ChP->chan < 16) 
	SETBITW(ChP->pBaseRegister + CHANNEL_INT_MASK, Bit(ChP->chan));
    else 
	SETBITW(ChP->pBaseRegister + CHANNEL_INT_MASK, Bit(ChP->chan - 16));
}

void uDisChannelInterrupt(tChan *ChP)
{
    if (ChP->chan < 16) 
	CLEARBITW(ChP->pBaseRegister + CHANNEL_INT_MASK, Bit(ChP->chan));
    else 
	CLEARBITW(ChP->pBaseRegister + CHANNEL_INT_MASK, Bit(ChP->chan - 16));
}

/*  ********************************************************************
 *  RCV_DATA_BYTE_STATUS Functions 
 *  Inputs:  The 16 bit byte|status read from RCV_DATA_BYTE_STATUS register (offset 0x0008) 
 *  Returns: 1 if set, else 0
 */
int uRcvByteParityError(unsigned short bytestatus)
{
    return bytestatus & RCV_BYTE_PARITY_ERROR;
}

int uRcvByteRcvOverrun(unsigned short bytestatus)
{
    return bytestatus & RCV_BYTE_RCV_OVERRUN;
}

int uRcvByteFramingError(unsigned short bytestatus)
{
    return bytestatus & RCV_BYTE_FRAMING_ERROR;
}

int uRcvByteBreakDetected(unsigned short bytestatus)
{
    return bytestatus & RCV_BYTE_BREAK_DETECTED;
}

int uRcvByteAddressByte(unsigned short bytestatus)
{
    return bytestatus & RCV_BYTE_ADDRESS_BYTE;
}

int uRcvByteTimeoutDetected(unsigned short bytestatus)
{
    return bytestatus & RCV_BYTE_TIMEOUT_DETECTED;
}


/**********************************************************************
 *   Enables/disables the XANY function of the receive processor on a port. 
 */
void uEnXany(tChan *ChP)
{
    WRITEB(URP_XANY_ENABLE, ChP->ucodeAddr + URP_XANY);
}

void uDisXany(tChan *ChP)
{
    WRITEB(URP_XANY_DISABLE, ChP->ucodeAddr + URP_XANY);
}

/******************************************************************************
 *   Enables/disables the receive processor transmitter on a port. 
 *   This is the Set XOFF and Set XON functions used in the Windows driver.  
 *   This puts the port in the XOFF or XON state, assuming software flow control is enabled.
 */
void uSetXon(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + UART_CTRL, RCV_PROC_XMIT_EN);
}

void uSetXoff(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + UART_CTRL, RCV_PROC_XMIT_EN);
}

/************************************************************************
 *   Returns the Transmitter XON/XOFF state when software flow control is active .
 *   Returns:  0 if XOFF'd, 1 if XON'd.
 */
int uXonXoffState(tChan *ChP)
{
    return READL(ChP->pBaseRegister + UART_CTRL) & RCV_PROC_XMIT_EN;
}


/***************************************************************************
 *   Enables/disables the One Stop Bit function on a port. 
 */
void uEnOneStopBit(tChan *ChP)
{
    SETBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, ONE_STOP_BIT_EN);
}

void uDisOneStopBit(tChan *ChP)
{
    CLEARBITL(ChP->pBaseRegister + XMIT_CTRL_RCV_CTRL, ONE_STOP_BIT_EN);
}

/***************************************************************************
 *   Loads the channel's Receive Processor microcode.
 */
void uLoadUcode(tChan *ChP)
{
    int i;

    for (i = 0; i < UCODESIZE; i++) {
	WRITEB(uCode[i], ChP->ucodeAddr + i );
    }
}


/*************************************************************************  
 *   Performs a global reset.  Do not need to clear the bit after setting it, or 
 *   wait after.  (BKA: not true!  we definitely need a delay, clear...maybe)
 *   Inputs:   CtlP pointer to controller struct.
 */
void uGlobalReset(tUnity *CtlP)
{
    WRITEW(1, CtlP->pBasePciMemory + GLOBAL_COMMAND);
    if (CtlP->has_2nd_asic) 
	WRITEW(1, CtlP->pBasePciMemory + 0x1000 + GLOBAL_COMMAND);
}

//===============================================================================================
